-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/_getdependencies.sql
-- SID: 1.21.2.14
-- Date Modified: 05/01/06 15:09:09
--
-- Module Description
--   Defines the _GetDependencies stored procedure

USE Object
GO

---------------------------------------------------------------------------------------------------
-- _GetDependencies
PRINT 'Creating procedure _GetDependencies'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = '_GetDependencies' AND type = 'P')
  DROP PROCEDURE _GetDependencies
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/_getdependencies.sql
-- SID: 1.21.2.14
-- Date Modified: 05/01/06 15:09:09
-- Procedure used by 'Business Impact' function to return an inverted hierarchy
-- where calling object is on the top, and all objects that are linked to it from
-- source links are shown underneath it.
CREATE PROCEDURE _GetDependencies
  @cid            ClassID,
  @id             ObjID,
  @securityLevel  INT = 0,
  @summarize      TINYINT = 0,  -- If 1 then results are returned in summary form
  @verbose        TINYINT = 0
AS
  SET NOCOUNT ON
  
  -- Temp table holding all generations of objects being collected
  CREATE TABLE #dependencies
  (
    src_cid                   ClassID         NOT NULL,
    src_id                    ObjID           NOT NULL,
    dst_cid                   ClassID         NOT NULL,                                             
    dst_id                    ObjID           NOT NULL,
    phy_cid                   ClassID         NOT NULL,
    phy_id                    ObjID           NOT NULL,
    gen                       INT             NOT NULL,
    node                      NVARCHAR(255)   NOT NULL
  )

  CREATE TABLE #dependencies2
  (
    src_cid                   ClassID         NOT NULL,
    src_id                    ObjID           NOT NULL,
    dst_cid                   ClassID         NOT NULL,
    dst_id                    ObjID           NOT NULL,
    phy_cid                   ClassID         NOT NULL,
    phy_id                    ObjID           NOT NULL,
    gen                       INT             NOT NULL,
    node                      NVARCHAR(255)   NOT NULL,
                                             
    keep                      INT             NOT NULL
  )

  -- Temp table for holding the objects collected in one generation. Needed to generate the IDENTITY
  -- values for the node ids.
  CREATE TABLE #gen
  (
    src_cid                   ClassID         NOT NULL,
    src_id                    ObjID           NOT NULL,
    dst_cid                   ClassID         NOT NULL,
    dst_id                    ObjID           NOT NULL,
    gen                       INT             NOT NULL,
    nodeid                    INT             NOT NULL IDENTITY(1,1)
  )

  -- Temp table for holding the objects collected in one generation. Needed to generate the IDENTITY
  -- values for the node ids.
  CREATE TABLE #gen1
  (
    src_cid                   ClassID         NOT NULL,
    src_id                    ObjID           NOT NULL,
    dst_cid                   ClassID         NOT NULL,
    dst_id                    ObjID           NOT NULL,
    gen                       INT             NOT NULL,
    nodeid                    INT             NOT NULL IDENTITY(1,1),
    link_type                 ClassID         NOT NULL
  )

  CREATE TABLE #work
  (
    src_cid                   ClassID         NOT NULL,
    src_id                    ObjID           NOT NULL,
    dst_cid                   ClassID         NOT NULL,
    dst_id                    ObjID           NOT NULL,
    phy_cid                   ClassID         NOT NULL,
    phy_id                    ObjID           NOT NULL,
    gen                       INT             NOT NULL,
    node                      NVARCHAR(255)   NOT NULL
  )

  CREATE TABLE #nodes
  (
    nodeid                    NVARCHAR(255)   NOT NULL,
    generation                INT             NOT NULL,
    cid                       ClassID         NOT NULL,
    id                        ObjID           NOT NULL,
    label                     NVARCHAR(255)   NOT NULL,
    name                      ObjName         NOT NULL,
    alertstateid              INT             NOT NULL,
    resid                     NVARCHAR(8)     NOT NULL,
    hasownershipnote          TINYINT         NOT NULL,
    priorityid                INT             NOT NULL,
    numchildren               INT             NOT NULL,
    phy_cid                   ClassID         NOT NULL,
    _flags                    INT             NOT NULL,
    _Desc                     NVARCHAR(255)   NOT NULL,
    _CurrentStateID           INT             NOT NULL DEFAULT 0,
    _DesiredStateID           INT             NOT NULL DEFAULT 0,
    _LinkedCID                ClassID         NULL DEFAULT NULL
  )

  DECLARE
    @cname                    NVARCHAR(24),
    @crl_id                   ObjID,
    @crl_usr_id               ObjID,
    @DEFAULT_GEN              INT,
    @del_flag                 INT,
    @err                      INT,
    @flag_direct_containment  TINYINT,
    @gen                      INT,
    @gen_str                  NVARCHAR(9),
    @grlidpathstr             NVARCHAR(32),
    @grl_id                   ObjID,
    @grl_usr_id               ObjID,
    @i                        INT,
    @idpathstr                NVARCHAR(32),
    @inodeid                  NVARCHAR(2000),
    @ins_count                INT,
    @linked_cid               ClassID,
    @linked_id                ObjID,
    @maxgen                   INT,
    @MaxRows                  INT,
    @now                      NVARCHAR(40),
    @numrows                  INT,
    @parentcid                ClassID,
    @parentid                 ObjID,
    @prev_cid                 ClassID,
    @rc                       INT,
    @rc2                      INT,
    @sql                      NVARCHAR(500),
    @start                    INT,
    @top                      INT,
    @totalrows                INT,
    @usrgrp                   NVARCHAR(255),
    @usrid                    NVARCHAR(255),
    @work_id                  ObjID,
    @work_node                NVARCHAR(255)

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetDependencies: @cid=''%s'', @id=%d, @securityLevel=%d: Starting', 0, 1, @now, @cid, @id, @securityLevel) WITH NOWAIT
  END

  -- Map NULL parameters to defaults  
  IF @securityLevel IS NULL
    SET @securityLevel = 0
  IF @summarize IS NULL
    SET @summarize = 0

  -- Lookup the maximum number of rows to return
  SELECT @MaxRows = CONVERT(INT, propertyvalue) 
  FROM SystemConfiguration WITH (NOLOCK)
  WHERE groupname = '_GetDescendents' -- This is NOT a mistake. We use the same property that _GetDescendents uses
    AND propertyname = 'RETURN_LIMIT'
  IF @MaxRows IS NULL OR @MaxRows < 0
    SET @MaxRows = 3000

  -- Lookup the maximum number of generations that will be searched
  SELECT @DEFAULT_GEN = CONVERT(INT, propertyvalue)
  FROM SystemConfiguration WITH (NOLOCK)
  WHERE groupname = '_GetDependencies'
    AND propertyname = 'DEFAULT_GENERATION'
  IF @DEFAULT_GEN IS NULL OR @DEFAULT_GEN < 0
    SET @DEFAULT_GEN = 6

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetDependencies: RETURN_LIMIT=%d DEFAULT_GENERATION=%d', 0, 1, @now, @MaxRows, @DEFAULT_GEN) WITH NOWAIT
  END

  -- If the calling object is an LOB then we resolve the LOB to the physical object before starting
  IF @cid = 'LOB'
  BEGIN
    SELECT @linked_cid = phy_cid, @linked_id = phy_id
    FROM lob_link AS l WITH (NOLOCK)
    WHERE l.src_id = 0 AND l.dst_id = @id
      AND l.phy_cid IS NOT NULL
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: [%s-%d] is linked to [%s-%d]', 0, 1, @now, @cid, @id, @linked_cid, @linked_id) WITH NOWAIT
    END
    IF @linked_cid IS NOT NULL AND @linked_cid <> 'LOB'
      SELECT @cid = @linked_cid, @id = @linked_id
  END

  -- Seed table with row for object and its immediate links
  SET @gen = 0
  SET @rc = 0
  SET @del_flag=0
  SET @ins_count=0
  INSERT INTO #dependencies VALUES ( @cid, @id, '', 0, @cid, @id, @gen, '00001' )
  SET @gen = @gen + 1

  -- If the user has permission to view the physical tree, or if the object is an LOB
  -- then populate #gen with the containment, parent_child, and propagation links to the object
  IF @securityLevel <> 0 OR @cid = 'LOB'
  BEGIN
    INSERT INTO #gen
      SELECT src_cid, src_id, dst_cid, dst_id, @gen 
      FROM link AS l WITH (NOLOCK)
        JOIN link_type AS lt WITH (NOLOCK) ON lt.link_type=l.link_type
      WHERE l.dst_cid=@cid AND l.dst_id=@id AND l.deleted=0
        AND (lt.flag_containment=1 OR lt.flag_parent_child=1 OR lt.flag_propagation=1)
        AND l.src_cid NOT IN ('LOBC', 'BUSC')
        AND l.dst_cid <> 'BATC'  --exclude the physical linked BATC under OS
        AND NOT (l.link_type = 'GENL' AND l.src_cid = 'LNKX')
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: Found %d object(s) at generation %d', 0, 1, @now, @rc, @gen) WITH NOWAIT
      IF @verbose >= 2
        SELECT '1' AS step, * FROM #gen
    END

    -- If this is a directly contained class we have look for the parent information in the object's record
    SELECT @cname = cname, @flag_direct_containment=flag_direct_containment FROM obj_class WITH (NOLOCK) WHERE cid=@cid
    IF @flag_direct_containment = 1
    BEGIN
      SET @sql = 'SELECT @parentcid = _ParentCID, @parentid = _ParentID FROM ' + @cname + '_V WITH (NOLOCK) WHERE id = ' + CAST(@id AS NVARCHAR(9))
      EXEC sp_executesql @sql, N'@parentcid ClassID OUTPUT, @parentid ObjID OUTPUT',
        @parentcid = @parentcid OUTPUT,
        @parentid  = @parentid OUTPUT
      INSERT INTO #gen
        VALUES ( @parentcid, @parentid, @cid, @id, @gen )
      SELECT @err =@@ERROR, @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: _GetDependencies: Found %d directly contained object(s) at generation %d', 0, 1, @now, @rc, @gen) WITH NOWAIT
        IF @verbose >= 2
          SELECT '2' AS step, * FROM #gen
      END
    END
  END -- IF @securityLevel <> 0 OR @cid = 'LOB'

  -- Add shortcuts linked to target
  INSERT INTO #gen
  SELECT l.src_cid, l.src_id, l.dst_cid, l.dst_id, @gen
  FROM link AS l WITH (NOLOCK)
  WHERE l.dst_cid = @cid AND l.dst_id = @id
    AND l.link_type = 'LOBL' AND l.deleted = 0
    AND l.dst_cid = 'LOB' AND l.src_cid = 'LOB'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetDependencies: Found %d shortcut(s)', 0, 1, @now, @rc) WITH NOWAIT
    IF @verbose >= 2
      SELECT '3.1' AS step, * FROM #gen
  END

  -- Find LOBs linked to non-LOB physical resource
  INSERT INTO #gen 
    SELECT DISTINCT l1.src_cid, l1.src_id, @cid, @id, @gen 
    FROM link AS l1 WITH (NOLOCK)
      JOIN link AS l2 WITH (NOLOCK) ON l2.dst_cid = l1.src_cid AND l2.dst_id = l1.src_id
    WHERE 
      l1.link_type = 'LOBL'
      AND l1.dst_cid = @cid
      AND l1.dst_id = @id
      AND l1.deleted = 0
      AND l2.link_type = 'LOBC'
      AND l2.deleted = 0
      AND l2.src_cid NOT IN ('LOBC', 'BUSC')
      AND l1.dst_cid <> 'LOB'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetDependencies: Found %d LOB(s) linked to non-LOB resources', 0, 1, @now, @rc) WITH NOWAIT
    IF @verbose >= 2
      SELECT '4' AS step, * FROM #gen
  END

  -- If this is a shortcut then treat target as belonging to first generation of dependents
  IF @cid = 'LOB' AND @linked_cid = 'LOB'
  BEGIN
    INSERT INTO #gen
      SELECT l2.dst_cid, l2.dst_id, l2.src_cid, l2.src_id, @gen
      FROM link AS l1 WITH (NOLOCK)
        JOIN link AS l2 WITH (NOLOCK) ON l2.dst_cid = l1.dst_cid AND l2.dst_id = l1.dst_id
      WHERE l1.link_type = 'LOBL'
        AND l1.src_cid   = @cid
        AND l1.src_id    = @id
        AND l1.dst_cid   = @linked_cid
        AND l1.deleted   = 0
        AND l2.link_type = 'LOBC'
        AND l2.deleted   = 0
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: Found %d LOB(s) linked to target LOB', 0, 1, @now, @rc) WITH NOWAIT
      IF @verbose >= 2
        SELECT '4.1' AS step, * FROM #gen
    END
  END

  -- Now get any shortcut LOBs that are parents
  INSERT INTO #gen
    SELECT l3.src_cid, l3.src_id, l2.dst_cid, l2.dst_id, @gen 
    FROM link AS l1 WITH (NOLOCK)
      JOIN link AS l2 WITH (NOLOCK) ON l2.dst_cid = l1.src_cid AND l2.dst_id = l1.src_id
      JOIN link AS l3 WITH (NOLOCK) ON l3.dst_cid = l2.src_cid AND l3.dst_id = l2.src_id
    WHERE
      l1.link_type = 'LOBL'
      AND l1.dst_cid = @cid
      AND l1.dst_id = @id
      AND l1.deleted = 0
      AND l2.link_type LIKE 'LOB[CL]'
      AND l2.deleted = 0
      AND l2.src_cid NOT IN ('LOBC', 'BUSC')
      AND l3.deleted = 0
      AND l3.link_type = 'LOBL'
      AND l3.src_cid = 'LOB'
      AND l3.dst_cid = 'LOB'
      AND l1.dst_cid = 'LOB'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetDependencies: Found %d shortcut(s) that are parents', 0, 1, @now, @rc) WITH NOWAIT
    IF @verbose >= 2
      SELECT '5' AS step, * FROM #gen
  END

  INSERT INTO #dependencies
    SELECT g.src_cid, g.src_id, g.dst_cid, g.dst_id, g.src_cid, g.src_id, g.gen, 
      '00001.' + REPLICATE('0', 5-LEN(CONVERT(NVARCHAR(9), nodeid))) + CONVERT(NVARCHAR(9), nodeid)
    FROM #gen AS g
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetDependencies: Total number of dependencies: %d', 0, 1, @now, @rc) WITH NOWAIT
    IF @verbose >= 2
      SELECT * FROM #dependencies
  END

  -- Now traverse up the physical and logical trees

  -- check @gen with @default_gen
  WHILE @rc > 0 AND @gen < @DEFAULT_GEN
  BEGIN
    -- Get the objects for the previous generation
    SET @gen = @gen + 1
    TRUNCATE TABLE #gen

    -- Follow propagation links
    INSERT INTO #gen
      SELECT DISTINCT l.src_cid, l.src_id, l.dst_cid, l.dst_id, @gen
      FROM link AS l WITH (NOLOCK)
        JOIN #dependencies AS prev ON l.dst_cid = prev.src_cid AND l.dst_id = prev.src_id
        JOIN link_type AS lt WITH (NOLOCK) ON lt.link_type = l.link_type
      WHERE
        prev.gen = @gen - 1
        AND l.src_cid NOT IN ('BUSC', 'LOBC')
        AND l.deleted = 0
        AND (lt.flag_containment=1 OR lt.flag_parent_child=1 OR lt.flag_propagation=1)
        AND l.dst_cid <> 'BATC'  --exclude the physical linked BATC under OS
        AND NOT (l.link_type = 'ASHL' AND l.src_cid = 'AGGR')
        AND NOT (l.link_type = 'GENL' AND l.src_cid = 'LNKX')
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: Found %d object(s) at generation %d', 0, 1, @now, @rc, @gen) WITH NOWAIT
      IF @verbose >= 2
        SELECT '6' AS step, * FROM #gen
    END

    -- Follow LOBL links to find shortcut LOB parents
    INSERT INTO #gen
      SELECT DISTINCT l2.src_cid, l2.src_id, l.dst_cid, l.dst_id, @gen
      FROM #dependencies AS d
        JOIN link AS l  ON l.dst_cid  = d.src_cid AND l.dst_id  = d.src_id
        JOIN link AS l2 ON l2.dst_cid = l.src_cid AND l2.dst_id = l.src_id
      WHERE d.gen = @gen - 1
        AND d.src_cid = 'LOB'
        AND l.deleted  = 0 AND l.src_cid  = 'LOB' AND l.link_type  = 'LOBC'
        AND l2.deleted = 0 AND l2.src_cid = 'LOB' AND l2.link_type = 'LOBL'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: Found %d shortcut parent(s) at generation %d', 0, 1, @now, @rc, @gen) WITH NOWAIT
      IF @verbose >= 2
        SELECT '7' AS step, * FROM #gen
    END

    -- If any of the objects at the previous generation are directly contained we have to look up their parents.
    -- For efficiency we only do this if the security allows the physical hierarchy to be traversed, or the
    -- object is an LOB.
    IF (@securityLevel <> 0 OR @cid = 'LOB') AND EXISTS
    (
      SELECT 1
      FROM #dependencies AS prev JOIN obj_class AS oc ON oc.cid = prev.src_cid AND oc.flag_direct_containment=1
      WHERE gen = @gen - 1
    )
    BEGIN
      SET @gen_str = CAST(@gen AS NVARCHAR)
      DECLARE directc INSENSITIVE CURSOR FOR
        SELECT DISTINCT oc.cid, oc.cname
        FROM #dependencies AS prev JOIN obj_class AS oc ON oc.cid = prev.src_cid AND oc.flag_direct_containment=1
        WHERE prev.gen = @gen - 1
      OPEN directc
      FETCH NEXT FROM directc INTO @prev_cid, @cname
      WHILE @@FETCH_STATUS <> -1
      BEGIN
        SET @sql =
        'INSERT INTO #gen ' +
          'SELECT DISTINCT c._ParentCID, c._ParentID, d.src_cid, d.src_id, ' + @gen_str + ' ' +
        'FROM ' + @cname + '_C AS c ' +
          'JOIN #dependencies AS d ON d.src_cid = ''' + @prev_cid + ''' AND d.src_id = c.id ' +
        'WHERE d.gen = ' + @gen_str + ' - 1 '
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: _GetDependencies: %s', 0, 1, @now, @sql) WITH NOWAIT
        END
        EXEC(@sql)
        SELECT @err = @@ERROR, @rc = @@ROWCOUNT
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: _GetDependencies: Found %d directly contained object(s) of class %s at generation %d', 0, 1, @now, @rc, @prev_cid, @gen) WITH NOWAIT
          IF @verbose >= 2
            SELECT '7.' + @prev_cid AS step, * FROM #gen
        END
        SET @rc = @rc + @@ROWCOUNT
        FETCH NEXT FROM directc INTO @prev_cid, @cname
      END
      DEALLOCATE directc
    END

    INSERT INTO #dependencies
      SELECT g.src_cid, g.src_id, g.dst_cid, g.dst_id, g.src_cid, g.src_id, g.gen,
        prev.node + '.' + REPLICATE('0', 5-LEN(CONVERT(NVARCHAR(9), nodeid))) + CONVERT(NVARCHAR(9), nodeid)
      FROM #dependencies prev, #gen g
      WHERE g.dst_cid = prev.src_cid AND g.dst_id = prev.src_id
        AND prev.gen = @gen-1
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: Seeded %d dependent(s) at generation %d', 0, 1, @now, @rc, @gen) WITH NOWAIT
      IF @verbose >= 2
        SELECT * FROM #dependencies
    END
      
  END -- WHILE @rc > 0 AND @gen < @DEFAULT_GEN

  -- If user is restricted_operator, show LOBs only in his CRL
  -- Else, filter out all CRL resources
  EXEC asisp_getContextUserInfo @usrid OUT, @usrgrp OUT
  EXEC asisp_getCrlIds @crl_id OUT, @crl_usr_id OUT
  EXEC asisp_getGrlIds @grl_id OUT, @grl_usr_id OUT
  SELECT @idpathstr = idpath FROM ObjPathCache WITH (NOLOCK) WHERE cid='LOB' AND id=@crl_id  ---  '%/LOB:' + CAST(@crl_id AS NVARCHAR)
  SELECT @grlidpathstr = idpath FROM ObjPathCache WHERE cid='LOB' AND id=@grl_id 

  IF @usrgrp = N'TBSM_Operators_Restricted' AND @crl_usr_id>0
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: Handling restricted operator filtering', 0, 1, @now) WITH NOWAIT
    END

    SET @i = 1
    SET @start = 1 
    SET @top=1 
    -- make a copy of the #dependencies table to work on 
    INSERT INTO #dependencies2 SELECT *, 0 AS keep FROM #dependencies
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: Copied %d dependent(s) for filtering', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- select * from #dependencies
    DECLARE iCursor CURSOR DYNAMIC SCROLL_LOCKS FOR
      SELECT node FROM #dependencies2 WHERE phy_cid='LOB' AND (phy_id=@crl_usr_id OR phy_id=@grl_usr_id)
    OPEN iCursor
    FETCH NEXT FROM iCursor INTO @inodeid
    WHILE @@FETCH_STATUS <> -1
    BEGIN      
      SET @i = 1
      SET @start = 1 
      SET @top=1 
      WHILE @i > 0
      BEGIN  
        SET @i = CHARINDEX('.',@inodeid,@start)
        IF @i > 0
        BEGIN
          -- PRINT ' found ' + substring(@inodeid, @top, @i-1)
          UPDATE #dependencies2 SET keep=1 WHERE node=SUBSTRING(@inodeid, @top, @i-1)
        END 
        SET @start=@i+1
      END     
loop:
      FETCH NEXT FROM iCursor INTO @inodeid
   
    END -- end of cursor 
    DEALLOCATE iCursor

    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: Finished restricted operator filtering', 0, 1, @now) WITH NOWAIT
    END

    ---Put the data back into the original table to continue
    TRUNCATE TABLE #dependencies
    INSERT INTO #dependencies 
    SELECT 
      n.src_cid,
      n.src_id ,
      n.dst_cid,
      n.dst_id  ,
      n.phy_cid ,
      n.phy_id ,
      n.gen  ,
      n.node 
    FROM #dependencies2 AS n
    WHERE n.keep=1
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: %d dependent(s) remaining after filter', 0, 1, @now, @rc) WITH NOWAIT
    END
  END -- IF @usrgrp = N'TBSM_Operators_Restricted' AND @crl_usr_id>0
  ELSE
  BEGIN
      

    INSERT INTO #work SELECT * FROM #dependencies
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: Copied %d dependent(s) for CRL/GRL filtering', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Delete any objects that have only cache entries that derive from the CRL
    DELETE d
    FROM #dependencies AS d
      LEFT OUTER JOIN ObjPathCache AS o WITH (NOLOCK) ON d.phy_cid = o.cid AND d.phy_id = o.id
        AND (o.idpath NOT LIKE @idpathstr+'/%' OR o.idpath NOT LIKE @grlidpathstr+'/%') 
        AND (o.idpath <> @idpathstr OR o.idpath <> @grlidpathstr)
    WHERE o.cid IS NULL AND d.phy_cid = 'LOB'
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: Deleted %d dependent(s) that are only in CRLs', 0, 1, @now, @rc) WITH NOWAIT
    END

    -- Cursor over the removed 'LOB' entries to look for LOBL links where the source is not in the CRL. 
    -- These entries must also be put back in the list.
    DECLARE workc INSENSITIVE CURSOR FOR
      SELECT phy_id, node 
      FROM #work WHERE node NOT IN ( SELECT node FROM #dependencies ) AND phy_cid = 'LOB' AND (phy_id <> @crl_id OR phy_id <> @grl_id)
      AND phy_id NOT IN ( SELECT dst_id FROM link WHERE link_type = 'LOBC' AND src_cid = 'LOB' AND (src_id = @crl_id OR src_id = @grl_id ) )    
    OPEN workc
    FETCH NEXT FROM workc INTO @work_id, @work_node
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      -- Look at the source id of all LOBLs where the fetched object is the destination. If any of these have
      -- cache entries that don't originate in the CRL, then re-insert the row with the fetched node.
      IF EXISTS
      (
        SELECT 1 
        FROM ObjPathCache AS o
        JOIN link AS l ON l.src_id = o.id AND l.src_cid = o.cid
        WHERE o.cid = 'LOB'
          AND l.link_type = 'LOBL' 
          AND l.dst_cid = 'LOB'
          AND l.dst_id = @work_id
          AND
          (
            idpath NOT LIKE @idpathstr+'/%' OR idpath NOT LIKE @grlidpathstr+'/%'
          )
          AND
          (
            idpath <> @idpathstr OR idpath <> @grlidpathstr
          )
      )
      BEGIN                 
        INSERT INTO #dependencies SELECT * FROM #work WHERE node = @work_node
        SELECT @err = @@ERROR, @rc = @@ROWCOUNT
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: _GetDependencies: Reinserted %d row(s) for LOB %d since it has a non-CRL path', 0, 1, @now, @rc, @work_id) WITH NOWAIT
        END
      END
      ELSE
      BEGIN
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: _GetDependencies: LOB %d does not have a non-CRL path', 0, 1, @now, @work_id) WITH NOWAIT
        END
      END
      
      FETCH NEXT FROM workc INTO @work_id, @work_node
    END
    DEALLOCATE workc

  END -- ELSE

  -- Now we need to resolve all LOBs to their linked physical objects (if they have any). This drives the display of the object type
  UPDATE d SET phy_cid = ll.phy_cid, phy_id = ll.phy_id 
    FROM #dependencies AS d JOIN lob_link AS ll ON d.phy_cid = 'LOB' AND d.phy_id = ll.dst_id
  WHERE
    ll.phy_cid IS NOT NULL
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetDependencies: Resolved %d LOB(s) to their linked objects', 0, 1, @now, @rc) WITH NOWAIT
  END

  SELECT @numrows = COUNT(*) FROM #dependencies
  IF @summarize = 0
  BEGIN
    -- Create the temp table used to display results to the caller. We could collapse this and #dependencies
    -- if there is a performance problem.
    IF @MaxRows = 0 OR @numrows < @MaxRows
    BEGIN
      INSERT INTO #nodes (nodeid, generation, cid, id, label, name, alertstateid, resid, hasownershipnote, priorityid, numchildren, phy_cid, _flags, _Desc, _CurrentStateID, _DesiredStateID)
        SELECT node, gen, src_cid, src_id, label, '', 1, '', 0, 0, 0, phy_cid, 0, '', 0, 0
        FROM #dependencies AS d
          JOIN obj_class AS c ON c.cid = d.phy_cid
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: _GetDependencies: Populated %d row(s) into result set from dependents', 0, 1, @now, @rc) WITH NOWAIT
      END
    END
    ELSE
    BEGIN
      SELECT @maxgen = MAX(gen) FROM #dependencies
      SET @gen = 0
      SET @totalrows = 0
      WHILE @totalrows <= @MaxRows AND @gen <= @maxgen
      BEGIN
        SELECT @numrows = COUNT(*) FROM #dependencies WHERE gen = @gen
        SET @totalrows = @totalrows + @numrows
        IF @totalrows <= @MaxRows
        BEGIN
          INSERT INTO #nodes (nodeid, generation, cid, id, label, name, alertstateid, resid, hasownershipnote, priorityid, numchildren, phy_cid, _flags, _Desc, _CurrentStateID, _DesiredStateID)
            SELECT node, gen, src_cid, src_id, label, '', 1, '', 0, 0, 0, phy_cid, 0, '', 0, 0
            FROM #dependencies AS d
              JOIN obj_class AS c ON c.cid = d.phy_cid
            WHERE d.gen = @gen
          SELECT @err = @@ERROR, @rc = @@ROWCOUNT
          IF @verbose >= 1
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
            RAISERROR('%s: _GetDependencies: Populated %d row(s) into result set from dependents of generation %d, %d total so far', 0, 1, @now, @rc, @gen, @totalrows) WITH NOWAIT
          END
        END
        ELSE
        BEGIN
          IF @verbose >= 1
          BEGIN
            SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
            RAISERROR('%s: _GetDependencies: Stopping at generation %d since adding %d rows at this level would bring total to %d, exceeding row limit of %d', 0, 1, @now, @gen, @numrows, @totalrows, @MaxRows) WITH NOWAIT
          END
        END
        SET @gen = @gen + 1
      END
    END
  END -- IF @summarize = 0
  ELSE
  BEGIN

    IF @numrows > 500
    BEGIN
      CREATE UNIQUE INDEX #dependencies_node ON #dependencies(src_cid, src_id, node)
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: _GetDependencies: Indexed node ids for summarization', 0, 1, @now) WITH NOWAIT
      END
    END

    -- Insert the first node for each dependent
    SET ROWCOUNT @MaxRows    
    INSERT INTO #nodes (nodeid, generation, cid, id, label, name, alertstateid, resid, hasownershipnote, priorityid, numchildren, phy_cid, _flags, _Desc, _CurrentStateID, _DesiredStateID)
      SELECT DISTINCT MIN(d.node), MIN(d.gen), d.src_cid, d.src_id, c.label, '', 1, '', 0, 0, 0, d.phy_cid, 0, '', 0, 0
      FROM #dependencies AS d
        JOIN obj_class AS c ON c.cid = d.phy_cid
      GROUP BY d.src_cid, d.src_id, d.phy_cid, c.label
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: Populated %d distinct row(s) into result set from dependents', 0, 1, @now, @rc) WITH NOWAIT
    END
    
  END -- ELSE of IF @summarize 0  
    
  -- Update the name and resid columns by cursoring over the distinct set of classes represented
  -- in #dependencies

  DECLARE attributes CURSOR FOR
    SELECT DISTINCT d.cid, c.cname
    FROM #nodes AS d
      JOIN obj_class AS c WITH (NOLOCK) ON c.cid = d.cid
  OPEN attributes
  FETCH NEXT FROM attributes INTO @cid, @cname
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC(
    'UPDATE n SET name = v.name, alertstateid=_AlertStateID, priorityid=_PriorityID, hasownershipnote=_HasOwnershipNote, n._flags=v._flags, n._Desc=ISNULL(v._Desc, ''''), n._CurrentStateID = v._CurrentStateID, n._DesiredStateID = v._DesiredStateID ' +
    'FROM #nodes AS n ' +
    'JOIN ' + @cname + '_V AS v ON v.cid = n.cid AND v.id = n.id ' +
    'WHERE n.cid=''' + @cid + ''' '
    )
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: Updated attribute information for %d row(s) of class %s', 0, 1, @now, @rc, @cid) WITH NOWAIT
    END
    FETCH NEXT FROM attributes INTO @cid, @cname
  END
  DEALLOCATE attributes
  
  DECLARE icons CURSOR FOR
    SELECT DISTINCT d.phy_cid, c.cname
    FROM #nodes AS d
      JOIN obj_class AS c WITH (NOLOCK) ON c.cid = d.phy_cid
  OPEN icons
  FETCH NEXT FROM icons INTO @cid, @cname
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    EXEC(
    'UPDATE #nodes SET resid = ISNULL(_LargeIconName, '''') FROM ' + @cname + '_S WHERE phy_cid = ''' + @cid + ''' '
    )
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: _GetDependencies: Updated icon information for %d row(s) of physical class %s', 0, 1, @now, @rc, @cid) WITH NOWAIT
    END
    
    FETCH NEXT FROM icons INTO @cid, @cname
  END
  DEALLOCATE icons

  -- Get the LinkedCID attribute from LineOfBusiness_V
  UPDATE n
  SET _LinkedCID = CASE l._LinkedCID WHEN '' THEN NULL ELSE l._LinkedCID END
  FROM #nodes AS n
    JOIN LineOfBusiness_V AS l ON l.id = n.id
  WHERE n.cid = 'LOB'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetDependencies: Updated LinkedCID for %d row(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  SELECT nodeid, generation, cid, id, label, name, alertstateid, resid, hasownershipnote, priorityid, numchildren, phy_cid, 'deleted' = 0, _flags, _Desc, _CurrentStateID, _DesiredStateID, _LinkedCID
  FROM #nodes
  ORDER BY nodeid
  SET @rc = @@ROWCOUNT
  
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: _GetDependencies: Finished. Number of rows returned = %d', 0, 1, @now, @rc) WITH NOWAIT
  END
  
  RETURN @rc
GO
