-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\asisp_add_ev_delete_triggers.sql
-- SID: 1.2
-- Date Modified: 04/10/21 09:26:30
--
-- Module Description
--   Defines the asisp_add_EV_delete_triggers stored procedure

USE Object
GO

-------------------------------------------------------------------------
-- asisp_add_EV_delete_triggers
PRINT 'Creating procedure asisp_add_EV_delete_triggers'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_add_EV_delete_triggers' AND type = 'P')
  DROP PROCEDURE asisp_add_EV_delete_triggers
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\asisp_add_ev_delete_triggers.sql
-- SID: 1.2
-- Date Modified: 04/10/21 09:26:30
-- Defines the delete trigger(s) for the specified class that insert records
-- into the twh_ResourceDelete table for each resource being deleted
CREATE PROCEDURE asisp_add_EV_delete_triggers
  @cid ClassID,
  @debug_print tinyint = 0
AS
  -- standard transaction logic
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION add_EV_at_delete_trigger
  ELSE
    SAVE TRANSACTION add_EV_at_delete_trigger

  DECLARE @cname NVARCHAR(54), @sql NVARCHAR(4000), @now NVARCHAR(40), @err INT, @trigger sysname
  DECLARE @table sysname

  IF @debug_print >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_add_EV_delete_triggers ''%s'': Starting', 0, 1, @now, @cid) WITH NOWAIT
  END
  
  SELECT @cid = RTRIM(@cid), @cname = RTRIM(cname) FROM obj_class WHERE cid = @cid AND flag_abstract = 0
  IF @cid IS NULL OR @cname IS NULL
  BEGIN
    RAISERROR('asisp_add_EV_delete_triggers: ERROR: "%s" is not a valid class id', 11, 1, @cid)
    ROLLBACK TRANSACTION add_EV_at_delete_trigger
    RETURN -30000
  END

  SET @table = @cname + '_C'
  IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = @table)
  BEGIN
    IF @debug_print >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_add_EV_delete_triggers: "%s" has no class table! No trigger added', 0, 1, @now, @cname) WITH NOWAIT
    END
    IF @tcount = 0
      COMMIT TRANSACTION add_EV_at_delete_trigger
    RETURN 0
  END

  -- Drop trigger on _C table
  SET @trigger = 'tD_' + @cname + '_C_EV'
  IF EXISTS (SELECT 1 FROM sysobjects WHERE name = @trigger AND type = 'TR')
  BEGIN
    IF @debug_print >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_add_EV_delete_triggers: Dropping old trigger %s', 0, 1, @now, @trigger) WITH NOWAIT
    END
    EXEC('DROP TRIGGER ' + @trigger)
    SET @err = @@ERROR
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_add_EV_delete_triggers: Error %d dropping old trigger %s', 3, 1, @now, @err, @trigger) WITH NOWAIT
      ROLLBACK TRANSACTION add_EV_at_delete_trigger
      RETURN -30000
    END
  END

  SET @sql = 
      '-- File: archive\db\sql\procs\asisp_add_ev_delete_triggers.sql'                                        + CHAR(13) + CHAR(10)
    + '-- SID: 1.2'                                                                                           + CHAR(13) + CHAR(10)
    + '-- Date Modified: 04/10/21 09:26:30'                                                                             + CHAR(13) + CHAR(10)
    + '-- Delete trigger for ' + @cname + '_C that inserts into the twh_ResourceDelete table                ' + CHAR(13) + CHAR(10)
    + 'CREATE TRIGGER '+ @trigger + ' ON ' + @cname + '_C FOR DELETE WITH APPEND'                             + CHAR(13) + CHAR(10)
    + 'AS'                                                                                                    + CHAR(13) + CHAR(10)
    + '  DECLARE @rc INT'                                                                                     + CHAR(13) + CHAR(10)
    + '  SET @rc = @@ROWCOUNT'                                                                                + CHAR(13) + CHAR(10)
    + '  IF @rc = 0'                                                                                          + CHAR(13) + CHAR(10)
    + '    RETURN'                                                                                            + CHAR(13) + CHAR(10)
    + '  SET NOCOUNT ON'                                                                                      + CHAR(13) + CHAR(10)
    + '  IF EXISTS (SELECT 1 FROM twh_data WHERE name = ''triggers_enabled'' and value = 1)'                  + CHAR(13) + CHAR(10)
    + '    INSERT INTO twh_ResourceDelete (cid, id)'                                                          + CHAR(13) + CHAR(10)
    + '      SELECT cid, id FROM deleted'                                                                     + CHAR(13) + CHAR(10)

  IF @debug_print >= 2
  BEGIN
    PRINT 'sql: ' + @sql
  END

  EXEC(@sql)       
  SET @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_add_EV_delete_triggers: Error %d creating trigger %s', 0, 1, @now, @err, @trigger) WITH NOWAIT
  END

  IF @tcount = 0
    COMMIT TRANSACTION add_EV_at_delete_trigger

  IF @debug_print >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_add_EV_delete_triggers ''%s'': Finished', 0, 1, @now, @cid) WITH NOWAIT
  END

  RETURN 1
GO
