-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\asisp_add_ev_insert_triggers.sql
-- SID: 1.3.4.10
-- Date Modified: 04/10/21 09:26:28
--
-- Module Description
--   Defines the asisp_add_EV_insert_triggers stored procedure

USE Object
GO

-------------------------------------------------------------------------
-- asisp_add_EV_insert_triggers
PRINT 'Creating procedure asisp_add_EV_insert_triggers'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_add_EV_insert_triggers' AND type = 'P')
  DROP PROCEDURE asisp_add_EV_insert_triggers
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\asisp_add_ev_insert_triggers.sql
-- SID: 1.3.4.10
-- Date Modified: 04/10/21 09:26:28
-- Defines the insert trigger(s) for the specified class that insert records
-- into the evQueueObject table for each attribute that has flag_rule_field set
CREATE PROCEDURE asisp_add_EV_insert_triggers
  @cid ClassID,
  @debug_print tinyint = 0
AS
  -- standard transaction logic
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION add_EV_at_insert_trigger
  ELSE
    SAVE TRANSACTION add_EV_at_insert_trigger

  DECLARE @cname NVARCHAR(54), @flag_consolidated TINYINT, @sql NVARCHAR(4000), @now NVARCHAR(40), @err INT, @trigger sysname
  DECLARE @flag_direct_containment TINYINT

  IF @debug_print >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_add_EV_insert_triggers ''%s'': Starting', 0, 1, @now, @cid) WITH NOWAIT
  END
  
  SELECT @cid = RTRIM(@cid), @cname = RTRIM(cname), @flag_consolidated = flag_consolidated,
         @flag_direct_containment = flag_direct_containment FROM obj_class WHERE cid = @cid AND flag_abstract = 0
  IF @cid IS NULL OR @cname IS NULL
  BEGIN
    RAISERROR('asisp_add_EV_insert_triggers: ERROR: "%s" is not a valid class id', 11, 1, @cid)
    ROLLBACK TRANSACTION add_EV_at_insert_trigger
    RETURN -30000
  END

  -- invoke alobGetAttributeValid with the bMigrate option to set the obj_attr.flag_rule_field
  EXEC alobGetAttributeValid @iClass = @cid, @bMigrate = 1

  DECLARE @table sysname, @attr_trigger TINYINT, @class_trigger TINYINT
  SET @table = @cname + '_A'
  IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = @table)
  BEGIN
    IF @debug_print >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_add_EV_insert_triggers: "%s" has no attribute table! No trigger added', 0, 1, @now, @cname) WITH NOWAIT
    END
    SET @attr_trigger = 0
  END
  ELSE
    SET @attr_trigger = 1

  SET @table = @cname + '_C'
  IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = @table)
  BEGIN
    IF @debug_print >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_add_EV_insert_triggers: "%s" has no class table! No trigger added', 0, 1, @now, @cname) WITH NOWAIT
    END
    SET @class_trigger = 0
    IF @tcount = 0
      COMMIT TRANSACTION add_EV_at_insert_trigger
    RETURN 0
  END
  ELSE
    SET @class_trigger = 1

  -- Drop old trigger(s)

  IF @flag_consolidated = 0 AND @attr_trigger = 1 -- If not consolidated then drop trigger on _A table
  BEGIN
    SET @trigger = 'tI_' + @cname + '_A_EV'
    IF EXISTS (SELECT 1 FROM sysobjects WHERE name = @trigger AND type = 'TR')
    BEGIN
      IF @debug_print >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_add_EV_insert_triggers: Dropping old trigger %s', 0, 1, @now, @trigger) WITH NOWAIT
      END
      EXEC('DROP TRIGGER ' + @trigger)
      SET @err = @@ERROR
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_add_EV_insert_triggers: Error %d dropping old trigger %s', 3, 1, @now, @err, @trigger) WITH NOWAIT
        ROLLBACK TRANSACTION add_EV_at_insert_trigger
        RETURN -30000
      END
    END
  END

  IF @class_trigger = 1
  BEGIN
    -- Drop trigger on _C table
    SET @trigger = 'tI_' + @cname + '_C_EV'
    IF EXISTS (SELECT 1 FROM sysobjects WHERE name = @trigger AND type = 'TR')
    BEGIN
      IF @debug_print >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_add_EV_insert_triggers: Dropping old trigger %s', 0, 1, @now, @trigger) WITH NOWAIT
      END
      EXEC('DROP TRIGGER ' + @trigger)
      SET @err = @@ERROR
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_add_EV_insert_triggers: Error %d dropping old trigger %s', 3, 1, @now, @err, @trigger) WITH NOWAIT
        ROLLBACK TRANSACTION add_EV_at_insert_trigger
        RETURN -30000
      END
    END

    SET @sql = 
      '-- File: archive\db\sql\asisp_add_ev_triggers.sql'                                                     + CHAR(13) + CHAR(10)
    + '-- SID: 1.3.4.10'                                                                                           + CHAR(13) + CHAR(10)
    + '-- Date Modified: 04/10/21 09:26:28'                                                                             + CHAR(13) + CHAR(10)
    + '-- Insert trigger for ' + @cname + '_C that inserts into the evQueueObject table for further evaluation' + CHAR(13) + CHAR(10)
    + 'CREATE TRIGGER '+ @trigger + ' ON ' + @cname + '_C FOR INSERT WITH APPEND'                             + CHAR(13) + CHAR(10)
    + 'AS'                                                                                                    + CHAR(13) + CHAR(10)
    + '  DECLARE @rc INT'                                                                                     + CHAR(13) + CHAR(10)
    + '  SET @rc = @@ROWCOUNT'                                                                                + CHAR(13) + CHAR(10)
    + '  IF @rc = 0'                                                                                          + CHAR(13) + CHAR(10)
    + '    RETURN'                                                                                            + CHAR(13) + CHAR(10)
    + '  SET NOCOUNT ON'                                                                                      + CHAR(13) + CHAR(10)
    + '  DECLARE @cid ClassID, @tran_id UNIQUEIDENTIFIER, @attribute_name NVARCHAR(24)'                       + CHAR(13) + CHAR(10)
    + '  SET @cid = ''' + @cid + ''''                                                                         + CHAR(13) + CHAR(10)
    + '  SET @tran_id = NEWID()'                                                                              + CHAR(13) + CHAR(10)
    +                                                                                                         + CHAR(13) + CHAR(10)
    + '  SET @attribute_name = ''name'''                                                                      + CHAR(13) + CHAR(10)
    + '  INSERT INTO evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
    + '    SELECT @cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, i.name'                                  + CHAR(13) + CHAR(10)
    + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
    + '      inserted AS i,'                                                                                  + CHAR(13) + CHAR(10)
    + '      ev_watch_insert_filter AS w'                                                                     + CHAR(13) + CHAR(10)
    + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
    + '      w.cid = @cid'                                                                                    + CHAR(13) + CHAR(10)
    + '      AND w.attribute_name = @attribute_name'                                                          + CHAR(13) + CHAR(10)
    +                                                                                                         + CHAR(13) + CHAR(10)
    + '  SET @attribute_name = ''ctime'''                                                                     + CHAR(13) + CHAR(10)
    + '  INSERT INTO evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
    + '    SELECT @cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, i.ctime'                                 + CHAR(13) + CHAR(10)
    + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
    + '      inserted AS i,'                                                                                  + CHAR(13) + CHAR(10)
    + '      ev_watch_insert_filter AS w'                                                                     + CHAR(13) + CHAR(10)
    + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
    + '      w.cid = @cid'                                                                                    + CHAR(13) + CHAR(10)
    + '      AND w.attribute_name = @attribute_name'                                                          + CHAR(13) + CHAR(10)
    +                                                                                                         + CHAR(13) + CHAR(10)
    + '  IF EXISTS (SELECT 1 FROM twh_data WHERE name = ''triggers_enabled'' and value = 1) BEGIN'            + CHAR(13) + CHAR(10)
    + '    INSERT INTO twh_ResourceCreate (cid, id, name, description, ctime)'                                + CHAR(13) + CHAR(10)
    + '      SELECT cid, id, name, description, ctime FROM inserted'                                          + CHAR(13) + CHAR(10)

    IF @flag_direct_containment = 1 AND @flag_consolidated = 1
      SET @sql = @sql
    + '    INSERT INTO twh_LinkCreate (link_type, src_cid, src_id, dst_cid, dst_id, ctime)'                   + CHAR(13) + CHAR(10)
    + '      SELECT "DRCT", _ParentCID, _ParentID, cid, id, ctime FROM inserted'                              + CHAR(13) + CHAR(10)

    SET @sql = @sql
    + '  END'                                                                                                 + CHAR(13) + CHAR(10)
  END
  --  Special logic for view based class attributes
  IF @cid = 'TRAN'   
  BEGIN
    SET @sql = @sql + '  SET @attribute_name = ''LastUpdateTime'''                                            + CHAR(13) + CHAR(10)
    + '  INSERT INTO evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
    + '    SELECT @cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, i.mtime' + CHAR(13) + CHAR(10)
    + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
    + '      inserted AS i,'                                                                                  + CHAR(13) + CHAR(10)
    + '      ev_watch_insert_filter AS w'                                                                     + CHAR(13) + CHAR(10)
    + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
    + '      w.cid = @cid'                                                                                    + CHAR(13) + CHAR(10)
    + '      AND w.attribute_name = @attribute_name'                                                          + CHAR(13) + CHAR(10)
    + '      AND w.attribute_name = @attribute_name'                                                          + CHAR(13) + CHAR(10)
    +                                                                                                         + CHAR(13) + CHAR(10)
  END

  IF @flag_consolidated = 0
  BEGIN
    IF @class_trigger = 1
    BEGIN
      IF @debug_print >= 2
        PRINT @sql + 'GO'
      EXEC(@sql)    
      SET @err = @@ERROR
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_add_EV_insert_triggers: Error %d creating trigger %s', 3, 1, @now, @err, @trigger) WITH NOWAIT
        ROLLBACK TRANSACTION add_EV_at_insert_trigger
        RETURN -30000
      END
    END

    -- If the class has no attributes with flag_rule_field set then no trigger is necessary
    IF NOT EXISTS
    (
      SELECT 1
      FROM obj_attr AS a
        JOIN isa_chain AS i ON i.base_cid = a.cid
        JOIN attr_type AS t ON t.atype = a.atype
      WHERE
        i.cid = @cid
        AND t.aname IN ( 'ASIString','BYTE', 'SHORT', 'USHORT', 'LONG', 'ULONG', 'FLOAT', 'DOUBLE', 'ASISystemTime' )
        AND a.flag_rule_field = 1
        AND NOT EXISTS
        (
          SELECT 1
          FROM obj_attr AS ab 
            JOIN isa_chain AS ib ON ib.base_cid = ab.cid
          WHERE
            ib.cid = i.cid
            AND ab.name = a.name
            AND ib.generation < i.generation
            AND ab.flag_rule_field = 0
        )
    )
    BEGIN
      IF @debug_print >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_add_EV_insert_triggers: "%s" has no attributes with flag_rule_field set', 0, 1, @now, @cname) WITH NOWAIT
      END
      IF @tcount = 0
        COMMIT TRANSACTION add_EV_at_insert_trigger
      RETURN 1
    END

    IF @attr_trigger = 1
    BEGIN
      SET @trigger = 'tI_' + @cname + '_A_EV'

      SET @sql = 
        '-- File: archive\db\sql\asisp_add_ev_triggers.sql'                               + CHAR(13) + CHAR(10)
      + '-- SID: 1.3.4.10'                                                                     + CHAR(13) + CHAR(10)
      + '-- Date Modified: 04/10/21 09:26:28'                                                       + CHAR(13) + CHAR(10)
      + '-- Insert trigger for ' + @cname + '_A that inserts into the evQueueObject table for further evaluation' + CHAR(13) + CHAR(10)
      + 'CREATE TRIGGER '+ @trigger + ' ON ' + @cname + '_A FOR INSERT WITH APPEND'       + CHAR(13) + CHAR(10)
      + 'AS'                                                                              + CHAR(13) + CHAR(10)
      + '  DECLARE @rc INT'                                                               + CHAR(13) + CHAR(10)
      + '  SET @rc = @@ROWCOUNT'                                                          + CHAR(13) + CHAR(10)
      + '  IF @rc = 0'                                                                    + CHAR(13) + CHAR(10)
      + '    RETURN'                                                                      + CHAR(13) + CHAR(10)
      + '  SET NOCOUNT ON'                                                                + CHAR(13) + CHAR(10)
      + '  DECLARE @cid ClassID, @tran_id UNIQUEIDENTIFIER, @attribute_name NVARCHAR(24)' + CHAR(13) + CHAR(10)
      + '  SET @cid = ''' + @cid + ''''                                                   + CHAR(13) + CHAR(10)
      + '  SET @tran_id = NEWID()'                                                        + CHAR(13) + CHAR(10)
    END

    IF @flag_direct_containment = 1
      SET @sql = @sql
    + '  IF EXISTS (SELECT 1 FROM twh_data WHERE name = ''triggers_enabled'' and value = 1)'                  + CHAR(13) + CHAR(10)
    + '    INSERT INTO twh_LinkCreate (link_type, src_cid, src_id, dst_cid, dst_id, ctime)'                   + CHAR(13) + CHAR(10)
    + '      SELECT "DRCT", i._ParentCID, i._ParentID, c.cid, i.id, c.ctime FROM inserted AS i'               + CHAR(13) + CHAR(10)
    + '             JOIN ' + @cname + '_C AS c ON c.id = i.id'                                                + CHAR(13) + CHAR(10)

    IF @class_trigger = 0
    BEGIN
      IF @tcount = 0
        COMMIT TRANSACTION add_EV_at_insert_trigger
      IF @debug_print >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: asisp_add_EV_insert_triggers ''%s'': Finished', 0, 1, @now, @cid) WITH NOWAIT
      END
    END
  END

  DECLARE @name NVARCHAR(24), @asql NVARCHAR(4000), @sql2 NVARCHAR(4000), @sql3 NVARCHAR(4000), @total_len INT, @n INT
  DECLARE @asql00 NVARCHAR(4000), @asql01 NVARCHAR(4000), @asql02 NVARCHAR(4000), @asql03 NVARCHAR(4000), @asql04 NVARCHAR(4000),
          @asql05 NVARCHAR(4000), @asql06 NVARCHAR(4000), @asql07 NVARCHAR(4000), @asql08 NVARCHAR(4000), @asql09 NVARCHAR(4000),
          @asql10 NVARCHAR(4000), @asql11 NVARCHAR(4000), @asql12 NVARCHAR(4000), @asql13 NVARCHAR(4000), @asql14 NVARCHAR(4000),
          @asql15 NVARCHAR(4000), @asql16 NVARCHAR(4000), @asql17 NVARCHAR(4000), @asql18 NVARCHAR(4000), @asql19 NVARCHAR(4000)
  SELECT  @asql00 = '', @asql01 = '', @asql02 = '', @asql03 = '', @asql04 = '',
          @asql05 = '', @asql06 = '', @asql07 = '', @asql08 = '', @asql09 = '',
          @asql10 = '', @asql11 = '', @asql12 = '', @asql13 = '', @asql14 = '',
          @asql15 = '', @asql16 = '', @asql17 = '', @asql18 = '', @asql19 = ''


  DECLARE attrc CURSOR FOR
    SELECT DISTINCT a.name
    FROM obj_attr AS a
      JOIN isa_chain AS i ON i.base_cid = a.cid
      JOIN attr_type AS t ON t.atype = a.atype
    WHERE
      i.cid = @cid
      AND t.aname IN ( 'ASIString', 'BYTE', 'SHORT', 'USHORT', 'LONG', 'ULONG', 'FLOAT', 'DOUBLE', 'ASISystemTime' )
      AND a.flag_rule_field = 1
      AND a.flag_no_attr_column = 0 
      AND a.name <> 'Desc'  -- special processing for this fellow as he is actually 'description' on consolidated classes
      AND NOT EXISTS
      (
        SELECT 1
        FROM obj_attr AS ab 
          JOIN isa_chain AS ib ON ib.base_cid = ab.cid
        WHERE
          ib.cid = i.cid
          AND ab.name = a.name
          AND ib.generation < i.generation
          AND ab.flag_rule_field = 0
      )
    ORDER BY
      a.name
  OPEN attrc
  SET @total_len = 0
  FETCH NEXT FROM attrc INTO @name
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SET @asql = CHAR(13) + CHAR(10)
    + '  SET @attribute_name = ''' + @name + ''''                                                             + CHAR(13) + CHAR(10)
    + '  INSERT INTO evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
    + '    SELECT @cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, i._' + @name + ''                        + CHAR(13) + CHAR(10)
    + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
    + '      inserted AS i,'                                                                                  + CHAR(13) + CHAR(10)
    + '      ev_watch_insert_filter AS w'                                                                     + CHAR(13) + CHAR(10)
    + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
    + '      w.cid = @cid'                                                                                    + CHAR(13) + CHAR(10)
    + '      AND w.attribute_name = @attribute_name'                                                          + CHAR(13) + CHAR(10)
    
    SET @total_len = @total_len + LEN(@asql)
    SET @n = @total_len / 3500


    IF @n = 0       SET @asql00 = @asql00 + @asql
    ELSE IF @n = 1  SET @asql01 = @asql01 + @asql
    ELSE IF @n = 2  SET @asql02 = @asql02 + @asql
    ELSE IF @n = 3  SET @asql03 = @asql03 + @asql
    ELSE IF @n = 4  SET @asql04 = @asql04 + @asql
    ELSE IF @n = 5  SET @asql05 = @asql05 + @asql
    ELSE IF @n = 6  SET @asql06 = @asql06 + @asql
    ELSE IF @n = 7  SET @asql07 = @asql07 + @asql
    ELSE IF @n = 8  SET @asql08 = @asql08 + @asql
    ELSE IF @n = 9  SET @asql09 = @asql09 + @asql
    ELSE IF @n = 10 SET @asql10 = @asql10 + @asql
    ELSE IF @n = 11 SET @asql11 = @asql11 + @asql
    ELSE IF @n = 12 SET @asql12 = @asql12 + @asql
    ELSE IF @n = 13 SET @asql13 = @asql13 + @asql
    ELSE IF @n = 14 SET @asql14 = @asql14 + @asql
    ELSE IF @n = 15 SET @asql15 = @asql15 + @asql
    ELSE IF @n = 16 SET @asql16 = @asql16 + @asql
    ELSE IF @n = 17 SET @asql17 = @asql17 + @asql
    ELSE IF @n = 18 SET @asql18 = @asql18 + @asql
    ELSE IF @n = 19 SET @asql19 = @asql19 + @asql
    ELSE
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_add_EV_insert_triggers ''%s'': Total length of attribute update code (%d) exceeded maximum allowed (80000)', 4, 1, @now, @cid, @total_len) WITH NOWAIT
      DEALLOCATE attrc
      ROLLBACK TRANSACTION add_EV_at_insert_trigger
      RETURN -30000
    END
    
   
    FETCH NEXT FROM attrc INTO @name
  END
  DEALLOCATE attrc

  IF @flag_consolidated = 1
  BEGIN
    SET @sql2 =
      '  SET @attribute_name = ''Desc'''                                                                      + CHAR(13) + CHAR(10)
    + '  INSERT INTO evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
    + '    SELECT @cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, i.description'                           + CHAR(13) + CHAR(10)
    + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
    + '      inserted AS i,'                                                                                  + CHAR(13) + CHAR(10)
    + '      ev_watch_insert_filter AS w'                                                                     + CHAR(13) + CHAR(10)
    + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
    + '      w.cid = @cid'                                                                                    + CHAR(13) + CHAR(10)
    + '      AND w.attribute_name = @attribute_name'                                                          + CHAR(13) + CHAR(10)
    +                                                                                                         + CHAR(13) + CHAR(10)
  END
  ELSE -- not consolidated
  BEGIN
    SET @sql2 =
      '  SET @attribute_name = ''Desc'''                                                                      + CHAR(13) + CHAR(10)
    + '  INSERT INTO evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
    + '    SELECT @cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, i._Desc'                                 + CHAR(13) + CHAR(10)
    + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
    + '      inserted AS i,'                                                                                  + CHAR(13) + CHAR(10)
    + '      ev_watch_insert_filter AS w'                                                                     + CHAR(13) + CHAR(10)
    + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
    + '      w.cid = @cid'                                                                                    + CHAR(13) + CHAR(10)
    + '      AND w.attribute_name = @attribute_name'                                                          + CHAR(13) + CHAR(10)
    +                                                                                                         + CHAR(13) + CHAR(10)

  END

  -- special logic for view based attributes
  IF @cid = 'OS'  -- 
  BEGIN
    SET @sql3 = '  SET @attribute_name = ''FTPUser'''                                                         + CHAR(13) + CHAR(10)
    + '  INSERT INTO evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
    + '    SELECT @cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, i._UploadUserID'                         + CHAR(13) + CHAR(10)
    + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
    + '      inserted AS i,'                                                                                  + CHAR(13) + CHAR(10)
    + '      ev_watch_insert_filter AS w'                                                                     + CHAR(13) + CHAR(10)
    + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
    + '      w.cid = @cid'                                                                                    + CHAR(13) + CHAR(10)
    + '      AND w.attribute_name = @attribute_name'                                                          + CHAR(13) + CHAR(10)
    + '  SET @attribute_name = ''FTPPassword'''                                                               + CHAR(13) + CHAR(10)
    + '  INSERT INTO evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
    + '    SELECT @cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, i._UploadPassword'                       + CHAR(13) + CHAR(10)
    + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
    + '      inserted AS i,'                                                                                  + CHAR(13) + CHAR(10)
    + '      ev_watch_insert_filter AS w'                                                                     + CHAR(13) + CHAR(10)
    + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
    + '      w.cid = @cid'                                                                                    + CHAR(13) + CHAR(10)
    + '      AND w.attribute_name = @attribute_name'                                                          + CHAR(13) + CHAR(10)
    +                                                                                                         + CHAR(13) + CHAR(10)
  END
  IF @cid = 'FILE'   
  BEGIN
    SET @sql3 = '  SET @attribute_name = ''Status'''                                                         + CHAR(13) + CHAR(10)
    + '  INSERT INTO evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
    + '    SELECT @cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, case i._Open when "OPE" then "Open" when "CLO" then "Closed" when "" then "----- " else i._Open end + "\" + case i._Enabled when "DIS" then "Disabled" when "ENA" then "Enabled" when "UNE" then "Unenabled" when "" then " -----" else i._Enabled end' + CHAR(13) + CHAR(10)
    + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
    + '      inserted AS i,'                                                                                  + CHAR(13) + CHAR(10)
    + '      ev_watch_insert_filter AS w'                                                                     + CHAR(13) + CHAR(10)
    + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
    + '      w.cid = @cid'                                                                                    + CHAR(13) + CHAR(10)
    + '      AND w.attribute_name = @attribute_name'                                                          + CHAR(13) + CHAR(10)
    +                                                                                                         + CHAR(13) + CHAR(10)
  END
  IF @cid = 'BTCY'   
  BEGIN
    SET @sql3 = '  SET @attribute_name = ''Duration'''                                                         + CHAR(13) + CHAR(10)
    + '  INSERT INTO evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
    + '    SELECT @cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, case i._CurrentStateID when 92 then datediff(mi, i._StartTime, getdate()) when 93 then datediff(mi, i._StartTime, i._StopTime) when 89 then datediff(mi, i._StartTime, i._StopTime) else 0 end' + CHAR(13) + CHAR(10)
    + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
    + '      inserted AS i,'                                                                                  + CHAR(13) + CHAR(10)
    + '      ev_watch_insert_filter AS w'                                                                     + CHAR(13) + CHAR(10)
    + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
    + '      w.cid = @cid'                                                                                    + CHAR(13) + CHAR(10)
    + '      AND w.attribute_name = @attribute_name'                                                          + CHAR(13) + CHAR(10)
    +                                                                                                         + CHAR(13) + CHAR(10)
  END


  IF @debug_print >= 2
  BEGIN
    PRINT 'sql: ' + @sql
    PRINT 'sql00: ' + @asql00
    PRINT 'sql01: ' + @asql01
    PRINT 'sql02: ' + @asql02
    PRINT 'sql03: ' + @asql03
    PRINT 'sql04: ' + @asql04
    PRINT 'sql05: ' + @asql05
    PRINT 'sql06: ' + @asql06
    PRINT 'sql07: ' + @asql07
    PRINT 'sql08: ' + @asql08
    PRINT 'sql09: ' + @asql09
    PRINT 'sql10: ' + @asql10
    PRINT 'sql11: ' + @asql11
    PRINT 'sql12: ' + @asql12
    PRINT 'sql13: ' + @asql13
    PRINT 'sql14: ' + @asql14
    PRINT 'sql15: ' + @asql15
    PRINT 'sql16: ' + @asql16
    PRINT 'sql17: ' + @asql17
    PRINT 'sql18: ' + @asql18
    PRINT 'sql19: ' + @asql19
    PRINT 'SQL2:  ' + @sql2
    PRINT 'SQL3:  ' + @sql3
  END

  EXEC(
  @sql + 
  @asql00 + @asql01 + @asql02 + @asql03 + @asql04 +
  @asql05 + @asql06 + @asql07 + @asql08 + @asql09 +
  @asql10 + @asql11 + @asql12 + @asql13 + @asql14 +
  @asql15 + @asql16 + @asql17 + @asql18 + @asql19 + @sql2 + @sql3
  )       
  SET @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_add_EV_insert_triggers: Error %d creating trigger %s', 0, 1, @now, @err, @trigger) WITH NOWAIT
  END

  IF @tcount = 0
    COMMIT TRANSACTION add_EV_at_insert_trigger

  IF @debug_print >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_add_EV_insert_triggers ''%s'': Finished', 0, 1, @now, @cid) WITH NOWAIT
  END

  RETURN 1
GO
