-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/asisp_add_ev_triggers.sql
-- SID: 1.4.4.4
-- Date Modified: 04/09/29 10:16:45
--
-- Module Description
--   Defines the asisp_add_EV_triggers stored procedure

Use Object
GO

-------------------------------------------------------------------------
-- asisp_add_EV_triggers
PRINT 'Creating procedure asisp_add_EV_triggers'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_add_EV_triggers' AND type = 'P')
  DROP PROCEDURE asisp_add_EV_triggers
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/asisp_add_ev_triggers.sql
-- SID: 1.4.4.4
-- Date Modified: 04/09/29 10:16:45
CREATE PROCEDURE asisp_add_EV_triggers
  @cid ClassID,
  @debug_print TINYINT = 0
AS
  -- Declaration
  DECLARE @iClass           ClassID
  DECLARE @bEnabled         INTEGER
  DECLARE @sCname           NVARCHAR( 48 )
  DECLARE @sSql             NVARCHAR( 2000 )

  -- Initalization
  SET     @iClass          = RTRIM( @cid )
  SET     @bEnabled        = 0
  SET     @sCname          = NULL
  SET     @sSql            = NULL

  -- Determine if @iClass is enabled for ABS -- set @bEnabled
  CREATE TABLE #check_class
  (
    iClass         ClassID           PRIMARY KEY ( iClass ),
    bEnabled       INTEGER           NULL
  )

  INSERT INTO #check_class (  iClass, bEnabled )
  SELECT                     @iClass,     NULL

  EXECUTE alobCheckClassEnabled '#check_class'

  SELECT  @bEnabled = bEnabled FROM #check_class WHERE iClass = @iClass
  DROP TABLE #check_class

  
  IF ( 1 = @bEnabled )
  BEGIN 
    EXEC asisp_add_EV_insert_triggers @cid, @debug_print
    EXEC asisp_add_EV_update_triggers @cid, @debug_print
    EXEC asisp_add_EV_delete_triggers @cid, @debug_print
  END
  ELSE
  BEGIN
    RAISERROR( 'asisp_add_EV_triggers: Triggers not created because %s is not a class that participates in ABS.', 0, 1, @iClass ) WITH NOWAIT

    -- Cleanup any triggers that have been created improperly
    SELECT @sCname = cname FROM obj_class WHERE cid = @iClass
    IF @sCname IS NOT NULL
    BEGIN 
      SET @sSql = '
IF OBJECT_ID( "tI_' + @sCname + '_C_EV") IS NOT NULL 
  DROP TRIGGER tI_' + @sCname + '_C_EV
IF OBJECT_ID( "tI_' + @sCname + '_A_EV") IS NOT NULL 
  DROP TRIGGER tI_' + @sCname + '_A_EV
IF OBJECT_ID( "tU_' + @sCname + '_C_EV") IS NOT NULL 
  DROP TRIGGER tU_' + @sCname + '_C_EV
IF OBJECT_ID( "tU_' + @sCname + '_A_EV") IS NOT NULL 
  DROP TRIGGER tU_' + @sCname + '_A_EV
'
      -- PRINT @sSql 
      EXEC ( @sSql )
    END
  END
GO
