-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\asisp_add_rodm_ev_class_table_delete_trigger.sql
-- SID: 1.3
-- Date Modified: 04/10/21 09:26:31
--
-- Module Description
--   Defines the ABS delete trigger for Rodm objects

USE RODM
GO

-------------------------------------------------------------------------
-- asisp_add_RODM_EV_class_table_delete_trigger
PRINT 'Creating procedure asisp_add_RODM_EV_class_table_delete_trigger'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_add_RODM_EV_class_table_delete_trigger' AND type = 'P')
  DROP PROCEDURE asisp_add_RODM_EV_class_table_delete_trigger
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\asisp_add_rodm_ev_class_table_delete_trigger.sql
-- SID: 1.3
-- Date Modified: 04/10/21 09:26:31
-- Defines the delete trigger(s) for the all RODM classes that insert records
-- into the twh_ResourceDelete table for each resource deleted
CREATE PROCEDURE asisp_add_RODM_EV_class_table_delete_trigger
  @verbose TINYINT = 0
AS
  -- standard transaction logic
  --DECLARE @verbose TINYINT
  --SET @verbose = 2
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION add_RODM_EV_delete_trigger
  ELSE
    SAVE TRANSACTION add_RODM_EV_delete_trigger

  DECLARE @sql NVARCHAR(4000), @now NVARCHAR(40), @err INT, @trigger sysname

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_add_RODM_EV_class_table_delete_trigger: Starting', 0, 1, @now) WITH NOWAIT
  END

  DECLARE @table sysname

  SET @table = 'RODMManagedObject'
  IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = @table)
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_add_RODM_EV_class_table_delete_trigger:  No trigger added', 0, 1, @now) WITH NOWAIT
    END
  END

  --Drop old trigger(s)

  SET @trigger = 'tD_' + @table + '_C_EV'

  SET @sql = 'IF OBJECT_ID( "' + @trigger + '") IS NOT NULL '+
  	     '  DROP TRIGGER ' + @trigger

  IF @verbose >= 2
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s:  asisp_add_RODM_EV_class_table_delete_trigger: DROP sql string is: %s', 0, 1, @now, @sql ) WITH NOWAIT
  END

  EXEC (@sql)

  SET @sql = '-- File: archive\db\sql\procs\asisp_add_rodm_ev_class_table_delete_trigger.sql'               + CHAR(13) + CHAR(10)
  + '-- SID: 1.3'                                                                                           + CHAR(13) + CHAR(10)
  + '-- Date Modified: 04/10/21 09:26:31'                                                                             + CHAR(13) + CHAR(10)
  + '-- Delete trigger for all RODM classes that inserts into twh_ResourceDelete'                           + CHAR(13) + CHAR(10)
  + 'CREATE TRIGGER '+ @trigger + ' ON ' + @table + ' FOR DELETE WITH APPEND'                               + CHAR(13) + CHAR(10)
  + 'AS'                                                                                                    + CHAR(13) + CHAR(10)
  + '  DECLARE @rc INT'                                                                                     + CHAR(13) + CHAR(10)
  + '  SET @rc = @@ROWCOUNT'                                                                                + CHAR(13) + CHAR(10)
  + '  IF @rc = 0'                                                                                          + CHAR(13) + CHAR(10)
  + '    RETURN'                                                                                            + CHAR(13) + CHAR(10)
  + '  SET NOCOUNT ON'                                                                                      + CHAR(13) + CHAR(10)
  + '  IF EXISTS (SELECT 1 FROM Object..twh_data WHERE name = ''triggers_enabled'' and value = 1)'          + CHAR(13) + CHAR(10)
  + '    INSERT INTO Object..twh_ResourceDelete (cid, id)'                                                  + CHAR(13) + CHAR(10)
  + '      SELECT cid, id FROM deleted'                                                                     + CHAR(13) + CHAR(10)

  IF @verbose >= 2
  BEGIN
    PRINT 'sql: '   + @sql
  END

  EXEC(@sql)       
  SET @err = @@ERROR
  IF @err < 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_add_RODM_EV_class_table_delete_trigger: Error %d creating trigger %s', 0, 1, @now, @err, @trigger) WITH NOWAIT
  END

  IF @tcount = 0
    COMMIT TRANSACTION add_RODM_EV_delete_trigger

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_add_RODM_EV_class_table_delete_trigger : Finished', 0, 1, @now) WITH NOWAIT
  END

  RETURN 0
GO
