-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\asisp_add_rodm_ev_class_table_insert_trigger.sql
-- SID: 1.12
-- Date Modified: 04/10/21 09:26:26
--
-- Module Description
--   Defines the ABS insert trigger for Rodm objects

USE RODM
GO

-------------------------------------------------------------------------
-- asisp_add_RODM_EV_class_table_insert_trigger
PRINT 'Creating procedure asisp_add_RODM_EV_class_table_insert_trigger'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_add_RODM_EV_class_table_insert_trigger' AND type = 'P')
  DROP PROCEDURE asisp_add_RODM_EV_class_table_insert_trigger
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\asisp_add_rodm_ev_class_table_insert_trigger.sql
-- SID: 1.12
-- Date Modified: 04/10/21 09:26:26
-- Defines the insert trigger(s) for the all RODM classes that insert records
-- into the evQueueObject table for each attribute that has flag_rule_field set
CREATE PROCEDURE asisp_add_RODM_EV_class_table_insert_trigger
  @verbose TINYINT = 0
AS
  -- standard transaction logic
  --DECLARE @verbose TINYINT
  --SET @verbose = 2
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION add_RODM_EV_insert_trigger
  ELSE
    SAVE TRANSACTION add_RODM_EV_insert_trigger

  DECLARE @sql NVARCHAR(4000), @now NVARCHAR(40), @err INT, @trigger sysname

  DECLARE @name NVARCHAR(24), @asql NVARCHAR(4000), @total_len INT, @n INT
  DECLARE @asql00 NVARCHAR(4000), @asql01 NVARCHAR(4000), @asql02 NVARCHAR(4000), @asql03 NVARCHAR(4000), @asql04 NVARCHAR(4000),
          @asql05 NVARCHAR(4000), @asql06 NVARCHAR(4000), @asql07 NVARCHAR(4000), @asql08 NVARCHAR(4000), @asql09 NVARCHAR(4000),
          @asql10 NVARCHAR(4000), @asql11 NVARCHAR(4000), @asql12 NVARCHAR(4000), @asql13 NVARCHAR(4000), @asql14 NVARCHAR(4000),
          @asql15 NVARCHAR(4000), @asql16 NVARCHAR(4000), @asql17 NVARCHAR(4000), @asql18 NVARCHAR(4000), @asql19 NVARCHAR(4000)
  SELECT  @asql00 = '', @asql01 = '', @asql02 = '', @asql03 = '', @asql04 = '',
          @asql05 = '', @asql06 = '', @asql07 = '', @asql08 = '', @asql09 = '',
          @asql10 = '', @asql11 = '', @asql12 = '', @asql13 = '', @asql14 = '',
          @asql15 = '', @asql16 = '', @asql17 = '', @asql18 = '', @asql19 = ''

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_add_RODM_EV_class_table_insert_trigger: Starting', 0, 1, @now) WITH NOWAIT
  END
  

  DECLARE @table sysname

  SET @table = 'RODMManagedObject'
  IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = @table)
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_add_RODM_EV_class_table_insert_trigger:  No trigger added', 0, 1, @now) WITH NOWAIT
    END
  END

  --Drop old trigger(s)

  SET @trigger = 'tI_' + @table + '_C_EV'

  SET @sql = 'IF OBJECT_ID( "' + @trigger + '") IS NOT NULL '+
  	     '  DROP TRIGGER ' + @trigger

  IF @verbose >= 2
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s:  asisp_add_RODM_EV_class_table_insert_trigger: DROP sql string is: %s', 0, 1, @now, @sql ) WITH NOWAIT
  END

  EXEC (@sql)

  SET @sql = '-- File: archive\db\sql\asisp_add_rodm_ev_triggers.sql'                                       + CHAR(13) + CHAR(10)
  + '-- SID: 1.12'                                                                                           + CHAR(13) + CHAR(10)
  + '-- Date Modified: 04/10/21 09:26:26'                                                                             + CHAR(13) + CHAR(10)
  + '-- Insert trigger for all RODM classes that inserts into the Object..evQueueObject table for further evaluation' + CHAR(13) + CHAR(10)
  + 'CREATE TRIGGER '+ @trigger + ' ON ' + @table + ' FOR INSERT WITH APPEND'                             + CHAR(13) + CHAR(10)
  + 'AS'                                                                                                    + CHAR(13) + CHAR(10)
  + '  DECLARE @rc INT'                                                                                     + CHAR(13) + CHAR(10)
  + '  SET @rc = @@ROWCOUNT'                                                                                + CHAR(13) + CHAR(10)
  + '  IF @rc = 0'                                                                                          + CHAR(13) + CHAR(10)
  + '    RETURN'                                                                                            + CHAR(13) + CHAR(10)
  + '  SET NOCOUNT ON'                                                                                      + CHAR(13) + CHAR(10)
  + '  DECLARE @tran_id UNIQUEIDENTIFIER, @attribute_name NVARCHAR(24)'                                     + CHAR(13) + CHAR(10)
  + '  SET @tran_id = NEWID()'                                                                              + CHAR(13) + CHAR(10)
  +                                                                                                         + CHAR(13) + CHAR(10)
  + '  SET @attribute_name = ''name'''                                                                      + CHAR(13) + CHAR(10)
  + '  INSERT INTO Object..evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
  + '    SELECT w.cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, i.name'                                 + CHAR(13) + CHAR(10)
  + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
  + '      inserted AS i'                                                                                   + CHAR(13) + CHAR(10)
  + '        JOIN Object..ev_watch_insert_filter AS w ON w.cid = i.cid'                                     + CHAR(13) + CHAR(10)
  + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
  + '      w.attribute_name = @attribute_name'                                                              + CHAR(13) + CHAR(10)
  +                                                                                                         + CHAR(13) + CHAR(10)
  + '  SET @attribute_name = ''ctime'''                                                                     + CHAR(13) + CHAR(10)
  + '  INSERT INTO Object..evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
  + '    SELECT w.cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, i.ctime'                                + CHAR(13) + CHAR(10)
  + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
  + '      inserted AS i'                                                                                   + CHAR(13) + CHAR(10)
  + '      JOIN Object..ev_watch_insert_filter AS w ON w.cid = i.cid'                                       + CHAR(13) + CHAR(10)
  + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
  + '      w.attribute_name = @attribute_name'                                                              + CHAR(13) + CHAR(10)
  +                                                                                                         + CHAR(13) + CHAR(10)
  + '  SET @attribute_name = ''Desc'''                                                                      + CHAR(13) + CHAR(10)
  + '  INSERT INTO Object..evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
  + '    SELECT w.cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, i.description'                          + CHAR(13) + CHAR(10)
  + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
  + '      inserted AS i'                                                                                   + CHAR(13) + CHAR(10)
  + '      JOIN Object..ev_watch_insert_filter AS w ON w.cid = i.cid'                                       + CHAR(13) + CHAR(10)
  + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
  + '      w.attribute_name = @attribute_name'                                                              + CHAR(13) + CHAR(10)
  +                                                                                                         + CHAR(13) + CHAR(10)
  + '  IF EXISTS (SELECT 1 FROM Object..twh_data WHERE name = ''triggers_enabled'' and value = 1)'          + CHAR(13) + CHAR(10)
  + '    INSERT INTO Object..twh_ResourceCreate (cid, id, name, description, ctime)'                        + CHAR(13) + CHAR(10)
  + '      SELECT cid, id, name, description, ctime FROM inserted'                                          + CHAR(13) + CHAR(10)
  +                                                                                                         + CHAR(13) + CHAR(10)

  DECLARE attrc CURSOR FOR
    SELECT DISTINCT a.name
    FROM obj_attr AS a
      JOIN isa_chain AS i ON i.base_cid = a.cid
      JOIN attr_type AS t ON t.atype = a.atype
    WHERE
      i.cid = 'APNN' -- Note, use 'APNN' as a model, as it contains all the RODM attributes
      AND t.aname IN ( 'ASIString', 'BYTE', 'SHORT', 'USHORT', 'LONG', 'ULONG', 'FLOAT', 'DOUBLE', 'ASISystemTime' )
      AND a.flag_rule_field = 1
      AND a.flag_no_attr_column = 0  --- ECS temporary
      AND a.name <> 'Desc'  --  must use class level 'description'
      AND a.name <> 'LatestDiscoveryDate'
      AND NOT EXISTS
      (
        SELECT 1
        FROM obj_attr AS ab 
          JOIN isa_chain AS ib ON ib.base_cid = ab.cid
        WHERE
          ib.cid = i.cid
          AND ab.name = a.name
          AND ib.generation < i.generation
          AND ab.flag_rule_field = 0
      )
    ORDER BY
      a.name
  OPEN attrc
  SET @total_len = 0
  FETCH NEXT FROM attrc INTO @name
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SET @asql = CHAR(13) + CHAR(10)
    + '  SET @attribute_name = ''' + @name + ''''                                                             + CHAR(13) + CHAR(10)
    + '  INSERT INTO Object..evQueueObject ( cid, id, attribute_name, tran_id, action, data_type, val_from, val_to )' + CHAR(13) + CHAR(10)
    + '    SELECT w.cid, i.id, @attribute_name, @tran_id, 1, 1, NULL, i._' + @name + ''                       + CHAR(13) + CHAR(10)
    + '    FROM '                                                                                             + CHAR(13) + CHAR(10)
    + '      inserted AS i'                                                                                   + CHAR(13) + CHAR(10)
    + '        JOIN Object..ev_watch_insert_filter AS w ON w.cid = i.cid'                                     + CHAR(13) + CHAR(10)
    + '    WHERE'                                                                                             + CHAR(13) + CHAR(10)
    + '      w.attribute_name = @attribute_name'                                                              + CHAR(13) + CHAR(10)
    
    SET @total_len = @total_len + LEN(@asql)
    SET @n = @total_len / 3500


    IF @n = 0       SET @asql00 = @asql00 + @asql
    ELSE IF @n = 1  SET @asql01 = @asql01 + @asql
    ELSE IF @n = 2  SET @asql02 = @asql02 + @asql
    ELSE IF @n = 3  SET @asql03 = @asql03 + @asql
    ELSE IF @n = 4  SET @asql04 = @asql04 + @asql
    ELSE IF @n = 5  SET @asql05 = @asql05 + @asql
    ELSE IF @n = 6  SET @asql06 = @asql06 + @asql
    ELSE IF @n = 7  SET @asql07 = @asql07 + @asql
    ELSE IF @n = 8  SET @asql08 = @asql08 + @asql
    ELSE IF @n = 9  SET @asql09 = @asql09 + @asql
    ELSE IF @n = 10 SET @asql10 = @asql10 + @asql
    ELSE IF @n = 11 SET @asql11 = @asql11 + @asql
    ELSE IF @n = 12 SET @asql12 = @asql12 + @asql
    ELSE IF @n = 13 SET @asql13 = @asql13 + @asql
    ELSE IF @n = 14 SET @asql14 = @asql14 + @asql
    ELSE IF @n = 15 SET @asql15 = @asql15 + @asql
    ELSE IF @n = 16 SET @asql16 = @asql16 + @asql
    ELSE IF @n = 17 SET @asql17 = @asql17 + @asql
    ELSE IF @n = 18 SET @asql18 = @asql18 + @asql
    ELSE IF @n = 19 SET @asql19 = @asql19 + @asql
    ELSE
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_add_RODM_EV_class_table_insert_trigger: Total length of attribute update code (%d) exceeded maximum allowed (70000)', 4, 1, @now, @total_len) WITH NOWAIT
      DEALLOCATE attrc
      ROLLBACK TRANSACTION add_RODM_EV_insert_trigger
      --RETURN -30000
    END
    
   
    FETCH NEXT FROM attrc INTO @name
  END
  DEALLOCATE attrc



  IF @verbose >= 2
  BEGIN
    PRINT 'sql: '   + @sql
    PRINT 'sql00: ' + @asql00
    PRINT 'sql01: ' + @asql01
    PRINT 'sql02: ' + @asql02
    PRINT 'sql03: ' + @asql03
    PRINT 'sql04: ' + @asql04
    PRINT 'sql05: ' + @asql05
    PRINT 'sql06: ' + @asql06
    PRINT 'sql07: ' + @asql07
    PRINT 'sql08: ' + @asql08
    PRINT 'sql09: ' + @asql09
    PRINT 'sql10: ' + @asql10
    PRINT 'sql11: ' + @asql11
    PRINT 'sql12: ' + @asql12
    PRINT 'sql13: ' + @asql13
    PRINT 'sql14: ' + @asql14
    PRINT 'sql15: ' + @asql15
    PRINT 'sql16: ' + @asql16
    PRINT 'sql17: ' + @asql17
    PRINT 'sql18: ' + @asql18
    PRINT 'sql19: ' + @asql19
  END

 EXEC(
  @sql + 
  @asql00 + @asql01 + @asql02 + @asql03 + @asql04 +
  @asql05 + @asql06 + @asql07 + @asql08 + @asql09 +
  @asql10 + @asql11 + @asql12 + @asql13 + @asql14 +
  @asql15 + @asql16 + @asql17 + @asql18 + @asql19
  )       
  SET @err = @@ERROR
  IF @err < 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_add_RODM_EV_class_table_insert_trigger: Error %d creating trigger %s', 0, 1, @now, @err, @trigger) WITH NOWAIT
  END

  IF @tcount = 0
    COMMIT TRANSACTION add_RODM_EV_insert_trigger

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_add_RODM_EV_class_table_insert_trigger : Finished', 0, 1, @now) WITH NOWAIT
  END

  RETURN 0
GO
