-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_deleteoldevent.sql
-- SID: 1.6.4.4
-- Date Modified: 04/11/14 11:54:57
--
-- Module description:
-- This procedure performs part of the maintenance of the BCP maintained history server setup.
-- It purges rows FROM the event residing on the live SQL server which
--  1) have been copied to the history server by asisp_moveevent.sql, and
--  2) are over seven days old, as a default

USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE id = object_id('asisp_deleteoldevent') AND sysstat & 0xf = 4)
  DROP PROCEDURE asisp_deleteoldevent
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_deleteoldevent.sql
-- SID: 1.6.4.4
-- Date Modified: 04/11/14 11:54:57

-- Module description:
-- This procedure performs part of the maintenance of the BCP maintained history server setup.
-- It purges rows FROM the event residing on the live SQL server which
--  1) have been copied to the history server by asisp_moveevent.sql, and
--  2) are over seven days old, as a default

CREATE PROCEDURE asisp_deleteoldevent
  @daystokeep INT = 7
AS
  DECLARE @tstamp BINARY(8)
  DECLARE @tbsmReporting TINYINT
  DECLARE @tdwReporting TINYINT

  DECLARE @curtime DATETIME
  SET @curtime = CURRENT_TIMESTAMP

  -- The TraditionalTBSM system configuration property is set to 1 by asisp_moveevent
  SELECT @tbsmReporting = ISNULL(CAST(propertyvalue AS TINYINT), 0)
    FROM SystemConfiguration WHERE groupname = 'ReportingComponent' AND propertyname = 'TraditionalTBSM'

  -- The TDWCrystal system configuration property is set to 1 by etl_prepare
  SELECT @tdwReporting = ISNULL(CAST(propertyvalue AS TINYINT), 0)
    FROM SystemConfiguration WHERE groupname = 'ReportingComponent' AND propertyname = 'TDWCrystal'

  IF @tbsmReporting <> 1 AND @tdwReporting <> 1
    RETURN /* nothing to do */

  ELSE IF @tbsmReporting = 1 AND @tdwReporting <> 1
    SELECT @tstamp=lastbcp_tstamp FROM event_lastbcp

  ELSE IF @tbsmReporting <> 1 AND @tdwReporting = 1
    SELECT @tstamp=CAST(CAST(last_xid AS BIGINT) AS TIMESTAMP) FROM twh_ExtractCtrl WHERE name = 'events'

  ELSE IF @tbsmReporting = 1 AND @tdwReporting = 1
    SELECT @tstamp = MIN(tstamp) FROM 
      (SELECT lastbcp_tstamp AS tstamp FROM event_lastbcp
       UNION
       SELECT CAST(CAST(last_xid AS BIGINT) AS TIMESTAMP) AS tstamp FROM twh_ExtractCtrl WHERE name = 'events') AS both

  DELETE event WHERE tstamp<=@tstamp AND DATEDIFF(day,ctime,@curtime)>=@daystokeep
GO
