-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_moveevent.sql
-- SID: 1.5.4.5
-- Date Modified: 04/11/14 11:55:00
--
-- Module description:
-- This procedure performs part of the maintenance of the BCP maintained history server setup.
-- It copies the event TABLE FROM the live server to the history server using BCP.
-- It also maintains a control value used by asisp_deleteoldevent FOR purging old event rows.

--CREATE asisp_moveevent PROCEDURE
USE Object
GO

IF EXISTS (SELECT * FROM sysobjects WHERE id = object_id('asisp_moveevent') AND sysstat & 0xf = 4)
  DROP PROCEDURE asisp_moveevent
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_moveevent.sql
-- SID: 1.5.4.5
-- Date Modified: 04/11/14 11:55:00

-- Module description:
-- This procedure performs part of the maintenance of the BCP maintained history server setup.
-- It copies the event TABLE FROM the live server to the history server using BCP.
-- It also maintains a control value used by asisp_deleteoldevent FOR purging old event rows.

CREATE PROCEDURE asisp_moveevent
  @targetserv sysname,
  @bcpdir NVARCHAR(255),
  @targetdb sysname='History',
  @targettable sysname='eventhist',
  @frompassword NVARCHAR(30)='',
  @topassword   NVARCHAR(30)='',
  @fromsauser   NVARCHAR(30)='sa',
  @tosauser     NVARCHAR(30)='sa'
AS

  DECLARE @rtn INT, @server sysname, @cmd NVARCHAR(255), @targetservlc sysname
  DECLARE @tstamp BINARY(8)
  DECLARE @rc INT

  SET NOCOUNT ON

  -- make sure the SystemConfiguration flag indicates that the traditional reporting
  -- system is being used - this enables asisp_deleteoldevent to clean up based on
  -- the data received by the reporting system
  IF NOT EXISTS (SELECT 1 FROM SystemConfiguration
                   WHERE groupname = 'ReportingComponent'
                   AND   propertyname = 'TraditionalTBSM'
                   AND   propertyvalue = '1')
    EXEC _SetSystemConfiguration @groupname = 'ReportingComponent', 
         @propertyname = 'TraditionalTBSM', 
         @propertyvalue = '1', 
         @propertydatatype = 'TINYINT'

  SELECT @server=LOWER(@@servername)
  SELECT @targetservlc = LOWER(@targetserv)
  IF @frompassword=''
    SELECT @frompassword='sa_' + @server
  IF @topassword=''
    SELECT @topassword='sa_' + @targetservlc
  IF @fromsauser=''
    SELECT @fromsauser='sa'
  IF @tosauser=''
    SELECT @tosauser='sa'

-----
-- Phases I: Copy event TABLE to eventbcp TABLE
--Clear eventbcp
  TRUNCATE TABLE eventbcp

--Load all events INTO eventbcp exit IF none to process
  SELECT @tstamp=lastbcp_tstamp FROM event_lastbcp
  INSERT INTO eventbcp SELECT agent_cid,
                              agent_id,
                              agent_ctime,
                              agent_seqno,
                              obj_cid,
                              obj_id,
                              cid,
                              id,
                              name,
                              ctime,
                              detail,
                              alertstate,
                              priority,
                              state,
                              tstamp FROM event WHERE tstamp>@tstamp
  SELECT @rc = @@ROWCOUNT
  IF @rc=0
    RETURN 0

-----
-- Phase II: bcp the eventbcp TABLE out
  SELECT @cmd='bcp Object..eventbcp_V out "' + @bcpdir + '\temp_event.txt" -S' + @server + ' -U' + @fromsauser + ' -P' + @frompassword + ' -c'
  EXEC @rtn=master..xp_cmdshell @cmd, no_output
  IF @rtn<>0
  BEGIN
    RAISERROR ('Unable to bcp eventbcp table out.', 16, 1)
    RETURN(-3000)
  END

-----
-- Phase III: bcp the eventbcp TABLE INTO history
  SELECT @cmd='bcp ' + @targetdb + '..' + @targettable + ' IN "' + @bcpdir + '\temp_event.txt" -S' + @targetservlc + ' -U' + @tosauser + ' -P' + @topassword + ' -c -b 1000 -m 10000 -e "' + @bcpdir + '\temp_event.err"'
  EXEC @rtn=master..xp_cmdshell @cmd, no_output
  IF @rtn<>0
  BEGIN
    RAISERROR ('Unable to bcp into eventhist table in server=%s, database=%s', 16, 1, @targetservlc, @targetdb)
    RETURN(-3000)
  END

-----
-- Phase IV: UPDATE last BCP timestamp
  UPDATE event_lastbcp SET lastbcp_tstamp=(SELECT max(tstamp) FROM eventbcp)

  RETURN(0)
GO
