-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_processstagedtecstatus.sql
-- SID: 1.8
-- Date Modified: 05/01/06 15:09:06
--
-- Module Description
--   Defines the asisp_processStagedTECStatus stored procedure
USE Object
GO

---------------------------------------------------------------------------------------------------
PRINT 'Creating procedure asisp_processStagedTECStatus'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_processStagedTECStatus' AND type = 'P' )
  DROP PROCEDURE asisp_processStagedTECStatus
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_processstagedtecstatus.sql
-- SID: 1.8
-- Date Modified: 05/01/06 15:09:06
-- Process StagedTECStatus table
CREATE PROCEDURE asisp_processStagedTECStatus
  @maxrows INT     = 1000,
  @verbose TINYINT = 0
AS
  SET NOCOUNT ON

  -- Used to process maxrows from StagedTECStatus
  DECLARE @updates TABLE
  (
    timestamp       BINARY(8)     NOT NULL PRIMARY KEY,
    ctime           DATETIME      NULL,
    noteid          INT           NOT NULL,
    Status          NVARCHAR(16)  NULL,
    ContextUserID   NVARCHAR(255)  NULL
  )

  DECLARE @int_req TABLE
  (
    request_guid       UNIQUEIDENTIFIER  NOT NULL DEFAULT NEWID(),
    noteid             INT               NOT NULL,
    eventcid           NVARCHAR(4)       NOT NULL,
    eventid            INT               NOT NULL,
    Status             NVARCHAR(16)      NULL,
    ContextUserID      NVARCHAR(260)     NULL,
    eventhandle        NVARCHAR(255)     NOT NULL,
    serverhandle       NVARCHAR(255)     NOT NULL,
    datereception      NVARCHAR(255)     NOT NULL,
    tecclass           NVARCHAR(255)     NOT NULL,
    eehost             NVARCHAR(255)     NULL
  )

  DECLARE @int_fields TABLE
  (
    request_guid       UNIQUEIDENTIFIER  NOT NULL,
    field_name         NVARCHAR(240)     NOT NULL,
    field_value        NVARCHAR(2000)    NULL,
    rowno              INT               NOT NULL IDENTITY
  )

  -- Variable declarations
  DECLARE
    @err            INT,
    @rc             INT,
    @now            NVARCHAR(40),
    @max_timestamp  TIMESTAMP,
    @def_admin      NVARCHAR(255),
    @usergroup NVARCHAR(255),
    @tcount         INT

  IF NOT EXISTS (SELECT 1 FROM StagedTECStatus WITH (NOLOCK))
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedTECStatus: no events to process', 0, 1, @now) WITH NOWAIT
    END
    RETURN 0
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: started - @maxrows=%d, @verbose=%d', 0, 1, @now, @maxrows, @verbose) WITH NOWAIT
  END
  
  -- Quick check to see if TEC ownership notification is has been turned off. If so then we simply delete any
  -- records in StagedTECStatus.
  IF NOT EXISTS
  (
    SELECT 1
    FROM SystemConfiguration
    WHERE groupname='TEC_EVENT_OWNERSHIP' AND propertyname='NOTIFY_TEC_OF_TBSM_OWNERSHIP' AND UPPER(propertyvalue)='YES'
  )
  BEGIN
    DELETE FROM StagedTECStatus
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedTECStatus: Error %d deleting from StagedTECStatus', 3, 1, @now, @err) WITH NOWAIT
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedTECStatus: TEC_EVENT_OWNERSHIP.NOTIFY_TEC_OF_TBSM_OWNERSHIP is not set to YES, discarding contents of StagedTECStatus', 0, 1, @now, @rc) WITH NOWAIT
    END
    RETURN @rc
  END

-----------------------------------------------------------------------------------------------------------------
--  read @maxrows from StagedTECStatus into @updates table
-----------------------------------------------------------------------------------------------------------------
  SET ROWCOUNT @maxrows
  INSERT INTO @updates 
    SELECT timestamp, ctime, noteid, Status, ContextUserID
      FROM StagedTECStatus        
      ORDER BY timestamp
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  SET ROWCOUNT 0
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: error %d populating worktable from first %d rows of StagedTECStatus', 3, 1, @now, @err, @maxrows) WITH NOWAIT
    RETURN -30000
  END
  IF @rc = 0
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: asisp_processStagedTECStatus: No events to process - return to main SEL proc', 0, 1, @now) WITH NOWAIT
    END
    RETURN
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: processing %d event(s) in worktable', 0, 1, @now, @rc) WITH NOWAIT
  END

  SELECT @max_timestamp = MAX(timestamp) FROM @updates
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0 OR @max_timestamp IS NULL
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: error selecting MAX(timestamp) from worktable. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 2
    SELECT '@updates' AS '@updates', * FROM @updates

  -- Delete from the worktable any notes whose resolution text matches the TEC_EVENT_OWNERSHIP.TEC_NOTE_RESOLUTION
  -- SystemConfiguration property.
  DELETE FROM e
  FROM @updates AS e
  JOIN Note_A AS n WITH (NOLOCK) ON n.id = e.noteid
  JOIN SystemConfiguration AS s WITH (NOLOCK) ON propertyvalue=CAST(n._Resolution AS NCHAR)
  WHERE s.groupname='TEC_EVENT_OWNERSHIP' AND s.propertyname='TEC_NOTE_RESOLUTION'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  SET ROWCOUNT 0
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: error %d removing TEC-resolved notes from the worktable', 3, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: Removed %d TEC-resolved note(s) from the worktable', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Populate a temporary table containing one row for each extended event that is owned
  -- by a note in the @updates table and which matches the profile for a TEC event, meaning
  -- that it has non-null values in the slots for eventhandle/serverhandle/datereception/tecclass.
  -- Each such event is turned into a separate IntegrationRequest.
  INSERT INTO @int_req
  (
    noteid,
    eventcid,
    eventid,
    Status,
    ContextUserID,
    eventhandle,
    serverhandle,
    datereception,
    tecclass,
    eehost
  )
  SELECT
    e.noteid,
    oe.eventcid,
    oe.eventid,
    e.Status,
    e.ContextUserID,
    ex._ExtEvSlot00, -- eventhandle
    ex._ExtEvSlot01, -- serverhandle
    ex._ExtEvSlot02, -- datereception
    ex._ExtEvSlot04, -- tecclass
    ex._ExtEvSlot05  -- eehost
  FROM @updates AS e
  JOIN OwnedEvent AS oe WITH (NOLOCK) ON oe.noteid = e.noteid
  JOIN ExtendedEvent_A AS ex WITH (NOLOCK) ON ex.eventcid = oe.eventcid AND ex.id = oe.eventid
  WHERE ex._ExtEvSlot00 IS NOT NULL -- eventhandle
    AND ex._ExtEvSlot01 IS NOT NULL -- serverhandle
    AND ex._ExtEvSlot02 IS NOT NULL -- datereception
    AND ex._ExtEvSlot04 IS NOT NULL -- tecclass
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: error %d looking up ExtendedEvent information for owned events', 3, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: Found %d owned ExtendedEvent(s)', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Figure out the ultimate value that gets inserted into the 'administrator' Integration Request
  -- slot. If any record got inserted into StagedTECStatus with a NULL value in ContextUserID
  -- we replace this with the current value returned by asisp_getContextUserID.
  IF EXISTS (SELECT 1 FROM @int_req WHERE ContextUserID IS NULL)
  BEGIN
    EXEC asisp_getContextUserInfo @def_admin OUTPUT, @usergroup OUTPUT
    IF @def_admin IS NULL OR @def_admin = '' SET @def_admin = 'SYSTEM'
    UPDATE @int_req SET ContextUserID = @def_admin WHERE ContextUserID IS NULL OR ContextUserID = ''
  END

  -- If the system is configured to use the display name from the UserProfiles table instead
  -- of the user id then replace this value in the @int_req table.
  IF EXISTS
  (
    SELECT 1 
    FROM SystemConfiguration WITH (NOLOCK)
    WHERE groupname='TEC_EVENT_OWNERSHIP' AND propertyname = 'DISPLAY_NAME' AND propertyvalue = 'YES'
  )
  BEGIN
    UPDATE i SET ContextUserID = ISNULL(p.display_name, i.ContextUserID)
    FROM @int_req AS i
    JOIN UserProfiles AS p WITH (NOLOCK) ON LOWER(p.userid) = LOWER(i.ContextUserID)
  END

  -- Populate temp table with the integration request fields
  INSERT INTO @int_fields(request_guid, field_name, field_value) SELECT request_guid, 'REQUEST_TEC_HOST',       eehost                  FROM @int_req
  INSERT INTO @int_fields(request_guid, field_name, field_value) SELECT request_guid, 'REQUEST_MESSAGE',        'TBSM ownership'        FROM @int_req
  INSERT INTO @int_fields(request_guid, field_name, field_value) SELECT request_guid, 'REQUEST_SERVER_HANDLE',  serverhandle            FROM @int_req
  INSERT INTO @int_fields(request_guid, field_name, field_value) SELECT request_guid, 'REQUEST_EVENT_HANDLE',   eventhandle             FROM @int_req
  INSERT INTO @int_fields(request_guid, field_name, field_value) SELECT request_guid, 'REQUEST_DATE_RECEPTION', datereception           FROM @int_req
  INSERT INTO @int_fields(request_guid, field_name, field_value) SELECT request_guid, 'REQUEST_TEC_CLASS',      tecclass                FROM @int_req
  INSERT INTO @int_fields(request_guid, field_name, field_value) SELECT request_guid, 'REQUEST_STATUS',         Status                  FROM @int_req
  INSERT INTO @int_fields(request_guid, field_name, field_value) SELECT request_guid, 'REQUEST_ADMINISTRATOR',  'TBSM:' + ContextUserID FROM @int_req
  
  -- Standard transaction logic
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION processStagedTECStatus
  ELSE
    SAVE TRANSACTION processStagedTECStatus

  -- Create the IntegrationRequest records. This is equivalent to the _OpenIntegrationRequest
  -- call followed by the update to IntegrationRequest.status that is made in asisp_sendStatusEvent.
  INSERT INTO IntegrationRequest
  (
    request_guid,
    request_type,
    method,
    status,
    open_datetime,
    end_datetime,
    evt_cid,
    evt_id
  )
  SELECT
    request_guid,
    'REQUEST_STATUS_EVENT',
    'REQUEST_ADD',
    200,
    CURRENT_TIMESTAMP,
    NULL,
    NULL,
    NULL
  FROM @int_req
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: error %d inserting into IntegrationRequest', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION processStagedTECStatus
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: Inserted %d row(s) into IntegrationRequest', 0, 1, @now, @rc) WITH NOWAIT
  END

  INSERT INTO IntegrationFields(request_guid, field_name, field_value)
    SELECT request_guid, field_name, field_value
    FROM @int_fields
    ORDER BY request_guid, rowno
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: error %d inserting into IntegrationFields', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION processStagedTECStatus
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: Inserted %d row(s) into IntegrationFields', 0, 1, @now, @rc) WITH NOWAIT
  END
  
-----------------------------------------------------------------------------------------------------------------
--  Finished - clean up
-----------------------------------------------------------------------------------------------------------------
  DELETE FROM StagedTECStatus WHERE timestamp <= @max_timestamp
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: error deleting processed records from StagedTECStatus. Error=%d', 3, 1, @now, @err) WITH NOWAIT
    ROLLBACK TRANSACTION processStagedTECStatus
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: asisp_processStagedTECStatus: completed - %d processed', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRANSACTION processStagedTECStatus

  RETURN @rc
GO
