-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_resetrodm.sql
-- SID: 1.19
-- Date Modified: 04/10/21 11:18:14
--
-- Module Description
--   Defines the asisp_resetRodm stored procedure

USE Object
GO
SET LANGUAGE 'English'
GO
SET QUOTED_IDENTIFIER OFF
SET ANSI_NULLS        OFF
GO


-------------------------------------------------------------------------
-- asisp_resetRodm
PRINT 'Creating procedure asisp_resetRodm'
GO

IF EXISTS ( SELECT * FROM sysobjects WHERE name = 'asisp_resetRodm' AND type = 'P')
  DROP PROCEDURE asisp_resetRodm
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/asisp_resetrodm.sql
-- SID: 1.19
-- Date Modified: 04/10/21 11:18:14
-- Remove all traces of RODM classes and their events from a database
CREATE PROCEDURE asisp_resetRodm
  @verbose  TINYINT = 1
 
AS
  -- Note this procedure does not change the following:
  -- PropagationSlot values for where OwnerCID = rodmClass AND OwnerID = 0
  --   although the rodmClass instances will be removed
  -- ObjectQueues..Pending%
  -- ObjectQueues..Processed%
  -- ObjectQueues..LastProcessed%

  SET NOCOUNT ON

  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_resetRodm
  ELSE BEGIN
    -- We can't run inside an outer transaction because we need to end the transaction
    -- to wait for asynchronous deletes to take place before we continue.
    RAISERROR('asisp_resetRodm cannot be run inside a SQL transaction', 11, 1)
    RETURN -1
    -- obsolete statement: SAVE TRAN asisp_resetRodm
  END

  DECLARE @err INT, @rc INT, @now NVARCHAR(40)


  -- Standard Announcement 
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm', 0, 1, @now ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Delete Container SNNT/GMFF objects, this will automatically: 
  --   1) close open notes
  --   2) negate all messages and exceptions
  --   3) in turn negate all dependent child events
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm Delete SNNT/GMFF objects.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action
  DECLARE @iObject ObjID
  DECLARE @cObject ClassID
  SET @rc = 0
  DECLARE activeSNNT CURSOR FOR 
    SELECT cid, id FROM SNANetwork_V WHERE deleted = 0
    UNION
    SELECT cid, id FROM GMFHSAggregateFolder_V WHERE deleted = 0
  OPEN activeSNNT
  FETCH NEXT FROM activeSNNT INTO @cObject, @iObject
  WHILE @@FETCH_STATUS <> -1
  BEGIN 
    SET @rc = @rc + 1
    -- Do not need to loadpropmatricies because propagation will be restarted.
    -- EXEC asisp_loadpropmatrices SNNT, @iObject  -- Seems to need to be 'fixed' for propagation to be correct on next discovery.
    EXEC @err = delete_obj @cObject, @iObject, @verbose
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR( '%s: asisp_resetRodm: error %d when attempting to delete_obj ''%s'', %d', 11, 1, @now, @err, @cObject, @iObject) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_resetRodm
      RETURN -30000
    END
    FETCH NEXT FROM activeSNNT INTO @cObject, @iObject
  END
  CLOSE activeSNNT
  DEALLOCATE activeSNNT

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm Deleted %d SNNT/GMFF objects.', 0, 1, @now, @rc ) WITH NOWAIT
  END
  -----------------------------------------------------------------------------------------------------
  -- Finish up transaction so that propagation can process the alert state changes
  IF @tcount = 0
    COMMIT TRAN asisp_resetRodm

  -- Need to wait if the SNNT objects have any child events on the enterprise, otherwise keep going

  WAITFOR DELAY '000:00:15'
  WHILE @rc > 0 AND EXISTS (SELECT id FROM SNANetwork_V WHERE deleted = 0
                            UNION
                            SELECT id FROM GMFHSAggregateFolder_V WHERE deleted = 0) BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Waiting for SNNT/GMFF objects to be logically deleted', 0, 1, @now ) WITH NOWAIT
    WAITFOR DELAY '000:00:15'
  END

  -----------------------------------------------------------------------------------------------------
/*
  -- Wait for propagation to finish processing pending RODM events
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Waiting for propagation to finish processing pending RODM events.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action
  WHILE ( SELECT eventno FROM LastProcessedEvent ) < @nCountEvent
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR( '%s: asisp_resetRodm:   Waiting...', 0, 1, @now ) WITH NOWAIT
    END
    WAITFOR DELAY '000:00:01'
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Propagation has finished processing pending RODM events.', 0, 1, @now ) WITH NOWAIT
  END
*/

  ----------------------------------------------------------------------------------------------------
  -- Start transaction again
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN asisp_resetRodm
  ELSE
    SAVE TRAN asisp_resetRodm

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related owned events
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related owned events.', 0, 1, @now ) WITH NOWAIT
  END

  -- UPDATE ownedevent status to 1 ('system closed') for those items currently marked 0 (open) 
  UPDATE OwnedEvent 
  SET
    status = 1
  FROM       OwnedEvent AS oe 
  INNER JOIN isa_chain  AS ic ON ic.cid = oe.parentcid
  WHERE
        (    ( ic.base_cid = 'RDMO' )
          OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
        )
    AND oe.status = 0
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to update all open RODM related owned events', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Action
  DELETE FROM OwnedEvent
  FROM       OwnedEvent AS oe 
  INNER JOIN isa_chain  AS ic ON ic.cid = oe.parentcid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related owned events', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  DELETE FROM NoteDependent
    FROM NoteDependent AS nd
    JOIN isa_chain     AS ic ON ic.cid = nd.depcid
    WHERE
         ( ic.base_cid = 'RDMO' )
      OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to delete all rows in NoteDependent which are related to RODM', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d RODM related owned events.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related Extended Events 
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related Extended Events.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM ExtendedEvent_C (deletion trigger will get ExtendedEvent_A)
  DELETE FROM ExtendedEvent_C
  FROM       ExtendedEvent_C AS eec
  INNER JOIN ExtendedEvent_A AS eea ON eea.id  = eec.id
  INNER JOIN isa_chain       AS ic  ON ic.cid  = eea._ParentCID
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related Extended Events.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from Extended Events.', 0, 1, @now, @rc ) WITH NOWAIT
  END



  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related Exception history
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related Exception history.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM ExceptionHistory_A
  DELETE FROM ExceptionHistory_A
  FROM       ExceptionHistory_A AS eha
  INNER JOIN Exception_A        AS ea  ON ea.id = eha._ParentID
  INNER JOIN isa_chain          AS ic ON ic.cid = ea._ParentCID
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related Execption history.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from Exception history attributes.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -- Action DELETE FROM ExceptionHistory_C
  DELETE FROM ExceptionHistory_C
  FROM            ExceptionHistory_C AS ehc
  WHERE
        NOT EXISTS ( 
          SELECT * 
          FROM ExceptionHistory_A AS eha 
          WHERE
                 eha.id = ehc.id
        )
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related Execption class history.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from Exception class history.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed all RODM related Exception history.', 0, 1, @now, @rc ) WITH NOWAIT
  END


  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related Child Event history
  -- Announcement
/*
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related Child Event history.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM ChildEventHistory_C (deletion trigger will get ChildEventHistory_A)
  DELETE FROM ChildEventHistory_C
  FROM       ChildEventHistory_C AS cehc
  INNER JOIN ChildEventHistory_A AS ceha ON ceha.id = cehc.id
  INNER JOIN ChildEvent_A        AS cea  ON cea.id  = ceha._ParentID
  INNER JOIN isa_chain           AS ic   ON ic.cid  = cea._ChildCID
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related Child Event history.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from Child Event history.', 0, 1, @now, @rc ) WITH NOWAIT
  END
*/
  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related CurrentMessageAlertStatePri
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related CurrentMessageAlertStatePri.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM CurrentMessageAlertStatePri
  DELETE FROM CurrentMessageAlertStatePri
  FROM       CurrentMessageAlertStatePri AS cmasp
  INNER JOIN isa_chain                   AS ic    ON ic.cid  = cmasp.cid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related CurrentMessageAlertStatePri.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from CurrentMessageAlertStatePri.', 0, 1, @now, @rc ) WITH NOWAIT
  END


  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related Messages
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related messages.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM Message_C (deletion trigger will get Message_A)
  DELETE FROM Message_C
  FROM       Message_C    AS mc
  INNER JOIN Message_A    AS ma ON ma.id   = mc.id
  INNER JOIN isa_chain    AS ic ON ic.cid  = ma._ParentCID
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related messages.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from messages.', 0, 1, @now, @rc ) WITH NOWAIT
  END


  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related Exceptions
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related exceptions.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM Exception_C (deletion trigger will get Exception_A)
  DELETE FROM Exception_C
  FROM       Exception_C  AS ec
  INNER JOIN Exception_A  AS ea ON ea.id   = ec.id
  INNER JOIN isa_chain    AS ic ON ic.cid  = ea._ParentCID
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related exceptions.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from exceptions.', 0, 1, @now, @rc ) WITH NOWAIT
  END


  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related alert states from child event parents
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related alert state history from child event parents.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM AlertStateHistory
  DELETE FROM AlertStateHistory 
  FROM AlertStateHistory AS ash
  INNER JOIN ChildEvent_A      AS cea ON cea.id  = ash.evt_id
  INNER JOIN isa_chain         AS ic  ON ic.cid  = cea._ChildCID
  WHERE
        (    ( ic.base_cid = 'RDMO' )
          OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
        )
    AND ( ash.evt_cid = 'CHEV' )
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related alert state history from child event parents.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from alert state history.', 0, 1, @now, @rc ) WITH NOWAIT
  END



  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related Child Events
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related child events.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM ChildEvent_C (deletion trigger will get ChildEvent_A)
  DELETE FROM ChildEvent_C 
  FROM       ChildEvent_C AS cec
  INNER JOIN ChildEvent_A AS cea ON cea.id   = cec.id
  INNER JOIN isa_chain    AS ic  ON ic.cid   = cea._ChildCID
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related child events.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from child events.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related AlertState History
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related Alert State history.', 0, 1, @now ) WITH NOWAIT
  END

  -- HACK: Delete from AlertStateHistory any record where the child event is based on a RODM object.
/*
cid  id          ctime                                                  AlertStateID evt_cid evt_id      evt_priority evt_ctime                                              evt_detail                                                                                                                                                                                                                                                      OldAlertStateID evt_name                                               evt_alertstate 
---- ----------- ------------------------------------------------------ ------------ ------- ----------- ------------ ------------------------------------------------------ --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- --------------- ------------------------------------------------------ -------------- 
ENT  1           2003-08-11 11:13:49.777                                3            CHEV    7           2            2003-08-11 11:13:49.000                                Tivoli Systems/USIBMNT                                                                                                                                                                                                                                          1               CHILDEVT                                               3
ENT  1           2003-08-11 11:19:27.490                                1            CHEV    7           2            2003-08-11 11:19:27.000                                Tivoli Systems/USIBMNT                                                                                                                                                                                                                                          3               CHILDEVT                                               -3
ENT  1           2003-08-11 11:24:01.947                                3            CHEV    14          2            2003-08-11 11:24:01.000                                Tivoli Systems/USIBMNT                                                                                                                                                                                                                                          1               CHILDEVT                                               3
ENT  1           2003-08-11 12:03:28.470                                1            CHEV    14          2            2003-08-11 12:03:28.000                                Tivoli Systems/USIBMNT                                                                                                                                                                                                                                          3               CHILDEVT                                               -3
ENT  1           2003-08-11 12:42:06.913                                3            CHEV    21          2            2003-08-11 12:42:06.000                                Tivoli Systems/USIBMNT                                                                                                                                                                                                                                          1               CHILDEVT                                               3
ENT  1           2003-08-11 12:42:34.353                                1            CHEV    21          2            2003-08-11 12:42:34.000                                Tivoli Systems/USIBMNT                                                                                                                                                                                                                                          3               CHILDEVT                                               -3
ENT  1           2003-08-11 12:59:31.267                                3            CHEV    28          2            2003-08-11 12:59:31.000                                Tivoli Systems/USIBMNT                                                                                                                                                                                                                                          1               CHILDEVT                                               3
ICND 3164        2003-08-11 12:59:30.723                                3            CHEV    22          2            2003-08-11 12:59:30.000                                Tivoli Systems/USIBMNT/USIBMNT.NTA7MVS/USIBMNT.NTA7MVS.NT50MVS                                                                                                                                                                                                  1               CHILDEVT                                               3
*/

/*
  SELECT ash.cid, ash.id, ash.evt_cid, ash.evt_id
  FROM       AlertStateHistory AS ash
  
  SELECT _ParentID, * FROM ChildEventHistory_V
  SELECT * FROM ChildEvent_V


  SELECT DISTINCT ash.cid, ash.id, ash.evt_cid, ash.evt_id
  FROM       AlertStateHistory AS ash
  INNER JOIN ChildEvent_A      AS cea ON cea.id  = ash.evt_id
  INNER JOIN isa_chain         AS ic  ON ic.cid  = cea._ChildCID
  WHERE
        (    ( ic.base_cid = 'RDMO' )
          OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
        )
    AND ( ash.evt_cid = 'CHEV' )


  SELECT DISTINCT ash.cid, ash.id, ash.evt_cid, ash.evt_id
  FROM       AlertStateHistory AS ash


  SELECT DISTINCT ash.cid, ash.id, ash.evt_cid, ash.evt_id
  FROM       AlertStateHistory AS ash
  INNER JOIN isa_chain         AS ic  ON ic.cid   = ash.cid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
*/

  -- Action DELETE FROM AlertStateHistory
  DELETE FROM AlertStateHistory
  FROM       AlertStateHistory AS ash
  INNER JOIN isa_chain         AS ic  ON ic.cid   = ash.cid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related Alert State history.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from Alert State history.', 0, 1, @now, @rc ) WITH NOWAIT
  END


  -----------------------------------------------------------------------------------------------------
  -- Remove RODMManagedLink 
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM RODMManagedLink records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM RODM..RODMManagedLink
  DELETE FROM RODM..RODMManagedLink
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM RODMManagedLink records.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from RODMManagedLink.', 0, 1, @now, @rc ) WITH NOWAIT
  END


  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM records from BatchDiscovery
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM records from Batch Discovery.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM DiscoveryBatch
  DELETE FROM DiscoveryBatch  
  FROM DiscoveryBatch  AS db
  WHERE
        db.FormatType = 5
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM Batch Discovery records.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from Batch Discovery.', 0, 1, @now, @rc ) WITH NOWAIT
  END


  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related ABS cache records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related ABS cache records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM alobCacheAttribute 
  DELETE FROM alobCacheAttribute 
  FROM       alobCacheAttribute AS aca
  INNER JOIN isa_chain          AS ic  ON ic.cid   = aca.iClass
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related ABS cache records.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from ABS cache.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related ABS queue records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related ABS queue records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM evQueueObject
  DELETE FROM evQueueObject
  FROM       evQueueObject AS eqo
  INNER JOIN isa_chain     AS ic  ON ic.cid   = eqo.cid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related ABS queue records.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from ABS queue.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related Staged Message records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related Staged Message records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM StagedMESG 
  DELETE FROM StagedMESG 
  FROM       StagedMESG AS sm
  INNER JOIN isa_chain  AS ic  ON ic.cid = sm.parent_cid 
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related Staged Message records.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from Staged Messages.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related Staged Exception records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related Staged Exception records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM StagedEXCP
  DELETE FROM StagedEXCP 
  FROM       StagedEXCP AS se
  INNER JOIN isa_chain  AS ic  ON ic.cid = se.parent_cid 
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related Staged Exception records.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from Staged Exceptions.', 0, 1, @now, @rc ) WITH NOWAIT
  END


  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related notes
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related notes.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM NoteHistory
  DELETE FROM NoteHistory
  FROM NoteHistory AS nh
  JOIN Note_C     AS nc  ON nc.id = nh.noteid
  JOIN Note_A     AS na  ON na.id  = nc.id
  JOIN isa_chain  AS ic  ON ic.cid = na._CreatorCID
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to delete from NoteHistory.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Action DELETE FROM Note_C
  DELETE FROM Note_C
  FROM       Note_C     AS nc
  INNER JOIN Note_A     AS na  ON na.id  = nc.id
  INNER JOIN isa_chain  AS ic  ON ic.cid = na._CreatorCID
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related notes.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from notes.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related LOBs
  -- Action Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Deleting RODM-related objects in Business Views', 0, 1, @now) WITH NOWAIT
  END

  DECLARE @lobID ObjID
  DECLARE @lobsDeleted INT
  SET @lobsDeleted = 0
  DECLARE activeLOB CURSOR FOR 
    SELECT DISTINCT lob.id 
      FROM LineOfBusiness_C lob
      JOIN link l ON l.src_id = lob.id
      JOIN isa_chain ic ON ic.cid = l.dst_cid
    WHERE l.link_type = 'LOBL' AND l.src_cid = 'LOB'
      AND lob.deleted = 0
      AND (ic.base_cid = 'RDMO' OR ic.cid IN ('GMFF', 'SNNT')) 
  OPEN activeLOB
  FETCH NEXT FROM activeLOB INTO @lobID
  WHILE @@FETCH_STATUS <> -1
  BEGIN 
    SET @lobsDeleted = @lobsDeleted + 1
    EXEC @err = delete_obj 'LOB', @lobID, @verbose
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR( '%s: asisp_resetRodm: error %d when attempting to delete_obj ''LOB'', %d', 11, 1, @now, @err, @lobID) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_resetRodm
      RETURN -30000
    END
    FETCH NEXT FROM activeLOB INTO @lobID
  END
  CLOSE activeLOB
  DEALLOCATE activeLOB

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Deleted %d RODM-related objects in Business Views', 0, 1, @now, @lobsDeleted) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Commit transaction now so that asisp_resetclass use TRUNCATE operator
  IF @tcount = 0
    COMMIT TRAN asisp_resetRodm

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related twh_ResourceDelete records
  -- Announcement - twh_ResourceDelete is used to record
  -- all physical deletes of resources for TDW.  asisp_resetRodm
  -- causes a problem because the table has a primary key made up
  -- of the resource cid and id.  We need to scrub these tables
  -- clean of RODM resources so we don't violate primary keys.
  -- There are other twh_* tables handled later but we want to
  -- treat twh_ResourceDelete special: we want RODM objects to show
  -- up in this table if triggers are in use.  First we'll wipe out
  -- resources that are already in the table (presumably from an earlier
  -- asisp_resetRodm).  Then when asisp_resetclass is doing its thing,
  -- rows will be added if TDW triggers are active.
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related twh_ResourceDelete records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM twh_ResourceDelete
  DELETE FROM twh_ResourceDelete
  FROM       twh_ResourceDelete AS t
  INNER JOIN isa_chain       AS ic  ON ic.cid  = t.cid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related records from twh_ResourceDelete.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from twh_ResourceDelete.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -----------------------------------------------------------------------------------------------------
  -- Reset RODM class tables
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove RODM resources.', 0, 1, @now, @rc ) WITH NOWAIT
  END
  DECLARE @iClass ClassID
  DECLARE rodmClasses CURSOR FOR 
    SELECT DISTINCT ic.cid 
    FROM       isa_chain  AS ic
    INNER JOIN obj_class  AS oc ON oc.cid = ic.cid
    WHERE
         (     ic.base_cid = 'RDMO' 
           AND oc.flag_abstract = 0 )
      OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))

  OPEN rodmClasses
  FETCH NEXT FROM rodmClasses INTO @iClass
  WHILE @@FETCH_STATUS <> -1
  BEGIN 
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR( '%s: asisp_resetRodm: Removing RODM resources for %s class.', 0, 1, @now, @iClass ) WITH NOWAIT
    END
    EXEC asisp_resetclass @iClass, @cleanbad=0 -- no real error checking from this procedure
    FETCH NEXT FROM rodmClasses INTO @iClass
  END
  CLOSE rodmClasses
  DEALLOCATE rodmClasses

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed RODM resources.', 0, 1, @now ) WITH NOWAIT
  END


  -----------------------------------------------------------------------------------------------------
  -- Remove Staged RODM Connection Status Messages
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all staged RODM Connection Status messages.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action TRUNCATE TABLE StagedRODMConnectionStatus
  TRUNCATE TABLE StagedRODMConnectionStatus
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all staged RODM Connection Status messages.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed staged RODM Connection Status messages.', 0, 1, @now ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove Staged RODM Response Messages
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all staged RODM Response messages.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action TRUNCATE TABLE StagedRODMResponseMessage
  TRUNCATE TABLE StagedRODMResponseMessage
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all staged RODM Response messages.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed staged RODM Response messages.', 0, 1, @now ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove Staged RODM States Messages
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all staged RODM States messages.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action TRUNCATE TABLE StagedRODMResponseMessage
  TRUNCATE TABLE StagedRODMStatesMessage
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all staged RODM States messages.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed staged RODM States messages.', 0, 1, @now ) WITH NOWAIT
  END
  
  -----------------------------------------------------------------------------------------------------
  -- Remove RODM Links from discovery processing
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM Links from discovery processing.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action TRUNCATE TABLE StagedRODMResponseMessage
  TRUNCATE TABLE RODMLoad..RODMLink
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM Links from discovery processing.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed RODM Links from discovery processing.', 0, 1, @now ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove RODM Scalar values from discovery processing
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM Scalar values from discovery processing.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action TRUNCATE TABLE StagedRODMResponseMessage
  TRUNCATE TABLE RODMLoad..RODMObjValScalar
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM Scalar values from discovery processing.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed RODM Scalar values from discovery processing.', 0, 1, @now ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove RODM Objects from discovery processing
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM Objects from discovery processing.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action TRUNCATE TABLE StagedRODMResponseMessage
  TRUNCATE TABLE RODMLoad..RODMObject
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM Objects from discovery processing.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed RODM Objects from discovery processing.', 0, 1, @now ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM RODMManagedObject records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM RODMManagedObject records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM RODM..RODMManagedObject
  DELETE FROM RODM..RODMManagedObject
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM RODMManagedObject records.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from RODMManagedObject.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Cleanup RODM old registration jobs
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM auto-registration jobs.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action EXEC msdb..sp_delete_job @job_id 
  DECLARE @job_id UNIQUEIDENTIFIER
  DECLARE @sJobId NVARCHAR(40)
  SET @rc = 0
  DECLARE jobsRodm CURSOR FOR 
    SELECT job_id FROM msdb..sysjobs WHERE name LIKE 'RODM Network Auto-register%'
  OPEN jobsRodm
  FETCH NEXT FROM jobsRodm INTO @job_id 
  WHILE @@FETCH_STATUS <> -1
  BEGIN 
    EXEC @err = msdb..sp_delete_job @job_id 
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove RODM auto-registration jobs with a job id of %s.', 11, 1, @now, @err, @sJobId ) WITH NOWAIT
      ROLLBACK TRANSACTION asisp_resetRodm
      RETURN -30000
    END
    SELECT @rc = @rc + 1
    FETCH NEXT FROM jobsRodm INTO @job_id 
  END
  CLOSE jobsRodm
  DEALLOCATE jobsRodm
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @sJobId = CONVERT( NVARCHAR(40), @job_id )
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM auto-registration jobs.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d RODM auto-registration jobs.', 0, 1, @now, @rc ) WITH NOWAIT
  END


  -----------------------------------------------------------------------------------------------------
  -- Cleanup thresholds on Enterprise resources
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM child events from enterprise resources.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action Cursor through enterprise objects clean child events
  SET @iClass = 'ENT'
  DECLARE activeENT CURSOR FOR 
    SELECT id FROM ENT_V WHERE deleted = 0
  OPEN activeENT
  FETCH NEXT FROM activeENT INTO @iObject
  WHILE @@FETCH_STATUS <> -1
  BEGIN 
    INSERT INTO StagedCMD (cmd, cid, id)
    SELECT
      'EXEC pa_updateObjectPriority @cid=''' + @iClass + ''', @id=' + CAST( @iObject AS NVARCHAR(9)), 
      @iClass,
      @iObject
    FROM isa_chain AS ic
    WHERE ic.cid = @iClass 
      AND ic.base_cid = 'MNO2' -- only for managed objects
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR( '%s: asisp_resetRodm: error %d inserting into StagedCMD for %s: %d.', 3, 1, @now, @err, @iClass, @iObject ) WITH NOWAIT
      RETURN -30000
    END
    FETCH NEXT FROM activeENT INTO @iObject
  END
  CLOSE activeENT
  DEALLOCATE activeENT

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed all RODM child events from enterprise resources.', 0, 1, @now, @rc ) WITH NOWAIT
  END


  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related ObjAlertCount records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related ObjAlertCount records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM ObjAlertCount
  DELETE FROM ObjAlertCount
  FROM       ObjAlertCount AS oac
  INNER JOIN isa_chain       AS ic  ON ic.cid  = oac.cid
  WHERE
       (( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' )))
    AND oac.nviolations > 0
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related records from ObjAlertCount.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from ObjAlertCount.', 0, 1, @now, @rc ) WITH NOWAIT
  END


  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related RootCauseState records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related RootCauseState records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM RootCauseState
  DELETE FROM RootCauseState
  FROM       RootCauseState AS rcs
  INNER JOIN isa_chain       AS ic  ON ic.cid  = rcs.cid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related records from RootCauseState.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from RootCauseState.', 0, 1, @now, @rc ) WITH NOWAIT
  END


  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related twh_AttrUpdate records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related twh_AttrUpdate records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM twh_AttrUpdate
  DELETE FROM twh_AttrUpdate
  FROM       twh_AttrUpdate AS t
  INNER JOIN isa_chain       AS ic  ON ic.cid  = t.cid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related records from twh_AttrUpdate.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from twh_AttrUpdate.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related twh_LinkCreate records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related twh_LinkCreate records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM twh_LinkCreate
  DELETE FROM twh_LinkCreate
  FROM       twh_LinkCreate AS t
  INNER JOIN isa_chain       AS ic  ON ic.cid  = t.src_cid OR ic.cid  = t.dst_cid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related records from twh_LinkCreate.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from twh_LinkCreate.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related twh_LinkDelete records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related twh_LinkDelete records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM twh_LinkDelete
  DELETE FROM twh_LinkDelete
  FROM       twh_LinkDelete AS t
  INNER JOIN isa_chain       AS ic  ON ic.cid  = t.src_cid OR ic.cid  = t.dst_cid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related records from twh_LinkDelete.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from twh_LinkDelete.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related twh_ResourceCreate records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related twh_ResourceCreate records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM twh_ResourceCreate
  DELETE FROM twh_ResourceCreate
  FROM       twh_ResourceCreate AS t
  INNER JOIN isa_chain       AS ic  ON ic.cid  = t.cid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related records from twh_ResourceCreate.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from twh_ResourceCreate.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -- Remove all RODM related twh_defer_ash records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related twh_defer_ash records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM twh_defer_ash
  DELETE FROM twh_defer_ash
  FROM       twh_defer_ash AS t
  INNER JOIN isa_chain       AS ic  ON ic.cid  = t.cid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related records from twh_defer_ash.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from twh_defer_ash.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related twh_defer_events records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related twh_defer_events records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM twh_defer_events
  DELETE FROM twh_defer_events
  FROM       twh_defer_events AS t
  INNER JOIN isa_chain       AS ic  ON ic.cid  = t.cid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related records from twh_defer_events.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from twh_defer_events.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Remove all RODM related twh_defer_notehist records
  -- Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Remove all RODM related twh_defer_notehist records.', 0, 1, @now ) WITH NOWAIT
  END

  -- Action DELETE FROM twh_defer_notehist
  DELETE FROM twh_defer_notehist
  FROM       twh_defer_notehist AS t
  INNER JOIN isa_chain       AS ic  ON ic.cid  = t.cid
  WHERE
       ( ic.base_cid = 'RDMO' )
    OR ( ic.cid      IN ( 'GMFF', 'SNNT' ))
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: error %d when attempting to remove all RODM related records from twh_defer_notehist.', 11, 1, @now, @err ) WITH NOWAIT
    ROLLBACK TRANSACTION asisp_resetRodm
    RETURN -30000
  END

  -- Result Announcement
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR( '%s: asisp_resetRodm: Removed %d rows from twh_defer_notehist.', 0, 1, @now, @rc ) WITH NOWAIT
  END

  -----------------------------------------------------------------------------------------------------
  -- Update ObjPathCache, need to force this so subsequent discovery is not contaminated by old ids.
  --   This does not need to be within a transaction, because it has its own.
  EXEC asisp_updateObjPathCache @optional=0


  -----------------------------------------------------------------------------------------------------
  SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
  RAISERROR( '', 0, 1 ) WITH NOWAIT
  RAISERROR( '%s: asisp_resetRodm: Processing complete', 0, 1, @now ) WITH NOWAIT
  RAISERROR( '', 0, 1 ) WITH NOWAIT

  SET NOCOUNT OFF
GO
