-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/_asisp_setuserid.sql
-- SID: 1.9.1.4
-- Date Modified: 05/01/06 15:09:00
--
-- _asisp_setuserid.sql
--
-- Module Description
--   Stores userid for use during task execution

USE Object
GO


----------------------------------------------------------------------------------------------------
-- asisp_setUserID
PRINT 'Creating procedure asisp_setUserID'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_setUserID' AND type = 'P')
  DROP PROCEDURE asisp_setUserID
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/_asisp_setuserid.sql
-- SID: 1.9.1.4
-- Date Modified: 05/01/06 15:09:00
-- Core PROCEDURE used by Java Console server
-- --------------------------------------------------
--
-- Updates the userid/pw in the task_id_file.
-- If the id/type combination already exists, the
-- value is updated.
--
-- Input:
--	userid	
--	password
--	type     (opt)  Type of userid/pw pair
--			Valid values are:
--				CONSOLE (default)
--				390
--				TME	
--      hostname (opt)  Java Console Server hostname
--
-- ReturnCodes:	
--	0		Procedure successed
--     ^0		Procedure failed
--     -1               Userid or password is NULL
--     -2               Unable to determine the users
--                      console login userid
-- --------------------------------------------------
CREATE PROCEDURE asisp_setUserID
 @userid      NVARCHAR(255),
 @password    NVARCHAR(64),
 @type        NVARCHAR(8),
 @hostname    NVARCHAR(255),
 @ReturnCode  INT = NULL OUTPUT
AS
  RAISERROR('asisp_setUserID: %s, %s, %s', 0, 1, @userid, @type, @hostname)

  IF (@userid IS NULL) or (@password IS NULL) RETURN -1

  --CREATE temp TABLE to hold xp_cmdshell output
  CREATE TABLE #output ( line NVARCHAR(500) NULL )

  DECLARE @cmd NVARCHAR(510)

  SET @cmd = 'tserver hostdata -u "' + @userid + '" -p "' + @password + '"'

  IF @type IS NOT NULL
  BEGIN
    SET @cmd = @cmd  + ' -t ' + @type

    IF (UPPER(@type) <> "CONSOLE")
    BEGIN
      DECLARE @consoleID NVARCHAR(255),@usergroup NVARCHAR(255)
      EXEC asisp_getContextUserInfo @consoleID OUTPUT, @usergroup OUTPUT

      IF @consoleID IS NOT NULL
      BEGIN
        SET @cmd = @cmd + ' -c "' + @consoleID + '"'
        -- RAISERROR('asisp_setUserID commad: %s', 0, 1, @cmd)
      END
      ELSE
      BEGIN
        RAISERROR('asisp_setUserID:  ERROR, unable to determine console userid', 0, 1)
        SET @ReturnCode = -2
      END
    END
  END

  EXEC @ReturnCode = Object..asisp_cmdshell '#output', @cmd

  -- SELECT line FROM #output

  RAISERROR('asisp_delUserID: returncode %d', 0, 1, @ReturnCode)

  RETURN @ReturnCode
GO
