-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/etl_getdeletelinks.sql
-- SID: 1.13
-- Date Modified: 05/01/07 10:34:46

-- Module description:
-- Create the etl_getupdateattr procedure which is used by TDW to return a result set of deleted links

USE Object
IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'etl_getdeletelinks')
  DROP PROCEDURE etl_getdeletelinks
GO
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}

-- File: archive/db/sql/procs/etl_getdeletelinks.sql
-- SID: 1.13
-- Date Modified: 05/01/07 10:34:46

-- Module description:
-- Create the etl_getupdateattr procedure which is used by TDW to return a result set of deleted links
-- The result set has the following format
--    src_cid         ClassID  NOT NULL
--    src_id          ObjID    NOT NULL
--    src_comp_id     DECIMAL(20,0)   NOT NULL
--    dst_cid         ClassID  NOT NULL
--    dst_id          ObjID    NOT NULL
--    dst_comp_id     DECIMAL(20,0)   NOT NULL
--    mtime           DATETIME NOT NULL
-- The procedure also updates the mtime and status columns of the twh_LinkDelete row of the twh_ExtractCtrl table

CREATE PROCEDURE etl_getdeletelinks
  @verbose TINYINT=0
AS
  SET NOCOUNT ON
  DECLARE @now NVARCHAR(40)
  DECLARE @err      INTEGER
  DECLARE @rc       INTEGER
  DECLARE @last_xid DECIMAL
  DECLARE @this_xid DECIMAL
  DECLARE @gmoffsetmin INT
  DECLARE @currgmdate DATETIME
  DECLARE @last_gmoffsetmin INT
  DECLARE @lastgmdate DATETIME
  DECLARE @GMT_CHANGE_TIME DATETIME
  DECLARE @process_gmtc TINYINT
  DECLARE @tcount INT

  CREATE TABLE #resultSet (
    src_cid         ClassID  NOT NULL,
    src_id          ObjID    NOT NULL,
    src_comp_id     DECIMAL(20,0)   NOT NULL,
    dst_cid         ClassID  NOT NULL,
    dst_id          ObjID    NOT NULL,
    dst_comp_id     DECIMAL(20,0)   NOT NULL,
    mtime           DATETIME NOT NULL
  )

  CREATE TABLE #link_types (
    link_type  LinkType NOT NULL
  )

  SET @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION etl_getdeletelinks
  ELSE
    SAVE TRANSACTION etl_getdeletelinks

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getdeletelinks starts', 0, 1, @now) WITH NOWAIT
  END

  -- standard transaction logic
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN etl_getdeletelinks
  ELSE
    SAVE TRAN etl_getdeletelinks

  UPDATE twh_ExtractCtrl WITH (HOLDLOCK) SET this_mtime = CURRENT_TIMESTAMP, status = 1
  WHERE xtable = 'twh_LinkDelete'

  SELECT @last_xid = last_xid, @this_xid = this_xid 
  FROM twh_ExtractCtrl
  WHERE xtable = 'twh_LinkDelete'

  -- Finish up transaction since remaining steps
  -- don't need to be atomic
  IF @tcount = 0
    COMMIT TRANSACTION etl_getdeletelinks

  -- create a table to contain the links we'll report. we only want to include
  -- certain link types in the result set.  others may get added by the triggers
  -- so we'll filter them out.

  INSERT INTO #link_types (link_type)
  SELECT DISTINCT l.link_type
  FROM derived_obj_link AS l
  JOIN link_type AS lt ON lt.link_type = l.link_type
  JOIN obj_class AS os ON os.cid = l.src_cid
  JOIN obj_class AS od ON od.cid = l.dst_cid
  JOIN isa_chain AS si ON si.cid = os.cid
  JOIN isa_chain AS di ON di.cid = od.cid
  WHERE os.flag_abstract = 0
    AND od.flag_abstract = 0
    AND (lt.flag_containment = 1 OR lt.flag_parent_child = 1)
    AND si.base_cid = 'MNO2'
    AND di.base_cid = 'MNO2'
  ORDER BY l.link_type
  SELECT @rc = @@ROWCOUNT, @err = @@ERROR
  IF @rc < 0 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getdeletelinks: error %d populating #link_types', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getdeletelinks added %d rows to #link_types', 0, 1, @now, @rc) WITH NOWAIT
  END

  INSERT INTO #resultSet (src_cid,   src_id,   src_comp_id, dst_cid,   dst_id,   dst_comp_id, mtime)
  SELECT  DISTINCT        d.src_cid, d.src_id, 0,           d.dst_cid, d.dst_id, 0,           d.ctime
  FROM twh_LinkDelete AS d WITH (NOLOCK)
  JOIN #link_types    AS l ON l.link_type = d.link_type
  WHERE d.xid > @last_xid 
    AND d.xid <= @this_xid
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err < 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getdeletelinks: error %d inserting into #resultSet', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getdeletelinks added %d rows to #resultSet', 0, 1, @now, @rc) WITH NOWAIT
  END

  UPDATE #resultSet SET src_comp_id = i.comp_id
  FROM #resultSet as r
    JOIN twh_Inventory AS i ON i.cid = r.src_cid AND i.id = r.src_id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err < 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getdeletelinks: error %d updating #resultSet for src', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getdeletelinks updated src_comp_id for %d rows in #resultSet', 0, 1, @now, @rc) WITH NOWAIT
  END

  UPDATE #resultSet SET dst_comp_id = i.comp_id
  FROM #resultSet as r
    JOIN twh_Inventory AS i ON i.cid = r.dst_cid AND i.id = r.dst_id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err < 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getdeletelinks: error %d updating #resultSet for dst', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getdeletelinks updated dst_comp_id for %d rows in #resultSet', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- adjust timestamps in result set to be GMT-based

  EXEC etl_getgmtvars 'deletelinks', @process_gmtc OUTPUT, @gmoffsetmin OUTPUT, @last_gmoffsetmin OUTPUT, @GMT_CHANGE_TIME OUTPUT, @verbose=@verbose

  IF @process_gmtc = 0 -- adjust by a single offset
    UPDATE #resultSet SET mtime = DATEADD(MI, @gmoffsetmin, mtime)
  ELSE                 -- adjust by the two offsets
    UPDATE #resultSet SET mtime = CASE
                                       WHEN mtime >= @GMT_CHANGE_TIME THEN DATEADD(MI, @gmoffsetmin, mtime)
                                       ELSE                                DATEADD(MI, @last_gmoffsetmin, mtime)
                                     END

  SELECT * FROM #resultSet

  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN etl_getdeletelinks


  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getdeletelinks returning', 0, 1, @now) WITH NOWAIT
  END

GO
