-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/etl_getnewobjs.sql
-- SID: 1.1.1.8
-- Date Modified: 04/11/12 11:46:39
--
-- Module Description:
--   Defines stored procedure etl_getnewobjs
USE Object
GO

PRINT 'Creating procedure etl_getnewobjs'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'etl_getnewobjs' AND type = 'P')
  DROP PROCEDURE etl_getnewobjs
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/etl_getnewobjs.sql
-- SID: 1.1.1.8
-- Date Modified: 04/11/12 11:46:39
--
-- This procedure returns the resource data for LOB and Service in TBSM database that are not loaded in TDW. 
-- It returns:
--  src_cid:            --NULL when it is root object 
--  src_id:             --NULL when it is root object 
--  cid:                --Object cid 
--  id:                 --Object id 
--  name:               --Object name 
--  comp_relntyp:       -- component relationship type in TDW
--  guid:		            -- TDW name GUID
--  Phys_label:         -- Physical class label
--  comp_src_type:      -- Source component type
--  comp_src_id:        -- Source component id, default 0
--  comp_target_type:   -- Target component type
--  comp_target_id:     -- Target component id, default 0
--  adddate:            -- datetime that obj created
--  mapped:             -- flag, default 0
CREATE PROCEDURE etl_getnewobjs
  @incremted  INT = 1,
  @verbose    TINYINT = 0
AS

  SET NOCOUNT ON
  --Get GM time offset
  DECLARE @gmoffsetmin INT
  EXEC master..xp_regread
    'HKEY_LOCAL_MACHINE',
    'SYSTEM\CurrentControlSet\Control\TimeZoneInformation',
    'ActiveTimeBias',
     @gmoffsetmin OUTPUT

  --First reset additions with addition in-process status (additions that didn't make it all the way to the warehouse)
  DELETE FROM twh_objs WHERE Status&1=1

  --Now find all appropriate new LOBs
  CREATE TABLE #newobjs
  (
    cid 	      ClassID,
    id 		      ObjID,
    src_cid   	ClassID NULL,
    src_id	    ObjID NULL,
    link_type	  NVARCHAR(10) NULL,
    name 	      ObjName NULL,
    label	      ObjName NULL,
    Phys_cid    ClassID NULL,
    Phys_label  ObjName NULL,
    guid	      NVARCHAR(254) NULL,
    ctime 	    DATETIME
  )

  IF @incremted = 0
  BEGIN
    -- insert all LOB objects
    INSERT INTO #newobjs (cid, id, src_cid, src_id, link_type, name, ctime)
      SELECT l.dst_cid, l.dst_id, l.src_cid, l.src_id, 'PCHILD', o.name, o.ctime
      FROM LineOfBusiness_V o, link l 
      WHERE l.link_type='LOBC' 
      AND l.dst_cid='LOB' 
      AND l.dst_id = o.id 
      AND l.deleted=0
      AND o.deleted=0
  END
  ELSE
  BEGIN 
    --insert LOB
    INSERT INTO #newobjs(cid, id, src_cid, src_id, link_type, name, ctime)
      SELECT l.dst_cid, l.dst_id, l.src_cid, l.src_id, 'PCHILD', o.name, o.ctime
      FROM LineOfBusiness_V o, link l 
      WHERE l.link_type='LOBC' 
        AND l.dst_cid='LOB' 
        AND l.dst_id=o.id 
        AND l.deleted=0
        AND o.deleted=0
        AND o.id NOT IN (SELECT Obj_id FROM twh_objs WHERE Obj_cid = 'LOB' AND Deleted_DT IS NULL)
  END

  -- Add children to LOB shortcut
  DECLARE @gen INT
  SET @gen = 1
  CREATE TABLE #lobl_lob
  (
    gen 	  SMALLINT,
    cid 	  ClassID,
    id 		  ObjID,
    src_cid       ClassID,
    src_id	  ObjID,
    ctime	  DATETIME
  )
  
  DECLARE @rc INT
  INSERT INTO #lobl_lob 
    SELECT @gen, l.dst_cid, l.dst_id, new.cid, new.id, l.ctime 
    FROM link AS l
    JOIN #newobjs AS new ON l.src_cid = new.cid AND l.src_id = new.id
      AND new.cid = 'LOB' AND l.link_type = 'LOBL' AND l.dst_cid = 'LOB'
    WHERE l.deleted = 0 
  SET @rc = @@ROWCOUNT
  WHILE @rc > 0
  BEGIN
    SET @gen = @gen + 1
    INSERT INTO #lobl_lob 
      SELECT @gen, l.dst_cid, l.dst_id, new.src_cid, new.src_id, l.ctime 
      FROM link AS l
      JOIN #lobl_lob AS new ON l.src_cid = new.cid AND l.src_id = new.id
        AND new.cid = 'LOB' AND l.link_type = 'LOBL' AND l.dst_cid = 'LOB'
      WHERE l.deleted = 0 AND new.gen = @gen -1
    SET @rc = @@ROWCOUNT
  END

  -- Insert data from #lobl_lob into #newobjs
  INSERT INTO #newobjs (cid, id, src_cid, src_id, link_type, ctime)
    SELECT DISTINCT l.dst_cid, l.dst_id, n.src_cid, n.src_id, 'PCHILD', n.ctime
    FROM #lobl_lob AS n JOIN link AS l ON n.cid = l.src_cid 
      AND n.id = l.src_id 
      AND l.dst_cid = 'LOB'
      AND l.link_type = 'LOBC'
      AND l.deleted = 0
    WHERE NOT EXISTS (SELECT 1 FROM twh_objs AS o WHERE n.cid=o.Obj_cid AND n.id=o.Obj_id AND n.src_cid=o.src_cid AND n.src_id=o.src_id)

  -- Add link to all the shortcuts for the new LOB
  INSERT INTO #newobjs (cid, id, src_cid, src_id, link_type, ctime)
    SELECT 'LOB', l2.dst_id, 'LOB', l.src_id, 'PCHILD', l.ctime
    FROM link AS l 
    JOIN link AS l2 ON l.src_cid = l2.src_cid
    AND l.dst_cid = l2.dst_cid
    AND l2.src_id = l.dst_id
    AND l.link_type = 'LOBL'
    AND l2.link_type = 'LOBC'
    AND l.deleted = 0
    AND l2.deleted = 0
    AND l2.dst_id IN (SELECT id FROM #newobjs WHERE name IS NOT NULL)
    WHERE l.src_cid = 'LOB' 
    AND l.dst_cid = 'LOB'

  -- Get Phys_cid and Phys_label in #newobjs
  UPDATE n SET n.Phys_cid = o.cid, Phys_label = o.label 
    FROM #newobjs AS n, lob_link AS l, obj_class AS o 
    WHERE n.cid = 'LOB' AND n.id = l.dst_id
    AND l.phy_cid = o.cid
    AND name IS NOT NULL

  --handle Service objects, Service objects are handled differently
  -- insert all Service objects
  INSERT INTO #newobjs (cid, id, label, ctime, name, guid)
    SELECT cid, id, name, ctime, _ServiceID, _ServiceGUID  
    FROM EVService_C 
    WHERE deleted = 0
      AND id NOT IN (SELECT Obj_id FROM twh_objs 
                       WHERE Obj_cid = 'EXES' 
                       AND Deleted_DT IS NULL
                       AND Name IS NOT NULL)

  -- insert Service relationship
  INSERT INTO #newobjs (cid, id, src_cid, src_id, link_type, ctime)
    SELECT l.dst_cid, l.dst_id, l.src_cid, l.src_id, m.twh, l.ctime 
    FROM link AS l JOIN twh_tbsm_map AS m ON l.link_type = m.tbsm AND m.map_type = 'RelnType' 
    WHERE src_cid = 'EXES' 
      AND (link_type = 'EVSL' OR link_type = 'EVSS')
      AND deleted = 0

  -- delete the existing Service link
  DELETE n FROM #newobjs AS n JOIN twh_objs AS o ON n.cid = o.Obj_cid
    AND n.id = o.Obj_id
    AND n.src_cid = o.src_cid
    AND n.src_id = o.src_id 
    AND Deleted_DT IS NULL

  --Insert new records into twh_objs in an addition in-process status.
  IF @incremted <> 0
    INSERT INTO twh_objs(Obj_cid, Obj_id, src_cid, src_id, Phys_cid, Name, CTIME, Status)
    SELECT DISTINCT  
      cid,
      id,
      src_cid,
      src_id,
      Phys_label,
      CASE cid WHEN 'EXES' THEN label ELSE name END,
      ctime,
      1
    FROM #newobjs --WHERE cid = 'LOB'

  CREATE TABLE #result
  (
    idnet             INT IDENTITY(1, 1),
    src_cid           ClassID NULL,
    src_id            ObjID NULL, 
    cid               ClassID,
    id                ObjID, 
    name              ObjName NULL, 
    comp_relntyp      NVARCHAR(254) NULL,
    Phys_label        NVARCHAR(254) NULL,
    guid              NVARCHAR(254) NULL,
    comp_src_type     NVARCHAR(254) NULL,
    comp_src_id       INT,
    comp_target_type  NVARCHAR(254) NULL,
    comp_target_id    INT,
    adddate           DATETIME NULL,
    mapped            INT,
    label             NVARCHAR(254) NULL,
    comp_corr_val     NVARCHAR(254),
    loading_flag      SMALLINT,
    src_comp_corr_val NVARCHAR(254) NULL
  )
   
  --Return records to the warehouse
  INSERT INTO #result
  SELECT DISTINCT
    CASE nn.src_cid WHEN 'LOBC' THEN '' ELSE nn.src_cid END AS src_cid, 
	  CASE nn.src_cid WHEN 'LOBC' THEN 0 ELSE nn.src_id END AS src_id, 
	  nn.cid,
	  nn.id, 
	  nn.name, 
    nn.link_type AS comp_relntyp,
	  nn.Phys_label,
	  nn.guid AS guid,
    m.twh AS comp_src_type,
    0 AS comp_src_id,
    m2.twh AS comp_target_type,
    0 AS comp_target_id,
    DATEADD(mi, @gmoffsetmin, nn.ctime) AS adddate,
    0 AS mapped,
    nn.label AS label,
    RTRIM(nn.cid) + CONVERT(NVARCHAR(40), nn.id) AS comp_corr_val,
    0,
    RTRIM(nn.src_cid) + CONVERT(NVARCHAR(40), nn.src_id) AS comp_corr_val
  FROM #newobjs AS nn
  LEFT OUTER JOIN twh_tbsm_map AS m ON m.map_type = 'ObjectType' AND nn.src_cid = m.tbsm AND m.enabled = 1
  JOIN twh_tbsm_map AS m2 ON nn.cid = m2.tbsm AND m2.map_type = 'ObjectType' AND m2.enabled = 1

  SELECT * FROM #result
GO
