-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/etl_getnewpubobjs.sql
-- SID: 1.24
-- Date Modified: 04/11/14 11:55:20

-- Module description:
-- Create the etl_getnewpubobjs procedure

USE Object
IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'etl_getnewpubobjs')
  DROP PROCEDURE etl_getnewpubobjs
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/etl_getnewpubobjs.sql
-- SID: 1.24
-- Date Modified: 04/11/14 11:55:20

-- Module description:
-- Create the etl_getnewpubobjs procedure which is used by
-- TDW to retrieve a list of new public resources.  It returns a result set with
-- the following columns:
--    ident            INT NOT NULL IDENTITY(1,1),   -- Used as an index when TDW processes the result set. Populated automatically by declaring the column in the temporary table with IDENTITY(1,1).  Each call to the stored procedure can generate a new series of numbers.
--    src_cid          ClassID NULL,                 -- Parent class ID, NULL when src_id = `ROOT'
--    src_id           ObjID NULL,                   -- Parent object ID, NULL when src_id = `ROOT'
--    cid              ClassID NOT NULL,             -- Resource type
--    id               ObjID NOT NULL,               -- Resource id
--    name             NVARCHAR(255) NULL,           -- The name will be constructed based on the Tivoli Common Data Model naming rules.  In some cases, this may turn out to be the name from the cname_C table.
--                                                   -- This will be NULL if a link is being added to an existing resource or we're trying to convey the fact that the resource has a LOBL link to a business system.
--    description      NVARCHAR(255) NULL,           -- This will be NULL if a link is being added to an existing resource or we're trying to convey the fact that the resource has a LOBL link to a business system.
--    comp_relntyp     NVARCHAR(254) NOT NULL,       -- `PCHILD' or `GTM_LB' Component relationship type in TDW. This column will be GTM_LB for a LOBL link and PCHILD otherwise.
--    comp_src_type    NVARCHAR(254) NULL,           -- Source component type. This will come from the comp_cd column of the twh_class_map table, based on src_cid from this table's row.
--    comp_src_id      INT NULL DEFAULT(0),          -- Source component id
--    comp_target_type NVARCHAR(254) NULL,           -- Target component type This will come from the comp_cd column of the twh_class_map table, based on cid from this table's row.
--    comp_target_id   INT NULL DEFAULT(0),          -- Target component id
--    adddate          DATETIME NOT NULL,            -- ctime from the cname_C (in the case of a resource) or link table (in the case of a link)
--    level            INT NOT NULL,                 -- This will come from the level column of the twh_class_map table, based on the cid from this table's row.
--    label            NVARCHAR(254) NULL,           -- This will come from the label column of the obj_class table, based on the cid from this table's row.  This will be NULL if a link is being added to an existing resource or we're trying to convey the fact that the resource has a LOBL link to a business system.
--    comp_core_val    NVARCHAR(254) NOT NULL,       -- cid '|' id The string will be a concatenation of the target resource's cid, a vertical bar, and the target resource's id.
--    loading_flag     SMALLINT NOT NULL DEFAULT(0), -- 0
--    src_comp_corr_val NVARCHAR(254) NULL           -- cid '|' id The string will be a concatenation of the source resource's cid, a vertical bar, and the source resource's id.
-- The procedure also updates the newpubobjs row of twh_ExtractCtrl: mtime is set to the current time
-- and status is changed to 1

CREATE PROCEDURE etl_getnewpubobjs
  @verbose TINYINT=0
AS
  SET NOCOUNT ON

  DECLARE @now NVARCHAR(40)
  DECLARE @this_obj_xid DECIMAL(18)
  DECLARE @last_obj_xid DECIMAL(18)
  DECLARE @this_lnk_xid DECIMAL(18)
  DECLARE @last_lnk_xid DECIMAL(18)
  DECLARE @rc INT
  DECLARE @err INT
  DECLARE @gmoffsetmin INT
  DECLARE @currgmdate DATETIME
  DECLARE @last_gmoffsetmin INT
  DECLARE @lastgmdate DATETIME
  DECLARE @GMT_CHANGE_TIME DATETIME
  DECLARE @process_gmtc TINYINT

  CREATE TABLE #newpubobjs (
    ident            INT NOT NULL IDENTITY(1,1),   -- Used as an index when TDW processes the result set. Populated automatically by declaring the column in the temporary table with IDENTITY(1,1).  Each call to the stored procedure can generate a new series of numbers.
    src_cid          ClassID NULL,                 -- Parent class ID, NULL when src_id = `ROOT'
    src_id           ObjID NULL,                   -- Parent object ID, NULL when src_id = `ROOT'
    cid              ClassID NOT NULL,             -- Resource type
    id               ObjID NOT NULL,               -- Resource id
    name             NVARCHAR(255) NULL,           -- The name will be constructed based on the Tivoli Common Data Model naming rules.  In some cases, this may turn out to be the name from the cname_C table.
                                                   -- This will be NULL if a link is being added to an existing resource or we're trying to convey the fact that the resource has a LOBL link to a business system.
    description      NVARCHAR(255) NULL,           -- This will be NULL if a link is being added to an existing resource or we're trying to convey the fact that the resource has a LOBL link to a business system.
    comp_relntyp     NVARCHAR(254) NOT NULL,       -- `PCHILD' or `GTM_LB' Component relationship type in TDW. This column will be GTM_LB for a LOBL link and PCHILD otherwise.
    comp_src_type    NVARCHAR(254) NULL,           -- Source component type. This will come from the comp_cd column of the twh_class_map table, based on src_cid from this table's row.
    comp_src_id      INT NULL DEFAULT(0),          -- Source component id
    comp_target_type NVARCHAR(254) NULL,           -- Target component type This will come from the comp_cd column of the twh_class_map table, based on cid from this table's row.
    comp_target_id   INT NULL DEFAULT(0),          -- Target component id
    adddate          DATETIME NOT NULL,            -- ctime from the cname_C (in the case of a resource) or link table (in the case of a link)
    level            INT NOT NULL,                 -- This will come from the level column of the twh_class_map table, based on the cid from this table's row.
    label            NVARCHAR(254) NULL,           -- This will come from the label column of the obj_class table, based on the cid from this table's row.  This will be NULL if a link is being added to an existing resource or we're trying to convey the fact that the resource has a LOBL link to a business system.
    comp_core_val    NVARCHAR(254) NOT NULL,       -- cid '|' id The string will be a concatenation of the target resource's cid, a vertical bar, and the target resource's id.
    loading_flag     SMALLINT NOT NULL DEFAULT(0), -- 0
    src_comp_corr_val NVARCHAR(254) NULL           -- cid '|' id The string will be a concatenation of the source resource's cid, a vertical bar, and the source resource's id.
  )

  -- create a table to contain the links we'll report. we only want to include
  -- certain link types in the result set.  others may get added by the triggers
  -- so we'll filter them out.

  CREATE TABLE #link_types (
    link_type  LinkType NOT NULL
  )

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getnewpubobjs starts', 0, 1, @now) WITH NOWAIT
  END

  IF @verbose >= 2 
  BEGIN
    SELECT 'twh_ExtractCtrl'='twh_ExtractCtrl',* FROM twh_ExtractCtrl WHERE name = 'newpubobjs'
  END

  -- indicate that a result set has been prepared for TDW.
  UPDATE twh_ExtractCtrl SET status = 1 WHERE name = 'newpubobjs'

  -- add resources to result set

  SELECT @this_obj_xid = this_xid, @last_obj_xid = last_xid FROM twh_ExtractCtrl WHERE name = 'newpubobjs' AND xtable = 'twh_ResourceCreate'
  SELECT @this_lnk_xid = this_xid, @last_lnk_xid = last_xid FROM twh_ExtractCtrl WHERE name = 'newpubobjs' AND xtable = 'twh_LinkCreate'

  INSERT INTO #link_types (link_type)
  SELECT 'DRCT' AS link_type
    UNION
  SELECT DISTINCT l.link_type
  FROM derived_obj_link AS l
  JOIN link_type AS lt ON lt.link_type = l.link_type
  JOIN obj_class AS os ON os.cid = l.src_cid
  JOIN obj_class AS od ON od.cid = l.dst_cid
  JOIN isa_chain AS si ON si.cid = os.cid
  JOIN isa_chain AS di ON di.cid = od.cid
  WHERE os.flag_abstract = 0
    AND od.flag_abstract = 0
    AND (lt.flag_containment = 1 OR lt.flag_parent_child = 1 OR lt.flag_propagation = 1 OR lt.flag_logical = 1)
    AND si.base_cid = 'MNO2'
    AND di.base_cid = 'MNO2'
  ORDER BY l.link_type
  SELECT @rc=@@ROWCOUNT, @err = @@ERROR

  IF @err < 0 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getnewpubobjs: error %d inserting into #link_types', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getnewpubobjs: inserted %d rows into #link_types', 0, 1, @now, @rc) WITH NOWAIT
  END

  INSERT INTO #newpubobjs (src_cid, src_id, cid, id, name, description, comp_relntyp, 
                           comp_src_type, comp_target_type, adddate, level, label, comp_core_val, src_comp_corr_val, comp_src_id)
    SELECT
      l.src_cid,
      l.src_id,
      ISNULL(l.dst_cid, o.cid),
      ISNULL(l.dst_id, o.id),
      o.name,
      o.description,
      CASE WHEN l.link_type IS NOT NULL AND l.link_type = 'LOBL' THEN 'GTM_LB' ELSE 'PCHILD' END,
      src_m.comp_cd,
      dst_m.comp_cd,
      ISNULL(o.ctime, l.ctime),
      dst_m.level,
      o.name, -- the "label" will always be the resource name
      RTRIM(ISNULL(l.dst_cid, o.cid)) + '|' + CAST(ISNULL(l.dst_id, o.id) AS NVARCHAR(10)),
      CASE WHEN l.src_cid IS NOT NULL THEN RTRIM(l.src_cid) + '|' + CAST(l.src_id AS NVARCHAR(10)) ELSE NULL END,
      CASE WHEN l.src_cid IS NULL THEN NULL ELSE ISNULL(i.comp_id, 0) END
    FROM twh_LinkCreate                l WITH (NOLOCK)
    JOIN #link_types                   lt    ON lt.link_type = l.link_type
    LEFT OUTER JOIN twh_class_map      src_m ON src_m.cid = l.src_cid
    FULL OUTER JOIN twh_ResourceCreate o WITH (NOLOCK) ON o.cid = l.dst_cid AND o.id = l.dst_id AND o.xid > @last_obj_xid AND o.xid <= @this_obj_xid
    JOIN twh_class_map                 dst_m ON dst_m.cid = ISNULL(l.dst_cid, o.cid)
    LEFT OUTER JOIN twh_Inventory      i ON i.cid = l.src_cid AND i.id = l.src_id
    WHERE dst_m._public = 1 AND l.dst_cid NOT IN ('LOB','EXES') AND (l.xid IS NULL OR (l.xid > @last_lnk_xid AND l.xid <= @this_lnk_xid))
      AND (src_m.cid IS NOT NULL OR l.src_cid = 'BUSC' or l.src_cid IS NULL)
  SELECT @rc=@@ROWCOUNT, @err = @@ERROR

  IF @err < 0 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getnewpubobjs: error %d inserting into #newpubobjs', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getnewpubobjs added %d resource rows to #newpubobjs', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- adjust timestamps in result set to be GMT-based
  EXEC etl_getgmtvars 'newpubobjs', @process_gmtc OUTPUT, @gmoffsetmin OUTPUT, @last_gmoffsetmin OUTPUT, @GMT_CHANGE_TIME OUTPUT

  IF @process_gmtc = 0 -- adjust by a single offset
    UPDATE #newpubobjs SET adddate = DATEADD(MI, @gmoffsetmin, adddate)
  ELSE                 -- adjust by the two offsets
    UPDATE #newpubobjs SET adddate = CASE
                                       WHEN adddate >= @GMT_CHANGE_TIME THEN DATEADD(MI, @gmoffsetmin, adddate)
                                       ELSE                                 DATEADD(MI, @last_gmoffsetmin, adddate)
                                     END

  -- return the result set

  SELECT * FROM #newpubobjs

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_getnewpubobjs returning', 0, 1, @now) WITH NOWAIT
  END
GO