-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/etl_prepare.sql
-- SID: 1.18
-- Date Modified: 04/11/14 11:55:26

-- Module description:
-- Create the etl_prepare procedure

USE Object
IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'etl_prepare')
  DROP PROCEDURE etl_prepare
GO
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}

-- File: archive/db/sql/procs/etl_prepare.sql
-- SID: 1.18
-- Date Modified: 04/11/14 11:55:26

-- Module description:
-- Create the etl_prepare procedure which is used by
-- TDW at the beginning of an extraction cycle.  It
-- performs initialization of the twh_ExtractCtrl table
-- and calls etl_sync (if necessary) and does not return 
-- any result sets.

CREATE PROCEDURE etl_prepare
  @verbose TINYINT=0
AS
  DECLARE @now             NVARCHAR(40)
  DECLARE @xtable          NVARCHAR(64)
  DECLARE @xid             NVARCHAR(16)
  DECLARE @last_xid        DECIMAL(18)
  DECLARE @last_xid_string NVARCHAR(18)
  DECLARE @sql             NVARCHAR(2000)
  DECLARE @ret             INTEGER
  DECLARE @err             INTEGER
  DECLARE @rc              INTEGER

  DECLARE @start DATETIME
  DECLARE @start1 DATETIME
  DECLARE @elapsed_ms INT
  DECLARE @sqlCounter1 INT
  DECLARE @sqlCounter2 INT
  DECLARE @sqlCounter3 INT
  DECLARE @sqlCounter4 INT
  DECLARE @sqlCounter5 INT
  DECLARE @sqlCounter6 INT
  DECLARE @sqlCounter7 INT
  DECLARE @sqlCounter8 INT
  DECLARE @sqlCounter9 INT
  DECLARE @sqlCounter10 INT
  DECLARE @sqlCounter11 INT
  DECLARE @sqlCounter12 INT
  DECLARE @sqlCounter13 INT

  CREATE TABLE #logicalDeletes (
    cid ClassID   NOT NULL, 
    id ObjID      NOT NULL, 
    maxid DECIMAL NOT NULL)

  CREATE TABLE #logicalCreates (
    cid ClassID NOT NULL, 
    id ObjID NOT NULL,
    maxid DECIMAL NOT NULL)

  CREATE TABLE #deleteThese (
    cid  ClassID NOT NULL,
    id   ObjID NOT NULL
  )

  SET @sqlCounter1 = 0
  SET @sqlCounter2 = 0
  SET @sqlCounter3 = 0
  SET @sqlCounter4 = 0
  SET @sqlCounter5 = 0
  SET @sqlCounter6 = 0
  SET @sqlCounter7 = 0
  SET @sqlCounter8 = 0
  SET @sqlCounter9 = 0
  SET @sqlCounter10 = 0
  SET @sqlCounter11 = 0
  SET @sqlCounter12 = 0
  SET @sqlCounter13 = 0

  SET NOCOUNT ON

  IF @verbose >= 2 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare starts', 0, 1, @now) WITH NOWAIT
  END

  SET @start  = CURRENT_TIMESTAMP
  SET @start1 = CURRENT_TIMESTAMP  

  -- Prune extraction tables for result sets that TDW
  -- has indicated it has received.
  -- Since some xtables have multiple rows, only deleted based on the minimum last_xid value (else we'll delete
  -- unprocessed rows)

  DECLARE prune_cursor CURSOR FOR 
    SELECT xtable, xid, MIN(last_xid) 
    FROM twh_ExtractCtrl 
    WHERE status = 0 AND prune = 1
    GROUP BY xtable,xid
    
  OPEN prune_cursor
  FETCH NEXT FROM prune_cursor INTO @xtable, @xid, @last_xid
  WHILE @@FETCH_STATUS = 0 
  BEGIN
    SET @last_xid_string = CAST(@last_xid AS NVARCHAR(18))
    IF @verbose >= 2 
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_prepare pruning %s for %s<=%s', 0, 1, @now, @xtable, @xid, @last_xid_string) WITH NOWAIT
    END

    SET @sql = 'DELETE FROM ' + @xtable + ' WHERE ' + @xid + ' <= ' + @last_xid_string

    EXEC(@sql)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_prepare: error %d executing sql statement: %s', 11, 1, @now, @err, @sql) WITH NOWAIT
      DEALLOCATE prune_cursor
      RETURN -30000
    END

    IF @verbose >= 2 
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_prepare executing: %s : deleted %d rows, last_xid_string = %s', 0, 1, @now, @sql, @rc, @last_xid_string) WITH NOWAIT
    END


    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
      SET @sqlCounter1 = @elapsed_ms + @sqlCounter1
      SET @start1 = CURRENT_TIMESTAMP
      RAISERROR('%s: etl_prepare. Pruning %s took %d ms', 0, 1, @now, @xtable, @elapsed_ms) WITH NOWAIT
    END

    FETCH NEXT FROM prune_cursor INTO @xtable, @xid, @last_xid
  END
  DEALLOCATE prune_cursor

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter1 = @elapsed_ms + @sqlCounter1
    SET @start1 = CURRENT_TIMESTAMP
    RAISERROR('%s: etl prepare. All pruning took %d ms', 0, 1, @now, @sqlCounter1) WITH NOWAIT
  END


  IF NOT EXISTS (SELECT 1 FROM twh_data WHERE name = 'triggers_enabled' AND value = '1')
  BEGIN
    EXEC @ret = etl_sync @verbose=@verbose
    IF @ret <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_prepare: error %d calling etl_sync.', 11, 1, @now, @ret) WITH NOWAIT
      RETURN -30000
    END
  END

  SET @start1 = CURRENT_TIMESTAMP  -- don't pick up etl_sync time (also miss the prune time)

  -- Update this_xid for each result set with the current maximum
  -- extraction id available.
  DECLARE this_set_cursor CURSOR FOR 
    SELECT xtable, xid FROM twh_ExtractCtrl
  OPEN this_set_cursor
  FETCH NEXT FROM this_set_cursor INTO @xtable, @xid
  WHILE @@FETCH_STATUS = 0 
  BEGIN
    IF @verbose >= 2 
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_prepare finding this_xid for %s', 0, 1, @now, @xtable) WITH NOWAIT
    END

    -- The event table has a tstamp column that must be converted to DECIMAL
    IF @xtable = 'event'
      SET @sql = '
        UPDATE twh_ExtractCtrl SET this_xid = ISNULL((SELECT MAX(CAST(CAST(' + @xid + ' AS BIGINT) AS DECIMAL(18))) FROM ' + @xtable + '), last_xid)
        WHERE xtable = ''' + @xtable + ''''
    ELSE
      SET @sql = '
        UPDATE twh_ExtractCtrl SET this_xid = ISNULL((SELECT MAX(' + @xid + ') FROM ' + @xtable + '), last_xid)
        WHERE xtable = ''' + @xtable + ''''
    IF @verbose >= 2 
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_prepare executing: %s', 0, 1, @now, @sql) WITH NOWAIT
    END
    EXEC(@sql)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_prepare: error %d executing sql statement: %s', 11, 1, @now, @err, @sql) WITH NOWAIT
      DEALLOCATE this_set_cursor
      RETURN -30000
    END

    FETCH NEXT FROM this_set_cursor INTO @xtable, @xid
  END
  DEALLOCATE this_set_cursor

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter2 = @elapsed_ms + @sqlCounter2
    SET @start1 = CURRENT_TIMESTAMP
    RAISERROR('%s: etl prepare. Updating twh_ExtractCtrl took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
  END


  IF @verbose >= 2 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare after finding this_xid for extraction tables', 0, 1, @now) WITH NOWAIT
    SELECT 'after setting this_xid',* FROM twh_ExtractCtrl
  END

  -- Update twh_data with the maximum depth of all resources.
  -- Ideally, you would think that we could use the gen column
  -- of ObjPathCache but that doesn't seem to contain accurate
  -- information.   What we'll do instead is count the number
  -- of components in the idpath column of resources in the 
  -- ObjPathCache table.  It would be nice if we could just
  -- count the number of slashes but that's not easy to do
  -- unless we do it in a loop.  What we can do easily instead
  -- is calculate the difference between the length of the idpath 
  -- and the length of the idpath with the slashes removed: this
  -- tells us how many slashes there are and we can just add one
  -- to that for a depth.
  DECLARE @gen INT
  SELECT @gen = MAX(LEN(idpath)-LEN(REPLACE(idpath, '/', ''))) 
    FROM ObjPathCache
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d setting @gen', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  UPDATE twh_data SET value = @gen 
    WHERE name = 'max_resource_depth'
    AND @gen > value
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d updating twh_data value', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter3 = @elapsed_ms + @sqlCounter3
    SET @start1 = CURRENT_TIMESTAMP
    RAISERROR('%s: etl prepare. Setting max depth took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
  END

  -- eliminate redundant data
  -- delete from LinkDelete if parent appears in ResourceDelete
  DELETE ld
  FROM twh_LinkDelete AS ld
    JOIN twh_ResourceDelete AS rd ON rd.cid = ld.src_cid AND rd.id = ld.src_id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d deleting from twh_LinkDelete', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  -- delete from LinkDelete if child appears in ResourceDelete
  DELETE ld
  FROM twh_LinkDelete AS ld
    JOIN twh_ResourceDelete AS rd ON rd.cid = ld.dst_cid AND rd.id = ld.dst_id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d deleting from twh_LinkDelete', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter4 = @elapsed_ms + @sqlCounter4
    SET @start1 = CURRENT_TIMESTAMP
    RAISERROR('%s: etl prepare. Deleting from twh_LinkDelete took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
  END


  -- delete from LinkDelete if logically deleted in AttrUpdate

  CREATE INDEX logicalCreatesIx ON #logicalCreates (cid, id)
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d creating index on #logicalCreates', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  INSERT INTO #logicalDeletes (cid, id, maxid)
  SELECT                       cid, id, max(xid) AS maxid
  FROM twh_AttrUpdate 
  WHERE attr_name = 'deleted' 
    AND attr_value = '1' 
  GROUP BY cid,id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d inserting into #logicalDeletes', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  CREATE INDEX logicalDeletesIx ON #logicalDeletes (cid, id)
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d creating index on #logicalDeletes', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  INSERT INTO #logicalCreates (cid, id, maxid)
  SELECT              cid, id, max(xid) AS maxid
  FROM twh_AttrUpdate 
  WHERE attr_name = 'deleted' 
    AND attr_value = '0' 
  GROUP BY cid,id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d inserting into #logicalCreates', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter5 = @elapsed_ms + @sqlCounter5
    SET @start1 = CURRENT_TIMESTAMP
    RAISERROR('%s: etl prepare. Setting up temp tables took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
  END

  --  Delete only those records where the latest entry in twh_AttrUpdate indicates a delete
  DELETE ld
  FROM twh_LinkDelete AS ld
    JOIN #logicalDeletes AS t1 ON (t1.cid = ld.dst_cid AND t1.id = ld.dst_id) OR (t1.cid = ld.src_cid AND t1.id = ld.src_id)
  WHERE NOT EXISTS (SELECT 1 FROM #logicalCreates AS t2 WHERE t2.cid = t1.cid AND t2.id = t1.id AND t2.maxid > t1.maxid)
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d deleting from twh_LinkDelete 2', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter6 = @elapsed_ms + @sqlCounter6
    SET @start1 = CURRENT_TIMESTAMP
    RAISERROR('%s: etl prepare. Deleting from twh_LinkDelete 1 took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
  END


  -- delete from AttrUpdate if logically deleted and also physically deleted (in ResourceDelete)
  DELETE a
  FROM twh_AttrUpdate AS a
    JOIN (SELECT t1.cid AS del_cid, t1.id AS del_id 
          FROM #logicalDeletes AS t1, #logicalCreates AS t2 
          WHERE (t1.cid = t2.cid AND t1.id = t2.id AND t1.maxid > t2.maxid)
          OR NOT EXISTS (SELECT 1 FROM #logicalCreates as t3 
                     WHERE t1.cid = t3.cid AND t1.id = t3.id)) AS del
      ON del.del_cid = a.cid AND del.del_id = a.id
    JOIN twh_ResourceDelete AS rd 
      ON rd.cid = del.del_cid AND rd.id = del.del_id
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d deleting from twh_AttrUpdate', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter7 = @elapsed_ms + @sqlCounter7
    SET @start1 = CURRENT_TIMESTAMP
    RAISERROR('%s: etl prepare. Deleting from twh_AttrUpdate took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
  END

  
  -- delete all records if a resource was created and physically deleted within the same extraction period

  -- resource must be in physical delete table
  INSERT INTO #deleteThese (cid, id)
  SELECT DISTINCT           d.cid, d.id
  FROM twh_ResourceDelete AS d
    JOIN twh_ResourceCreate AS c ON c.cid = d.cid AND c.id = d.id AND d.ctime > c.ctime
  WHERE NOT EXISTS (SELECT 1 FROM #logicalDeletes AS t WHERE t.cid = d.cid AND t.id = d.id)
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d inserting into #deleteThese', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter8 = @elapsed_ms + @sqlCounter8
    SET @start1 = CURRENT_TIMESTAMP
    RAISERROR('%s: etl prepare. Setting up temp delete tables took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
  END

  DELETE c
  FROM twh_ResourceCreate AS c
    JOIN #deleteThese AS d ON c.id = d.id AND c.cid = d.cid
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d deleting from twh_ResourceCreate', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter9 = @elapsed_ms + @sqlCounter9
    SET @start1 = CURRENT_TIMESTAMP
    RAISERROR('%s: etl prepare. Deleting from twh_ResourceCreate took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
  END

  DELETE c
  FROM twh_ResourceDelete AS c
    JOIN #deleteThese AS d ON c.id = d.id AND c.cid = d.cid
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d deleting from twh_ResourceDelete', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter10 = @elapsed_ms + @sqlCounter10
    SET @start1 = CURRENT_TIMESTAMP
    RAISERROR('%s: etl prepare. Deleting from twh_ResourceDelete took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
  END

  DELETE c
  FROM twh_AttrUpdate AS c
    JOIN #deleteThese AS d ON c.id = d.id AND c.cid = d.cid 
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d deleting from twh_AttrUpdate', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END


  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter11 = @elapsed_ms + @sqlCounter11
    SET @start1 = CURRENT_TIMESTAMP
    RAISERROR('%s: etl prepare. Deleting from twh_AttrUpdate took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
  END


  -- Re-load the private class info: "Public" TDW classes always stay in
  -- the twh_class_map table.  Since new classes can be created
  -- by TBSM service or the customer, we will account for them
  -- by repopulating the table.
  DELETE FROM twh_class_map WHERE _public = 0
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter12 = @elapsed_ms + @sqlCounter12
    SET @start1 = CURRENT_TIMESTAMP
    RAISERROR('%s: etl prepare. Deleting from twh_class_map took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
  END
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d deleting from twh_class_map', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  INSERT INTO twh_class_map
    SELECT cid, 'GTM_RESOURCE', label, description, 1, 0
    FROM obj_class 
    WHERE flag_abstract = 0
    AND cid NOT IN (SELECT cid FROM twh_class_map) 
    AND cid IN (SELECT cid FROM isa_chain WHERE base_cid = 'MNO2')
    AND cid NOT IN ('GMDM', 'GMGM', 'GMGN', 'GMGX', 'GMMF', 'GMSC')
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d inserting into twh_class_map', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    SET @sqlCounter13 = @elapsed_ms + @sqlCounter13
    SET @start1 = CURRENT_TIMESTAMP
    RAISERROR('%s: etl prepare. Inserting into twh_class_map took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
  END


  -- Update the ReportingComponent/TDWCrystal system configuration property if it is not already set
  -- The property is used to let asisp_deleteoldevent know that TDW extraction has been performed and
  -- the procedure should take into account what data from the event table has been received by TDW
  -- when pruning the event table.

  IF @verbose >= 2 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare setting ReportingComponent/TDWCrystal system configuration property', 0, 1, @now) WITH NOWAIT
  END

  EXEC @ret =  _SetSystemConfiguration @groupname = 'ReportingComponent', 
       @propertyname = 'TDWCrystal', 
       @propertyvalue = '1', 
       @propertydatatype = 'TINYINT'
  IF @ret <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare: error %d calling _SetSystemConfiguration.', 11, 1, @now, @ret) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start, CURRENT_TIMESTAMP)
    RAISERROR('%s:   etl_prepare took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
    RAISERROR('%s:     etl_prepare sqlCounter1  took %d ms. All prunes.', 0, 1, @now, @sqlCounter1) WITH NOWAIT
    RAISERROR('%s:     etl_prepare sqlCounter2  took %d ms. Updating twh_ExtractCtrl', 0, 1, @now, @sqlCounter2) WITH NOWAIT
    RAISERROR('%s:     etl_prepare sqlCounter3  took %d ms. Setting Maximum depth', 0, 1, @now, @sqlCounter3) WITH NOWAIT
    RAISERROR('%s:     etl_prepare sqlCounter4  took %d ms. Deleting from twh_LinkDelete', 0, 1, @now, @sqlCounter4) WITH NOWAIT
    RAISERROR('%s:     etl_prepare sqlCounter5  took %d ms. Setting up temp tables', 0, 1, @now, @sqlCounter5) WITH NOWAIT
    RAISERROR('%s:     etl_prepare sqlCounter6  took %d ms. Deleting from twh_LinkDelete 1', 0, 1, @now, @sqlCounter6) WITH NOWAIT
    RAISERROR('%s:     etl_prepare sqlCounter7  took %d ms. Deleting from twh_AttrUpdate', 0, 1, @now, @sqlCounter7) WITH NOWAIT
    RAISERROR('%s:     etl_prepare sqlCounter8  took %d ms. Setting up temp delete tables', 0, 1, @now, @sqlCounter8) WITH NOWAIT
    RAISERROR('%s:     etl_prepare sqlCounter9  took %d ms. Deleting from twh_ResourceCreate', 0, 1, @now, @sqlCounter9) WITH NOWAIT
    RAISERROR('%s:     etl_prepare sqlCounter10 took %d ms. Deleting from twh_ResourceDelete', 0, 1, @now, @sqlCounter10) WITH NOWAIT
    RAISERROR('%s:     etl_prepare sqlCounter11 took %d ms. Deleting from twh_AttrUpdate', 0, 1, @now, @sqlCounter11) WITH NOWAIT
    RAISERROR('%s:     etl_prepare sqlCounter12 took %d ms. Deleting from twh_class_map', 0, 1, @now, @sqlCounter12) WITH NOWAIT
    RAISERROR('%s:     etl_prepare sqlCounter13 took %d ms. Insert into twh_class_map', 0, 1, @now, @sqlCounter13) WITH NOWAIT
  END


  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_prepare returning', 0, 1, @now) WITH NOWAIT
  END
GO