-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/etl_sync.sql
-- SID: 1.25
-- Date Modified: 04/12/01 15:04:27

-- Module description:
-- Create the etl_sync procedure

USE Object
IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'etl_sync')
  DROP PROCEDURE etl_sync
GO
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}

-- File: archive/db/sql/procs/etl_sync.sql
-- SID: 1.25
-- Date Modified: 04/12/01 15:04:27

-- Module description:
-- Create the etl_sync procedure which is used to
-- to synchronize TDW with TBSM: information on
-- existing TBSM resources are saved in the
-- extraction tables which get returned in a
-- result set when TDW calls other etl_* procedures.

CREATE PROCEDURE etl_sync
  @verbose TINYINT=0
AS
  DECLARE @now   NVARCHAR(40)
  DECLARE @cid   ClassID
  DECLARE @cname NVARCHAR(24)
  DECLARE @sql   NVARCHAR(4000)
  DECLARE @err   INTEGER
  DECLARE @rc    INTEGER
  DECLARE @count INTEGER
  DECLARE @ret   INTEGER

  DECLARE @start DATETIME
  DECLARE @start1 DATETIME
  DECLARE @elapsed_ms INT
  DECLARE @sqlCounter1 INT
  DECLARE @sqlCounter2 INT
  DECLARE @sqlCounter3 INT
  DECLARE @sqlCounter4 INT
  DECLARE @sqlCounter5 INT
  DECLARE @sqlCounter6 INT
  DECLARE @sqlCounter7 INT
  DECLARE @sqlCounter8 INT
  DECLARE @sqlCounter9 INT
  DECLARE @sqlCounter10 INT
  DECLARE @sqlCounter11 INT
  DECLARE @sqlCounter12 INT

  DECLARE @lastLinkTime DATETIME
  DECLARE @thisLinkTime DATETIME

  CREATE TABLE #link_types (
    link_type  LinkType NOT NULL
  )

  CREATE TABLE #newlinks (cid ClassID NOT NULL, id ObjID NOT NULL)

  SET NOCOUNT ON

  SET @sqlCounter1 = 0
  SET @sqlCounter2 = 0
  SET @sqlCounter3 = 0
  SET @sqlCounter4 = 0
  SET @sqlCounter5 = 0
  SET @sqlCounter6 = 0
  SET @sqlCounter7 = 0
  SET @sqlCounter8 = 0
  SET @sqlCounter9 = 0
  SET @sqlCounter10 = 0
  SET @sqlCounter11 = 0
  SET @sqlCounter12 = 0

  SET @start = CURRENT_TIMESTAMP
  SET @start1 = CURRENT_TIMESTAMP

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_sync starts', 0, 1, @now) WITH NOWAIT
  END

  INSERT INTO #link_types (link_type)
  SELECT DISTINCT l.link_type
  FROM derived_obj_link AS l
  JOIN link_type AS lt ON lt.link_type = l.link_type
  JOIN obj_class AS os ON os.cid = l.src_cid
  JOIN obj_class AS od ON od.cid = l.dst_cid
  JOIN isa_chain AS si ON si.cid = os.cid
  JOIN isa_chain AS di ON di.cid = od.cid
  WHERE os.flag_abstract = 0
    AND od.flag_abstract = 0
    AND (lt.flag_containment = 1 OR lt.flag_parent_child = 1 OR lt.flag_propagation = 1 OR lt.flag_logical = 1)
    AND si.base_cid = 'MNO2'
    AND di.base_cid = 'MNO2'
  ORDER BY l.link_type
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_sync: error %d inserting into #link_types', 11, 1, @now, @err) WITH NOWAIT
    RETURN -30000
  END

  DECLARE class_cursor CURSOR FOR 
    SELECT cid, cname FROM obj_class WHERE 
           flag_abstract = 0 AND cid IN (SELECT cid FROM isa_chain WHERE base_cid = 'MNO2')
                             AND cid <> 'LOB'
  OPEN class_cursor
  FETCH NEXT FROM class_cursor INTO @cid, @cname
  WHILE @@FETCH_STATUS = 0
  BEGIN

    IF @verbose >= 2 
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_sync processing cid=%s, cname=%s', 0, 1, @now, @cid, @cname) WITH NOWAIT
    END

    SET @count = 0
    SET @sql = 'IF EXISTS (SELECT 1 FROM ' + @cname + '_V  WITH (NOLOCK))
                  SET @count = 1'

    EXEC @ret = sp_executesql @sql, N'@count INT OUTPUT',@count = @count OUTPUT
    IF @ret <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_sync: error %d calling sp_executesql.', 11, 1, @now, @ret) WITH NOWAIT
      RETURN -30000
    END

    IF @verbose >= 2 
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_sync:  %s, %d', 0, 1, @now, @sql, @count) WITH NOWAIT
    END

    IF (@count > 0)
    BEGIN


      SET @start1 = CURRENT_TIMESTAMP
      -- populate twh_ResourceCreate with records not in TDW
      SET @sql = 'INSERT INTO twh_ResourceCreate (cid,   id,   name,   description, ctime)
                  SELECT                        c.cid, c.id, c.name,   c._Desc,     c.ctime
      FROM ' + @cname + '_V AS c WITH (NOLOCK)
      WHERE NOT EXISTS (SELECT 1 FROM twh_Inventory AS i WHERE c.id = i.id AND c.cid =i.cid)
        AND NOT EXISTS (SELECT 1 FROM twh_ResourceCreate AS r WHERE c.id = r.id AND c.cid = r.cid)
      OPTION(MAXDOP 1)'

      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: etl_sync: error %d executing sql statement: %s', 11, 1, @now, @err, @sql) WITH NOWAIT
        DEALLOCATE class_cursor
        RETURN -30000
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter1 = @elapsed_ms + @sqlCounter1
        SET @start1 = CURRENT_TIMESTAMP
        RAISERROR('%s: etl_sync. Insert %d rows into twh_ResourceCreate for class %s took %d ms', 0, 1, @now, @rc,@cid, @elapsed_ms) WITH NOWAIT
      END

      -- populate twh_LinkCreate with records not in TDW 
      SET @sql = 
      'INSERT INTO twh_LinkCreate (link_type,   src_cid,   src_id,   dst_cid,   dst_id,   ctime)
       SELECT                      l.link_type, l.src_cid, l.src_id, l.dst_cid, l.dst_id, l.ctime
       FROM link AS l WITH (NOLOCK)
         JOIN #link_types AS lt ON lt.link_type = l.link_type
         JOIN ' + @cname + '_V AS c WITH (NOLOCK) ON l.dst_cid = ''' + @cid + ''' AND c.id = l.dst_id
       WHERE NOT EXISTS (SELECT 1 FROM twh_Inventory AS i WHERE c.id = i.id AND c.cid =i.cid)
         AND NOT EXISTS (SELECT 1 FROM twh_LinkCreate AS l1 WHERE l1.link_type = l.link_type
                                                            AND  l1.src_cid   = l.src_cid
                                                            AND  l1.src_id    = l.src_id
                                                            AND  l1.dst_cid   = l.dst_cid
                                                            AND  l1.dst_id    = l.dst_id)
       OPTION(MAXDOP 1)'

      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: etl_sync: error %d inserting into twh_LinkCreate: %s', 11, 1, @now, @err) WITH NOWAIT
        DEALLOCATE class_cursor
        RETURN -30000
      END

      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter2 = @elapsed_ms + @sqlCounter2
        SET @start1 = CURRENT_TIMESTAMP
        RAISERROR('%s: etl_sync. Insert %d rows into twh_LinkCreate for class %s took %d ms', 0, 1, @now, @rc, @cid, @elapsed_ms) WITH NOWAIT
      END

      -- populate twh_LinkDelete for each resource in TBSM not in TDW and link is deleted
      SET @sql = 
      'INSERT INTO twh_LinkDelete (link_type,   src_cid,   src_id,   dst_cid,   dst_id,   ctime)
       SELECT                      l.link_type, l.src_cid, l.src_id, l.dst_cid, l.dst_id, l.ctime
       FROM link AS l WITH (NOLOCK)
         JOIN #link_types AS lt ON lt.link_type = l.link_type
         JOIN ' + @cname + '_V AS c WITH (NOLOCK) ON l.dst_cid = ''' + @cid + ''' AND c.id = l.dst_id
       WHERE l.deleted = 1
         AND NOT EXISTS (SELECT 1 FROM twh_Inventory AS i WHERE c.id = i.id AND c.cid =i.cid)
         AND NOT EXISTS (SELECT 1 FROM twh_LinkDelete AS l1 WHERE l1.link_type = l.link_type
                                                            AND  l1.src_cid   = l.src_cid
                                                            AND  l1.src_id    = l.src_id
                                                            AND  l1.dst_cid   = l.dst_cid
                                                            AND  l1.dst_id    = l.dst_id)
       OPTION(MAXDOP 1)'

      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: etl_sync: error %d inserting into twh_LinkDelete: %s', 11, 1, @now, @err, @sql) WITH NOWAIT
        DEALLOCATE class_cursor
        RETURN -30000
      END

      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter3 = @elapsed_ms + @sqlCounter3
        SET @start1 = CURRENT_TIMESTAMP
        RAISERROR('%s: etl_sync. Insert %d rows into twh_LinkDelete for class %s took %d ms', 0, 1, @now, @rc, @cid, @elapsed_ms) WITH NOWAIT
      END

      -- populate twh_AttrUpdate for each resource that:
      --   o is marked as deleted in TBSM but not marked as deleted in TDW
      --   o is not marked as deleted in TBSM but is marked as deleted in TDW
      --   o is marked as deleted in TBSM but does not exist in TDW
      SET @sql = 'INSERT INTO twh_AttrUpdate (cid,   id,   attr_name,   attr_value,                        ctime)
                  SELECT                    c.cid, c.id, ''deleted'',  CAST(c.deleted AS NVARCHAR(1024)), ISNULL(c.mtime, c.ctime)
      FROM ' + @cname + '_V AS c WITH (NOLOCK)
      LEFT OUTER JOIN twh_Inventory AS i ON i.cid = c.cid AND i.id = c.id
      WHERE i.id IS NULL AND c.deleted = 1
      OPTION(MAXDOP 1)'

      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: etl_sync: error %d executing sql statement: %s', 11, 1, @now, @err, @sql) WITH NOWAIT
        DEALLOCATE class_cursor
        RETURN -30000
      END

      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter4 = @elapsed_ms + @sqlCounter4
        SET @start1 = CURRENT_TIMESTAMP
        RAISERROR('%s: etl_sync. Insert %d rows into twh_AttrUpdate 0 for class %s took %d ms', 0, 1, @now, @rc, @cid, @elapsed_ms) WITH NOWAIT
      END


      SET @sql = 'INSERT INTO twh_AttrUpdate (cid,   id,   attr_name,   attr_value,                        ctime)
                  SELECT                     c.cid, c.id, ''deleted'',  CAST(c.deleted AS NVARCHAR(1024)), ISNULL(c.mtime, c.ctime)
      FROM ' + @cname + '_V AS c WITH (NOLOCK)
      JOIN twh_Inventory AS i ON i.cid = c.cid AND i.id = c.id
      WHERE i.deleted <> c.deleted
      OPTION(MAXDOP 1)'

      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: etl_sync: error %d executing sql statement: %s', 11, 1, @now, @err, @sql) WITH NOWAIT
        DEALLOCATE class_cursor
        RETURN -30000
      END

      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter11 = @elapsed_ms + @sqlCounter11
        SET @start1 = CURRENT_TIMESTAMP
        RAISERROR('%s: etl_sync. Insert %d rows into twh_AttrUpdate for class %s took %d ms', 0, 1, @now, @rc, @cid, @elapsed_ms) WITH NOWAIT
      END


      -- populate twh_LinkCreate with links for resources that are being un-deleted: they are marked as
      -- deleted in TDW but they are not logically-deleted in TBSM.
      SET @sql = 'INSERT INTO twh_LinkCreate (link_type,   src_cid,   src_id,   dst_cid,   dst_id,   ctime)
                  SELECT                    l.link_type, l.src_cid,   l.src_id, l.dst_cid, l.dst_id, 
                         CASE WHEN l.ctime > ISNULL(c.mtime, c.ctime) THEN l.ctime ELSE ISNULL(c.mtime, c.ctime) END
      FROM ' + @cname + '_V AS c WITH (NOLOCK)
      JOIN link          AS l WITH (NOLOCK) ON l.dst_cid = c.cid AND l.dst_id = c.id
      JOIN #link_types   AS lt ON lt.link_type = l.link_type
      JOIN twh_Inventory AS i ON i.cid = c.cid AND i.id = c.id
      WHERE c.deleted = 0 AND i.deleted = 1 AND l.deleted = 0
      OPTION(MAXDOP 1)'

      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: etl_sync: error %d executing sql statement: %s', 11, 1, @now, @err, @sql) WITH NOWAIT
        DEALLOCATE class_cursor
        RETURN -30000
      END

      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter5 = @elapsed_ms + @sqlCounter5
        SET @start1 = CURRENT_TIMESTAMP
        RAISERROR('%s: etl_sync. Insert %d rows into twh_LinkCreate for class %s took %d ms', 0, 1, @now, @rc, @cid, @elapsed_ms) WITH NOWAIT
      END


      -- populate twh_AttrUpdate for each resource that was renamed and in both 
      SET @sql = 'INSERT INTO twh_AttrUpdate (cid,   id,   attr_name,   attr_value, ctime)
                  SELECT                    c.cid, c.id,   ''name'',    c.name, ISNULL(c.mtime, c.ctime)
      FROM ' + @cname + '_V AS c WITH (NOLOCK) JOIN twh_Inventory AS i ON c.id = i.id AND c.cid = i.cid
                                                         AND c.name <> i.name
      WHERE c.deleted = 0
      OPTION(MAXDOP 1)'

      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: etl_sync: error %d executing sql statement: %s', 11, 1, @now, @err, @sql) WITH NOWAIT
        DEALLOCATE class_cursor
        RETURN -30000
      END

      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter6 = @elapsed_ms + @sqlCounter6
        SET @start1 = CURRENT_TIMESTAMP
        RAISERROR('%s: etl_sync. Insert %d rows into twh_AttrUpdate 2 for class %s took %d ms', 0, 1, @now, @rc, @cid, @elapsed_ms) WITH NOWAIT
      END

/*
   143279: etl_updateattr has been changed to eliminate description changes from its result set.
   This is because TBSM can save up to 255 characters but TDW only stores 254 BYTES (*not* characters!).
   Not only does this cause problems for a 255-length description but some languages use two or more
   bytes per character which reduces the actual number of characters even more.  Since we're not reporting
   the changes to descriptions, we don't want to record the differences each time etl_sync runs.  We won't
   even create the records that say the descriptions have changed.

      -- populate twh_AttrUpdate for each resource whose description changed and in both 
      SET @sql = 'INSERT INTO twh_AttrUpdate (cid,   id,   attr_name,   attr_value, ctime)
                  SELECT                      c.cid, c.id, ''description'',    c._Desc, ISNULL(c.mtime, c.ctime)
      FROM ' + @cname + '_V AS c WITH (NOLOCK) JOIN twh_Inventory AS i ON c.id = i.id AND c.cid = i.cid
                                                         AND c._Desc <> i.description
      WHERE c.deleted = 0
      OPTION(MAXDOP 1)'

      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: etl_sync: error %d executing sql statement: %s', 11, 1, @now, @err, @sql) WITH NOWAIT
        DEALLOCATE class_cursor
        RETURN -30000
      END

      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter7 = @elapsed_ms + @sqlCounter7
        SET @start1 = CURRENT_TIMESTAMP
        RAISERROR('%s: etl_sync. Insert %d rows into twh_AttrUpdate 3 for class %s took %d ms', 0, 1, @now, @rc, @cid, @elapsed_ms) WITH NOWAIT
      END
*/
    END  -- IF @count > 0
          -- populate twh_ResourceDelete for each resource in TDW but not in TBSM
    SET @sql = 'INSERT INTO twh_ResourceDelete (cid,   id)
                SELECT                         i.cid, i.id
    FROM twh_Inventory AS i
    WHERE i.cid = ''' + @cid + '''
      AND NOT EXISTS (SELECT 1 FROM ' + @cname + '_V AS c WITH (NOLOCK) WHERE c.id = i.id)
      AND NOT EXISTS (SELECT 1 FROM twh_ResourceDelete AS r WHERE r.id = i.id
                                                              AND r.cid = i.cid)
    OPTION(MAXDOP 1)'

    EXEC(@sql)
    SELECT @err = @@ERROR, @rc = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_sync: error %d executing sql statement: %s', 11, 1, @now, @err, @sql) WITH NOWAIT
      DEALLOCATE class_cursor
      RETURN -30000
    END
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
      SET @sqlCounter9 = @elapsed_ms + @sqlCounter9
      SET @start1 = CURRENT_TIMESTAMP
      RAISERROR('%s: etl_sync. Insert %d rows into twh_ResourceDelete for class %s took %d ms', 0, 1, @now, @rc, @cid, @elapsed_ms) WITH NOWAIT
    END

    FETCH NEXT FROM class_cursor INTO @cid, @cname
  END
  DEALLOCATE class_cursor

  -- handle directly contained links
  DECLARE class_cursor2 CURSOR FOR 
    SELECT c.cid, c.cname FROM obj_class AS c
    JOIN isa_chain AS i ON i.cid = c.cid
    WHERE c.flag_direct_containment = 1 
      AND c.flag_abstract = 0
      AND i.base_cid = 'MNO2'
  OPEN class_cursor2
  FETCH NEXT FROM class_cursor2 INTO @cid, @cname
  WHILE @@FETCH_STATUS = 0 
  BEGIN

    IF @verbose >= 2 
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_sync directly linked class processing cid=%s, cname=%s', 0, 1, @now, @cid, @cname) WITH NOWAIT
    END

    SET @count = 0
    SET @sql = 'IF EXISTS (SELECT 1 FROM ' + @cname + '_V  WITH (NOLOCK))
                  SET @count = 1'

    EXEC @ret = sp_executesql @sql, N'@count INT OUTPUT',@count = @count OUTPUT
    IF @ret <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_sync: error %d calling sp_executesql 2.', 11, 1, @now, @ret) WITH NOWAIT
      RETURN -30000
    END

    IF @verbose >= 2 
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: etl_sync:  %s, %d', 0, 1, @now,  @sql, @count) WITH NOWAIT
    END

    IF (@count > 0)
    BEGIN

      IF (@verbose >= 2) 
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: etl_sync adding to twh_LinkCreate for directly-linked cid=%s', 0, 1, @now, @cid) WITH NOWAIT
      END

      SET @start1 = CURRENT_TIMESTAMP
      -- populate twh_LinkCreate with records not in TDW
      SET @sql = 
      'INSERT INTO twh_LinkCreate (link_type,   src_cid,      src_id,      dst_cid,   dst_id,   ctime)
       SELECT                     ''DRCT'',    c._ParentCID, c._ParentID, c.cid,     c.id,     c.ctime
       FROM ' + @cname + '_V AS c WITH (NOLOCK)
       WHERE NOT EXISTS (SELECT 1 FROM twh_Inventory AS i WHERE c.id = i.id AND c.cid =i.cid)
         AND NOT EXISTS (SELECT 1 FROM twh_LinkCreate AS l WHERE l.link_type = ''DRCT''
                                                            AND  l.src_cid = c._ParentCID
                                                            AND  l.src_id  = c._ParentID
                                                            AND  l.dst_cid = c.cid
                                                            AND  l.dst_id  = c.id)
       OPTION(MAXDOP 1)'

      EXEC(@sql)
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: etl_sync: error %d inserting into twh_LinkCreate: %s', 11, 1, @now, @err) WITH NOWAIT
        DEALLOCATE class_cursor2
        RETURN -30000
      END

      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
        SET @sqlCounter10 = @elapsed_ms + @sqlCounter10
        SET @start1 = CURRENT_TIMESTAMP
        RAISERROR('%s: etl_sync. Insert %d rows into twh_LinkCreate DRCT for class %s took %d ms', 0, 1, @now, @rc, @cid, @elapsed_ms) WITH NOWAIT
      END



    END -- IF @count > 0

    FETCH NEXT FROM class_cursor2 INTO @cid, @cname
  END
  DEALLOCATE class_cursor2

  -- Tell TDW about new links to existing resources.  First, we'll make a list of all existing
  -- resources that have new links to them.

  SET @start1 = CURRENT_TIMESTAMP

  SELECT @lastLinkTime = MIN(prevlastenddate) FROM twh_date WHERE loadtype LIKE 'link/new%objs'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0 OR @rc = 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_sync: error %d retrieving @lastLinkTime', 11, 1, @now, @ret) WITH NOWAIT
    RETURN -30000
  END

  SELECT @thisLinkTime = MAX(ctime) FROM link WITH (NOLOCK)
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_sync: error %d retrieving @thisLinkTime', 11, 1, @now, @ret) WITH NOWAIT
    RETURN -30000
  END

  UPDATE twh_date SET lastenddate = @thisLinkTime WHERE loadtype LIKE 'link/new%objs'
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @err <> 0 OR @rc = 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_sync: error %d updating twh_date', 11, 1, @now, @ret) WITH NOWAIT
    RETURN -30000
  END

  CREATE INDEX newlinksIx ON #newlinks (cid, id)
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_sync: error %d creating newlinksIx index', 11, 1, @now, @ret) WITH NOWAIT
    RETURN -30000
  END

  INSERT INTO #newlinks
    SELECT DISTINCT i.cid, i.id
      FROM twh_Inventory AS i
      JOIN link         AS l ON i.cid = l.dst_cid AND i.id = l.dst_id
      WHERE l.deleted = 0 AND l.ctime > @lastLinkTime AND l.ctime <= @thisLinkTime
    OPTION(MAXDOP 1)
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_sync: error %d populating #newlinks', 11, 1, @now, @ret) WITH NOWAIT
    RETURN -30000
  END

  -- Tell TDW about all currently existing links to the resources we found
  INSERT INTO twh_LinkCreate (link_type,   src_cid,   src_id,   dst_cid,   dst_id,   ctime)
    SELECT                    l.link_type, l.src_cid, l.src_id, l.dst_cid, l.dst_id, l.ctime
    FROM link      AS l WITH (NOLOCK)
    JOIN #newlinks AS o ON o.cid = l.dst_cid AND o.id = l.dst_id
    WHERE l.deleted = 0 AND 
          l.ctime > @lastLinkTime AND
          l.ctime <= @thisLinkTime AND
          NOT EXISTS (SELECT 1 FROM twh_LinkCreate AS lc WHERE
                        lc.link_type = l.link_type AND
                        lc.src_cid   = l.src_cid   AND
                        lc.src_id    = l.src_id    AND
                        lc.dst_cid   = l.dst_cid   AND
                        lc.dst_id    = l.dst_id)
    OPTION(MAXDOP 1)
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_sync: error %d populating twh_LinkCreate from #newlinks', 11, 1, @now, @ret) WITH NOWAIT
    RETURN -30000
  END

  -- Tell TDW about all deleted links to the resources we found
  INSERT INTO twh_LinkDelete (link_type,   src_cid,   src_id,   dst_cid,   dst_id,   ctime)
    SELECT                    l.link_type, l.src_cid, l.src_id, l.dst_cid, l.dst_id, CURRENT_TIMESTAMP
    FROM link      AS l WITH (NOLOCK)
    JOIN #newlinks AS o ON o.cid = l.dst_cid AND o.id = l.dst_id
    WHERE l.deleted = 1 AND NOT EXISTS (SELECT 1 FROM twh_LinkCreate AS lc WHERE
                                          lc.link_type = l.link_type AND
                                          lc.src_cid   = l.src_cid   AND
                                          lc.src_id    = l.src_id    AND
                                          lc.dst_cid   = l.dst_cid   AND
                                          lc.dst_id    = l.dst_id)
    OPTION(MAXDOP 1)
  SELECT @err = @@ERROR
  IF @err <> 0
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_sync: error %d populating twh_LinkDelete from #newlinks', 11, 1, @now, @ret) WITH NOWAIT
    RETURN -30000
  END

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start1, CURRENT_TIMESTAMP)
    RAISERROR('%s: etl_sync took %d ms to account for new links for existing resources', 0, 1, @now, @elapsed_ms) WITH NOWAIT
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    SET @elapsed_ms = DATEDIFF(MS, @start, CURRENT_TIMESTAMP)
    RAISERROR('%s:   etl_sync took %d ms', 0, 1, @now, @elapsed_ms) WITH NOWAIT
    RAISERROR('%s:     etl_sync sqlCounter1  took %d ms. Insert into twh_ResourceCreate', 0, 1, @now, @sqlCounter1) WITH NOWAIT
    RAISERROR('%s:     etl_sync sqlCounter2  took %d ms. Insert into twh_LinkCreate', 0, 1, @now, @sqlCounter2) WITH NOWAIT
    RAISERROR('%s:     etl_sync sqlCounter3  took %d ms. Insert into twh_LinkDelete', 0, 1, @now, @sqlCounter3) WITH NOWAIT
    RAISERROR('%s:     etl_sync sqlCounter4  took %d ms. Insert into twh_AttrUpdate 0', 0, 1, @now, @sqlCounter4) WITH NOWAIT
    RAISERROR('%s:     etl_sync sqlCounter11 took %d ms. Insert into twh_AttrUpdate ', 0, 1, @now, @sqlCounter11) WITH NOWAIT
    RAISERROR('%s:     etl_sync sqlCounter5  took %d ms. Insert into twh_AttrUpdate 1', 0, 1, @now, @sqlCounter5) WITH NOWAIT
    RAISERROR('%s:     etl_sync sqlCounter6  took %d ms. Insert into twh_AttrUpdate 2', 0, 1, @now, @sqlCounter6) WITH NOWAIT
    RAISERROR('%s:     etl_sync sqlCounter7  took %d ms. Insert into twh_AttrUpdate 3', 0, 1, @now, @sqlCounter7) WITH NOWAIT
    RAISERROR('%s:     etl_sync sqlCounter8  took %d ms. Insert into twh_LinkDelete', 0, 1, @now, @sqlCounter8) WITH NOWAIT
    RAISERROR('%s:     etl_sync sqlCounter9  took %d ms. Insert into twh_ResourceDelete', 0, 1, @now, @sqlCounter9) WITH NOWAIT
    RAISERROR('%s:     etl_sync sqlCounter10 took %d ms. Insert into twh_LinkCreate for DRCT', 0, 1, @now, @sqlCounter10) WITH NOWAIT
  END

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_sync returning', 0, 1, @now)
  END
GO