-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/etl_updateextctl.sql
-- SID: 1.11
-- Date Modified: 04/11/15 18:37:37

-- Module description: This procs is used to update the extract control table twh_ExtracCtrl 
-- in TBSM database based on the extract control in central data warehouse. It ise use when
-- installing TBSM 3.2.0.0 WEP and doing Maintenance and fixes. 
-- Create the etl_updateextctl procedure

USE Object
IF EXISTS (SELECT * FROM sysobjects WHERE type = 'P' AND name = 'etl_updateextctl')
  DROP PROCEDURE etl_updateextctl
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/etl_updateextctl.sql
-- SID: 1.11
-- Date Modified: 04/11/15 18:37:37

-- Module description:
-- Create the etl_updateextctl procedure which is used by
-- TDW to update the extraction controls.

CREATE PROCEDURE etl_updateextctl
  @verbose TINYINT=0
AS

  SET NOCOUNT ON
  DECLARE @now NVARCHAR(40), @max_id DECIMAL(18,0)

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_updateextctl starts', 0, 1, @now) WITH NOWAIT
  END

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: updating event extraction control ', 0, 1, @now) WITH NOWAIT
  END

  -- handle event 
  IF EXISTS (SELECT 1 FROM twh_tmp_extctl WHERE extctl_source = 'ETL1_events')
    UPDATE t SET this_xid = p.extctl_to_int,
      t.last_xid = p.extctl_from_int
      FROM twh_ExtractCtrl t, twh_tmp_extctl AS p 
      WHERE t.name = 'events'
      AND p.extctl_source = 'ETL1_events'
      AND t.this_xid > p.extctl_to_int
  ELSE
  BEGIN
    SELECT @max_id = CAST(MAX(tstamp) AS BIGINT)
      FROM event
    IF @max_id IS NULL
      SELECT @max_id = 1
    UPDATE t SET this_xid = @max_id,
      t.last_xid = @max_id
      FROM twh_ExtractCtrl t 
      WHERE t.name = 'events'
    -- load message in Message_V at init time for note and alert state history correlation
    IF NOT EXISTS (SELECT 1 FROM twh_ExtractCtrl WHERE name = 'init_mesg')
      INSERT INTO twh_ExtractCtrl 
        SELECT 'init_mesg', 
        'Message_V', 
        'id', 
        0, 
        1, 
        max(id),
        GETDATE(),
        max(id),
        GETDATE(),
        GETDATE()
        FROM Message_V
    ELSE
    BEGIN
      SELECT @max_id = max(id) FROM Message_V
      UPDATE twh_ExtractCtrl SET this_xid = @max_id, 
        last_xid = @max_id,
        last_mtime = GETDATE(),
        this_mtime = GETDATE(),
        status = 1
        WHERE name = 'init_mesg'
    END

    -- load exception in Exception_V at init time for note and alert state history correlation
    IF NOT EXISTS (SELECT 1 FROM twh_ExtractCtrl WHERE name = 'init_excp')
      INSERT INTO twh_ExtractCtrl 
        SELECT 'init_excp', 
        'Exception_V', 
        'id', 
        0, 
        1, 
        max(id),
        GETDATE(),
        max(id),
        GETDATE(),
        GETDATE()
        FROM Exception_V
    ELSE
    BEGIN
      SELECT @max_id = max(id) FROM Exception_V
      UPDATE twh_ExtractCtrl SET this_xid = @max_id,
        last_xid = @max_id,
        last_mtime = GETDATE(),
        this_mtime = GETDATE(),
        status = 1
        WHERE name = 'init_excp'
    END
    -- load child event in ChildEvent_V at init time for note and alert state history correlation
    IF NOT EXISTS (SELECT 1 FROM twh_ExtractCtrl WHERE name = 'init_chev')
      INSERT INTO twh_ExtractCtrl 
        SELECT 'init_chev', 
        'ChildEvent_V', 
        'id', 
        0, 
        1, 
        max(id),
        GETDATE(),
        max(id),
        GETDATE(),
        GETDATE()
        FROM ChildEvent_V
    ELSE
    BEGIN
      SELECT @max_id = max(id) FROM ChildEvent_V
      UPDATE twh_ExtractCtrl SET this_xid = @max_id,
        last_xid = @max_id,
        last_mtime = GETDATE(),
        this_mtime = GETDATE(),
        status = 1
        WHERE name = 'init_chev'
    END
  END

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: updating NoteHistory extraction control ', 0, 1, @now) WITH NOWAIT
  END
  -- handle note 
  IF EXISTS (SELECT 1 FROM twh_tmp_extctl WHERE extctl_source = 'ETL1_notehistory')
    UPDATE t SET this_xid = p.extctl_to_int,
      t.last_xid = p.extctl_from_int
      FROM twh_ExtractCtrl t, twh_tmp_extctl AS p 
      WHERE t.name = 'notehistory'
      AND p.extctl_source = 'ETL1_notehistory'
      AND t.this_xid > p.extctl_to_int
  ELSE
  BEGIN
    SELECT @max_id = MAX(ident)
      FROM NoteHistory
    UPDATE t SET this_xid = @max_id,
      t.last_xid = @max_id
      FROM twh_ExtractCtrl t 
      WHERE t.name = 'notehistory'
  END

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: updating AlertStateHistory extraction control ', 0, 1, @now) WITH NOWAIT
  END

  -- handle AlertStateHistory 
  IF EXISTS (SELECT 1 FROM twh_tmp_extctl WHERE extctl_source = 'ETL1_ash')
    UPDATE t SET this_xid = p.extctl_to_int,
      t.last_xid = p.extctl_from_int
      FROM twh_ExtractCtrl t, twh_tmp_extctl AS p 
      WHERE t.name = 'ash'
      AND p.extctl_source = 'ETL1_ash'
      AND t.this_xid > p.extctl_to_int
  ELSE
  BEGIN
    SELECT @max_id = MAX(ash_id)
      FROM AlertStateHistory
    UPDATE t SET this_xid = @max_id,
      t.last_xid = @max_id
      FROM twh_ExtractCtrl t 
      WHERE t.name = 'ash'
  END

  -- clean up the tracking tables
  TRUNCATE TABLE twh_AttrUpdate
  TRUNCATE TABLE twh_defer_ash
  TRUNCATE TABLE twh_defer_events
  TRUNCATE TABLE twh_defer_notehist
  TRUNCATE TABLE twh_LinkCreate
  TRUNCATE TABLE twh_LinkDelete
  TRUNCATE TABLE twh_ResourceCreate
  TRUNCATE TABLE twh_ResourceDelete
  -- reset extraction control
  UPDATE twh_ExtractCtrl SET this_xid = 0,
        last_xid = 0,
        last_mtime = GETDATE(),
        this_mtime = GETDATE(),
        status = 0
  WHERE name IN ('newpubobjs', 'newprivobjs', 'updateattr', 'deletedobjs', 'deletedlinks')

  -- handle the extract control data in twh_date
  -- for link/newpubobjs
  IF NOT EXISTS (SELECT 1 FROM twh_date WHERE loadtype = 'link/newpubobjs')
  BEGIN
    IF EXISTS (SELECT 1 FROM twh_tmp_extctl WHERE extctl_source = 'ETL1_linkc')
       INSERT INTO twh_date 
         SELECT 'link/newpubobjs',
         extctl_to_dttm,
         extctl_from_dttm,
         0
         FROM twh_tmp_extctl WHERE extctl_source = 'ETL1_linkc'
    ELSE 
      INSERT INTO twh_date (loadtype, lastenddate, prevlastenddate, status)
         VALUES ('link/newpubobjs', '1970/01/01', '1970/01/01', 0)
  END
  ELSE
    UPDATE t SET prevlastenddate = c.extctl_from_dttm,
        t.lastenddate = c.extctl_to_dttm
        FROM twh_date AS t 
        JOIN twh_tmp_extctl AS c ON t.loadtype = 'link/newpubobjs'
        AND c.extctl_source = 'ETL1_linkc'
  -- for link/newprivobjs
  IF NOT EXISTS (SELECT 1 FROM twh_date WHERE loadtype = 'link/newprivobjs')
  BEGIN
    IF EXISTS (SELECT 1 FROM twh_tmp_extctl WHERE extctl_source = 'ETL1_linkp')
       INSERT INTO twh_date 
         SELECT 'link/newprivobjs',
         extctl_to_dttm,
         extctl_from_dttm,
         0
         FROM twh_tmp_extctl WHERE extctl_source = 'ETL1_linkp'
    ELSE 
      INSERT INTO twh_date (loadtype, lastenddate, prevlastenddate, status)
         SELECT 'link/newprivobjs', '1970/01/01', '1970/01/01', 0
  END
  ELSE
    UPDATE t SET prevlastenddate = c.extctl_from_dttm,
        t.lastenddate = c.extctl_to_dttm
        FROM twh_date AS t 
        JOIN twh_tmp_extctl AS c ON t.loadtype = 'link/newprivobjs'
        AND c.extctl_source = 'ETL1_linkp'

  -- cleanup for the entries with *_mtime is NULL
  UPDATE twh_ExtractCtrl SET last_mtime = GETDATE()  
    WHERE  last_mtime IS NULL
  UPDATE twh_ExtractCtrl SET this_mtime = getdate()
    WHERE this_mtime IS NULL

  IF @verbose >= 1 
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: etl_updateextctl done ', 0, 1, @now) WITH NOWAIT
  END
GO
