-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/tbsmd/sql/procs/gemsp_invoketask.sql
-- SID: 1.25.1.3
-- Date Modified: 05/01/06 15:09:02

USE Object
GO

SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON
GO

IF EXISTS (SELECT * FROM sysobjects WHERE id = object_id(N'[dbo].[gemsp_invokeTask]') AND OBJECTPROPERTY(id, N'IsProcedure') = 1)
DROP PROCEDURE [dbo].[gemsp_invokeTask]
GO


-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/tbsmd/sql/procs/gemsp_invoketask.sql
-- SID: 1.25.1.3
-- Date Modified: 05/01/06 15:09:02
CREATE PROCEDURE gemsp_invokeTask
 @cid         ClassID,
 @id          ObjID,
 @tskname     NVARCHAR(255),
 @tsklib      NVARCHAR(255),
 @tskargs     NVARCHAR(255),
 @cmd         NVARCHAR(540) OUTPUT,
 @timeout     NVARCHAR(8) = NULL
AS
  RAISERROR('gemsp_invokeTask: @cid=%s, @id=%d, @tskname=%s, @tsklib=%s, @tskargs=%s,
	    @cmd=%s, timeout=%s OUT', 0, 1, @cid, @id, @tskname, @tsklib, @tskargs, @cmd, @timeout)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN gemsp_invokeTask
  ELSE
    SAVE TRAN gemsp_invokeTask

  /* Returns a NULL IN @cmd IF @hostname AND @tmeoid IS NULL */
  DECLARE @hostname NVARCHAR(64), @eehost NVARCHAR(64), @tmeoid NVARCHAR(64), @ctype NVARCHAR(32)
  DECLARE @data4 NVARCHAR(255)
  DECLARE @print_string NVARCHAR(1024)

  SELECT @ctype=comptype FROM GEMLookupCID WHERE cid=@cid
  EXEC gemsp_getAttribute @cid, @id, '_EEhost', @eehost OUTPUT

  DECLARE @userID NVARCHAR(255)
  DECLARE @usergroup NVARCHAR(255)
  EXEC asisp_getContextUserInfo @userID OUTPUT, @usergroup OUTPUT



  IF @ctype='os/390'
  BEGIN
    EXEC gemsp_getAttribute @cid, @id, '_NetviewDomainName', @hostname OUTPUT
    IF @hostname IS NOT NULL
    BEGIN
      SELECT @tskargs = replace(@tskargs, '|', ' ')
      SELECT @cmd = 'tgmtask -d 390 -h "' + @eehost + '" -n "' + @hostname + '" '
      IF @userID IS NOT NULL SET @cmd = @cmd + ' -u "' + @userID + '" -k "********" '
      SET @cmd = @cmd + ' "' + @tskname + ' ' + @tskargs + '"'

      -- commit transaction
      IF @tcount = 0
        COMMIT TRAN gemsp_invokeTask
      RETURN
    END
  END
  ELSE
  BEGIN
    IF @ctype='generic'
    BEGIN
      DECLARE @pcid ClassID
      SELECT @pcid=base_cid FROM isa WHERE cid=@cid

      IF @pcid='GMGN'
      BEGIN
        SET @cmd = 'echo Tasks are not supported for Generic events (GMGN)'

	-- commit transaction
	IF @tcount = 0
  	  COMMIT TRAN gemsp_invokeTask
        RETURN
      END
      ELSE IF @pcid='GMGX'
      BEGIN
        EXEC gemsp_getAttribute @cid, @id, '_MgedSystemName', @hostname OUTPUT

        EXEC gemsp_getAttribute @cid, @id, '_Data4', @data4 OUTPUT
        IF @data4 IS NOT NULL AND @data4 <> ''
        BEGIN
          IF SUBSTRING(@data4,1,4) <> 'OID=' AND SUBSTRING(@data4,1,6) <> 'LABEL='
            SET @data4 = NULL
        END
      END
    END

    IF @ctype='gem' OR @ctype='dm'
      EXEC gemsp_getAttribute @cid, @id, '_MgedSystemName', @hostname OUTPUT
    IF @ctype='gem'
      EXEC gemsp_getAttribute @cid, @id, '_TmeEndpointOid', @tmeoid OUTPUT

    /*get the timeout parameter from the SystemConfiguration table. If it is not found we will default
      the timeout value to 60 seconds. The datatype is carried forward from the hardcoded value originally
      used is this procedure.*/
    IF @timeout IS NULL OR @timeout <= 0
    BEGIN
      DECLARE @timeout_sysconfig NVARCHAR(8)
      SET @timeout_sysconfig = NULL
      SELECT @timeout_sysconfig = propertyvalue
      FROM SystemConfiguration
      WHERE groupname = 'TEC_TASKS' AND propertyname = 'timeout'

      IF @timeout_sysconfig IS NOT NULL
        SET @timeout = @timeout_sysconfig
      ELSE
        SET @timeout = '60'
    END

    IF @hostname IS NOT NULL AND @hostname <> '' AND @hostname <> ' '
    BEGIN
      IF @tmeoid IS NOT NULL
        SELECT @cmd = 'tgmtask -t ' + @timeout + ' -h "' + @eehost + '" -o "' + @tmeoid  + '" -r "' + @hostname + '" '
      ELSE
      BEGIN
        IF @data4 IS NOT NULL
          SELECT @cmd = 'tgmtask -t ' + @timeout + ' -h "' + @eehost + '" -w "' + @data4  + '" -r "' + @hostname + '" '
        ELSE
          SELECT @cmd = 'tgmtask -t ' + @timeout + ' -h "' + @eehost + '" -r "' + @hostname + '" '
      END
      IF @userID IS NOT NULL SET @cmd = @cmd + ' -u "' + @userID + '" -k "********" '
      SET @cmd = @cmd + ' "' + @tskname + '|' +  @tsklib + '|' + @tskargs + '"'
    END
    ELSE
    BEGIN
      IF @tmeoid IS NOT NULL
        SELECT @cmd = 'tgmtask -t ' + @timeout + ' -h "' + @eehost + '" -o "' + @tmeoid + '" '
      ELSE
      BEGIN
        IF @data4 IS NOT NULL
          SELECT @cmd = 'tgmtask -t ' + @timeout + ' -h "' + @eehost + '" -w "' + @data4 + '" '
      END
      IF @userID IS NOT NULL SET @cmd = @cmd + ' -u "' + @userID + '" -k "********" '
      SET @cmd = @cmd + ' "' + @tskname + '|' +  @tsklib + '|' + @tskargs + '"'
    END
  END

  SET @print_string = 'TBSM4001I: Task command built IS: ' + @cmd
  EXEC gemsp_print @print_string

  -- commit transaction
  IF @tcount = 0
    COMMIT TRAN gemsp_invokeTask

GO
SET QUOTED_IDENTIFIER  OFF    SET ANSI_NULLS  ON
GO
