-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/gtm_setreportappurl
-- SID: 1.3
-- Date Modified: 04/11/15 06:38:16
--
-- Module Description
-- Defines the gtm_setreportappurl stored procedure
USE Object
GO

---------------------------------------------------------------------------------------------------
PRINT 'Creating procedure gtm_setreportappurl'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'gtm_setreportappurl' AND type = 'P')
  DROP PROCEDURE gtm_setreportappurl
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/gtm_setreportappurl
-- SID: 1.3
-- Date Modified: 04/11/15 06:38:16
--
-- Module Description
-- Defines the gtm_setreportappurl stored procedure
-- Called to set the launch URL for Reporting to point to either the IIS server or the 
-- Crystal Enterprise server.
-- exec gtm_setreportappurl 'bmeyer', 'iis',0
-- exec gtm_setreportappurl 'test', 'eportfolio',0
-- exec gtm_setreportappurl 'test', 'xxx',1
--
-- Input Parameters:
--   @report_server_name - The name or either the IIS server or the Crystal Enterprise Server.
--   @report_app_name    - A value to indicate if the URL should launch to the IIS based reporting application or
--                         Crystal ePorfolio application. Only valid value are 'IIS' or 'ePortfolio'. This value 
--                         is not case sensative.
--   @ssl_enabled        - A value of 1 to indicate that https should be used rather than http.
--   @verbose            - The flag to activate more detailed information concerning the processing within this proc
--
-- Output:
--   @err       - Zero or negative value to indicate success or failure
CREATE PROCEDURE gtm_setreportappurl
   @report_server_name  NVARCHAR(255),
   @report_app_name     NVARCHAR(255),
   @ssl_enabled         TINYINT = 0,
   @verbose             INT = 0
AS
  
  DECLARE @now NVARCHAR(40)
  DECLARE @url_lob    NVARCHAR(255)
  DECLARE @url_phys   NVARCHAR(255)
  DECLARE @url_prefix NVARCHAR(255)
  DECLARE @row_count  INTEGER
  DECLARE @err        INTEGER

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: gtm_setreportappurl: @report_server_name=%s, @report_app_name=%s, @ssl_enabled=%d, @verbose=%d', 0, 1, @now, @report_server_name, @report_app_name, @ssl_enabled, @verbose) WITH NOWAIT
  END

  IF @report_server_name is null or (@report_server_name = '')
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: gtm_setreportappurl: Missing input parameter @report_server_name.', 11, 1, @now) WITH NOWAIT
    RETURN -30000
  END

  IF @report_app_name is null or (UPPER(LTRIM(RTRIM(@report_app_name))) not in('IIS','EPORTFOLIO'))
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: gtm_setreportappurl: Missing or invalid input parameter @report_app_name. Only the values IIS and ePortfolio are allowed.', 11, 1, @now) WITH NOWAIT
    RETURN -30000
  END

  -- Set the url prefix based on ssl enablement.
  IF @ssl_enabled = 1 
    SET @url_prefix = 'https://'
  ELSE
    SET @url_prefix = 'http://'
         
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: gtm_setreportappurl: Using @url_prefix=%s.', 0, 1, @now,@url_prefix ) WITH NOWAIT
  END

  --  Standard transaction logic
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT

  IF @tcount = 0
    BEGIN TRANSACTION gtm_setreportappurl
  ELSE
    SAVE TRANSACTION gtm_setreportappurl

  IF EXISTS( SELECT 1 FROM MenuItem WHERE UPPER(name) = 'HISTORY' AND obj_cid in('LOB','MNO2') AND obj_id = 0
             AND ( UPPER(control_id) LIKE 'HTTP://%' or UPPER(control_id) LIKE 'HTTPS://%'))
  BEGIN
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: gtm_setreportappurl: MenuItem table contains valid rows to update.', 0, 1, @now) WITH NOWAIT
    END

    -- Set the 2 url values based on the @report_app_name.   
    IF UPPER(@report_app_name) = 'IIS' 
    BEGIN
       -- Sample urls to launch reports from the IIS WEB app.
       -- http://<servername>/asi/Reports/LOBEventsSel.asp?lobid=%id%&pagesource=javaconsole
       -- http://<servername>/asi/Reports/physEventsSel.asp?cid=%cid%&id=%id%&pagesource=javaconsole 
       SET @url_lob  = @url_prefix + LTRIM(RTRIM(@report_server_name)) + '/asi/Reports/LOBEventsSel.asp?lobid=%id%&pagesource=javaconsole'
       SET @url_phys = @url_prefix + LTRIM(RTRIM(@report_server_name)) + '/asi/Reports/physEventsSel.asp?cid=%cid%&id=%id%&pagesource=javaconsole'
    END
    ELSE
    BEGIN
      -- Sample urls to launch reports from the Crystal ePortfolio WEB app. Both the LOB and PHYS uls are the same
      -- since there is no mechanism to directly access a individual report at this time.
      -- http://<servername>/asi/Reports/physEventsSel.asp?cid=%cid%&id=%id%&pagesource=javaconsole 
      SET @url_lob  = @url_prefix + LTRIM(RTRIM(@report_server_name)) + '/crystal/enterprise9/eportfolio/default.htm?cid=%cid%&id=%id%&pagesource=javaconsole'
      SET @url_phys = @url_lob
    END

    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: gtm_setreportappurl: Using @url_lob=%s to launch LOB report for @report_app_name=%s on @report_server_name=%s.', 0, 1, @now,@url_lob,@report_app_name,@report_server_name) WITH NOWAIT
      RAISERROR('%s: gtm_setreportappurl: Using @url_phys=%s to launch PHYS report from the for @report_app_name=%s on @report_server_name=%s.', 0, 1, @now,@url_phys,@report_app_name,@report_server_name) WITH NOWAIT
      RAISERROR('%s: gtm_setreportappurl: Updating MenuItem table to launch LOB report from a @url_lob=%s, @report_server_name=%s, @report_app_name=%s.', 0, 1, @now,@url_lob,@report_server_name,@report_app_name) WITH NOWAIT
    END

    -- Update the LOB url first.
    UPDATE MenuItem SET control_id = @url_lob WHERE UPPER(name) = 'HISTORY' AND obj_cid = 'LOB' AND obj_id = 0 
    SELECT @err = @@ERROR, @row_count = @@ROWCOUNT

    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: gtm_setreportappurl: Update of MenuItem table failed when updating the LOB report launch @url_lob=%s, @report_server_name=%s, @report_app_name=%s with @@ERROR=%d.', 11, 1, @now,@url_lob,@report_server_name,@report_app_name,@err) WITH NOWAIT
      ROLLBACK TRANSACTION gtm_setreportappurl
      RETURN -30000
    END

    -- Should only be updating 1 row so if the number of rows affected is not 1 then there was problem.
    -- Roll back the transaction to undo the update.
    IF @row_count <> 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: gtm_setreportappurl: Update of MenuItem affected more rows than exspected when updating the LOB report launch @url_lob=%s, @report_server_name=%s, @report_app_name=%s.', 11, 1, @now,@url_lob,@report_server_name,@report_app_name) WITH NOWAIT
      ROLLBACK TRANSACTION gtm_setreportappurl
      RETURN -30000
    END

    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: gtm_setreportappurl: Update of the MenuItem table LOB report launch control_id value with @url_lob=%s, @report_server_name=%s, @report_app_name=%s completed without error.', 0, 1, @now,@url_lob,@report_server_name,@report_app_name) WITH NOWAIT
    END    

    -- Update the PHYS url next
    UPDATE MenuItem SET control_id = @url_phys WHERE UPPER(name) = 'HISTORY' AND obj_cid = 'MNO2' AND obj_id = 0
    SELECT @err = @@ERROR, @row_count = @@ROWCOUNT
    IF @err <> 0
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: gtm_setreportappurl: Update of MenuItem table failed when updating the PHYS report launch @url_phys=%s, @report_server_name=%s, @report_app_name=%s with @@ERROR=%d.', 11, 1, @now,@url_phys,@report_server_name,@report_app_name,@err) WITH NOWAIT
      ROLLBACK TRANSACTION gtm_setreportappurl
      RETURN -30000
    END

    -- Should only be updating 1 row so if the number of rows affected is not 1 then there was problem.
    -- Roll back the transaction to undo the update.
    IF @row_count <> 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: gtm_setreportappurl: Update of MenuItem affected more rows than exspected when updating the PHYS report launch @url_phys=%s, @report_server_name=%s, @report_app_name=%s.', 11, 1, @now,@url_phys,@report_server_name,@report_app_name) WITH NOWAIT
      ROLLBACK TRANSACTION gtm_setreportappurl
      RETURN -30000
    END

    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: gtm_setreportappurl: Update of the MenuItem table PHYS report launch control_id value with @url_phys=%s, @report_server_name=%s, @report_app_name=%s completed without error.', 0, 1, @now, @url_phys, @report_server_name,@report_app_name) WITH NOWAIT
    END    

  END
  ELSE
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: gtm_setreportappurl: The MenuItem table does not contain and valid rows to update.', 11, 1, @now) WITH NOWAIT
    ROLLBACK TRANSACTION gtm_setreportappurl
    RETURN -30000
  END

  -- Commit transaction.
  IF @tcount = 0
  BEGIN
    IF @verbose = 1
    BEGIN
       SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
       RAISERROR('%s: gtm_setreportappurl: Committing gtm_setreportappurl transaction.', 0, 1, @now) WITH NOWAIT
    END
    COMMIT TRANSACTION gtm_setreportappurl
  END

GO