-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tecstatusupdate.sql
-- SID: 1.5.4.8
-- Date Modified: 05/01/06 15:09:03
--
-- Module Description
--   Defines the TECStatusUpdate stored procedure
-- this proc is deprecated in TBSM 3.1, the code is now
-- in asisp_processStagedTECStatus

USE Object
GO

---------------------------------------------------------------------------------------------------
PRINT 'Creating procedure TECStatusUpdate'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'TECStatusUpdate' AND type = 'P')
  DROP PROCEDURE TECStatusUpdate
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/procs/tecstatusupdate.sql
-- SID: 1.5.4.8
-- Date Modified: 05/01/06 15:09:03
-- Called to send a status update to TEC for events that originated from TEC
-- this proc is deprecated in TBSM 3.1, the code is now
-- in asisp_processStagedTECStatus
CREATE PROCEDURE TECStatusUpdate
  @noteid           ObjID,
  @Status           NVARCHAR(16),
  @ContextUserID    NVARCHAR(40) = NULL,
  @verbose          TINYINT = 0
AS
  DECLARE @now NVARCHAR(40)
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: TECStatusUpdate @noteid=%d, @Status=%s, @ContextUserID=%s.', 0, 1, @now, @noteid, @Status, @ContextUserID) WITH NOWAIT
  END

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRANSACTION TECStatusUpdate
  ELSE
    SAVE TRANSACTION TECStatusUpdate

  --
  -- If the TEC notification enabled
  IF EXISTS ( SELECT *
              FROM SystemConfiguration WITH (NOLOCK)
              WHERE groupname='TEC_EVENT_OWNERSHIP' AND propertyname='NOTIFY_TEC_OF_TBSM_OWNERSHIP' AND UPPER(propertyvalue)='YES' )
  BEGIN
    DECLARE @eventhandle   NVARCHAR(16)
    DECLARE @serverhandle  NVARCHAR(16)
    DECLARE @datereception NVARCHAR(16)
    DECLARE @eehost        NVARCHAR(128)
    DECLARE @tecclass      NVARCHAR(255)
    DECLARE @cid           ClassID
    DECLARE @id            ObjID
    DECLARE @rc            INTEGER
    
    --
    -- Lets see if we need to notify TEC of this change
    IF NOT EXISTS ( SELECT * FROM SystemConfiguration WITH (NOLOCK)
                    WHERE groupname='TEC_EVENT_OWNERSHIP' AND propertyname='TEC_NOTE_RESOLUTION' AND propertyvalue=(SELECT CAST(_Resolution AS NCHAR) FROM Note_A where id=@noteid) )
    BEGIN
      -- DECLARE @postemsgconf NVARCHAR(255)
      -- SELECT  @postemsgconf=propertyvalue FROM SystemConfiguration WITH (NOLOCK) WHERE groupname='TEC_EVENT_OWNERSHIP' AND propertyname='TEC_CONFIG_DIRECTORY'

      DECLARE TECStatusUpdate_C INSENSITIVE CURSOR FOR SELECT _ExtEvSlot00, _ExtEvSlot01, _ExtEvSlot02, _ExtEvSlot04, _ExtEvSlot05
      FROM ExtendedEvent_A AS ex
      JOIN OwnedEvent AS oe ON
        ex.id = oe.eventid
        AND oe.noteid = @noteid
        AND ex.eventcid = oe.eventcid
      OPEN TECStatusUpdate_C
      FETCH NEXT FROM TECStatusUpdate_C INTO @eventhandle, @serverhandle, @datereception, @tecclass, @eehost
      WHILE @@FETCH_STATUS <> -1
      BEGIN

        IF @eventhandle IS NOT NULL AND @serverhandle IS NOT NULL AND @datereception IS NOT NULL AND @tecclass IS NOT NULL
        BEGIN
          --DECLARE @cmd NVARCHAR(500)
          --DECLARE @ReturnCode INT
          DECLARE @administrator NVARCHAR(260)

          -- Form command string
          --SELECT @cmd = 'postemsg -f ' + @postemsgconf + @eehost + '.conf '
          --SET @cmd = @cmd + ' -m "TBSM ownership" '
          --SET @cmd = @cmd + ' o_server_handle=' + @serverhandle
          --SET @cmd = @cmd + ' o_event_handle=' + @eventhandle
          --SET @cmd = @cmd + ' o_date_reception=' + @datereception
          --SET @cmd = @cmd + ' o_class=' + @tecclass

          SET @administrator = @ContextUserID
          IF @administrator IS NULL
            EXEC asisp_getContextUserID @administrator OUTPUT

          --IF @administrator IS NOT NULL AND @administrator <> '' SET @cmd = @cmd + ' new_admin=TBSM:' + @administrator
          --ELSE SET @cmd = @cmd + ' new_admin=TBSM'

          IF @administrator IS NOT NULL AND @administrator <> '' 
          BEGIN
            IF EXISTS (SELECT * FROM SystemConfiguration WITH (NOLOCK) WHERE groupname='TEC_EVENT_OWNERSHIP' AND propertyname = 'DISPLAY_NAME' AND propertyvalue = 'YES')
            BEGIN
              DECLARE @admin NVARCHAR(255)
              SELECT @admin=display_name FROM UserProfiles WITH (NOLOCK) WHERE LOWER(userid)=LOWER(@administrator)
              IF @admin IS NOT NULL AND @admin <> '' 
              BEGIN
                SET @administrator = 'TBSM:' + @admin
              END
              ELSE
              BEGIN
                SET @administrator = 'TBSM:' + @administrator
              END  
            END
            ELSE
            BEGIN
              SET @administrator = 'TBSM:' + @administrator
            END  
          END
          ELSE
          BEGIN
            SET @administrator = 'TBSM'
          END  

          --SET @cmd = @cmd + ' new_status=' + @Status
          --SET @cmd = @cmd + ' TBSM_setemsg_status_event TBSM'

          --RAISERROR('CMD: %s', 0, 1, @cmd)

          -- Get the _CreatorCID and _CreatorObjID from the @noteid passed in to pass to asisp_sendStatusEvent.
          SELECT @cid = _CreatorCID, @id = _CreatorObjID FROM Note_V WHERE id = @noteid
          IF @cid = '' or @cid IS NULL 
            SET @cid = 'ROOT'

          IF @id IS NULL 
            SET @id = 0
    
          -- Call asisp_sendStatusEvent
          EXEC @rc = Object..asisp_sendStatusEvent @cid, @id, @eehost, 'TBSM ownership',
             @serverhandle, @eventhandle, @datereception, @tecclass, @Status, @administrator 

          --EXEC @ReturnCode = master..xp_cmdshell @cmd, NO_OUTPUT

        END
        ELSE
        BEGIN
          RAISERROR('TECStatusUpdate invalid attributes: eventhandle=%s, serverhandle=%s, date=%s, class=%s', 0, 1, @eventhandle, @serverhandle, @datereception, @tecclass)
        END

        FETCH NEXT FROM TECStatusUpdate_C INTO @eventhandle, @serverhandle, @datereception, @tecclass, @eehost
      END
      DEALLOCATE TECStatusUpdate_C
    END
  END
  -- End of TEC notification

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: TECStatusUpdate: completed.', 0, 1, @now) WITH NOWAIT
  END

  -- Finish up transaction
  IF @tcount = 0
    COMMIT TRANSACTION TECStatusUpdate
GO