-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\procs\twh_getlobstates.sql
-- SID: 1.1.1.11
-- Date Modified: 05/01/17 15:40:01
--
-- Module Description
--   Defines the twh_getgmt_time stored procedure

USE Object
GO

---------------------------------------------------------------------------------------------------
-- twh_getlobstates
PRINT 'Creating procedure twh_getlobstates'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'twh_getlobstates' AND type = 'P')
  DROP PROCEDURE twh_getlobstates
GO
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}

-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-- File: archive\db\sql\procs\twh_getlobstates.sql
-- SID: 1.1.1.11
-- Date Modified: 05/01/17 15:40:01

CREATE PROCEDURE twh_getlobstates
@update_ctrl INT = 0, --update extraction control only
@verbose INT = 0
AS
  SET NOCOUNT ON
  DECLARE @lastenddate DATETIME, @thisenddate DATETIME, @msmt_ctrl INT
  DECLARE @tempdate DATETIME, @cnt INT, @daysold INT, @now NVARCHAR(40)

  IF EXISTS(SELECT 1 FROM twh_date WHERE status&1=1 AND loadtype='lobstate')
  BEGIN 
    --Reverse tracking tables for measurements that didn't get all the way to the CDW
    UPDATE twh_date SET status=status-1, lastenddate=prevlastenddate, prevlastenddate=NULL
      WHERE status&1=1 AND loadtype='lobstate'

    -- Clean up unsucessful load
    UPDATE TWH_MSMT_CTRL SET Loaded_DT = Last_Loaded_DT, Obj_State = Last_Obj_State
      WHERE Last_Loaded_DT IS NOT NULL
    UPDATE twh_gmt_time SET this_gmt = last_gmt, status = status -1 WHERE status = 1
  END

  --Get GM time offset
  DECLARE @gmoffsetmin INT, @last_gmoffsetmin INT, @GMT_CHANG_TIME DATETIME, @process_gmtc TINYINT

  EXEC master..xp_regread 
    'HKEY_LOCAL_MACHINE',
    'SYSTEM\CurrentControlSet\Control\TimeZoneInformation',
    'ActiveTimeBias', 
     @gmoffsetmin OUTPUT
  SELECT @last_gmoffsetmin = last_gmt FROM twh_gmt_time

  -- get @thisenddate
  SET @thisenddate = GETDATE()

  --Get last cutoff (ending) date if it exists
  SELECT @lastenddate=lastenddate FROM twh_date WHERE status&1=0 AND loadtype='lobstate'

  --Summarize lobs to be processed along with their date ranges
  CREATE TABLE #templob ( id INT IDENTITY(1,1) PRIMARY KEY, 
	comp_id         INT NULL,
        tab_id          INT NULL,
	lobid 		INT, 
	tbsm_st 	NVARCHAR(40) NULL,
	ctime		DATETIME,
	duration 	INT NULL, --duration in the time frame in minute 
	priority	TINYINT NULL,
	not_count	TINYINT NULL,
        ash_id          INT NULL,
        gmt_time	DATETIME NULL
  )
  CREATE INDEX i_#templob_1 ON #templob(ash_id)  
  CREATE INDEX i_#templob ON #templob(lobid, ctime, ash_id)
  CREATE INDEX i_#templob_2 ON #templob(ctime, priority)
  CREATE INDEX i_#templob_3 ON #templob(tbsm_st)

  --If no last cutoff date exists (ie the first load) check for days to keep sent from twh
  --and calculate cutoff date going back that many days and rounding.
  IF @lastenddate IS NULL
  BEGIN
    SELECT @daysold=pmsmtc_age_in_days FROM twh_prune_msmt_control WHERE tmsum_cd='P'
    IF NOT @daysold IS NULL
      SET @lastenddate=DATEADD(d, -@daysold, getdate())
    SET @msmt_ctrl = 0
  END
  ELSE IF @update_ctrl = 1
    SET @msmt_ctrl = 0

  IF @msmt_ctrl = 0
  BEGIN
    IF @verbose = 1
    BEGIN
      RAISERROR('Process the initial load with @increament = 0', 0, 1)
    END
    -- insert the last state of LOB objects from ObjectStateHist
    INSERT INTO #templob(lobid, ctime)
      SELECT Obj_id,
      MAX(CTIME)
      FROM ObjectStateHist 
      WHERE Obj_cid = 'LOB'
      AND CTIME <= @lastenddate
      AND Obj_id IN (SELECT Obj_id FROM twh_objs WHERE Deleted_DT IS NULL)
      GROUP BY Obj_id

    UPDATE t SET t.tbsm_st = obj.State_Nm, 
        t.tab_id = obj.ID,
	t.not_count = CASE obj.State_flag WHEN 0 THEN 1 ELSE 0 END
      FROM #templob AS t JOIN ObjectStateHist AS obj ON obj.Obj_id = t.lobid 
      AND obj.Obj_cid = 'LOB' 
      AND t.ctime = obj.CTIME

    -- insert the last state of LOB objects from AlertStateHistory
    INSERT INTO #templob(lobid, ctime)
      SELECT id,
      MAX(ctime)
      FROM AlertStateHistory WITH (NOLOCK)
      WHERE cid = 'LOB'
      AND ctime <= @lastenddate
      AND id IN (SELECT DISTINCT Obj_id FROM twh_objs WHERE Obj_cid = 'LOB' AND Deleted_DT IS NULL)
      GROUP BY id

    UPDATE t SET t.tbsm_st = a.name,  
      t.not_count = CASE obj.AlertStateID WHEN 1 THEN 1 ELSE 0 END
      FROM #templob AS t JOIN AlertStateHistory AS obj WITH (NOLOCK) ON t.lobid = obj.id 
      AND obj.cid = 'LOB' 
      AND t.ctime = obj.ctime 
      AND obj.AlertStateID > 0
      JOIN AlertState_V AS a ON a.id = obj.AlertStateID

    -- Update priority based on twh_tbsm_map
    UPDATE t SET t.priority = m.priority FROM #templob AS t
      JOIN twh_tbsm_map AS m ON t.tbsm_st = m.tbsm
      AND m.map_type = 'MSMTType'

    -- Delete the state that don't need to count
    DELETE #templob 
      WHERE not_count = 1

    -- use the state for the @lastenddate
    UPDATE #templob SET ctime = @lastenddate
    -- assume any object does not have state recorded in neither AlertStateHistory nor AlertStateHistory
    -- as availible 
   
    -- insert TWH_MSMT_CTRL table here if @update_ctrl = 1
    IF @update_ctrl = 1
    BEGIN
      INSERT TWH_MSMT_CTRL(cid, id, Source_name, Obj_State, Priority, Loaded_DT)
        SELECT 'LOB', 
        lobid,
        CASE tbsm_st WHEN 'Owned' THEN 'ObjectStateHist' 
                     WHEN 'Maint' THEN 'ObjectStateHist' 
                     ELSE 'AlertStateHistory' END, 
        tbsm_st,
        priority,
        @lastenddate
        FROM #templob WHERE not_count = 0  
      RETURN 
    END        
  END
  ELSE
    --get object starting states from TWH_MSMT_CTRL 
    INSERT INTO #templob(lobid, tbsm_st, ctime, not_count)
      SELECT id,
      Obj_State,
      @lastenddate,
      0     
      FROM TWH_MSMT_CTRL 
      WHERE Loaded_DT = @lastenddate
      AND id IN (SELECT DISTINCT Obj_id FROM twh_objs WHERE Obj_cid = 'LOB' AND Deleted_DT IS NULL)

  IF @verbose > 0
  BEGIN
    SELECT @now = GETDATE()
    RAISERROR('%s :Collecting data between:', 0 , 1, @now)
    SELECT CONVERT(NVARCHAR(50), @lastenddate) + ' and ' + CONVERT(NVARCHAR(50), @thisenddate)
  END

  --get object alert state from AlertStateHistory between @thisenddate and @lastenddate 
  INSERT INTO #templob(lobid, tbsm_st, ctime, priority, not_count, ash_id)
    SELECT his.id,
    a.name,
    his.ctime,
    0,
    CASE his.AlertStateID WHEN 1 THEN 1 ELSE 0 END,
    his.ash_id    
    FROM AlertStateHistory AS his WITH (NOLOCK)
    JOIN AlertState_V AS a ON a.id = his.AlertStateID
    WHERE his.cid = 'LOB'
    AND his.AlertStateID IS NOT NULL
    AND his.id IN (SELECT DISTINCT Obj_id FROM twh_objs WHERE Obj_cid = 'LOB' AND Deleted_DT IS NULL) 
    AND his.ctime >= @lastenddate AND his.ctime < @thisenddate

  -- remove the multible rows with the same datetime. The last alert state is taken.
  DELETE t FROM #templob AS t (INDEX=i_#templob)
    JOIN #templob AS t1 (INDEX=i_#templob) ON t.lobid = t1.lobid
    AND t.ctime = t1.ctime 
    AND t.ash_id < t1.ash_id
 
  --get object states from ObjectStateHist between @thisenddate and @lastenddate 
  INSERT INTO #templob(lobid, tbsm_st, ctime, priority, not_count)
    SELECT Obj_id,
      State_Nm,
      CTIME,
      0,
      CASE State_flag WHEN 0 THEN 1 ELSE 0 END
      FROM ObjectStateHist AS his 
      WHERE CTIME >= @lastenddate AND CTIME < @thisenddate
      AND Obj_cid = 'LOB'

  -- Update priority based on twh_tbsm_map
  UPDATE t SET t.priority = m.priority FROM #templob AS t (INDEX=i_#templob_3)
    JOIN twh_tbsm_map AS m ON t.tbsm_st = m.tbsm
    AND m.map_type = 'MSMTType'

  IF @verbose >0
  BEGIN
    SELECT @now = GETDATE()
    RAISERROR('%s :Processing data for: ', 0 , 1, @now)
    SELECT CONVERT(NVARCHAR(40), COUNT(*)) + ' rows' FROM #templob 
    IF @verbose > 1
      SELECT * FROM #templob
  END

  -- working tables to calculate duration
  --calculate the dur 
  CREATE TABLE #tempstate ( ident INT IDENTITY(1,1) PRIMARY KEY, 
        lobid 		INT, 
	ctime		DATETIME,
        tbsm_st		NVARCHAR(40),
	dur 	INT NULL, --duration in the time frame in minute 
        gmt_time DATETIME NULL
     )
  CREATE INDEX i_tempstate_1 ON #tempstate (lobid, ident, tbsm_st) 
  CREATE INDEX i_tempstate_2 ON #tempstate (ctime, lobid) 
  --temp table to remove multiple records
  CREATE TABLE #tempstatep ( ident INT IDENTITY(1,1) PRIMARY KEY, 
        lobid 		INT, 
	ctime		DATETIME,
        tbsm_st		NVARCHAR(40)
     )
  CREATE INDEX i_tempstatep_1 ON #tempstatep (lobid, ident, tbsm_st) 
  -- hold last state
  CREATE TABLE #laststate ( lobid INT, dt DATETIME, priority SMALLINT) 
  -- used for gmt changes
  CREATE TABLE #tempstate1 ( ident INT PRIMARY KEY, 
        lobid 		INT, 
	ctime		DATETIME,
        tbsm_st		NVARCHAR(40),
	dur 	INT NULL, --duration in the time frame in minute 
     	)
  -- used for gmt changes
  CREATE TABLE #tempstate2 ( ident INT PRIMARY KEY, 
        lobid 		INT, 
	ctime		DATETIME,
        tbsm_st		NVARCHAR(40),
	dur 	INT NULL, --duration in the time frame in minute 
     	)

  DECLARE @pri TINYINT
  SElECT @pri = MAX (priority) FROM #templob
  WHILE @pri >0
  BEGIN
    TRUNCATE TABLE #tempstatep
    TRUNCATE TABLE #tempstate

    -- remove the multiple records with the same states
    INSERT #tempstatep(lobid, ctime, tbsm_st) 
      SELECT lobid, ctime, tbsm_st 
      FROM #templob WHERE priority = @pri
      ORDER BY lobid, ctime, tbsm_st ASC
    DELETE t1 FROM #tempstatep AS t1 (INDEX = i_tempstatep_1) 
      JOIN #tempstatep AS t2 (INDEX = i_tempstatep_1) ON t1.lobid = t2.lobid 
      AND t1.ident = t2.ident + 1
      AND t1.tbsm_st = t2.tbsm_st

    -- insert data into working table
    INSERT #tempstate(lobid, ctime, tbsm_st) 
      SELECT lobid, ctime, tbsm_st 
      FROM #tempstatep
      ORDER BY ident

    IF @gmoffsetmin <> @last_gmoffsetmin
    BEGIN
      SELECT @GMT_CHANG_TIME = propertyvalue 
        FROM SystemConfiguration 
        WHERE propertyname = 'GMT_CHANGE_TIME'
        AND groupname = 'TWH'
      IF (DATEDIFF(mi, @lastenddate, @GMT_CHANG_TIME) > 0) AND (DATEDIFF(mi, @GMT_CHANG_TIME, @thisenddate)>0) 
          SELECT @process_gmtc = 1 
    END
--select @gmoffsetmin, @last_gmoffsetmin, @GMT_CHANG_TIME, @lastenddate, @thisenddate, @process_gmtc
    IF @process_gmtc = 1  
    BEGIN
      TRUNCATE TABLE #tempstate1
      TRUNCATE TABLE #tempstate2
      -- process the data with the last GMT time
      INSERT #tempstate1(ident, lobid, ctime, tbsm_st) 
        SELECT ident, lobid, ctime, tbsm_st 
        FROM #tempstate WHERE ctime < @GMT_CHANG_TIME
        ORDER BY ident ASC
      -- update duration for the objects in both #tempstate1 and #tempstate2
      UPDATE t1 SET t1.dur = DATEDIFF(mi, t1.ctime, t2.ctime) 
        FROM #tempstate1 AS t1
        JOIN #tempstate1 AS t2 ON t1.lobid = t2.lobid
        AND t1.ident+1 = t2.ident
      -- insert the last state for twh_msmt_ctrl 
      INSERT #laststate SELECT lobid, ctime, @pri 
        FROM #tempstate1 WHERE dur IS NULL
        AND lobid NOT IN (SELECT lobid FROM #tempstate WHERE ctime >= @GMT_CHANG_TIME) 
      -- populate for the rest of the time cal 
      INSERT #tempstate2(ident, lobid, ctime, tbsm_st) 
        SELECT ident, lobid, ctime, tbsm_st 
        FROM #tempstate1 WHERE dur IS NULL
        AND lobid NOT IN (SELECT lobid FROM #tempstate WHERE ctime >= @GMT_CHANG_TIME) 
      -- update duration for the objects in #tempstate1 and not in #tempstate2
      UPDATE t1 SET t1.dur = DATEDIFF(mi, t1.ctime, @GMT_CHANG_TIME) 
        FROM #tempstate1 AS t1 WHERE dur IS NULL
      -- update dur and gmt_time in #tempstate
      UPDATE t SET t.dur = t1.dur
        FROM #tempstate AS t
        JOIN #tempstate1 AS t1 ON t.ident = t1.ident
      UPDATE t SET t.gmt_time = DATEADD(mi, @last_gmoffsetmin, t.ctime)  
        FROM #tempstate AS t
        JOIN #tempstate2 AS t1 ON t.ident = t1.ident
        AND t.gmt_time IS NULL

      -- process data with new GMT time
      INSERT #tempstate2(ident, lobid, ctime, tbsm_st) 
        SELECT ident, lobid, ctime, tbsm_st 
        FROM #tempstate WHERE ctime >= @GMT_CHANG_TIME
        ORDER BY ident ASC

      -- update duration for the objects in both #tempstate1 and #tempstate2
      UPDATE t1 SET t1.dur = DATEDIFF(mi, t1.ctime, t2.ctime) 
        FROM #tempstate2 AS t1
        JOIN #tempstate2 AS t2 ON t1.lobid = t2.lobid
        AND t1.ident+1 = t2.ident
      -- insert the last state for twh_msmt_ctrl 
      INSERT #laststate SELECT lobid, ctime, @pri 
        FROM #tempstate2 WHERE dur IS NULL
      -- update duration for the objects in #tempstate1 and not in #tempstate2
      UPDATE t1 SET t1.dur = DATEDIFF(mi, t1.ctime, @thisenddate) 
        FROM #tempstate2 AS t1 WHERE dur IS NULL
      -- update dur and gmt_time in #tempstate
      UPDATE t SET t.dur = t.dur + t1.dur
        FROM #tempstate AS t
        JOIN #tempstate2 AS t1 ON t.ident = t1.ident
      UPDATE t SET t.gmt_time = DATEADD(mi, @gmoffsetmin, t.ctime)  
        FROM #tempstate AS t
        JOIN #tempstate2 AS t1 ON t.ident = t1.ident
        AND t.gmt_time IS NULL
      -- update #templob table
      UPDATE org SET org.duration = t1.dur,
        org.gmt_time = t1.gmt_time 
        FROM #templob AS org 
        JOIN #tempstate AS t1 ON org.ctime = t1.ctime
        AND org.lobid = t1.lobid
        AND org.priority = @pri
    END
    ELSE
    BEGIN
      -- update duration for the objects in both #tempstate1 and #tempstate2
      UPDATE t1 SET t1.dur = DATEDIFF(mi, t1.ctime, t2.ctime) 
        FROM #tempstate AS t1
        JOIN #tempstate AS t2 ON t1.lobid = t2.lobid
        AND t1.ident+1 = t2.ident

      -- insert the last state for twh_msmt_ctrl 
      INSERT #laststate SELECT lobid, ctime, @pri 
        FROM #tempstate WHERE dur IS NULL
    
      -- update duration for the objects in #tempstate1 and not in #tempstate2
      UPDATE t1 SET t1.dur = DATEDIFF(mi, t1.ctime, @thisenddate) 
        FROM #tempstate AS t1 WHERE dur IS NULL

      -- update #templob table
      UPDATE org SET org.duration = t1.dur 
        FROM #templob AS org (INDEX=i_#templob_2)
        JOIN #tempstate AS t1 (INDEX=i_tempstate_2) ON org.ctime = t1.ctime
        AND org.lobid = t1.lobid
        AND org.priority = @pri

    END   

    IF @verbose >0
    BEGIN
      SELECT @now = GETDATE()
      RAISERROR('%s :Processing data for:', 0 , 1, @now)
      SELECT CONVERT(NVARCHAR(40), COUNT(*)) + ' rows in #templob table, with priority = ', @pri FROM #templob 
      IF @verbose > 1
      BEGIN
        SELECT @now = GETDATE()
        RAISERROR('%s :Processing data for priority = %d', 0 , 1, @now, @pri)
        SELECT * FROM #tempstate
      END
    END
    SET @pri = @pri -1
  END

  --Update tracking tables.  twh_date records this end date as the lastenddate in an in-process status.
  --This also sets the last alertstate on twh_objs to the most recent reported for each lob.
  DECLARE @tcount TINYINT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN twh_getlobstates
  ELSE 
    SAVE TRAN twh_getlobstates

  -- update twh_date table
  IF EXISTS(SELECT * FROM twh_date)
    UPDATE twh_date SET status=status+1, prevlastenddate=lastenddate, lastenddate=@thisenddate
    WHERE loadtype='lobstate' AND status&1=0
  ELSE
    INSERT twh_date(loadtype, lastenddate, status) SELECT 'lobstate', @thisenddate, 1

  --update TWH_MSMT_CTRL table
  DECLARE last_state CURSOR FOR SELECT DISTINCT priority FROM #laststate 
  OPEN last_state
  FETCH NEXT FROM last_state INTO @pri
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    -- update or insert the active states into TWH_MSMT_CTRL table
    UPDATE c SET c.Last_Obj_State = c.Obj_State, 
      c.Obj_State = t.tbsm_st,
      c.Source_name = CASE @pri WHEN 1 THEN 'AlertStateHistory' ELSE 'ObjectStateHist' END,
      Last_Loaded_DT = Loaded_DT,
      Loaded_DT = @thisenddate
      FROM TWH_MSMT_CTRL AS c JOIN #laststate AS las ON c.id = las.lobid AND c.cid = 'LOB'
      AND c.Priority = las.priority
      JOIN #templob AS t on las.lobid = t.lobid 
      AND las.dt = t.ctime
      AND las.priority = t.priority 
      AND t.priority = @pri
      AND t.not_count = 0

    INSERT TWH_MSMT_CTRL(cid, id, Source_name, Obj_State, Loaded_DT, Priority) SELECT 'LOB', 
      a.lobid, 
      CASE @pri WHEN 1 THEN 'AlertStateHistory' ELSE 'ObjectStateHist' END,
      tbsm_st,
      @thisenddate,
      @pri
      FROM #laststate AS a JOIN #templob AS t ON a.lobid = t.lobid 
      AND a.dt = t.ctime
      AND t.not_count = 0
      AND a.priority = t.priority  
      WHERE a.priority = @pri
      AND a.lobid NOT IN (SELECT id FROM TWH_MSMT_CTRL WHERE Priority = @pri)
    FETCH NEXT FROM last_state INTO @pri
  END
  DEALLOCATE last_state

  IF @gmoffsetmin <> @last_gmoffsetmin
    UPDATE twh_gmt_time SET this_gmt = @gmoffsetmin, 
    last_gmt = @last_gmoffsetmin,
    mtime = GETDATE(),
    status = 1 

  -- commit tran
  IF @tcount = 0
    COMMIT TRAN twh_getlobstates

  -- update comp_id
  UPDATE t SET t.comp_id = m.comp_id FROM #templob AS t 
    JOIN twh_comp_map AS m ON t.lobid = m.id 

  --This returns the percentage data to the warehouse with identity column.
  CREATE TABLE #result(
    ident                INT IDENTITY (1,1),
    cid                  ClassID,
    id                   ObjID,
    msmt_strt_dt         DATETIME,
    msmt_dur_min         INT,
    msmt_priority        SMALLINT,
    etl_end_dttm         DATETIME,
    msmt_type            NVARCHAR(40),
    comp_id              INT NULL,
    loading_flag         SMALLINT
    )

  IF @process_gmtc IS NOT NULL
    INSERT INTO #result 
      SELECT
      'LOB' AS cid,
      t1.lobid AS id,
      t1.gmt_time AS msmt_strt_dt,
      t1.duration AS msmt_dur_min,
      t1.priority AS msmt_priority,
      DATEADD(mi, @gmoffsetmin, @thisenddate) AS etl_end_dttm,
      a.twh AS msmt_type,
      t1.comp_id AS comp_id,
      0 AS loading_flag
      FROM #templob AS t1 JOIN twh_tbsm_map AS a ON t1.tbsm_st = a.tbsm 
      AND a.map_type = 'MSMTType' 
      AND a.enabled = 1
      AND t1.duration IS NOT NULL
      WHERE t1.not_count = 0
      ORDER BY t1.priority
  ELSE
    INSERT INTO #result 
      SELECT
      'LOB' AS cid,
      t1.lobid AS id,
      DATEADD(mi, @gmoffsetmin, t1.ctime) AS msmt_strt_dt,
      t1.duration AS msmt_dur_min,
      t1.priority AS msmt_priority,
      DATEADD(mi, @gmoffsetmin, @thisenddate) AS etl_end_dttm,
      a.twh AS msmt_type,
      t1.comp_id AS comp_id,
      0 AS loading_flag
      FROM #templob AS t1 JOIN twh_tbsm_map AS a ON t1.tbsm_st = a.tbsm 
      AND a.map_type = 'MSMTType' 
      AND a.enabled = 1
      AND t1.duration IS NOT NULL
      WHERE t1.not_count = 0
      ORDER BY t1.priority
  -- add the data for the end of the measurement period if the state is availible
  DECLARE @strt_dttm DATETIME
  SELECT @strt_dttm = DATEADD(mi, -1, @thisenddate)
  INSERT INTO #result
    SELECT 'LOB' AS cid,
      t1.id AS id,
      DATEADD(mi, @gmoffsetmin, @strt_dttm) AS msmt_strt_dt,
      1 AS msmt_dur_min,
      1 AS msmt_priority,
      DATEADD(mi, @gmoffsetmin, @thisenddate) AS etl_end_dttm,
      'Available' AS msmt_type,
      i.comp_id AS comp_id,
      0 AS loading_flag
      FROM LineOfBusiness_V AS t1 WITH (NOLOCK) JOIN twh_Inventory AS i ON i.cid = 'LOB' 
      AND i.id = t1.id 
      WHERE t1.deleted = 0
      AND t1.id NOT IN (SELECT id FROM TWH_MSMT_CTRL WHERE cid = 'LOB' AND 
        Loaded_DT = @thisenddate)

  SELECT * FROM #result 
GO

