-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\schedule_task_bcp_live.sql
--
-- SID: 1.9.4.1
-- 
-- Date Modified: 04/10/19 08:14:31
--
-- APAR: OW53355
-- Module Description:
--  This is part of History Server Set up using BCP Approach. It addresses APAR OW5335
--  This setup program should be run on the live server.
--  The "Copy Backups to Hist Server" job should not be on the live server. Check to insure
--  that it's not defined here.
--  This file will create one scheduled task:
--    - "Move events to History Database" Move the entries in event table every 5 min.
--  and will delete any previously scheduled "Copy Backups to Hist Server" which is now run
--  on the History server instead of the Live server.
--
-- REPLACE "XXXX" with live server name such as "server5"; 
-- REPLACE "HHHH" with hist server name such as "server4"; 
-- REPLACE "FFFF" with live server network drive backup path such as "e$\mssql7"
--

-- ----------------------------------------------------------------------------------------------
-- ---- 1. ------ Check to see if the 'Copy Backups to Hist Server' job is defined. -------------
-- ----------------------------------------------------------------------------------------------


BEGIN TRANSACTION

IF (SELECT COUNT(*) FROM msdb.dbo.syscategories WHERE name = N'History Server Maintenance') < 1
   EXECUTE msdb.dbo.sp_add_category @name = N'History Server Maintenance'

IF (SELECT COUNT(*) FROM msdb.dbo.sysjobs WHERE name = N'Copy Backups to Hist Server') > 0
BEGIN
   PRINT N'The job "Copy Backups to Hist Server" is defined to this server.'
   PRINT N'It must be manually deleted using Enterprise Manager.'
END
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
   IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:
GO

-- --------------------------------------------------------------------------
-- ---- 2. ------ Add the 'Move events to History Database' job -------------
-- --------------------------------------------------------------------------

BEGIN TRANSACTION
   DECLARE @JobID BINARY(16)
   DECLARE @ReturnCode INT
   SELECT @ReturnCode = 0

IF (SELECT COUNT(*) FROM msdb.dbo.syscategories WHERE name = N'History Server Maintenance') < 1
   EXECUTE msdb.dbo.sp_add_category @name = N'History Server Maintenance'

IF (SELECT COUNT(*) FROM msdb.dbo.sysjobs WHERE name = N'Move events to History Database') > 0
BEGIN
   PRINT 'The job "Move events to History Database" is already defined.'
   PRINT 'It must be manually deleted using Enterprise Manager in order for updates to be applied.'
END


ELSE

BEGIN
PRINT N'Creating job "Move events to History Database"'

   -- Add the job
   EXECUTE @ReturnCode = msdb.dbo.sp_add_job @job_id = @JobID OUTPUT,
	@job_name = N'Move events to History Database',
	@owner_login_name = N'sa',
        @description = N'No description available',
        @category_name = N'History Server Maintenance',
        @enabled = 1,
	@notify_level_email = 0,
	@notify_level_page = 0,
	@notify_level_netsend = 0,
	@notify_level_eventlog = 2,
	@delete_level = 0
   IF (@@ERROR <> 0 OR @ReturnCode <>0) GOTO QuitWithRollback

   -- Add the job step
   EXECUTE @ReturnCode = msdb.dbo.sp_add_jobstep @job_id = @JobID,
	@step_id = 1,
	@step_name = N'Move events to History Database',
	@command = N'asisp_moveevent ''HHHH'', ''\\XXXX\FFFF\backup'', ''History'', ''eventhist''',
	@database_name = N'Object',
	@server = N'',
	@database_user_name = N'',
	@subsystem = N'TSQL',
	@cmdexec_success_code = 0,
	@flags = 0,
	@retry_attempts = 0,
	@retry_interval = 1,
	@output_file_name = N'',
	@on_success_step_id = 0,
	@on_success_action = 1,
	@on_fail_step_id = 0,
	@on_fail_action = 2
   IF (@@ERROR <> 0 OR @ReturnCode <>0) GOTO QuitWithRollback

   -- Identify which step to start at 
   EXECUTE @ReturnCode = msdb.dbo.sp_update_job @job_id = @JobID, @start_step_id = 1
   IF (@@ERROR <> 0 OR @ReturnCode <>0) GOTO QuitWithRollback

   -- Add the job schedule
   EXECUTE @ReturnCode = msdb.dbo.sp_add_jobschedule @job_id = @JobID,
	@name = N'Copy Interval',
	@enabled = 1,
	@freq_type = 4,                          -- When job is to be run: 4=Daily
	@freq_interval = 1,                      -- Days the job is to be run: 1=Daily
	@freq_subday_type = 4,                   -- 4=freq_subday_interval is in Minutes
	@freq_subday_interval = 5,               -- 5=Every five minutes 
	@freq_relative_interval = 0,
	@freq_recurrence_factor = 0,
 	@active_start_date = 20000410,
	@active_start_time = 020000,             -- Run at Two am
	@active_end_date = 99991231,
	@active_end_time = 235959
   IF (@@ERROR <> 0 OR @ReturnCode <>0) GOTO QuitWithRollback

   -- Add the Target Servers	
   EXECUTE @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @JobID, @server_name = N'(local)'
   IF (@@ERROR <> 0 OR @ReturnCode <>0) GOTO QuitWithRollback

END
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
   IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:
GO

-- Note: the "Delete old entries in event table" job (which calls
-- asisp_deleteoldevent is now created by tracks\schedule_deleteoldevent.sql 
-- regardless of which reporting system is in use (TDW or traditional TBSM) 
-- or even if the customer is using any reporting system.  The job always 
-- runs once a day.