-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\tables\childevent_c.sql
-- SID: 1.21
-- Date Modified: 04/11/30 17:08:41
--
-- Module Description
--   Defines the ChildEvent_C table
USE Object
GO

-- Migrate data from old ChildEvent table
IF EXISTS (SELECT * FROM syscolumns WHERE id = OBJECT_ID('ChildEvent_C') AND name = 'name')
BEGIN
  RAISERROR('Old ChildEvent_C table exists, renaming', 0, 1) WITH NOWAIT

  EXEC sp_rename 'ChildEvent_C', 'ChildEvent_C_OLD'
  EXEC sp_rename 'CHEV_PK',      'CHEV_PK_OLD'
END

---------------------------------------------------------------------------------------------------
-- ChildEvent_C

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'ChildEvent_C' AND type = 'U')
BEGIN
  PRINT 'Creating table ChildEvent_C'

  CREATE TABLE ChildEvent_C
  (
    id              ObjID         NOT NULL  IDENTITY(1,1),
    ctime           DATETIME      NOT NULL  CONSTRAINT CHEV_ctime_DF        DEFAULT (CURRENT_TIMESTAMP),
    dtime           DATETIME      NULL      CONSTRAINT CHEV_dtime_DF        DEFAULT (NULL),
    deleted         TINYINT       NOT NULL  CONSTRAINT CHEV_deleted_DF      DEFAULT (0),
    _ParentCID      ClassID       NOT NULL,
    _ParentID       ObjID         NOT NULL,
    _ChildCID       ClassID       NOT NULL,
    _ChildID        ObjID         NOT NULL,
    _EventCID       ClassID       NOT NULL,
    _AlertState     INT           NOT NULL,
    _EventPriority  INT           NOT NULL  CONSTRAINT CHEV_EventPriority_DF   DEFAULT '',
    _Priority       INT           NOT NULL,
    _EventCount     INT           NOT NULL  CONSTRAINT CHEV_EventCount_DF   DEFAULT (1),
    _EventDetail    NVARCHAR(255) NOT NULL,
    _DeleteDetail   NVARCHAR(255) NULL      CONSTRAINT CHEV_DeleteDetail_DF DEFAULT (NULL)
  ) ON [PRIMARY];
  ALTER TABLE ChildEvent_C ADD CONSTRAINT CHEV_PK PRIMARY KEY NONCLUSTERED (id) ON [PRIMARY];

  CREATE UNIQUE CLUSTERED INDEX CHEV_TYPE ON ChildEvent_C 
    ( _ParentCID, _ParentID, _ChildCID, _ChildID, _EventCID, _AlertState, _EventPriority, id ) 
  WITH FILLFACTOR = 10 ON [PRIMARY];
END

IF (SELECT OrigFillFactor FROM sysindexes WHERE name = 'CHEV_TYPE' AND id = OBJECT_ID('ChildEvent_C')) <> 10
BEGIN
  PRINT 'Adding FillFactor 10 to CHEV_TYPE index'
  DBCC DBREINDEX (ChildEvent_C, 'CHEV_TYPE', 10)
END

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ChildEvent_C_OLD' AND type = 'U')
BEGIN

  IF EXISTS (SELECT 1 FROM ChildEvent_C_OLD)
  BEGIN
    RAISERROR('Migrating data from old ChildEvent_C', 0, 1) WITH NOWAIT
    SET XACT_ABORT ON

    BEGIN TRANSACTION

    -- temp table
    CREATE TABLE #objs
    (
      cid ClassID,
      id  ObjID PRIMARY KEY (cid, id),
      Priority INT NULL
    )

    CREATE TABLE #chev_ep
    (
      chev_id       INT,
      EventPriority INT           NULL,
    )

    -- Populate the #objs table with a distinct set of child cid values
    INSERT INTO #objs (cid, id)
    SELECT DISTINCT _ChildCID, _ChildID
    FROM ChildEvent_C_OLD AS c
      JOIN ChildEvent_A AS a ON a.id = c.id
    WHERE c.deleted = 0


    -- Get the resource priorities
    DECLARE @c_cid ClassID, @c_cname sysname
    DECLARE c_objs INSENSITIVE CURSOR FOR
      SELECT DISTINCT RTRIM(oc.cid), oc.cname
      FROM obj_class AS oc
        JOIN #objs AS o ON o.cid = oc.cid
    OPEN c_objs
    FETCH NEXT FROM c_objs INTO @c_cid, @c_cname
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      EXEC('
        UPDATE o
        SET Priority = obj._PriorityID
        FROM #objs AS o
          JOIN ' + @c_cname + '_V AS obj ON obj.id = o.id AND o.cid = ''' + @c_cid + '''
      ')
      FETCH NEXT FROM c_objs INTO @c_cid, @c_cname
    END
    DEALLOCATE c_objs

    -- For those resources whose priority is 6, we do not have to compute the EventPriority
    INSERT INTO #chev_ep (chev_id, EventPriority)
    SELECT c.id, a._Priority
    FROM ChildEvent_C_OLD AS c
      JOIN ChildEvent_A AS a ON a.id = c.id
      JOIN #objs AS o ON o.cid = a._ChildCID AND o.id = a._ChildID
    WHERE c.deleted = 0
      AND o.Priority = 6

    -- Compute the EventPriority for Exceptions
    INSERT INTO #chev_ep (chev_id, EventPriority)
    SELECT c.id, e_v._Priority
    FROM ChildEvent_C_OLD AS c
      JOIN ChildEvent_A AS a ON a.id = c.id
      JOIN #objs AS o ON o.cid = a._ChildCID AND o.id = a._ChildID
      JOIN ChildEventHistory_V AS h_v ON h_v._ParentID = c.id
      JOIN Exception_V AS e_v ON e_v._ParentCID = a._ChildCID AND e_v._ParentID = a._ChildID
        AND h_v._EventDetail = e_v._EXCP_NAME AND e_v.deleted = 0
    WHERE o.Priority <> 6
       AND a._EventCID = 'EXCP'
       AND c.deleted = 0

    -- Compute the EventPriority for ChildEvents
    INSERT INTO #chev_ep (chev_id, EventPriority)
    SELECT c.id, c_v._Priority
    FROM ChildEvent_C_OLD AS c
      JOIN ChildEvent_A AS a ON a.id = c.id
      JOIN #objs AS o ON o.cid = a._ChildCID AND o.id = a._ChildID
      JOIN ChildEventHistory_V AS h_v ON h_v._ParentID = c.id
      JOIN (
        SELECT DISTINCT c_c.id, c_a._ParentCID, c_a._ParentID, c_a._EventDetail, c_a._Priority
        FROM ChildEvent_C_OLD AS c_c
          JOIN ChildEvent_A AS c_a ON c_c.id = c_a.id
        WHERE c_c.deleted = 0
      ) AS c_v ON c_v._ParentCID = a._ChildCID AND c_v._ParentID = a._ChildID AND h_v._EventDetail = c_v._EventDetail
    WHERE o.Priority <> 6
      AND a._EventCID = 'CHEV'
      AND c.deleted = 0

    -- Compute the EventPriority for Messages
    INSERT INTO #chev_ep (chev_id, EventPriority)
    SELECT c.id, m._Priority
    FROM ChildEvent_C_OLD AS c
      JOIN ChildEvent_A AS a ON a.id = c.id
      JOIN #objs AS o ON o.cid = a._ChildCID AND o.id = a._ChildID
      JOIN ChildEventHistory_V AS h_v ON h_v._ParentID = c.id
      JOIN Message_V AS m ON m._ParentCID = a._ChildCID AND m._ParentID = a._ChildID
        AND h_v._EventDetail = m._MESSAGE
      JOIN CurrentMessageAlertStatePri AS cmap ON cmap.mesg_id = m.id
    WHERE o.Priority <> 6
      AND a._EventCID = 'MESG'
      AND c.deleted = 0

    -- Handle any unresolvable Priorities
    -- Since we have no idea what the event priority is, just assume it is the same as the CHEV priority
    INSERT INTO #chev_ep (chev_id, EventPriority)
    SELECT c.id, a._Priority
    FROM ChildEvent_C_OLD AS c
      JOIN ChildEvent_A AS a ON a.id = c.id
      LEFT OUTER JOIN #chev_ep AS e ON e.chev_id = c.id
    WHERE e.chev_id IS NULL
      AND c.deleted = 0

    -- Insert the ChildEvent_C records
    -- The identity columns will not match, but this is acceptable since there will be more CHEVs under the
    -- new configuration than the old, and the ids are not referenced anywhere else.
    INSERT INTO ChildEvent_C
    (
      ctime, _ParentCID, _ParentID, _ChildCID, _ChildID, _EventCID, _AlertState, _EventPriority, _Priority, _EventDetail
    )
    SELECT DISTINCT
     c.ctime, a._ParentCID, a._ParentID, a._ChildCID, a._ChildID, a._EventCID, a._AlertState, ep.EventPriority, a._Priority, a._EventDetail
    FROM ChildEvent_C_OLD AS c
      JOIN ChildEvent_A AS a ON a.id = c.id
      JOIN #chev_ep AS ep ON ep.chev_id = c.id
    ORDER BY c.ctime

    COMMIT TRANSACTION

  END

  RAISERROR('Dropping obsolete table ChildEvent_A', 0, 1) WITH NOWAIT
  EXEC('IF EXISTS (SELECT * FROM sysobjects WHERE name = ''ChildEvent_A'' AND type = ''U'') DROP TABLE ChildEvent_A')
  RAISERROR('Dropping obsolete view ChildEvent_V', 0, 1) WITH NOWAIT
  EXEC('IF EXISTS (SELECT * FROM sysobjects WHERE name = ''ChildEvent_V'' AND type = ''V'') DROP VIEW  ChildEvent_V')

  RAISERROR('Dropping obsolete table ChildEventHistory_C', 0, 1) WITH NOWAIT
  EXEC('IF EXISTS (SELECT * FROM sysobjects WHERE name = ''ChildEventHistory_C'' AND type = ''U'') DROP TABLE ChildEventHistory_C')
  RAISERROR('Dropping obsolete table ChildEventHistory_A', 0, 1) WITH NOWAIT
  EXEC('IF EXISTS (SELECT * FROM sysobjects WHERE name = ''ChildEventHistory_A'' AND type = ''U'') DROP TABLE ChildEventHistory_A')
  RAISERROR('Dropping obsolete view ChildEventHistory_V', 0, 1) WITH NOWAIT
  EXEC('IF EXISTS (SELECT * FROM sysobjects WHERE name = ''ChildEventHistory_V'' AND type = ''V'') DROP VIEW  ChildEventHistory_V')

  DROP TABLE ChildEvent_C_OLD

END
GO

--
-- Triggers
--

-------------------------------------------------------------------------------------
-- tI_ChildEvent_C
-- This trigger is not used, so drop it if it exists
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tI_ChildEvent_C' AND type = 'TR')
  DROP TRIGGER tI_ChildEvent_C
GO

-------------------------------------------------------------------------------------
-- tI_ChildEvent_C_Prop
PRINT 'Creating trigger tI_ChildEvent_C_Prop'
GO

-------------------------------------------------------------------------------------
-- tI_ChildEvent_C_Prop
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tI_ChildEvent_C_Prop' AND type = 'TR')
  DROP TRIGGER tI_ChildEvent_C_Prop
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\tables\childevent_c.sql
-- SID: 1.21
-- Date Modified: 04/11/30 17:08:41
CREATE TRIGGER tI_ChildEvent_C_Prop ON ChildEvent_C FOR INSERT WITH APPEND
AS
  DECLARE @err INT, @now NVARCHAR(40), @rc INT
  SET @rc = @@ROWCOUNT

  -- set up the @verbose variable
  DECLARE @verbose TINYINT
  SET @verbose = 0

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ChildEvent_C_Prop: Processing %d record(s)', 0, 1, @now, @rc) WITH NOWAIT
  END -- IF @verbose >= 1

  IF @rc = 0
    RETURN


  -- Insert into the event_V view
  INSERT INTO event_V
  SELECT 
    i._ParentCID, i._ParentID, i.ctime, '', i._ParentCID, i._ParentID, 'CHEV', 
    ISNULL(i._EventDetail,' '), i.ctime, i._EventDetail, i._AlertState, i._Priority, 1, i.id
  FROM inserted AS i
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ChildEvent_C_Prop: Inserted %d record(s) into event_V', 0, 1, @now, @rc) WITH NOWAIT
  END -- IF @verbose >= 1
  
  CREATE TABLE #psu
  (
    id          INT NULL,
    OwnerCID    ClassID NULL,
    OwnerID     ObjID NULL,
    MatrixIndex INT NULL,
    ArrayIndex  INT NULL,
    OldCritical INT NOT NULL,
    NewCritical INT NOT NULL,
    OldHigh     INT NOT NULL,
    NewHigh     INT NOT NULL,
    OldMedium   INT NOT NULL,
    NewMedium   INT NOT NULL,
    OldLow      INT NOT NULL,
    NewLow      INT NOT NULL,
    OldIgnore   INT NOT NULL,
    NewIgnore   INT NOT NULL
  )

RecomputeDeltas:
  INSERT INTO #psu
    SELECT
      p.id,
      i._ParentCID,
      i._ParentID,
      1, -- EXCP
      i._AlertState,
      ISNULL(p.Critical, 0),
      ISNULL(p.Critical, 0) + (SELECT ISNULL(SUM(i._EventCount), 0) FROM inserted AS i WHERE i._ParentCID = p.OwnerCID AND i._ParentID = p.OwnerID AND i._AlertState = p.ArrayIndex AND i._Priority = 1),
      ISNULL(p.High, 0),
      ISNULL(p.High, 0)     + (SELECT ISNULL(SUM(i._EventCount), 0) FROM inserted AS i WHERE i._ParentCID = p.OwnerCID AND i._ParentID = p.OwnerID AND i._AlertState = p.ArrayIndex AND i._Priority = 2),
      ISNULL(p.Medium, 0),
      ISNULL(p.Medium, 0)   + (SELECT ISNULL(SUM(i._EventCount), 0) FROM inserted AS i WHERE i._ParentCID = p.OwnerCID AND i._ParentID = p.OwnerID AND i._AlertState = p.ArrayIndex AND i._Priority = 3),
      ISNULL(p.Low, 0),
      ISNULL(p.Low, 0)      + (SELECT ISNULL(SUM(i._EventCount), 0) FROM inserted AS i WHERE i._ParentCID = p.OwnerCID AND i._ParentID = p.OwnerID AND i._AlertState = p.ArrayIndex AND i._Priority = 4),
      ISNULL(p.Ignore, 0),
      ISNULL(p.Ignore, 0)   + (SELECT ISNULL(SUM(i._EventCount), 0) FROM inserted AS i WHERE i._ParentCID = p.OwnerCID AND i._ParentID = p.OwnerID AND i._AlertState = p.ArrayIndex AND i._Priority = 5)
    FROM inserted AS i
      LEFT OUTER JOIN PropagationSlot AS p ON i._ParentCID   = p.OwnerCID
                                          AND i._ParentID    = p.OwnerID
                                          AND i._AlertState  = p.ArrayIndex
                                          AND p.MatrixIndex  = 0 -- ChildEvent matrix
                                          AND p.SettingIndex = 0 -- Counts
  SELECT @err = @@ERROR, @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ChildEvent_C_Prop: Computed %d update(s) for events', 0, 1, @now, @rc) WITH NOWAIT
    IF @verbose >= 2    
      SELECT * FROM #psu
  END -- IF @verbose >= 1

  IF EXISTS (SELECT 1 FROM #psu)
  BEGIN  

    -- Create any missing PropagationSlot entries
    IF EXISTS (SELECT 1 FROM #psu WHERE id IS NULL)
    BEGIN
      CREATE TABLE #newps
      (
        OwnerCID    ClassID NOT NULL,
        OwnerID     ObjID NOT NULL
      )
      INSERT INTO #newps
        SELECT DISTINCT OwnerCID, OwnerID
        FROM #psu
        WHERE id IS NULL
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: tI_ChildEvent_C_Prop: Error %d computed ids of objects with missing PropagationSlots', 11, 1, @now, @err) WITH NOWAIT
        ROLLBACK TRANSACTION
        RETURN
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: tI_ChildEvent_C_Prop: Found %d object(s) with missing PropagationSlots', 0, 1, @now, @rc) WITH NOWAIT
      END

      INSERT INTO PropagationSlot
      (
        OwnerCID,
        OwnerID,
        MatrixIndex,
        SettingIndex,
        ArrayIndex,
        ResourceCID,
        ResourceID
      )
      SELECT
        n.OwnerCID,
        n.OwnerID,
        p.MatrixIndex,
        0,
        p.ArrayIndex,
        CASE n.OwnerCID WHEN 'LOB' THEN NULL ELSE n.OwnerCID END,
        CASE n.OwnerCID WHEN 'LOB' THEN 0 ELSE n.OwnerID END
      FROM #newps AS n
        JOIN PropagationSlot AS p ON p.OwnerCID = n.OwnerCID AND p.OwnerID = 0 AND p.SettingIndex = 1
      SELECT @err = @@ERROR, @rc = @@ROWCOUNT
      IF @err <> 0
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: tI_ChildEvent_C_Prop: Error %d creating missing PropagationSlots', 11, 1, @now, @err) WITH NOWAIT
        ROLLBACK TRANSACTION
        RETURN
      END
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: tI_ChildEvent_C_Prop: Created %d PropagationSlot(s)', 0, 1, @now, @rc) WITH NOWAIT
      END

      TRUNCATE TABLE #psu
      GOTO RecomputeDeltas
    END

    DELETE FROM #psu
    WHERE OldCritical = NewCritical
      AND OldHigh     = NewHigh
      AND OldMedium   = NewMedium
      AND OldLow      = NewLow
      AND OldIgnore   = NewIgnore    
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tI_ChildEvent_C_Prop: Deleted %d update(s) with no counter deltas', 0, 1, @now, @rc) WITH NOWAIT
    END -- IF @verbose >= 1
  
    -- Update the PropagationSlot table with the new count values
    UPDATE p SET
      Critical = u.NewCritical,
      High     = u.NewHigh,
      Medium   = u.NewMedium,
      Low      = u.NewLow,
      Ignore   = u.NewIgnore,
      mtime    = CURRENT_TIMESTAMP
    FROM PropagationSlot AS p
      JOIN #psu AS u ON u.id = p.id
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tI_ChildEvent_C_Prop: Updated %d row(s) in PropagationSlot', 0, 1, @now, @rc) WITH NOWAIT
    END -- IF @verbose >= 1
  END
  
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ChildEvent_C_Prop: Finished', 0, 1, @now) WITH NOWAIT
  END
GO

-------------------------------------------------------------------------------------
-- tI_ChildEvent_C_PBT
PRINT 'Creating trigger tI_ChildEvent_C_PBT'
GO

-------------------------------------------------------------------------------------
-- tI_ChildEvent_C_PBT
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tI_ChildEvent_C_PBT' AND type = 'TR')
  DROP TRIGGER tI_ChildEvent_C_PBT
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\tables\childevent_c.sql
-- SID: 1.21
-- Date Modified: 04/11/30 17:08:41
-- Inserts a record into StagedPBT for each distinct parent object and PBTUsedBy record.
CREATE TRIGGER tI_ChildEvent_C_PBT ON ChildEvent_C FOR INSERT
AS
  -- Quick check to return immediately if no rows inserted
  IF @@ROWCOUNT = 0
    RETURN

  DECLARE @now NVARCHAR(40), @rc INT, @verbose TINYINT
  SET @verbose = 0
    
  IF @verbose >= 1
  BEGIN
    SELECT @rc = COUNT(*) FROM inserted
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ChildEvent_C_PBT: Starting. ROWCOUNT=%d', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Do the insert
  -- The _ParentCID/_ParentID columns of the inserted rows are joined to PBTUsedBy obj_cid/obj_id.
  -- PBTUsedBy is joined to PercentageBasedThreshold to filter-out PBTs that are not enabled.
  INSERT INTO StagedPBT ( id )
    SELECT DISTINCT u.id
    FROM inserted AS i
    JOIN PBTUsedBy AS u ON u.obj_cid = i._ParentCID AND u.obj_id = i._ParentID
    JOIN PercentageBasedThreshold AS p ON p.id = u.pbt_id
    WHERE u.enabled = 1
      AND p.enabled = 1
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ChildEvent_C_PBT: Finished. Inserted %d record(s) into StagedPBT', 0, 1, @now, @rc) WITH NOWAIT
  END
GO

-------------------------------------------------------------------------------------
-- tU_ChildEvent_C
-- This trigger is not used, so drop it if it exists
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_ChildEvent_C' AND type = 'TR')
  DROP TRIGGER tU_ChildEvent_C
GO

PRINT 'Creating trigger tU_ChildEvent_C_Prop'
GO

-------------------------------------------------------------------------------------
-- tU_ChildEvent_C_Prop

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_ChildEvent_C_Prop' AND type = 'TR')
  DROP TRIGGER tU_ChildEvent_C_Prop
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\tables\childevent_c.sql
-- SID: 1.21
-- Date Modified: 04/11/30 17:08:41
CREATE TRIGGER tU_ChildEvent_C_Prop ON ChildEvent_C FOR UPDATE WITH APPEND
AS

  DECLARE @rc INT
  SET @rc = @@ROWCOUNT
  IF @rc = 0
    RETURN

  -- set up the @verbose variable
  DECLARE @verbose TINYINT
  SET @verbose = 0

  DECLARE @now NVARCHAR(40)
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ChildEvent_C_Prop: Processing %d records updated in ChildEvent_C', 0, 1, @now, @rc) WITH NOWAIT
  END -- IF @verbose >= 1

  -- Do not allow updates of AlertState or Priority
  IF UPDATE(_AlertState) OR UPDATE(_Priority)
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ChildEvent_C: ERROR: Updating of AlertState and Priority columns is not allowed', 11, 1, @now) WITH NOWAIT
    ROLLBACK TRANSACTION
    RETURN
  END -- IF UPDATE(_AlertState) OR UPDATE(_Priority)

  --
  -- Handle child events being deleted
  --
  IF UPDATE(deleted)
  BEGIN


    -- Insert into the event_V view
    INSERT INTO event_V
    SELECT 
      i._ParentCID, i._ParentID, ISNULL(i.dtime, CURRENT_TIMESTAMP), '', i._ParentCID, i._ParentID, 'CHEV', 
      ISNULL(i._DeleteDetail, ' '), ISNULL(i.dtime, CURRENT_TIMESTAMP), ISNULL(i._DeleteDetail, ' '), i._AlertState, i._Priority, 0, i.id
    FROM inserted AS i
    JOIN deleted AS d ON d.id = i.id
    WHERE i.deleted = 1 AND d.deleted = 0
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tU_ChildEvent_C_Prop: Inserted %d record(s) into event_V', 0, 1, @now, @rc) WITH NOWAIT
    END -- IF @verbose >= 1

    -- Update the PropagationSlot entries for the ChildEvents whose EventCount values have changed
    UPDATE p SET
      Critical = p.Critical + (SELECT ISNULL(SUM(0 - d._EventCount), 0) FROM deleted AS d WHERE d._ParentCID = p.OwnerCID AND d._ParentID = p.OwnerID AND d._AlertState = p.ArrayIndex AND d._Priority = 1 AND d.deleted = 0),
      High     = p.High     + (SELECT ISNULL(SUM(0 - d._EventCount), 0) FROM deleted AS d WHERE d._ParentCID = p.OwnerCID AND d._ParentID = p.OwnerID AND d._AlertState = p.ArrayIndex AND d._Priority = 2 AND d.deleted = 0),
      Medium   = p.Medium   + (SELECT ISNULL(SUM(0 - d._EventCount), 0) FROM deleted AS d WHERE d._ParentCID = p.OwnerCID AND d._ParentID = p.OwnerID AND d._AlertState = p.ArrayIndex AND d._Priority = 3 AND d.deleted = 0),
      Low      = p.Low      + (SELECT ISNULL(SUM(0 - d._EventCount), 0) FROM deleted AS d WHERE d._ParentCID = p.OwnerCID AND d._ParentID = p.OwnerID AND d._AlertState = p.ArrayIndex AND d._Priority = 4 AND d.deleted = 0),
      Ignore   = p.Ignore   + (SELECT ISNULL(SUM(0 - d._EventCount), 0) FROM deleted AS d WHERE d._ParentCID = p.OwnerCID AND d._ParentID = p.OwnerID AND d._AlertState = p.ArrayIndex AND d._Priority = 5 AND d.deleted = 0),
      mtime    = CURRENT_TIMESTAMP
    FROM PropagationSlot AS p
      JOIN inserted AS i ON i._ParentCID = p.OwnerCID AND i._ParentID = p.OwnerID AND i._AlertState = p.ArrayIndex
      JOIN deleted AS d ON d.id = i.id
    WHERE p.MatrixIndex  = 0 -- ChildEvent
      AND p.SettingIndex = 0
      AND d.deleted = 0 AND i.deleted = 1
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tU_ChildEvent_C_Prop: Updated %d PropagationSlot entries', 0, 1, @now, @rc) WITH NOWAIT
    END -- IF @verbose >= 1

  END -- IF UPDATE(deleted)
  
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ChildEvent_C_Prop: Finished', 0, 1, @now) WITH NOWAIT
  END
GO

-------------------------------------------------------------------------------------
-- tU_ChildEvent_C_PBT
PRINT 'Creating trigger tU_ChildEvent_C_PBT'
GO

-------------------------------------------------------------------------------------
-- tU_ChildEvent_C_PBT
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_ChildEvent_C_PBT' AND type = 'TR')
  DROP TRIGGER tU_ChildEvent_C_PBT
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\tables\childevent_c.sql
-- SID: 1.21
-- Date Modified: 04/11/30 17:08:41
-- Inserts a record into StagedPBT for each distinct parent object and PBTUsedBy record
-- for ChildEvent_C records that are being marked for deletion.
CREATE TRIGGER tU_ChildEvent_C_PBT ON ChildEvent_C FOR UPDATE
AS
  -- Quick check to return immediately if no rows inserted
  IF @@ROWCOUNT = 0
    RETURN

  DECLARE @now NVARCHAR(40), @rc INT, @verbose TINYINT
  SET @verbose = 0
    
  IF @verbose >= 1
  BEGIN
    SELECT @rc = COUNT(*) FROM inserted
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ChildEvent_C_PBT: Starting. ROWCOUNT=%d', 0, 1, @now, @rc) WITH NOWAIT
  END

  -- Do the insert.
  -- The _ParentCID/_ParentID columns of the inserted rows are joined to PBTUsedBy obj_cid/obj_id.
  -- PBTUsedBy is joined to PercentageBasedThreshold to filter-out PBTs that are not enabled.
  INSERT INTO StagedPBT ( id )
    SELECT DISTINCT u.id
    FROM inserted AS i
    JOIN deleted AS d ON d.id = i.id
    JOIN PBTUsedBy AS u ON u.obj_cid = i._ParentCID AND u.obj_id = i._ParentID
    JOIN PercentageBasedThreshold AS p ON p.id = u.pbt_id
    WHERE d.deleted = 0
      AND i.deleted = 1
      AND u.enabled = 1
      AND p.enabled = 1
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ChildEvent_C_PBT: Finished. Inserted %d record(s) into StagedPBT', 0, 1, @now, @rc) WITH NOWAIT
  END
GO
