-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/event.sql
-- SID: 1.13.1.3
-- Date Modified: 04/09/23 18:31:17
--
-- Resides in: \TivoliManager\sql
----
-- This SQL validates and maintains the Object..event table residing on the
-- Live server.
--
-- Any changes to this table should be reflected in the following other table definitions.
--    Object..event              Can either be a table or a view
--    Object..eventbcp
--    Object..event_legacy
--    EventHistory..event_table
--    History..eventhist
-------------
IF NOT EXISTS (SELECT * FROM master..sysdatabases WHERE name = 'Object')
  BEGIN
    RAISERROR ('Object database IS NOT present ON this server', 0, 11)
  END
  ELSE

  BEGIN
  USE Object

  IF EXISTS (SELECT * FROM syscolumns WHERE id = object_id('event')
     AND (    (name = 'agent_seqno' AND (xusertype <> 231 OR length <> 100))   -- nvarchar(50)
           OR (name = 'detail'      AND (xusertype <> 231 OR length <> 510)) ) -- nvarchar(255)
     AND NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'event' AND type = 'V') )
     BEGIN
       PRINT 'Obsolete Object..event TABLE discovered. Rename scheduled.'
       EXEC sp_rename 'event', 'eventOLD', 'OBJECT'
       -- ---------------------------------------------------------
       -- The following rename is required to allow the table constraint name
       -- to be defined in the create table verb below
       -- ---------------------------------------------------------
       EXEC sp_rename 'event_PK', 'eventOLD_PK', 'OBJECT'
     END
  -- ----------------------
  -- Create the event TABLE if the event TABLE or VIEW does NOT exist.
  -- If it's a view, then we're running a Linked server history system, so leave it alone.
  -- ----------------------
  IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'event' AND (type = 'U' OR type = 'V') )
    BEGIN
     PRINT 'Creating Object..event TABLE'
     -- File: archive/db/sql/event.sql
     -- SID: 1.13.1.3
     -- Date Modified: 04/09/23 18:31:17
     CREATE TABLE event
     (
      agent_cid     ClassID,
      agent_id      ObjID,
      agent_ctime   DATETIME,
      agent_seqno   NVARCHAR(50),
      obj_cid       ClassID,
      obj_id        ObjID,
      cid           ClassID,
      id            NUMERIC(16) identity,
      name          ObjName,
      ctime         DATETIME,
      detail        NVARCHAR(255) NULL,
      alertstate    INT,
      priority      INT,
      state         INT,
      tstamp        timestamp
      CONSTRAINT event_PK primary key clustered (tstamp)
     )
    END

  IF EXISTS (SELECT * FROM sysobjects WHERE name = 'eventOLD' AND type = 'U')
    BEGIN
       PRINT 'Migrating Object..event TABLE'
       SET IDENTITY_INSERT event ON
       SET ANSI_WARNINGS OFF
       INSERT INTO event (agent_cid, agent_id, agent_ctime, agent_seqno,
         obj_cid, obj_id, cid, id, name, ctime, detail, alertstate, priority,
         state)
        SELECT agent_cid, agent_id, agent_ctime, agent_seqno,
         obj_cid, obj_id, cid, id, name, ctime, detail, alertstate, priority,
         state FROM eventOLD
       PRINT 'Dropping Object..eventOLD'
       DROP TABLE eventOLD
    END
  -- add evt_id column for event table
  IF EXISTS (SELECT * FROM sysobjects WHERE name = 'event' AND type = 'U')
    IF NOT EXISTS (SELECT * FROM syscolumns WHERE name = 'evt_id' AND OBJECT_NAME(id) = 'event')
      ALTER TABLE event ADD evt_id INTEGER NULL
END
