-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\tables\exceptionhistory_c.sql
-- SID: 1.6
-- Date Modified: 04/09/23 18:15:56
--
-- Module Description
--   Defines the ExceptionHistory_C table
USE Object
GO

SET XACT_ABORT ON

-- Look for table with old structure. If it exists, the records will be moved to the new table.

BEGIN TRANSACTION UpgradeExceptionHistory

DECLARE @now NVARCHAR(40), @rc INT

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ExceptionHistory_C_OLD' AND type = 'U')
BEGIN
  SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
  RAISERROR('%s: ExceptionHistory_C_OLD ALREADY EXISTS, ABORTING!', 11, 1, @now) WITH NOWAIT
END

IF EXISTS (SELECT * FROM syscolumns WHERE id = OBJECT_ID('ExceptionHistory_C') AND name = 'description')
BEGIN

  DECLARE cnstc INSENSITIVE CURSOR FOR
    SELECT OBJECT_NAME(parent_obj), name
    FROM sysobjects
    WHERE name IN
    (
      'EXCH_ctime_DF',
      'EXCH_mtime_DF',
      'EXCH_deleted_DF',
      'EXCH_SEQ_NO_DF',
      'EXCH_TIME_DF',
      'EXCH_SEQ_NO_DF',
      'EXCH_EXCP_CD_DF',
      'EXCH_ParentID_DF',
      'EXCH_EXCP_NAME_DF',
      'EXCH_PK'
    )
  OPEN cnstc
  DECLARE @table sysname, @cnst sysname
  FETCH NEXT FROM cnstc INTO @table, @cnst
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: Dropping constraint %s from old %s', 0, 1, @now, @cnst, @table) WITH NOWAIT
    EXEC('ALTER TABLE ' + @table + ' DROP CONSTRAINT ' + @cnst)
    FETCH NEXT FROM cnstc INTO @table, @cnst
  END
  DEALLOCATE cnstc

  EXEC sp_rename ExceptionHistory_C, ExceptionHistory_C_OLD
END

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'ExceptionHistory_C' AND type = 'U')
BEGIN
  PRINT 'Creating table ExceptionHistory_C'
  CREATE TABLE ExceptionHistory_C
  (
    id             ObjID         NOT NULL IDENTITY(1,1),
    ctime          DATETIME      NOT NULL CONSTRAINT EXCH_ctime_DF DEFAULT (CURRENT_TIMESTAMP),
    mtime          DATETIME      NOT NULL CONSTRAINT EXCH_mtime_DF DEFAULT (CURRENT_TIMESTAMP),
    deleted        TINYINT       NOT NULL CONSTRAINT EXCH_deleted_DF DEFAULT (0),
    _TIME          DATETIME      NOT NULL CONSTRAINT EXCH_TIME_DF DEFAULT GETDATE(),
    _SEQ_NO        NVARCHAR(50)  NULL     CONSTRAINT EXCH_SEQ_NO_DF DEFAULT (NULL),
    _EXCP_CD       NVARCHAR(255)     NULL CONSTRAINT EXCH_EXCP_CD_DF DEFAULT (NULL),
    _ParentID      INT           NOT NULL CONSTRAINT EXCH_ParentID_DF DEFAULT 0,
    _EXCP_NAME     NVARCHAR(255) NOT NULL CONSTRAINT EXCH_EXCP_NAME_DF DEFAULT ('')
  ) ON [PRIMARY];
  ALTER TABLE ExceptionHistory_C ADD CONSTRAINT EXCH_PK PRIMARY KEY CLUSTERED (id) ON [PRIMARY];
  CREATE UNIQUE INDEX EXCH_C_TIME ON ExceptionHistory_C (_TIME, id) ON [PRIMARY];
END

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ExceptionHistory_C_OLD' AND type = 'U')
BEGIN
  IF     (SELECT COUNT(*) FROM ExceptionHistory_C) = 0 
     AND (SELECT COUNT(*) FROM ExceptionHistory_C_OLD) > 0
     AND EXISTS (SELECT * FROM sysobjects WHERE name = 'ExceptionHistory_A' AND type = 'U')
  BEGIN
    SET IDENTITY_INSERT ExceptionHistory_C ON
    INSERT INTO ExceptionHistory_C (id, ctime, mtime, deleted, _TIME, _SEQ_NO,
      _EXCP_CD, _ParentID, _EXCP_NAME)
    SELECT
      c.id, c.ctime, c.mtime, c.deleted, a._TIME, a._SEQ_NO,
      a._EXCP_CD, a._ParentID, a._EXCP_NAME
    FROM
      ExceptionHistory_C_OLD AS c JOIN ExceptionHistory_A AS a ON a.id = c.id
    SET IDENTITY_INSERT ExceptionHistory_C OFF
  END
  -- Get rid of old Exception_C table
  DROP TABLE ExceptionHistory_C_OLD
END

-- We drop the old ExceptionHistory_A table in this module instead of in exceptionhistory_a.sql because it can
-- only be dropped after the successful migration of its data into the new ExceptionHistory_C table.
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ExceptionHistory_A' AND type = 'U')
BEGIN
  PRINT 'Dropping obsolete ExceptionHistory_A table'
  EXEC('DROP TABLE ExceptionHistory_A')
END

COMMIT TRANSACTION UpgradeExceptionHistory
GO


------------------------------------------------------------------------------------------------------------------------
-- tI_ExceptionHistory_C

IF EXISTS (SELECT * FROM sysobjects WHERE name  = 'tI_ExceptionHistory_C' AND type = 'TR')
  DROP TRIGGER tI_ExceptionHistory_C
GO

PRINT 'Creating trigger tI_ExceptionHistory_C'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/exceptionhistory_c.sql
-- SID: 1.5
-- Date Modified: 04/07/19 20:56:46
CREATE TRIGGER tI_ExceptionHistory_C ON ExceptionHistory_C FOR INSERT WITH APPEND
AS
--First insert from exceptions with valid OS in _SEQ_NO using os as agent
-- find os id
  DECLARE @flag_containment INT, @os_id ObjID, @objname NVARCHAR(24), @objid ObjID
  SELECT @flag_containment = flag_direct_containment, @objname=cname, @objid=e._ParentID
  FROM obj_class o, inserted a, Exception_V e
  WHERE o.cid = e._ParentCID AND e.id=a._ParentID

  IF @flag_containment = 1
  BEGIN
    DECLARE @midcid ClassID, @midid ObjID
    DECLARE @id_str NVARCHAR(9)
    SET @id_str = CAST(@objid AS NVARCHAR)
    EXEC("
      DECLARE directc INSENSITIVE CURSOR FOR SELECT _ParentCID, _ParentID FROM " + @objname + "_V WHERE id = " + @id_str
    )
    OPEN directc
    FETCH NEXT FROM directc INTO @midcid, @midid
    DEALLOCATE directc
    SELECT @os_id=src_id FROM containment 
      WHERE dst_cid = @midcid AND dst_id = @midid AND src_cid='OS' AND deleted=0
  END

  ELSE
    SELECT @os_id = src_id 
      FROM containment c, inserted a, Exception_V e
      WHERE c.dst_cid=e._ParentCID AND c.dst_id = e._ParentID 
        AND c.src_cid = 'OS' AND c.deleted=0 AND e.id = a._ParentID

  INSERT event_V
  SELECT 
    'OS', os.id, a._TIME, a._SEQ_NO, e._ParentCID, e._ParentID, 'EXCP', 
    o._EXCP_NAME, a._TIME, a._EXCP_CD, isnull(e._AlertState,0), isnull(e._Priority,0), 0, a._ParentID
  FROM inserted a, ExceptionHistory_C o, Exception_V e, OS_C os 
  WHERE 
    a.id=o.id AND e.id=a._ParentID AND os.name=SUBSTRING(a._SEQ_NO,1,4) 
    AND os.id = @os_id AND os.deleted=0

  --Next insert from exceptions without valid os in _SEQ_NO using parent of history as agent 
  INSERT event_V 
  SELECT 
    e._ParentCID, e._ParentID, a._TIME, a._SEQ_NO, e._ParentCID, e._ParentID, 'EXCP', 
    o._EXCP_NAME, a._TIME, a._EXCP_CD, ISNULL(e._AlertState,0), ISNULL(e._Priority,0), 0, a._ParentID
  FROM inserted a, ExceptionHistory_C o, Exception_V e 
  WHERE 
    a.id=o.id AND e.id=a._ParentID AND a.id NOT IN 
      (SELECT a.id FROM inserted a, OS_C os WHERE SUBSTRING(a._SEQ_NO,1,4)=os.name AND os.id = @os_id AND os.deleted=0) 
    
    --   if customer has defined rules in the AT_Rule table then queue messages to PendingAutoTicketEvent table 
    --   for processing by DeQueue AT Events SQL Server task/job
  IF EXISTS (SELECT 1 FROM AT_Rule WITH (NOLOCK))
  BEGIN
    INSERT PendingAutoTicketEvent
     (obj_cid,obj_id,obj_name,event_cid,event_id,event_name,event_ctime,
      event_detail,event_alertstate,event_priority)
    SELECT ev._ParentCID,ev._ParentID,ISNULL(ev._OBJECT_NAME,''),'EXCP', a.id, eh._EXCP_NAME,a._TIME,
           eh._EXCP_CD, ev._AlertState,ev._Priority
    FROM inserted a
      JOIN ExceptionHistory_C eh ON eh.id = a.id
      JOIN Exception_V ev ON ev.id = eh._ParentID 
    WHERE ev.deleted = 0
  END

  --Uncommenting these will enable delete of history 
  --delete from ExceptionHistory_A where id in (select id from inserted) 
  --delete from ExceptionHistory_C where id in (select id from inserted) 

GO

