-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/message_c.sql
-- SID: 1.5.1.7
-- Date Modified: 04/09/23 18:10:58
--
-- Module Description
--   Defines the Message_C table
USE Object
GO

SET XACT_ABORT ON

-- Look for table with old structure. If it exists, the records will be moved to the new table.

BEGIN TRANSACTION UpgradeMessage

DECLARE @now NVARCHAR(40), @rc INT

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'Message_C_OLD' AND type = 'U')
BEGIN
  SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
  RAISERROR('%s: Message_C_OLD ALREADY EXISTS, ABORTING!', 11, 1, @now) WITH NOWAIT
END

IF EXISTS (SELECT * FROM syscolumns WHERE id = OBJECT_ID('Message_C') AND name = 'description')
BEGIN
  DECLARE cnstc INSENSITIVE CURSOR FOR
    SELECT OBJECT_NAME(parent_obj), name
    FROM sysobjects
    WHERE name IN
    (
      'MESG_C_PK',
      'MESG_mtime_DF',
      'MESG_ctime_DF',
      'MESG_deleted_DF',
      'MESG_SEQ_NO_DF',
      'MESG_OBJ_TYPE_ID_DF',
      'MESG_OBJECT_NAME_DF',
      'MESG_NATIVE_KEY_DF',
      'MESG_A_DISCOVERY_IND_DF',
      'MESG_A_MESSAGE_DF',
      'MESG_A_STATE_DF',
      'MESG_A_StateID_DF',
      'MESG_A_REASON_CD_DF'
    )
  OPEN cnstc
  DECLARE @table sysname, @cnst sysname
  FETCH NEXT FROM cnstc INTO @table, @cnst
  WHILE @@FETCH_STATUS <> -1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: Dropping constraint %s from old %s', 0, 1, @now, @cnst, @table) WITH NOWAIT
    EXEC('ALTER TABLE ' + @table + ' DROP CONSTRAINT ' + @cnst)
    FETCH NEXT FROM cnstc INTO @table, @cnst
  END
  DEALLOCATE cnstc

  IF EXISTS (SELECT * FROM sysindexes WHERE name = 'iMESG_NAME')
    DROP INDEX Message_C.iMESG_NAME
  IF EXISTS (SELECT * FROM sysindexes WHERE name = 'Message_MsgID')
    DROP INDEX Message_A.Message_MsgID

  EXEC sp_rename Message_C, Message_C_OLD
END

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'Message_C' AND type = 'U')
BEGIN
  PRINT 'Creating table Message_C'

  CREATE TABLE Message_C
  (
    id             ObjID         NOT NULL IDENTITY(1,1),
    name           ObjName       NOT NULL,
    ctime          DATETIME      NOT NULL CONSTRAINT MESG_ctime_DF DEFAULT (CURRENT_TIMESTAMP),
    mtime          DATETIME      NOT NULL CONSTRAINT MESG_mtime_DF DEFAULT (CURRENT_TIMESTAMP),
    deleted        TINYINT       NOT NULL CONSTRAINT MESG_deleted_DF DEFAULT (0),
    _TIME          DATETIME      NOT NULL,
    _SEQ_NO        NVARCHAR(50)      NULL CONSTRAINT MESG_SEQ_NO_DF DEFAULT (NULL),
    _OBJ_TYPE_ID   NVARCHAR(4)   NOT NULL CONSTRAINT MESG_OBJ_TYPE_ID_DF DEFAULT (''),
    _OBJECT_NAME   NVARCHAR(54)  NOT NULL CONSTRAINT MESG_OBJECT_NAME_DF DEFAULT (''),
    _NATIVE_KEY    NVARCHAR(10)  NOT NULL CONSTRAINT MESG_NATIVE_KEY_DF DEFAULT (''),
    _DISCOVERY_IND NVARCHAR(4)       NULL CONSTRAINT MESG_A_DISCOVERY_IND_DF         DEFAULT 'MON',
    _MESSAGE       NVARCHAR(255)     NULL CONSTRAINT MESG_A_MESSAGE_DF               DEFAULT '',
    _STATE         NVARCHAR(40)      NULL CONSTRAINT MESG_A_STATE_DF                 DEFAULT '',
    _StateID       INT               NULL CONSTRAINT MESG_A_StateID_DF               DEFAULT 0,
    _REASON_CD     NVARCHAR(255)     NULL CONSTRAINT MESG_A_REASON_CD_DF             DEFAULT '',
    _AlertState    INT           NOT NULL,
    _Priority      INT           NOT NULL,
    _ParentCID     NVARCHAR(4)   NOT NULL,
    _ParentID      INT           NOT NULL,
    _flags         INT           NOT NULL CONSTRAINT MESG_A_flags_DF DEFAULT 0
  ) ON [PRIMARY];
  ALTER TABLE Message_C ADD CONSTRAINT MESG_C_PK PRIMARY KEY CLUSTERED (id) ON [PRIMARY];
END

IF NOT EXISTS (SELECT 1 FROM sysindexes WHERE name = 'MESG_C_Parent_UQ')
BEGIN
  PRINT 'Creating index MESG_C_Parent_UQ on Message_C'
  CREATE UNIQUE INDEX MESG_C_Parent_UQ ON Message_C (_ParentCID, _ParentID, id)
END

IF NOT EXISTS (SELECT 1 FROM sysindexes WHERE name = 'MESG_C_TIME')
BEGIN
  PRINT 'Creating index MESG_C_TIME on Message_C'
  CREATE UNIQUE INDEX MESG_C_TIME ON Message_C (_TIME, id)
END
  
IF NOT EXISTS (SELECT 1 FROM sysindexes WHERE name = 'Message_MsgID')
BEGIN
  PRINT 'Creating index Message_MsgID on Message_C'
  CREATE INDEX Message_MsgID ON Message_C (_OBJ_TYPE_ID, _MESSAGE)
END
  
IF NOT EXISTS (SELECT 1 FROM sysindexes WHERE name = 'MESG_C_PCID_PID_DATE')
BEGIN
  PRINT 'Creating index MESG_C_PCID_PID_DATE on Message_C'
  CREATE INDEX MESG_C_PCID_PID_DATE ON Message_C (_ParentCID, _ParentID, _TIME)
END
  
IF NOT EXISTS (SELECT 1 FROM sysindexes WHERE name = 'MESG_C_PCID_PID_DATE_ALERTSTATE')
BEGIN
  PRINT 'Creating index MESG_C_PCID_PID_DATE_ALERTSTATE on Message_C'
  CREATE INDEX MESG_C_PCID_PID_DATE_ALERTSTATE ON Message_C (_ParentCID, _ParentID, _TIME, _AlertState)
END
  
IF NOT EXISTS (SELECT 1 FROM sysindexes WHERE name = 'MESG_C_by_Alert_Pri')
BEGIN
  PRINT 'Creating index MESG_C_by_Alert_Pri on Message_C'
  CREATE UNIQUE INDEX MESG_C_by_Alert_Pri ON Message_C(_ParentCID, _ParentID, _AlertState, _Priority, id)
END
  
IF NOT EXISTS (SELECT 1 FROM sysindexes WHERE name = 'MESG_C_by_MESSAGE_Pri')
BEGIN
  PRINT 'Creating index MESG_C_by_MESSAGE_Pri on Message_C'
  CREATE UNIQUE INDEX MESG_C_by_MESSAGE_Pri ON Message_C(_ParentCID, _ParentID, _MESSAGE, _Priority, id)
END

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'Message_C_OLD' AND type = 'U')
BEGIN
  IF     (SELECT COUNT(*) FROM Message_C) = 0 
     AND (SELECT COUNT(*) FROM Message_C_OLD) > 0
     AND EXISTS (SELECT * FROM sysobjects WHERE name = 'Message_A' AND type = 'U')
  BEGIN
    SET IDENTITY_INSERT Message_C ON
    INSERT INTO Message_C (id, name, ctime, mtime, deleted, _TIME, _SEQ_NO, _OBJ_TYPE_ID,
      _OBJECT_NAME, _NATIVE_KEY, _DISCOVERY_IND, _MESSAGE, _STATE, _StateID, _REASON_CD,
      _AlertState, _Priority, _ParentCID, _ParentID)
    SELECT
      c.id, c.name, c.ctime, c.mtime, c.deleted, a._TIME, a._SEQ_NO, a._OBJ_TYPE_ID,
      a._OBJECT_NAME, a._NATIVE_KEY, a._DISCOVERY_IND, a._MESSAGE, a._STATE, a._StateID, a._REASON_CD,
      a._AlertState, a._Priority, a._ParentCID, a._ParentID
    FROM Message_C_OLD AS c JOIN Message_A AS a ON a.id = c.id
    SET IDENTITY_INSERT Message_C OFF
  END
  -- Get rid of old Exception_C table
  DROP TABLE Message_C_OLD
END

-- We drop the old Message_A table in this module instead of in exception_a.sql because it can
-- only be dropped after the successful migration of its data into the new Message_C table.
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'Message_A' AND type = 'U')
BEGIN
  PRINT 'Dropping obsolete Message_A table'
  EXEC('DROP TABLE Message_A')
END

-- Add the _flags column to Message_C if needed. This is normally handled by the migration code above
-- but is also being done here in case an earlier version of this module was already applied.
IF NOT EXISTS (SELECT 1 FROM syscolumns AS c WHERE OBJECT_NAME(id) = 'Message_C' AND name = '_flags')
BEGIN
  PRINT 'Adding _flags column to Message_C'
  ALTER TABLE Message_C ADD
    _flags         INT           NOT NULL CONSTRAINT MESG_A_flags_DF DEFAULT 0    
END

COMMIT TRANSACTION UpgradeMessage
GO

--
-- Triggers
--

--------------------------------------------------------------------------------------------
-- tI_Message_C

PRINT 'Creating trigger tI_Message_C'
GO 

IF EXISTS (SELECT * FROM sysobjects WHERE type='TR' AND name='tI_Message_C') 
  DROP TRIGGER tI_Message_C
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/message_c.sql
-- SID: 1.5.1.6
-- Date Modified: 04/07/28 17:42:10
CREATE TRIGGER tI_Message_C ON Message_C FOR INSERT AS 

  --First insert from messages with valid OS in _SEQ_NO using os as agent
  -- find os id
  DECLARE @flag_containment INT, @os_id ObjID, @objname NVARCHAR(24), @objid ObjID
  SELECT @flag_containment = flag_direct_containment, @objname=cname, @objid=a._ParentID
  FROM obj_class o, inserted a
  WHERE o.cid = a._ParentCID

  IF @flag_containment = 1
  BEGIN
    DECLARE @midcid ClassID, @midid ObjID
    DECLARE @id_str NVARCHAR(9)
    SET @id_str = CAST(@objid AS NVARCHAR)
    EXEC("
      DECLARE directc INSENSITIVE CURSOR FOR SELECT _ParentCID, _ParentID FROM " + @objname + "_V WHERE id = " + @id_str
    )
    OPEN directc
    FETCH NEXT FROM directc INTO @midcid, @midid
    DEALLOCATE directc
    SELECT @os_id=src_id FROM containment 
      WHERE dst_cid = @midcid AND dst_id = @midid AND src_cid='OS' AND deleted=0
  END

  ELSE
    SELECT @os_id = src_id 
      FROM containment c, inserted a 
      WHERE c.dst_cid=a._ParentCID AND c.dst_id = a._ParentID 
        AND c.src_cid = 'OS' AND c.deleted=0

  INSERT event_V 
  SELECT 
    'OS', os.id, a._TIME, a._SEQ_NO, a._ParentCID, a._ParentID, 'MESG', 
    a._MESSAGE, a._TIME, a._REASON_CD, ISNULL(a._AlertState,0), ISNULL(a._Priority,0), ISNULL(a._StateID,0), a.id 
  FROM inserted a, OS_C os 
  WHERE 
    os.name=SUBSTRING(a._SEQ_NO,1,4) AND os.id = @os_id AND os.deleted=0
  --Next insert from messages without valid os in _SEQ_NO using parent of message as agent 
  INSERT event_V 
  SELECT 
    a._ParentCID, a._ParentID, a._TIME, a._SEQ_NO, a._ParentCID, a._ParentID, 'MESG', 
    a._MESSAGE, a._TIME, a._REASON_CD, ISNULL(a._AlertState,0), ISNULL(a._Priority,0), ISNULL(a._StateID,0), a.id 
  FROM inserted a 
  WHERE 
    a.id NOT IN (SELECT a.id FROM inserted a, OS_C os WHERE SUBSTRING(a._SEQ_NO,1,4)=os.name AND os.id = @os_id AND os.deleted=0) 

  -- If customer has defined rules in the AT_Rule table then queue messages to PendingAutoTicketEvent table 
  -- for processing by DeQueue AT Events SQL Server task/job

  IF EXISTS (SELECT 1 FROM AT_Rule WITH (NOLOCK))
  BEGIN
    INSERT PendingAutoTicketEvent
      (obj_cid,obj_id,obj_name,event_cid,event_id,event_name,event_ctime,event_detail,event_alertstate, event_priority,event_state)
    SELECT m._ParentCID,m._ParentID,ISNULL(m._OBJECT_NAME,''),'MESG',m.id,m._MESSAGE,m._TIME, m._REASON_CD,
           isnull(m._AlertState,0),isnull(m._Priority,0),isnull(m._StateID,0)
    FROM Message_C m, inserted a WHERE m.id = a.id 
  END


GO 


--------------------------------------------------------------------------------------------
-- tU_Message_C
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_Message_C' AND type = 'TR')
  DROP TRIGGER tU_Message_C 
GO

PRINT 'Creating trigger tU_Message_C'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\message_c.sql
-- SID: 1.5.1.6
-- Date Modified: 04/07/28 17:42:10
CREATE TRIGGER tU_Message_C ON Message_C FOR UPDATE AS
  IF UPDATE(id)
  BEGIN
    RAISERROR('Cannot UPDATE id IN Message_C', 0, 1)
    ROLLBACK TRAN
    RETURN
  END

  IF UPDATE(deleted)
  BEGIN
   UPDATE ExtendedEvent_C SET ExtendedEvent_C.deleted =inserted.deleted 
     FROM inserted WHERE ExtendedEvent_C.id=inserted.id and ExtendedEvent_C.eventcid='MESG'
  END

GO

--------------------------------------------------------------------------------------------
-- tD_Message_C
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tD_Message_C' AND type = 'TR')
  DROP TRIGGER tD_Message_C
GO

PRINT 'Creating trigger tD_Message_C'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\message_c.sql
-- SID: 1.5.1.6
-- Date Modified: 04/07/28 17:42:10
-- Module Description
--   Delete trigger for the Message_A table.
CREATE TRIGGER tD_Message_C ON Message_C FOR DELETE
AS
  UPDATE oe SET status = 1
  FROM OwnedEvent AS oe
    JOIN deleted AS d ON d.id = oe.eventid
  WHERE oe.eventcid = 'MESG'
    AND oe.status = 0

  DELETE ExtendedEvent_C FROM deleted WHERE ExtendedEvent_C.id=deleted.id AND ExtendedEvent_C.eventcid='MESG'

  UPDATE cmap SET mesg_id = 0
  FROM CurrentMessageAlertStatePri AS cmap
    JOIN deleted AS d ON d.id = cmap.mesg_id
GO

--------------------------------------------------------------------------------------------
-- tI_Message_C_Prop
IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tI_Message_C_Prop' AND type = 'TR')
  DROP TRIGGER tI_Message_C_Prop
GO

PRINT 'Creating trigger tI_Message_C_Prop'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\message_c.sql
-- SID: 1.5.1.6
-- Date Modified: 04/07/28 17:42:10
--
-- This trigger handles the ProcessingPattern field in the MessageDescription table
CREATE TRIGGER tI_Message_C_Prop ON Message_C FOR INSERT WITH APPEND
AS

  DECLARE @rc INT
  SET @rc = @@ROWCOUNT

  DECLARE @c_mesg_id ObjID, @c_cid ClassID, @c_id ObjID, @c_ReasonCD NVARCHAR(2000), @c_ProcessingPattern NVARCHAR(255), @c_SearchExpression NVARCHAR(255)
  DECLARE @TIME DATETIME, @SEQ_NO NVARCHAR(50), @STATE NVARCHAR(100), @MESSAGE NVARCHAR(255), @DISCOVERY_IND NVARCHAR(4), @AlertState INT, @Priority INT, @StateID INT
  DECLARE @cmd NVARCHAR(2000), @h INT, @b_match TINYINT, @err INT, @r INT
  -- Cursor over the ProcessingPattern entries
  DECLARE c_mesg INSENSITIVE CURSOR FOR
    SELECT m.id, m._ParentCID, m._ParentID,
      m._REASON_CD,
      CASE WHEN md.obj_cid IS NOT NULL THEN md.ProcessingPattern ELSE md_class.ProcessingPattern END,
      CASE WHEN md.obj_cid IS NOT NULL THEN md.SearchExpression ELSE md_class.SearchExpression END,
      m._TIME, m._SEQ_NO, m._STATE, m._MESSAGE, m._DISCOVERY_IND, m._AlertState, m._Priority, m._StateID
    FROM inserted AS m
      -- join to get the class-level definition
      LEFT OUTER JOIN MessageDescription_T AS md_class ON m._MESSAGE = md_class.MsgID AND m._ParentCID = md_class.obj_cid AND md_class.obj_id = 0
      -- join to get the instance-level definition
      LEFT OUTER JOIN MessageDescription_T AS md ON m._MESSAGE = md.MsgID AND m._ParentCID = md.obj_cid AND m._ParentID = md.obj_id
    WHERE (md.obj_cid IS NOT NULL OR md_class.obj_cid IS NOT NULL)
      -- Make sure the ProcessingPattern is not null or empty.  If it is, it does not need to be processed.
      AND CASE WHEN md.obj_cid IS NOT NULL THEN md.ProcessingPattern ELSE md_class.ProcessingPattern END IS NOT NULL
      AND CASE WHEN md.obj_cid IS NOT NULL THEN md.ProcessingPattern ELSE md_class.ProcessingPattern END <> ''
  OPEN c_mesg
  FETCH NEXT FROM c_mesg INTO @c_mesg_id, @c_cid, @c_id, @c_ReasonCD, @c_ProcessingPattern, @c_SearchExpression,
    @TIME, @SEQ_NO, @STATE, @MESSAGE, @DISCOVERY_IND, @AlertState, @Priority, @StateID
  WHILE @@FETCH_STATUS <> -1
  BEGIN

    -- If the SearchExpression field is not empty, check if it matches the _REASON_CD field
    SET @b_match = 1
    IF @c_SearchExpression IS NOT NULL AND @c_SearchExpression <> ''
    BEGIN
      SET @b_match = 0
      SET @h = NULL
      EXEC @r = master..xp_regexmatch @c_SearchExpression, @c_ReasonCD, @h OUTPUT
      SET @err = @@ERROR
      IF @err <> 0 OR @r <> 0
      BEGIN
        RAISERROR('tU_Message_C: ERROR calling xp_regexmatch ''%s'', ''%s'' @@ERROR=%d, return code = %d.', 0, 1, @c_SearchExpression, @c_ReasonCD, @err, @r) WITH NOWAIT
        ROLLBACK TRAN
        RETURN
      END

      -- if there was a match, set the variable and free the memory
      IF @h IS NOT NULL AND @h > 0
      BEGIN
        SET @b_match = 1
        EXEC master..xp_regexmatcharrayfree @h
      END
    END

    -- Only continue if the match was successful
    IF @b_match > 0
    BEGIN

      -- Make supported substitutions in the ProcessingPattern field
      SET @cmd = @c_ProcessingPattern
      IF CHARINDEX('@', @c_ProcessingPattern) > 0
      BEGIN
        SET @cmd = REPLACE(@cmd, '@dst_id', CAST(@c_mesg_id AS NVARCHAR(9)))
        SET @cmd = REPLACE(@cmd, '@src_cid', @c_cid)
        SET @cmd = REPLACE(@cmd, '@src_id', CAST(@c_id AS NVARCHAR(9)))
        SET @cmd = REPLACE(@cmd, '@TIME', CONVERT(NVARCHAR(100), @TIME, 121))
        SET @cmd = REPLACE(@cmd, '@DATE', CONVERT(NVARCHAR(100), @TIME, 121))
        SET @cmd = REPLACE(@cmd, '@MSEC', CAST(DATEPART(MS, @TIME) AS NVARCHAR(9)))
        SET @cmd = REPLACE(@cmd, '@SEQ_NO', @SEQ_NO)
        SET @cmd = REPLACE(@cmd, '@STATE', @STATE)
        SET @cmd = REPLACE(@cmd, '@MESSAGE', @MESSAGE)
        SET @cmd = REPLACE(@cmd, '@REASON_CD', @c_ReasonCD)
        SET @cmd = REPLACE(@cmd, '@DISCOVERY_IND', @DISCOVERY_IND)
        SET @cmd = REPLACE(@cmd, '@AlertState', CAST(@AlertState AS NVARCHAR(9)))
        SET @cmd = REPLACE(@cmd, '@Priority', CAST(@Priority AS NVARCHAR(9)))
        SET @cmd = REPLACE(@cmd, '@State', CAST(@StateID AS NVARCHAR(9)))
      END

      -- Insert the row
      INSERT INTO StagedCMD (cid, id, cmd)
      VALUES (@c_cid, @c_id, @cmd)

    END

    FETCH NEXT FROM c_mesg INTO @c_mesg_id, @c_cid, @c_id, @c_ReasonCD, @c_ProcessingPattern, @c_SearchExpression,
      @TIME, @SEQ_NO, @STATE, @MESSAGE, @DISCOVERY_IND, @AlertState, @Priority, @StateID
  END
  DEALLOCATE c_mesg

GO
