-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/note_a.sql
-- SID: 1.22
-- Date Modified: 05/01/18 13:27:46
--
-- Module Description
--   Defines the Note_A table and its triggers.
--
USE Object
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = N'Note_A' AND type = 'U')
BEGIN
  PRINT 'Creating table Note_A'
  
  CREATE TABLE Note_A
  (
    id                ObjID         NOT NULL  CONSTRAINT NOTE_A_PK                  PRIMARY KEY(id),
    _AlertState       NVARCHAR(60)  NULL      CONSTRAINT NOTE_A_AlertState_DF       DEFAULT ''
                                              CONSTRAINT NOTE_A_AlertState_CK       CHECK   (_AlertState='' or _AlertState like 'GENC/STCR-1/ALRS-%/T/Desc'),
    _Body             NTEXT         NULL      CONSTRAINT NOTE_A_Body_DF             DEFAULT '',
    _CreatorCID       NVARCHAR(4)   NULL      CONSTRAINT NOTE_A_CreatorCID_DF       DEFAULT '',
    _CreatorObjID     INT           NULL      CONSTRAINT NOTE_A_CreatorObjID_DF     DEFAULT 0,
    _Date             DATETIME      NULL      CONSTRAINT NOTE_A_Date_DF             DEFAULT CURRENT_TIMESTAMP,
    _DurationMinutes  INT           NULL      CONSTRAINT NOTE_A_DurationMinutes_DF  DEFAULT 0,
    _ExpirationTime   DATETIME      NULL      CONSTRAINT NOTE_A_ExpirationTime_DF   DEFAULT null,
    _Flags            INT           NULL      CONSTRAINT NOTE_A_Flags_DF            DEFAULT 1,
    _ProblemID        NVARCHAR(255) NULL      CONSTRAINT NOTE_A_ProblemID_DF        DEFAULT '',
    _Resolution       NTEXT         NULL      CONSTRAINT NOTE_A_Resolution_DF       DEFAULT '',
    _Status           NVARCHAR(255) NULL      CONSTRAINT NOTE_A_Status_DF           DEFAULT 'Open'
                                              CONSTRAINT NOTE_A_Status_CK           CHECK   (_Status in ('', 'Open', 'Closed', 'SystemClosed')),
    _Subject          NVARCHAR(255) NULL      CONSTRAINT NOTE_A_Subject_DF          DEFAULT '',
    _Time             DATETIME      NULL      CONSTRAINT NOTE_A_Time_DF             DEFAULT CURRENT_TIMESTAMP,
    _Type             NVARCHAR(255) NULL      CONSTRAINT NOTE_A_Type_DF             DEFAULT 'Problem'
                                              CONSTRAINT NOTE_A_Type_CK             CHECK   (_Type in ('Problem', 'Informational', 'Ownership')),
    _UserID           NVARCHAR(255) NULL      CONSTRAINT NOTE_A_UserID_DF           DEFAULT '',
    _Originator       NVARCHAR(255) NULL      CONSTRAINT NOTE_A_Originator_DF       DEFAULT ''
  )
  GRANT SELECT ON Note_A TO PUBLIC
END
GO

-- Add _Originator column
IF EXISTS (SELECT * FROM sysobjects AS so JOIN syscolumns AS sc ON so.id = sc.id 
   WHERE so.name = 'Note_A' AND sc.name = '_Originator')
BEGIN
  PRINT 'Column ''_Originator'' already exists in the Note_A table.'
END
ELSE
BEGIN
  PRINT 'Adding column ''_Originator'' to Note_A...'

  -- add the column
  ALTER TABLE Note_A ADD _Originator NVARCHAR(255) NULL CONSTRAINT NOTE_A_Originator_DF DEFAULT ""
  IF @@ERROR <> 0
    PRINT 'ERROR adding column ''_Originator'' to Note_A.'
  ELSE
    PRINT 'Sucessfully added column ''_Originator'' to Note_A.'
END
GO


---------------------------------------------------------------------------------------------------
-- create_tU_Note_A
PRINT 'Creating procedure create_tU_Note_A'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'create_tU_Note_A' AND type = 'P')
  DROP PROCEDURE create_tU_Note_A
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/note_a.sql
-- SID: 1.22
-- Date Modified: 05/01/18 13:27:46
-- Procedure for creating the tU_Note_A trigger. This trigger is dropped and readded during
-- database reset, so its definition is maintained in this procedure.
CREATE PROCEDURE create_tU_Note_A
AS
  EXEC("
-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/note_a.sql
-- SID: 1.22
-- Date Modified: 05/01/18 13:27:46
-- Custom TRIGGER ON Note_A.
-- *** IMPORTANT NOTE FOR DEVELOPERS ***
-- This trigger is created dynamically by the create_tU_Note_A stored procedure.
-- If you have any changes you want to make persistent you have to change the
-- procedure and rerun it, not this trigger.  The reason for using the procedure
-- is that asisp_resetdb drops this trigger (and a couple of others) prior to
-- resetting the database and then uses the procedure to recreate the trigger when
-- it done.  We could now disable the triggers temporarily rather than deleting
-- and recreating.
CREATE TRIGGER tU_Note_A
  ON Note_A FOR UPDATE
AS
  DECLARE
    @sql        NVARCHAR(2000),
    @cid        ClassID,
    @id         ObjID,
    @rc         INT,
    @UserID     NVARCHAR(255),
    @ctime      DATETIME,
    @ctime_str  NVARCHAR(40),
    @eventcid   ClassID,
    @eventid    ObjID,
    @closeCount INT

  SELECT @rc = @@ROWCOUNT
  --RAISERROR('tU_Note_A: @@ROWCOUNT=%d', 0, 1, @rc)

-- Notify TEC if an open note is transferred to a new user
 IF UPDATE(_UserID)
    BEGIN
      IF EXISTS
      (
        SELECT 1
        FROM SystemConfiguration WITH (NOLOCK)
        WHERE groupname = 'TEC_EVENT_OWNERSHIP'
          AND propertyname = 'NOTIFY_TEC_OF_TBSM_OWNERSHIP'
          AND UPPER(propertyvalue) = 'YES'
      )
      BEGIN
 
        INSERT INTO StagedTECStatus (noteid, Status, ContextUserID)
          SELECT i.id, 'ACK', ISNULL(i._UserID,'')
          FROM inserted AS i
          JOIN deleted  AS d ON d.id = i.id
          WHERE i._Type   = 'Ownership'
            AND i._Status = 'Open'
            AND d._Status = 'Open'
            AND EXISTS
            (
              SELECT 1  
              FROM OwnedEvent      AS oe WITH (NOLOCK)
              JOIN ExtendedEvent_A AS ex WITH (NOLOCK) ON ex.id = oe.eventid
                                                      AND ex.eventcid = oe.eventcid  -- TEC event
              WHERE oe.noteid = i.id AND oe.status = 0 )
      END
    END

  IF UPDATE(_Status)
  BEGIN
    -- ignore changes to Flags if Status is changing
    UPDATE n SET _Flags = d._Flags
    FROM Note_A n
    JOIN inserted i ON i.id = n.id
    JOIN deleted d ON d.id = i.id
    WHERE d._Flags <> i._Flags AND d._Status <> i._Status

    -- Check for ownership notes being closed
    UPDATE oe SET status = 3 /* closed */, closetime = CURRENT_TIMESTAMP
    FROM OwnedEvent AS oe
    JOIN inserted AS i ON i.id = oe.noteid
    JOIN deleted AS d ON d.id = i.id
    WHERE i._Type = 'Ownership'
      AND d._Status = 'Open'
      AND i._Status = 'Closed'

    SET @closeCount=@@ROWCOUNT

    -- check for events that have not been cleared
    IF EXISTS
    (
      SELECT 1
      FROM inserted AS i 
      JOIN deleted  AS d ON d.id = i.id
      WHERE d._Status = 'Open'
        AND i._Status = 'Closed'
        AND (d._Flags & 1) = 0
    )
    BEGIN
      DECLARE Note_A_Flag_Notes CURSOR FOR
        SELECT i.id, i._UserID
        FROM inserted AS i
        JOIN deleted  AS d ON d.id = i.id
        WHERE (d._Flags & 1) = 0
          AND d._Status = 'Open'
          AND i._Status = 'Closed'
      OPEN Note_A_Flag_Notes
      FETCH NEXT FROM Note_A_Flag_Notes INTO @id, @UserID
      WHILE @@FETCH_STATUS <> -1
      BEGIN
        SELECT @ctime = ctime FROM Note_C WHERE id = @id
        SELECT @ctime_str = CONVERT(NVARCHAR(30), @ctime, 101) + ' ' + CONVERT(NVARCHAR(8), @ctime, 108)

        DECLARE Note_A_Flag_Events INSENSITIVE CURSOR FOR
          SELECT eventcid, eventid FROM OwnedEvent WHERE noteid = @id
        OPEN Note_A_Flag_Events
        FETCH NEXT FROM Note_A_Flag_Events INTO @eventcid, @eventid
        WHILE @@FETCH_STATUS <> -1
        BEGIN
          IF (@eventcid = 'EXCP' AND EXISTS (SELECT 1 FROM Exception_V WHERE id = @eventid AND deleted = 0))
          BEGIN
            SELECT @sql = N'EXEC asisp_negate' + @eventcid + N' ' + CONVERT(NVARCHAR(9), @eventid) + N', @UserID, ''' + @ctime_str + N''''
            EXEC sp_executesql @sql, N'@UserID NVARCHAR(255)', @UserID = @UserID
          END
          ELSE IF @eventcid = 'MESG'
          BEGIN -- clear a message
            -- We may not want to simply call asisp_negateMESG, as the one we are trying to clear may
            -- no longer be the most relevant message on the resource.
            IF EXISTS (SELECT 1 FROM CurrentMessageAlertStatePri WHERE mesg_id = @eventid)
            BEGIN
              SET @sql = N'EXEC asisp_negate' + @eventcid + N' ' + CONVERT(NVARCHAR(9), @eventid) + N', @UserID, ''' + @ctime_str + N''''
              EXEC sp_executesql @sql, N'@UserID NVARCHAR(255)', @UserID = @UserID
            END
          END -- clear a message

          FETCH NEXT FROM Note_A_Flag_Events INTO @eventcid, @eventid
        END
        DEALLOCATE Note_A_Flag_Events
        FETCH NEXT FROM Note_A_Flag_Notes INTO @id, @UserID
      END
      DEALLOCATE Note_A_Flag_Notes
    END

    -- Check whether we need to notify TEC
    IF @closeCount > 0
    BEGIN
      IF EXISTS
      (
        SELECT 1
        FROM SystemConfiguration WITH (NOLOCK)
        WHERE groupname = 'TEC_EVENT_OWNERSHIP'
          AND propertyname = 'NOTIFY_TEC_OF_TBSM_OWNERSHIP'
          AND UPPER(propertyvalue) = 'YES'
      )
      BEGIN
       DECLARE @usergroup NVARCHAR(255), @ContextUserID NVARCHAR(255)
       EXEC asisp_getContextUserInfo @ContextUserID OUTPUT, @usergroup OUTPUT

        INSERT INTO StagedTECStatus (noteid, Status, ContextUserID)
          SELECT i.id, 'CLOSED', ISNULL(@ContextUserID,'')
          FROM inserted AS i
          JOIN deleted  AS d ON d.id = i.id
          WHERE i._Type   = 'Ownership'
            AND i._Status = 'Closed'
            AND d._Status = 'Open'
            AND EXISTS
            (
              SELECT 1  
              FROM OwnedEvent      AS oe WITH (NOLOCK)
              JOIN ExtendedEvent_A AS ex WITH (NOLOCK) ON ex.id = oe.eventid
                                                      AND ex.eventcid = oe.eventcid  -- TEC event
              WHERE oe.noteid = i.id
            )
      END
    END

    -- Check for ownership notes being reopened
    UPDATE oe SET status = 0 /* Open */, closetime = NULL
    FROM OwnedEvent AS oe
      JOIN inserted AS i ON i.id = oe.noteid
      JOIN deleted AS d ON d.id = i.id
    WHERE i._Type = 'Ownership'
      AND d._Status <> 'Open'
      AND i._Status = 'Open'
      AND oe.status > 1    -- we only update status if it is not 0(open) or 1(cleared)
    SET @rc = @@ROWCOUNT
    IF @rc = 0 -- no events are outstanding, so the note cannot be reopened
    BEGIN
      IF EXISTS
      (
         SELECT 1
         FROM
         inserted AS i
         JOIN deleted AS d ON d.id = i.id
         WHERE i._Type = 'Ownership'
           AND d._Status <> 'Open'
           AND i._Status = 'Open'
      )
      BEGIN
        RAISERROR('tU_Note_A: cannot reopen note(s) since all owned events have cleared', 11, 1)
        ROLLBACK TRANSACTION
        RETURN
      END
    END
  END

  -- Attribute update notification
  IF UPDATE(_Status)
  BEGIN
    DECLARE @oldStatus NVARCHAR(16), @newStatus NVARCHAR(16)
    IF @rc = 1
    BEGIN
      SELECT
        @id        = i.id,
        @oldStatus = d._Status,
        @newStatus = i._Status
      FROM inserted AS i
      JOIN deleted AS d ON d.id = i.id
      IF @oldStatus <> @newStatus
        EXEC asisp_notifyObjAttrUpdated NOTE, @id, @newStatus, 'Status'
    END
    ELSE -- multirow update
    BEGIN
      -- Lookup ids for attributes being updated so that asisp_notifyObjAttrUpdated does not have to
      DECLARE @StatusID INT
      EXEC @StatusID = asisp_stringtoid 'Status'
      DECLARE Note_A_updatec CURSOR FOR
        SELECT i.id, d._Status, i._Status
        FROM inserted AS i
        JOIN deleted AS d ON d.id = i.id
      OPEN Note_A_updatec
      FETCH NEXT FROM Note_A_updatec INTO @id, @oldStatus, @newStatus
      WHILE @@FETCH_STATUS <> -1
      BEGIN
        IF @oldStatus <> @newStatus
          EXEC asisp_notifyObjAttrUpdated NOTE, @id, @newStatus, 'Status', @StatusID
        FETCH NEXT FROM Note_A_updatec INTO @id, @oldStatus, @newStatus
      END
      DEALLOCATE Note_A_updatec
    END
  END

  IF UPDATE(_Body)
  BEGIN
    IF @rc = 1
    BEGIN
      SELECT
        @id = i.id
      FROM
        inserted AS i
      EXEC asisp_notifyObjAttrInvalidated NOTE, @id, 'Body'
    END
    ELSE -- multirow update
    BEGIN
      -- Lookup ids for attributes being updated so that asisp_notifyObjAttrUpdated does not have to
      DECLARE @BodyID INT
      EXEC @BodyID = asisp_stringtoid 'Body'
      DECLARE Note_A_updatec CURSOR FOR SELECT id FROM inserted
      OPEN Note_A_updatec
      FETCH NEXT FROM Note_A_updatec INTO @id
      WHILE @@FETCH_STATUS <> -1
      BEGIN
        EXEC asisp_notifyObjAttrInvalidated NOTE, @id, 'Body', @BodyID
        FETCH NEXT FROM Note_A_updatec INTO @id
      END
      DEALLOCATE Note_A_updatec
    END
  END

  -- Update problem id
  IF UPDATE(_ProblemID)
  BEGIN
    DECLARE @oldProblemID NVARCHAR(255), @newProblemID NVARCHAR(255)
    DECLARE @notehistory_id INT
    DECLARE @Executer NVARCHAR(255)
    DECLARE @userRole NVARCHAR(255)

    -- Get the executer and role from the user context
    EXEC asisp_getContextUserInfo @Executer OUTPUT, @userRole OUTPUT
    IF @Executer IS NULL
      SET @Executer = 'SYSTEM'

    IF @rc = 1
    BEGIN
      SELECT
        @id = i.id,
        @oldProblemID = d._ProblemID, @newProblemID = i._ProblemID
      FROM inserted AS i
      JOIN deleted AS d ON d.id = i.id

      IF @oldProblemID <> @newProblemID
      BEGIN
        -- update note history. The action name 'TicketAssociated' is the name from the 
        -- NoteHistoryActions enumeration
        INSERT INTO NoteHistory (noteid, Action, Executer)
          VALUES (@id, 'TicketAssociated', @Executer)

        SELECT @notehistory_id = MAX(ident) FROM NoteHistory

        INSERT INTO NoteHistoryDetail (NoteHistory_id, ProblemID)
           VALUES (@notehistory_id, @newProblemID)
      END
    END
    ELSE -- multirow update
    BEGIN
      DECLARE Note_A_updatec CURSOR FOR 
        SELECT i.id, d._ProblemID, i._ProblemID
        FROM inserted AS i
        JOIN deleted AS d ON d.id = i.id
      OPEN Note_A_updatec
      FETCH NEXT FROM Note_A_updatec INTO @id, @oldProblemID, @newProblemID
      WHILE @@FETCH_STATUS <> -1
      BEGIN
        IF @oldProblemID <> @newProblemID
        BEGIN
          -- update note history. The action name 'TicketAssociated' is the name from the 
          -- NoteHistoryActions enumeration
          INSERT INTO NoteHistory (noteid, Action, Executer)
            VALUES (@id, 'TicketAssociated', @Executer)

          SELECT @notehistory_id = SCOPE_IDENTITY()

          INSERT INTO NoteHistoryDetail (NoteHistory_id, ProblemID)
            VALUES (@notehistory_id, @newProblemID)
        END
        FETCH NEXT FROM Note_A_updatec INTO @id, @oldProblemID, @newProblemID
      END
      DEALLOCATE Note_A_updatec
    END
  END

   -- handle events when the 'clear events' flag is changed for a note
  IF UPDATE(_Flags)
  BEGIN
    DECLARE @_Flags INT
    DECLARE Note_A_Flag_Notes CURSOR FOR
      SELECT i.id, i._Flags, i._UserID
      FROM inserted AS i
      JOIN deleted AS d ON d.id = i.id
      WHERE (i._Flags & 1) <> (d._Flags & 1) AND (i._Status = 'Open')
    OPEN Note_A_Flag_Notes
    FETCH NEXT FROM Note_A_Flag_Notes INTO @id, @_Flags, @UserID
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      SELECT @ctime = ctime FROM Note_C WHERE id = @id
      SELECT @ctime_str = CONVERT(NVARCHAR(30), @ctime, 101) + ' ' + CONVERT(NVARCHAR(8), @ctime, 108)

      DECLARE Note_A_Flag_Events INSENSITIVE CURSOR FOR
        SELECT eventcid, eventid FROM OwnedEvent WHERE noteid = @id
      OPEN Note_A_Flag_Events
      FETCH NEXT FROM Note_A_Flag_Events INTO @eventcid, @eventid
      WHILE @@FETCH_STATUS <> -1
      BEGIN
        IF (@_Flags & 1) = 1
        BEGIN -- clear event
          IF @eventcid = 'EXCP' AND EXISTS (SELECT 1 FROM Exception_V WHERE id = @eventid AND deleted = 0)
          BEGIN
            SELECT @sql = N'EXEC asisp_negate' + @eventcid + N' ' + CONVERT(NVARCHAR(9), @eventid) + N', @UserID, ''' + @ctime_str + N''''
            EXEC sp_executesql @sql, N'@UserID NVARCHAR(255)', @UserID = @UserID
          END
          ELSE IF @eventcid = 'MESG'
          BEGIN -- clear a message
            -- We may not want to simply call asisp_negateMESG, as this creates another ownership note.
            -- If there isn't already an ownership note, it's appropriate to call asisp_negateMESG
            -- If we already have an ownership note, the proper solution is to reassert it.
            -- First, see if there is an ownership message.  This is easily found in the 
            -- CurrentMessageAlertStatePri table.
            DECLARE @ownership_mesg_id INT, @ownercid ClassID, @ownerid ObjID
            SET @ownership_mesg_id = NULL

            SELECT @ownercid = m._ParentCID, @ownerid = m._ParentID
            FROM Message_V AS m WITH (NOLOCK)
            WHERE m.id = @eventid

            SELECT @ownership_mesg_id = c.mesg_id
            FROM CurrentMessageAlertStatePri AS c
            WHERE c.cid = @ownercid AND c.id = @ownerid

            IF (@ownership_mesg_id = @eventid)
            BEGIN
              SELECT @sql = N'EXEC asisp_negate' + @eventcid + N' ' + CONVERT(NVARCHAR(9), @eventid) + N', @UserID, ''' + @ctime_str + N''''
              EXEC sp_executesql @sql, N'@UserID NVARCHAR(255)', @UserID = @UserID
            END
            ELSE
            BEGIN -- Reassert the ownership message
              --PRINT 'tU_Note_A: Re-asserting ownership message ' + CONVERT(NVARCHAR(9), @ownership_mesg_id)
              UPDATE c SET 
                AlertState = m._AlertState, 
                Priority   = m._Priority,
                mesg_id    = m.id            
              FROM CurrentMessageAlertStatePri AS c
                JOIN Message_V AS m ON m._ParentCID = c.cid AND m._ParentID = c.id
              WHERE m.id = @ownership_mesg_id                 
            END -- reassert the ownership message
          END -- clear a message
        END -- clear event
        ELSE
        BEGIN -- reassert event
          IF @eventcid = 'EXCP' AND EXISTS (SELECT * FROM Exception_V WHERE id = @eventid AND deleted = 1)
          BEGIN
            --PRINT 'tU_Note_A: reasserting EXCP ' + CAST(@eventid AS NVARCHAR)
            UPDATE Exception_V SET deleted = 0 WHERE id = @eventid
          END
          ELSE IF @eventcid = 'MESG'
          BEGIN
            --PRINT 'tU_Note_A: reasserting MESG ' + CAST(@eventid AS NVARCHAR)
            UPDATE c SET 
              AlertState = m._AlertState, 
              Priority   = m._Priority,
              mesg_id    = m.id            
            FROM CurrentMessageAlertStatePri AS c
              JOIN Message_V AS m ON m._ParentCID = c.cid AND m._ParentID = c.id
            WHERE m.id = @eventid             
          END -- reassert a message
        END -- else, assert an event
        FETCH NEXT FROM Note_A_Flag_Events INTO @eventcid, @eventid
      END -- process events owned by the event
      DEALLOCATE Note_A_Flag_Events
      FETCH NEXT FROM Note_A_Flag_Notes INTO @id, @_Flags, @UserID
    END
    DEALLOCATE Note_A_Flag_Notes
  END

  -- update the mtime column
  UPDATE n
  SET mtime = CURRENT_TIMESTAMP
  FROM Note_C AS n
    JOIN inserted AS i ON i.id = n.id
")
GO

---------------------------------------------------------------------------------------
-- 
--   Drops and then dynamically creates trigger tU_Note_A
--
---------------------------------------------------------------------------------------

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_Note_A' AND type = 'TR') 
BEGIN
  PRINT 'Dropping Trigger tU_Note_A ...'
  DROP TRIGGER tU_Note_A
END

PRINT 'Calling create_tU_Note_A procedure to create trigger tU_Note_A'
EXEC create_tU_Note_A
