-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/objalertcount.sql
-- SID: 1.31
-- Date Modified: 04/12/15 17:28:45
--
-- Module Description
--   Defines the ObjAlertCount table
USE Object
GO

---------------------------------------------------------------------------------------------------
-- ObjAlertCount

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'ObjAlertCount' AND type = 'U')
BEGIN
  PRINT 'Creating table ObjAlertCount'

  CREATE TABLE ObjAlertCount
  (
    cid         ClassID  NOT NULL,
    id          ObjID    NOT NULL,
    event_cid   ClassID  NOT NULL CONSTRAINT ObjAlertCount_event_cid_CK  CHECK (event_cid IN ('MESG', 'EXCP', 'CHEV')),
    alertstate  INT      NOT NULL CONSTRAINT ObjAlertCount_alertstate_CK CHECK (alertstate IN (2, 3)),
    priority    INT      NOT NULL CONSTRAINT ObjAlertCount_priority_CK   CHECK (priority IN (1, 2, 3, 4)),
    nviolations INT      NOT NULL DEFAULT 0,
    mtime       DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP
  ) ON [PRIMARY];

  ALTER TABLE ObjAlertCount ADD CONSTRAINT ObjAlertCount_PK PRIMARY KEY (cid, id, event_cid, alertstate, priority)
  CREATE INDEX iObjAlertCount_cid_id ON ObjAlertCount (cid, id)

  --
  -- Migrate the data
  --
  EXEC asisp_populateObjAlertCount
END

GO

--
-- Triggers
--

------------------------------------------------------------------------------------
-- Trigger tI_ObjAlertCount

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tI_ObjAlertCount' AND type = 'TR')
  DROP TRIGGER tI_ObjAlertCount
GO

PRINT 'Creating trigger tI_ObjAlertCount'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/objalertcount.sql
-- SID: 1.31
-- Date Modified: 04/12/15 17:28:45
CREATE TRIGGER tI_ObjAlertCount ON ObjAlertCount FOR INSERT WITH APPEND
AS
  DECLARE @ashist TABLE
  (
    cid             NVARCHAR(4)   NOT NULL,
    id              INT           NOT NULL,
    AlertStateID    TINYINT       NOT NULL,
    evt_cid         NVARCHAR(4)   NOT NULL,
    evt_id          INT           NOT NULL,
    evt_priority    TINYINT       NOT NULL,
    evt_ctime       DATETIME      NOT NULL,
    evt_detail      NVARCHAR(255) NOT NULL,
    OldAlertStateID TINYINT       NULL DEFAULT (NULL),
    evt_name        NVARCHAR(255) NULL DEFAULT (''),
    evt_alertstate  SMALLINT      NOT NULL,
    phy_cid         NVARCHAR(4)   NULL,
    phy_id          INT           NULL,
    exch_id         INT           NULL
  )

  DECLARE
    @c_cid    ClassID,
    @c_cname  NVARCHAR(24),
    @c_fc     INT,
    @now      NVARCHAR(40),
    @rc       INT,
    @rc_total INT,
    @sql      NVARCHAR(4000),
    @verbose  TINYINT

  SET @rc = @@ROWCOUNT
  IF @rc = 0
    RETURN

  -- Verbose setting
  SET @verbose = 0

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Processing %d inserts.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- init work tables
  DELETE FROM ObjAlertCountWT    WITH (TABLOCKX)
  DELETE FROM ObjAlertCountDelta WITH (TABLOCKX)
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Cleared ObjAlertCountWT and ObjAlertCountDelta.', 0, 1, @now) WITH NOWAIT
  END -- @verbose >= 1

  -- Put the inserted records into the ObjAlertCountWT table
  INSERT INTO ObjAlertCountWT WITH (HOLDLOCK) (cid, id, event_cid, alertstate, priority, nviolations)
  SELECT cid, id, event_cid, alertstate, priority, nviolations
  FROM inserted
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Inserted %d row(s) into the ObjAlertCountWT table.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Get the changes where nviolations went above 0
  INSERT INTO ObjAlertCountDelta WITH (HOLDLOCK) (cid, id, alertstate)
  SELECT DISTINCT i.cid, i.id, 0
  FROM ObjAlertCountWT AS i
  WHERE i.nviolations > 0
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Inserted %d row(s) into the ObjAlertCountDelta table.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Update the alertstate column of the delta table
  UPDATE d
  SET d.alertstate = ISNULL((SELECT MAX(alertstate) FROM ObjAlertCount AS oac WHERE oac.cid = d.cid AND oac.id = d.id AND oac.nviolations > 0), 1)
  FROM ObjAlertCountDelta AS d
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Updated the alertstate column in ObjAlertCountDelta for %d row(s).', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Update the old_alertstate column of the delta table
  UPDATE d SET 
    old_alertstate = ISNULL((
      SELECT MAX(oac.alertstate)
      FROM ObjAlertCount AS oac
        LEFT OUTER JOIN ObjAlertCountWT AS i ON i.cid = oac.cid AND i.id = oac.id
          AND i.event_cid = oac.event_cid AND i.alertstate = oac.alertstate AND i.priority = oac.priority
          AND i.trigger_action = 0
      WHERE i.cid IS NULL
        AND oac.cid = d.cid AND oac.id = d.id AND oac.nviolations > 0
    ), 1)
  FROM ObjAlertCountDelta AS d
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Updated the old_alertstate column in ObjAlertCountDelta for %d row(s).', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Determine which resources are in maintenance mode
  UPDATE d SET in_maint = 1
  FROM ObjAlertCountDelta AS d
    JOIN MaintTrueState AS m ON m.cid = d.cid AND m.id = d.id
  WHERE m.enabled = 1
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Updated %d ObjAlertCountDelta row(s) where the resource is in maintenance mode.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Update the AlertStateID columns of objects whose records have just been inserted
  DECLARE c_oac INSENSITIVE CURSOR FOR
    SELECT DISTINCT RTRIM(oc.cid), oc.cname, oc.flag_consolidated
    FROM obj_class AS oc
      JOIN ObjAlertCountDelta AS d ON d.cid = oc.cid
    WHERE d.in_maint = 0
      AND d.alertstate <> d.old_alertstate
  OPEN c_oac
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Opened cursor c_oac.', 0, 1, @now) WITH NOWAIT
  END -- @verbose >= 1
  SET @rc_total = 0
  FETCH NEXT FROM c_oac INTO @c_cid, @c_cname, @c_fc
  WHILE @@FETCH_STATUS <> -1
  BEGIN

    -- Update the AlertState
    SET @sql =
      'UPDATE o SET _AlertStateID = d.alertstate ' +
      'FROM ' + @c_cname + CASE @c_fc WHEN 0 THEN '_A' ELSE '_C' END + ' AS o ' +
        'JOIN ObjAlertCountDelta AS d ON d.cid = ''' + @c_cid + ''' AND d.id = o.id '+
      'WHERE d.in_maint = 0 ' +
        'AND d.alertstate <> o._AlertStateID '
    EXEC(@sql)
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tI_ObjAlertCount: Updated the AlertState for %d objects of type %s.', 0, 1, @now, @rc, @c_cid) WITH NOWAIT
    END -- @verbose >= 1

    SET @rc_total = @rc + @rc_total

    FETCH NEXT FROM c_oac INTO @c_cid, @c_cname, @c_fc
  END
  DEALLOCATE c_oac
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Updated the AlertState for %d row(s).', 0, 1, @now, @rc_total) WITH NOWAIT
  END -- @verbose >= 1

  -- Send notifications
  INSERT INTO PendingNotification (type, fields, detail)
  SELECT et.type,
    CAST(oc.cno AS NVARCHAR(9)) + '|' +
    CAST(d.id AS NVARCHAR(9))   + '|' +
    CAST(map.id AS NVARCHAR(9)),
    CAST(d.alertstate AS NVARCHAR(9))
  FROM EventType AS et,
    id_string_map AS map,
    obj_class AS oc
    JOIN ObjAlertCountDelta AS d ON d.cid = oc.cid
  WHERE et.name = 'ObjAttrUpdated'
    AND map.string = 'AlertStateID'
    AND d.in_maint = 0
    AND d.alertstate <> d.old_alertstate
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Inserted %d PendingNotification row(s) for AlertState updates.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Update the AlertState column in the MaintTrueState table
  UPDATE m SET AlertState = d.alertstate
  FROM MaintTrueState AS m
    JOIN ObjAlertCountDelta AS d ON d.cid = m.cid AND d.id = m.id
  WHERE m.enabled = 1 AND d.in_maint = 1
    AND m.AlertState <> d.alertstate
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Updated the AlertState for %d row(s) in MaintTrueState.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  --
  -- AlertStateHistory inserts
  --

  -- Insert into the AlertStateHistory table for Messages
  INSERT INTO @ashist (cid, id, AlertStateID, OldAlertStateID, evt_cid, evt_id, evt_priority, evt_alertstate, evt_ctime, evt_detail, evt_name)
  SELECT DISTINCT d.cid, d.id, d.alertstate, d.old_alertstate, i.event_cid, c.mesg_id, i.priority, c.AlertState, m._TIME, m._REASON_CD, m._MESSAGE
  FROM ObjAlertCountDelta AS d
    JOIN ObjAlertCountWT AS i ON i.cid = d.cid AND i.id = d.id
    JOIN CurrentMessageAlertStatePri AS c ON c.cid = i.cid AND c.id = i.id AND i.event_cid = 'MESG'
    JOIN Message_V AS m WITH (NOLOCK) ON m.id = c.mesg_id
  WHERE i.nviolations > 0 AND i.trigger_action = 0
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Computed %d row(s) of inserts into AlertStateHistory for Messages', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Insert into the AlertStateHistory table for Exceptions
  INSERT INTO @ashist (cid, id, AlertStateID, OldAlertStateID, evt_cid, evt_id, evt_priority, evt_alertstate, evt_ctime, evt_detail)
  SELECT DISTINCT d.cid, d.id, d.alertstate, d.old_alertstate, i.event_cid, 0, i.priority, i.alertstate, CURRENT_TIMESTAMP, ''    
  FROM ObjAlertCountDelta AS d
    JOIN ObjAlertCountWT AS i ON i.cid = d.cid AND i.id = d.id
  WHERE i.nviolations > 0
    AND i.event_cid = 'EXCP'
    AND i.trigger_action = 0
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Computed %d row(s) of inserts into AlertStateHistory for Exceptions', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1
  
  -- Fill in exception information
  IF @rc > 0
  BEGIN
    -- Lookup last exception ids for objects that own the exceptions
    UPDATE i SET evt_id = (
      SELECT ISNULL(MAX(c.id),0)
      FROM Exception_V AS c 
      WHERE c._ParentCID  = i.cid
        AND c._ParentID   = i.id 
        AND c._Priority   = i.evt_priority
        AND c._AlertState = i.evt_alertstate
        AND c.deleted = 0
    )
    FROM @ashist AS i
    WHERE i.evt_cid = 'EXCP'
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tI_ObjAlertCount: Updated exception id %d for row(s) of AlertStateHistory inserts', 0, 1, @now, @rc) WITH NOWAIT
    END -- @verbose >= 1
  
    -- If we still have unresolved exception ids, then lookup LOBs linked to objects with non-LOB only exceptions
    IF EXISTS (SELECT 1 FROM @ashist WHERE evt_id = 0)
    BEGIN
      -- It has proven to be much faster to add the phy_cid and phy_id columns to the work table rather
      -- than do the join to lob_link in the subsequent queries.
      UPDATE a SET phy_cid = l.phy_cid, phy_id = l.phy_id
      FROM @ashist AS a
      JOIN lob_link AS l ON l.dst_id = a.id
      WHERE a.cid = 'LOB'
        AND l.src_id = 0
      SET @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: tI_ObjAlertCount: Looked up resource cid/id for %d LOB AlertStateHistory insert(s)', 0, 1, @now, @rc) WITH NOWAIT
      END -- @verbose >= 1

      UPDATE i SET evt_id = (
        SELECT ISNULL(MAX(c.id),0)
        FROM Exception_V AS c 
        WHERE c._ParentCID  = i.phy_cid
          AND c._ParentID   = i.phy_id 
          AND c._Priority   = i.evt_priority
          AND c._AlertState = i.evt_alertstate
          AND c.deleted     = 0
          AND c._flags & 16 = 0
      )
      FROM @ashist AS i
      WHERE i.cid     = 'LOB'
        AND i.evt_cid = 'EXCP'
        AND i.evt_id  = 0
      SET @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: tI_ObjAlertCount: Updated exception id %d for linked LOB row(s) of AlertStateHistory inserts', 0, 1, @now, @rc) WITH NOWAIT
      END -- @verbose >= 1
    END -- If we still have unresolved exception ids, then lookup LOBs linked to objects with non-LOB only exceptions

    -- Lookup latest exception history id
    UPDATE a SET exch_id = (
      SELECT MAX(h.id)
      FROM ExceptionHistory_V AS h
      WHERE h._ParentID = a.evt_id
    )
    FROM @ashist AS a
    WHERE a.evt_cid = 'EXCP'
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tI_ObjAlertCount: Updated exception history id for %d row(s) of AlertStateHistory inserts', 0, 1, @now, @rc) WITH NOWAIT
    END -- @verbose >= 1

    -- Lookup exception details
    UPDATE a SET
      evt_ctime  = h._TIME,
      evt_detail = h._EXCP_CD,
      evt_name   = e._EXCP_NAME
    FROM @ashist AS a
    JOIN Exception_V AS e WITH (NOLOCK) ON e.id = a.evt_id
    JOIN ExceptionHistory_V AS h WITH (NOLOCK) ON h.id = a.exch_id
    WHERE a.evt_cid = 'EXCP'
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tI_ObjAlertCount: Looked up exception details for %d row(s) of AlertStateHistory inserts', 0, 1, @now, @rc) WITH NOWAIT
    END -- @verbose >= 1
  END -- Fill in exception information

  -- Insert into the AlertStateHistory table for ChildEvents
  INSERT INTO @ashist (cid, id, AlertStateID, OldAlertStateID, evt_cid, evt_id, evt_priority, evt_alertstate, evt_ctime, evt_detail)
  SELECT DISTINCT d.cid, d.id, d.alertstate, d.old_alertstate, i.event_cid,
    ISNULL((
      SELECT MAX(c.id)
      FROM ChildEvent_C AS c 
      WHERE c._ParentCID = i.cid AND c._ParentID = i.id AND c._AlertState = i.alertstate AND c._Priority = i.priority
        AND c.deleted = 0
    ), 0) AS 'evt_id',
    i.priority, i.alertstate, CURRENT_TIMESTAMP, ''
  FROM ObjAlertCountDelta AS d
    JOIN ObjAlertCountWT AS i ON i.cid = d.cid AND i.id = d.id
  WHERE i.nviolations > 0
    AND i.event_cid = 'CHEV' AND i.trigger_action = 0
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Computed %d row(s) of inserts into AlertStateHistory for ChildEvents', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Lookup ChildEvent details
  IF @rc > 0
  BEGIN
    UPDATE a SET
      evt_ctime  = CASE c.deleted WHEN 0 THEN c.ctime ELSE c.mtime END,
      evt_detail = ISNULL(CASE c.deleted WHEN 0 THEN c._EventDetail ELSE c._DeleteDetail END,'')
    FROM @ashist AS a
    JOIN ChildEvent_V AS c WITH (NOLOCK) ON c.id = a.evt_id
    WHERE a.evt_cid = 'CHEV'
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tI_ObjAlertCount: Looked up child event details for %d row(s) of AlertStateHistory inserts', 0, 1, @now, @rc) WITH NOWAIT
    END -- @verbose >= 1    
  END -- Lookup ChildEvent details
    
  INSERT INTO AlertStateHistory(cid, id, AlertStateID, OldAlertStateID, evt_cid, evt_id, evt_priority, evt_alertstate, evt_ctime, evt_detail, evt_name)
  SELECT cid, id, AlertStateID, OldAlertStateID, evt_cid, evt_id, evt_priority, evt_alertstate, evt_ctime, evt_detail, evt_name
  FROM @ashist
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Inserted %d total row(s) into AlertStateHistory', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Insert into the RootCauseState table
  INSERT INTO RootCauseState (cid, id, alertstate, mtime)
  SELECT d.cid, d.id, d.alertstate, CURRENT_TIMESTAMP
  FROM ObjAlertCountDelta AS d
    LEFT OUTER JOIN RootCauseState AS r ON r.cid = d.cid AND r.id = d.id
  WHERE r.cid IS NULL
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Inserted %d entries into the RootCauseState table.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1
  
  -- Update the RootCauseState table
  UPDATE r
  SET alertstate = d.alertstate, mtime = CURRENT_TIMESTAMP
  FROM ObjAlertCountDelta AS d
    JOIN RootCauseState AS r ON r.cid = d.cid AND r.id = d.id
  WHERE d.old_alertstate <> d.alertstate
    AND r.alertstate <> d.alertstate
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Updated %d entries in the RootCauseState table.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Cleanup work tables
  TRUNCATE TABLE ObjAlertCountWT
  TRUNCATE TABLE ObjAlertCountDelta
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tI_ObjAlertCount: Truncated ObjAlertCountWT and ObjAlertCountDelta.', 0, 1, @now) WITH NOWAIT
  END -- @verbose >= 1

GO

------------------------------------------------------------------------------------
-- Trigger tU_ObjAlertCount

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_ObjAlertCount' AND type = 'TR')
  DROP TRIGGER tU_ObjAlertCount
GO

PRINT 'Creating trigger tU_ObjAlertCount'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 2003,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/objalertcount.sql
-- SID: 1.31
-- Date Modified: 04/12/15 17:28:45
CREATE TRIGGER tU_ObjAlertCount ON ObjAlertCount FOR UPDATE WITH APPEND
AS
  DECLARE @ashist TABLE
  (
    cid             NVARCHAR(4)   NOT NULL,
    id              INT           NOT NULL,
    AlertStateID    TINYINT       NOT NULL,
    evt_cid         NVARCHAR(4)   NOT NULL,
    evt_id          INT           NOT NULL,
    evt_priority    TINYINT       NOT NULL,
    evt_ctime       DATETIME      NOT NULL,
    evt_detail      NVARCHAR(255) NOT NULL,
    OldAlertStateID TINYINT       NULL DEFAULT (NULL),
    evt_name        NVARCHAR(255) NULL DEFAULT (''),
    evt_alertstate  SMALLINT      NOT NULL,
    phy_cid         NVARCHAR(4)   NULL,
    phy_id          INT           NULL,
    exch_id         INT           NULL
  )

  DECLARE
    @c_cid    ClassID,
    @c_cname  NVARCHAR(24),
    @c_fc     INT,
    @now      NVARCHAR(40),
    @rc       INT,
    @rc_temp  INT,
    @rc_total INT,
    @sql      NVARCHAR(4000),
    @verbose  TINYINT

  SET @rc = @@ROWCOUNT
  IF @rc = 0
    RETURN

  -- set up the @verbose option
  SET @verbose = 0

  -- Verify that only the nviolations and mtime columns are being updated
  IF UPDATE(cid) OR UPDATE(id) OR UPDATE(event_cid) OR UPDATE(alertstate) OR UPDATE(priority)
  BEGIN
    RAISERROR('tU_ObjAlertCount: ERROR: Attempt to update non-updatable field.', 11, 1) WITH NOWAIT
    ROLLBACK TRANSACTION
    RETURN
  END

  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Starting. ROWCOUNT=%d', 0, 1, @now, @rc) WITH NOWAIT
    IF @verbose >= 2
    BEGIN
      SELECT 'deleted ' AS 'deleted',  * FROM deleted
      SELECT 'inserted' AS 'inserted', * FROM inserted
    END
  END -- @verbose >= 1

  -- init work tables
  DELETE FROM ObjAlertCountWT    WITH (TABLOCKX)
  DELETE FROM ObjAlertCountDelta WITH (TABLOCKX)
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Cleared ObjAlertCountWT and ObjAlertCountDelta.', 0, 1, @now) WITH NOWAIT
  END -- @verbose >= 1

  -- Put the inserted entries into the ObjAlertCountWT table
  INSERT INTO ObjAlertCountWT WITH (HOLDLOCK) (cid, id, event_cid, alertstate, priority, nviolations, trigger_action)
  SELECT cid, id, event_cid, alertstate, priority, nviolations, 0
  FROM inserted
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Inserted %d row(s) from inserted into the ObjAlertCountWT table.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1
  
  -- Put the deleted entries into the ObjAlertCountWT table
  INSERT INTO ObjAlertCountWT WITH (HOLDLOCK) (cid, id, event_cid, alertstate, priority, nviolations, trigger_action)
  SELECT cid, id, event_cid, alertstate, priority, nviolations, 1
  FROM deleted
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Inserted %d row(s) from deleted into the ObjAlertCountWT table.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Get the changes where nviolations went above 0 or to 0
  INSERT INTO ObjAlertCountDelta WITH (HOLDLOCK) (cid, id, alertstate)
  SELECT DISTINCT i.cid, i.id, 0
  FROM ObjAlertCountWT AS i
    JOIN ObjAlertCountWT AS d ON i.cid = d.cid AND i.id = d.id
      AND i.event_cid = d.event_cid AND i.alertstate = d.alertstate AND i.priority = d.priority
      AND i.trigger_action = 0 AND d.trigger_action = 1
  WHERE (i.nviolations > 0 AND d.nviolations = 0 OR d.nviolations > 0 AND i.nviolations = 0)
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Inserted %d row(s) into the ObjAlertCountDelta table.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Update the alertstate column of the delta table
  UPDATE d SET 
    alertstate = ISNULL((
      SELECT MAX(alertstate)
      FROM ObjAlertCount AS oac
      WHERE oac.cid = d.cid AND oac.id = d.id AND oac.nviolations > 0
    ), 1)
  FROM ObjAlertCountDelta AS d
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Updated the alertstate column in ObjAlertCountDelta for %d row(s).', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Update the old_alertstate column
  UPDATE d SET
    old_alertstate = ISNULL((
      SELECT MAX(del.alertstate)
      FROM ObjAlertCountWT AS del
      WHERE del.nviolations > 0
        AND del.trigger_action = 1
        AND del.cid = d.cid
        AND del.id = d.id
    ), 1)
  FROM ObjAlertCountDelta AS d
  SET @rc = @@ROWCOUNT

  -- Get the alertstate from the ObjAlertCount table
  UPDATE d SET 
    old_alertstate_temp = ISNULL((
      SELECT MAX(oac.alertstate)
      FROM ObjAlertCount AS oac
        LEFT OUTER JOIN ObjAlertCountWT AS i ON i.cid = oac.cid
                                            AND i.id = oac.id
                                            AND i.event_cid = oac.event_cid
                                            AND i.alertstate = oac.alertstate
                                            AND i.priority = oac.priority
                                            AND i.trigger_action = 0
      WHERE i.cid IS NULL
        AND oac.nviolations > 0
        AND oac.cid = d.cid
        AND oac.id = d.id
    ), 1)
  FROM ObjAlertCountDelta AS d
  SET @rc_temp = @@ROWCOUNT
  SET @rc = @rc + @rc_temp
  -- Choose the higher of the two
  UPDATE d SET
    old_alertstate = CASE WHEN d.old_alertstate_temp > d.old_alertstate THEN d.old_alertstate_temp 
                          ELSE d.old_alertstate
                     END
  FROM ObjAlertCountDelta AS d
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Updated the old_alertstate column in ObjAlertCountDelta for %d row(s).', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Determine which resources are in maintenance mode
  UPDATE d SET in_maint = 1
  FROM ObjAlertCountDelta AS d
    JOIN MaintTrueState AS m ON m.cid = d.cid AND m.id = d.id
  WHERE m.enabled = 1
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Updated %d ObjAlertCountDelta row(s) where the resource is in maintenance mode.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Update the AlertStateID columns of objects whose records have just been inserted
  DECLARE c_oac INSENSITIVE CURSOR FOR
    SELECT DISTINCT RTRIM(oc.cid), oc.cname, oc.flag_consolidated
    FROM obj_class AS oc
      JOIN ObjAlertCountDelta AS d ON d.cid = oc.cid
    WHERE d.in_maint = 0
      AND d.alertstate <> d.old_alertstate
  OPEN c_oac
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Opened cursor c_oac.', 0, 1, @now) WITH NOWAIT
  END -- @verbose >= 1
  SET @rc_total = 0 
  FETCH NEXT FROM c_oac INTO @c_cid, @c_cname, @c_fc
  WHILE @@FETCH_STATUS <> -1
  BEGIN

    -- Perform the update
    SET @sql = 
      'UPDATE o ' +
      'SET _AlertStateID = d.alertstate ' +
      'FROM ' + @c_cname + CASE @c_fc WHEN 0 THEN '_A' ELSE '_C' END + ' AS o ' +
        'JOIN ObjAlertCountDelta AS d ON d.cid = ''' + @c_cid + ''' AND d.id = o.id ' +
      'WHERE d.in_maint = 0 ' +
        'AND d.alertstate <> o._AlertStateID '
    EXEC(@sql)
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tU_ObjAlertCount: Updated the AlertState for %d objects of type %s.', 0, 1, @now, @rc, @c_cid) WITH NOWAIT
    END -- @verbose >= 1

    SET @rc_total = @rc + @rc_total

    -- Send notifications
    SET @sql =
    'INSERT INTO PendingNotification (type, fields, detail) ' +
    'SELECT et.type, ' +
      'CAST(oc.cno AS NVARCHAR(9)) + ''|'' + ' +
      'CAST(d.id AS NVARCHAR(9))   + ''|'' + ' +
      'CAST(map.id AS NVARCHAR(9)), ' +
      'CAST(o._AlertStateID AS NVARCHAR(9)) ' +
    'FROM EventType AS et WITH (NOLOCK), ' +
      'id_string_map AS map WITH (NOLOCK), ' +
      'Meta..obj_class_table_BASE AS oc WITH (NOLOCK) ' +
      'JOIN ObjAlertCountDelta AS d ON d.cid = oc.cid ' +
      'JOIN ' + @c_cname + '_V AS o ON o.id = d.id AND d.cid = ''' + @c_cid + ''' ' +
    'WHERE et.name = ''ObjAttrUpdated'' ' +
      'AND map.string = ''AlertStateID'' ' +
      'AND d.in_maint = 0 ' +
      'AND d.alertstate <> d.old_alertstate '
    EXEC(@sql)
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tU_ObjAlertCount: Inserted %d PendingNotification row(s) for objects of type %s.', 0, 1, @now, @rc, @c_cid) WITH NOWAIT
    END -- @verbose >= 1

    FETCH NEXT FROM c_oac INTO @c_cid, @c_cname, @c_fc
  END
  DEALLOCATE c_oac
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Updated the AlertState for %d objects.', 0, 1, @now, @rc) WITH NOWAIT
  END
  
  -- Update the AlertState column in the MaintTrueState table
  UPDATE m SET AlertState = d.alertstate
  FROM MaintTrueState AS m
    JOIN ObjAlertCountDelta AS d ON d.cid = m.cid AND d.id = m.id
  WHERE m.enabled = 1 AND d.in_maint = 1
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Updated the AlertState for %d row(s) in MaintTrueState.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  --
  -- AlertStateHistory inserts
  --

  -- Insert into the AlertStateHistory table for Messages
  INSERT INTO @ashist (cid, id, AlertStateID, OldAlertStateID, evt_cid, evt_id, evt_priority, evt_alertstate, evt_ctime, evt_detail, evt_name)
  SELECT DISTINCT delta.cid, delta.id, delta.alertstate, delta.old_alertstate, i.event_cid, c.mesg_id, i.priority, c.AlertState, m._TIME, m._REASON_CD, m._MESSAGE
  FROM ObjAlertCountDelta AS delta
    JOIN ObjAlertCountWT AS i ON i.cid = delta.cid AND i.id = delta.id
    JOIN ObjAlertCountWT AS d ON d.cid = i.cid AND d.id = i.id
      AND i.event_cid = d.event_cid AND i.alertstate = d.alertstate AND i.priority = d.priority
    JOIN CurrentMessageAlertStatePri AS c ON c.cid = i.cid AND c.id = i.id AND i.event_cid = 'MESG'
    JOIN Message_V AS m WITH (NOLOCK) ON m.id = c.mesg_id
  WHERE ((i.nviolations > 0 AND d.nviolations = 0) OR (d.nviolations > 0 AND i.nviolations = 0))
    AND i.trigger_action = 0
    AND d.trigger_action = 1
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Computed %d row(s) of inserts into AlertStateHistory for Messages', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Insert into the AlertStateHistory table for Exceptions
  INSERT INTO @ashist (cid, id, AlertStateID, OldAlertStateID, evt_cid, evt_id, evt_priority, evt_alertstate, evt_ctime, evt_detail)
  SELECT DISTINCT delta.cid, delta.id, delta.alertstate, delta.old_alertstate, i.event_cid, 0, i.priority,
    CASE WHEN d.nviolations > i.nviolations THEN -1*i.alertstate ELSE i.alertstate END,
    CURRENT_TIMESTAMP, 
    ''
  FROM ObjAlertCountDelta AS delta
    JOIN ObjAlertCountWT AS i ON i.cid = delta.cid AND i.id = delta.id
    JOIN ObjAlertCountWT AS d ON d.cid = i.cid AND d.id = i.id
                             AND i.event_cid  = d.event_cid
                             AND i.alertstate = d.alertstate
                             AND i.priority   = d.priority
  WHERE ((i.nviolations > 0 AND d.nviolations = 0) OR (d.nviolations > 0 AND i.nviolations = 0))
    AND i.trigger_action = 0
    AND i.event_cid = 'EXCP'
    AND d.trigger_action = 1
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Computed %d row(s) of inserts into AlertStateHistory for Exceptions', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1
  
  -- Fill in exception information
  IF @rc > 0
  BEGIN
    -- Lookup last exception ids for objects that own the exceptions
  
    -- For new exceptions
    UPDATE i SET evt_id = (
      SELECT ISNULL(MAX(c.id),0)
      FROM Exception_V AS c 
      WHERE c._ParentCID  = i.cid
        AND c._ParentID   = i.id 
        AND c._Priority   = i.evt_priority
        AND c._AlertState = i.evt_alertstate
        AND c.deleted = 0
    )
    FROM @ashist AS i
    WHERE i.evt_alertstate > 1
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tU_ObjAlertCount: Updated exception id for %d row(s) of AlertStateHistory inserts', 0, 1, @now, @rc) WITH NOWAIT
    END -- @verbose >= 1
  
    -- For cleared exceptions
    UPDATE i SET evt_id = (
      SELECT ISNULL(MAX(c.id),0)
      FROM Exception_V AS c 
      WHERE c._ParentCID  = i.cid
        AND c._ParentID   = i.id 
        AND c._Priority   = i.evt_priority
        AND c._AlertState = (-1*i.evt_alertstate)
        AND c.deleted = 1
    )
    FROM @ashist AS i
    WHERE i.evt_alertstate < 0
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tU_ObjAlertCount: Updated cleared exception id for %d row(s) of AlertStateHistory inserts', 0, 1, @now, @rc) WITH NOWAIT
    END -- @verbose >= 1

    -- If we still have unresolved exception ids, then lookup LOBs linked to objects with non-LOB only exceptions
    IF EXISTS (SELECT 1 FROM @ashist WHERE cid = 'LOB' AND evt_id = 0)
    BEGIN
      -- It has proven to be much faster to add the phy_cid and phy_id columns to the work table rather
      -- than do the join to lob_link in the subsequent queries.
      UPDATE a SET phy_cid = l.phy_cid, phy_id = l.phy_id
      FROM @ashist AS a
      JOIN lob_link AS l ON l.dst_id = a.id
      WHERE a.cid = 'LOB'
        AND l.src_id = 0
      SET @rc = @@ROWCOUNT
      IF @verbose >= 1
      BEGIN
        SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
        RAISERROR('%s: tU_ObjAlertCount: Looked up resource cid/id for %d LOB AlertStateHistory insert(s)', 0, 1, @now, @rc) WITH NOWAIT
      END -- @verbose >= 1

      -- New exceptions
      IF EXISTS (SELECT 1 FROM @ashist WHERE cid = 'LOB' AND evt_id = 0 AND evt_alertstate > 1)
      BEGIN
        UPDATE i SET evt_id = (
          SELECT ISNULL(MAX(c.id),0)
          FROM Exception_V AS c
          WHERE c._ParentCID  = i.phy_cid
            AND c._ParentID   = i.phy_id
            AND c._Priority   = i.evt_priority
            AND c._AlertState = i.evt_alertstate
            AND c._flags & 16 = 0
            AND c.deleted     = 0
        )
        FROM @ashist AS i
        WHERE i.cid = 'LOB'
          AND i.evt_id = 0
          AND i.evt_alertstate > 1
        SET @rc = @@ROWCOUNT
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: tU_ObjAlertCount: Updated exception id for %d linked LOB row(s) of AlertStateHistory inserts', 0, 1, @now, @rc) WITH NOWAIT
        END -- @verbose >= 1
      END -- New exceptions

      -- Cleared exceptions
      IF EXISTS (SELECT 1 FROM @ashist WHERE cid = 'LOB' AND evt_id = 0 AND evt_alertstate < 0)
      BEGIN
        UPDATE i SET evt_id = (
          SELECT ISNULL(MAX(c.id),0)
          FROM Exception_V AS c
          WHERE c._ParentCID  = i.phy_cid
            AND c._ParentID   = i.phy_id
            AND c._Priority   = i.evt_priority
            AND c._AlertState = (-1*i.evt_alertstate)
            AND c._flags & 16 = 0
            AND c.deleted     = 1
        )
        FROM @ashist AS i
        WHERE i.cid = 'LOB'
          AND i.evt_id = 0
          AND i.evt_alertstate < 0
        SET @rc = @@ROWCOUNT
        IF @verbose >= 1
        BEGIN
          SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
          RAISERROR('%s: tU_ObjAlertCount: Updated cleared exception id for %d linked LOB row(s) of AlertStateHistory inserts', 0, 1, @now, @rc) WITH NOWAIT
        END -- @verbose >= 1
      END -- Cleared exceptions
    END -- If we still have unresolved exception ids, then lookup LOBs linked to objects with non-LOB only exceptions

    -- Lookup latest exception history id
    UPDATE a SET exch_id = (
      SELECT MAX(h.id)
      FROM ExceptionHistory_V AS h
      WHERE h._ParentID = a.evt_id
    )
    FROM @ashist AS a
    WHERE a.evt_cid = 'EXCP'
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tU_ObjAlertCount: Updated exception history id for %d row(s) of AlertStateHistory inserts', 0, 1, @now, @rc) WITH NOWAIT
    END -- @verbose >= 1

    -- Lookup exception details
    UPDATE a SET
      evt_ctime  = h._TIME,
      evt_detail = h._EXCP_CD,
      evt_name   = e._EXCP_NAME
    FROM @ashist AS a
    JOIN Exception_V AS e WITH (NOLOCK) ON e.id = a.evt_id
    JOIN ExceptionHistory_V AS h WITH (NOLOCK) ON h.id = a.exch_id
    WHERE a.evt_cid = 'EXCP'
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tU_ObjAlertCount: Looked up exception details for %d row(s) of AlertStateHistory inserts', 0, 1, @now, @rc) WITH NOWAIT
      IF @verbose >= 2
        SELECT '@ashist' AS '@ashist', * FROM @ashist
    END -- @verbose >= 1
  END -- Fill in exception information
      
  -- Insert into the AlertStateHistory table for ChildEvents
  INSERT INTO @ashist (cid, id, AlertStateID, OldAlertStateID, evt_cid, evt_id, evt_priority,
    evt_alertstate, evt_ctime, evt_detail)
  SELECT DISTINCT delta.cid, delta.id, delta.alertstate, delta.old_alertstate, i.event_cid,
    CASE i.nviolations WHEN 0 THEN
      ISNULL((
        SELECT MAX(c.id)
        FROM ChildEvent_C AS c 
        WHERE c._ParentCID = i.cid AND c._ParentID = i.id AND c._AlertState = i.alertstate AND c._Priority = i.priority
          AND c.deleted = 1
          AND dtime = (
            SELECT MAX(dtime)
            FROM ChildEvent_C AS c_i
            WHERE c_i._ParentCID = i.cid AND c_i._ParentID = i.id
              AND c_i._AlertState = i.alertstate AND c_i._Priority = i.priority
              AND c_i.deleted = 1
          )
      ), 0)
    ELSE
      ISNULL((
        SELECT MAX(c.id)
        FROM ChildEvent_C AS c 
        WHERE c._ParentCID = i.cid AND c._ParentID = i.id AND c._AlertState = i.alertstate AND c._Priority = i.priority
          AND c.deleted = 0
      ), 0)
    END AS 'evt_id',
    i.priority, CASE WHEN d.nviolations > i.nviolations THEN -1*i.alertstate ELSE i.alertstate END,
    CURRENT_TIMESTAMP, ''
  FROM ObjAlertCountDelta AS delta
    JOIN ObjAlertCountWT AS i ON i.cid = delta.cid AND i.id = delta.id AND i.trigger_action = 0
    JOIN ObjAlertCountWT AS d ON d.cid = i.cid AND d.id = i.id
      AND i.event_cid = d.event_cid AND i.alertstate = d.alertstate AND i.priority = d.priority
      AND d.trigger_action = 1
  WHERE ((i.nviolations > 0 AND d.nviolations = 0) OR (d.nviolations > 0 AND i.nviolations = 0))
    AND i.event_cid = 'CHEV'
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Computed %d row(s) of inserts into AlertStateHistory for ChildEvents', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Lookup ChildEvent details
  IF @rc > 0
  BEGIN
    UPDATE a SET
      evt_ctime  = CASE c.deleted WHEN 0 THEN c.ctime ELSE c.mtime END,
      evt_detail = ISNULL(CASE c.deleted WHEN 0 THEN c._EventDetail ELSE c._DeleteDetail END,'')
    FROM @ashist AS a
    JOIN ChildEvent_V AS c WITH (NOLOCK) ON c.id = a.evt_id
    WHERE a.evt_cid = 'CHEV'
    SET @rc = @@ROWCOUNT
    IF @verbose >= 1
    BEGIN
      SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
      RAISERROR('%s: tU_ObjAlertCount: Looked up child event details for %d row(s) of AlertStateHistory inserts', 0, 1, @now, @rc) WITH NOWAIT
    END -- @verbose >= 1    
  END -- Lookup ChildEvent details

  INSERT INTO AlertStateHistory(cid, id, AlertStateID, OldAlertStateID, evt_cid, evt_id, evt_priority, evt_alertstate, evt_ctime, evt_detail, evt_name)
  SELECT cid, id, AlertStateID, OldAlertStateID, evt_cid, evt_id, evt_priority, evt_alertstate, evt_ctime, evt_detail, evt_name
  FROM @ashist
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Inserted %d total row(s) into AlertStateHistory', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Update the mtime column
  UPDATE oac SET mtime = CURRENT_TIMESTAMP
  FROM ObjAlertCountWT AS i
    JOIN ObjAlertCount AS oac ON oac.cid = i.cid AND oac.id = i.id
      AND i.event_cid = oac.event_cid AND i.alertstate = oac.alertstate AND i.priority = oac.priority
      AND i.trigger_action = 0
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Updated the mtime column for %d row(s).', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Insert into the RootCauseState table
  INSERT INTO RootCauseState (cid, id, alertstate, mtime)
  SELECT d.cid, d.id, d.alertstate, CURRENT_TIMESTAMP
  FROM ObjAlertCountDelta AS d
    LEFT OUTER JOIN RootCauseState AS r ON r.cid = d.cid AND r.id = d.id
  WHERE r.cid IS NULL
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Inserted %d entries into the RootCauseState table.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- Update the RootCauseState table
  UPDATE r
  SET alertstate = d.alertstate, mtime = CURRENT_TIMESTAMP
  FROM ObjAlertCountDelta AS d
    JOIN RootCauseState AS r ON r.cid = d.cid AND r.id = d.id
  WHERE d.old_alertstate <> d.alertstate
    AND r.alertstate <> d.alertstate
  SET @rc = @@ROWCOUNT
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Updated %d entries in the RootCauseState table.', 0, 1, @now, @rc) WITH NOWAIT
  END -- @verbose >= 1

  -- cleanup work tables
  TRUNCATE TABLE ObjAlertCountWT
  TRUNCATE TABLE ObjAlertCountDelta
  IF @verbose >= 1
  BEGIN
    SET @now = CONVERT(NVARCHAR, CURRENT_TIMESTAMP, 121)
    RAISERROR('%s: tU_ObjAlertCount: Truncated ObjAlertCountWT and ObjAlertCountDelta.', 0, 1, @now) WITH NOWAIT
  END -- @verbose >= 1
GO
