-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/twh_data.sql
-- SID: 1.2
-- Date Modified: 04/09/03 09:40:48

-- Create the twh_data table and populate with the default data values.

USE Object
IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = 'twh_data') BEGIN
  CREATE TABLE twh_data (
    name   NVARCHAR(64) NOT NULL PRIMARY KEY, -- The name the data is known by.  This should be one of the 
                                              -- following: max_resource_depth, triggers_enabled
    value  NVARCHAR(2000),                    -- The value of the datum.  A value of NULL will appear as
                                              -- the empty string.  An numeric value is converted to NVARCHAR.
    type   NVARCHAR(16) NOT NULL,             -- The type of data held in this row.  The possible values will be
                                              -- 'CHAR' and 'INT'
    ctime  DATETIME NOT NULL DEFAULT(CURRENT_TIMESTAMP), -- The time when the row was added to the table
    mtime  DATETIME NOT NULL DEFAULT(CURRENT_TIMESTAMP)  -- The time when the row was last updated
  )

  INSERT INTO twh_data (name,value,type)
  SELECT 'max_resource_depth', '0', 'INT'

  INSERT INTO twh_data (name,value,type)
  SELECT 'triggers_enabled', '0', 'INT'
END

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_twh_data' AND type = 'TR')
  DROP TRIGGER tU_twh_data

PRINT 'Creating trigger tU_twh_data'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/twh_data.sql
-- SID: 1.2
-- Date Modified: 04/09/03 09:40:48

-- Update the mtime column whenever another column in the table is updated.

CREATE TRIGGER tU_twh_data ON twh_data FOR UPDATE WITH APPEND
AS

  IF (UPDATE(name) OR UPDATE(value) OR UPDATE(type))
    UPDATE data SET mtime = CURRENT_TIMESTAMP
      FROM twh_data data
      JOIN inserted i    ON i.name = data.name
GO