-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/twh_defer_ash.sql
-- SID: 1.3
-- Date Modified: 04/12/01 09:57:10

-- Create the table to hold rows from the
-- etl_getash result set that do not have
-- a comp_id yet.

USE Object
IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = 'twh_defer_ash') BEGIN
  PRINT 'Creating twh_defer_ash'

  CREATE TABLE twh_defer_ash (
    ashid            INT NOT NULL,            -- The id associated with the alert state history. This will
                                              -- come from the ash_id column of the AlertStateHistory table.
    AlertStateID     INT NOT NULL,            -- The new TBSM alert state associated with the event: 1=Green, 
                                              -- 2=Yellow, 3=Red. This will come from the AlertStateID column 
                                              -- of the AlertStateHistory table
    OldAlertStateID  INT NOT NULL,            -- The old TBSM alert state associated with the event: 1=Green, 
                                              -- 2=Yellow, 3=Red. This will come from the OldAlertStateID column of the AlertStateHistory table
    ctime            DATETIME NOT NULL,       -- The time the alert state history record was created.
    event_type       NVARCHAR(32) NOT NULL,   -- `Alert State Change'
    evt_corr_val     NVARCHAR(40) NOT NULL,   -- `ASH\' + ashid
    twh_ash_id       INT NOT NULL,            -- 0
    sevt_corr_val    NVARCHAR(40) NOT NULL,   -- evt_cid + `\' + evt_id
    twh_sevt_id      INT NOT NULL,            -- 0
    cid              ClassID NOT NULL,        -- Class ID of the resource the alert state history record is 
                                              -- associated with. This isn't really necessary for TDW so it isn't 
                                              -- included in the result set but it simplifies things for TBSM 
                                              -- when we don't know the comp_id and have to defer the result set 
                                              -- row.
    id               ObjID NOT NULL,          -- Object ID of the resource te alert state history record is 
                                              -- associated with. This isn't really necessary for TDW but it 
                                              -- simplifies things for TBSM when we don't know the comp_id and 
                                              -- have to defer the result set row.
    comp_id          BIGINT NOT NULL,         -- The TDW component id associated with the resource the event 
                                              -- belongs to. This will come from the comp_id column of the 
                                              -- twh_Inventory table, based on the cid and id of the resource 
                                              -- the alert state history record is associated with.
                                              -- The value can be zero in this table, which means that the
                                              -- comp_id is not known yet.  It might also be non-zero which
                                              -- means it was included in the last result set but TDW has not
                                              -- acknowledged successful and complete processing of the result
                                              -- set so it hasn't been removed from the deferred table yet.
    age              INT NOT NULL DEFAULT(0), -- the number of times the event has stayed in the deferred table
    evt_detail       NVARCHAR(255) NOT NULL   -- from the AlertStateHistory table
  )
END
