-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/twh_defer_events.sql
-- SID: 1.5
-- Date Modified: 04/10/22 13:31:47

-- Create the table to hold rows from the
-- etl_getevents result set that do not have
-- a comp_id yet.

USE Object
IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = 'twh_defer_events') BEGIN
  PRINT 'Creating twh_defer_events'

  CREATE TABLE twh_defer_events (
    obj_cid             ClassID NOT NULL,        -- Class id of the resource the event belongs to
    obj_id              ObjID NOT NULL,          -- Object id of the resource the event belongs to
    comp_id             BIGINT NOT NULL,         -- The TDW component id associated with the resource the event belongs to. 
                                              -- This will come from the comp_id column of the twh_Inventory table, based
                                              -- on the cid and id of the resource the event belongs to.
                                              -- This value will usually be zero - that's why it got deferred in this
                                              -- table.  When it can be resolved using twh_Inventory, we'll take it out
                                              -- of this table.  Rows with a non-zero comp_id are also possible: these
                                              -- are rows that have been resolved and sent to TDW but TDW has not acknowledged
                                              -- successful processing of the result set yet.
    cid                 ClassID NOT NULL,        -- Class id of the event (`MESG', `EXCP', or `CHEV')
    id                  ObjID NOT NULL,          -- Object id of the event
    event_type          ObjName NOT NULL,        -- This will come from the twh column of the twh_tbsm_map table, based on the event cid
    name                NVARCHAR(255) NOT NULL,  -- Name associated with the event
    ctime               DATETIME NOT NULL,       -- The time the event was created
    detail              NVARCHAR(255) NULL,      -- 
    alertstate          INT NOT NULL,            -- The TBSM alert state associated with the event: 1=Green, 2=Yellow, 3=Red.
    priority            INT NOT NULL,            -- The TBSM priority associated with the event: 1=Critical, 2=High, 3=Medium, 4=Low, 5=Ignore.
    twh_cid_id_val      NVARCHAR(40) NULL,       -- cid + `\' + CONVERT(NVARCHAR(40), evt_id) or NULL if evt_id = 0
    twh_evt_corr_val    NVARCHAR(40),            -- `gtmevt\' + CONVERT(NVARCHAR(40), id)
    twh_evt_id          INT NOT NULL,
    age                 INT NOT NULL DEFAULT(0)  -- the number of times the event has stayed in the deferred table
  )
END
