-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/twh_extractctrl.sql
-- SID: 1.10
-- Date Modified: 04/10/21 09:07:12

-- Create the table to control the TDW extraction process.
-- It contains extraction id's, the status of result sets,
-- etc.

USE Object
IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = 'twh_ExtractCtrl') BEGIN
  PRINT 'Creating twh_ExtractCtrl'

  CREATE TABLE twh_ExtractCtrl (
    name             NVARCHAR(64) NOT NULL,       -- The type of result set this control record is for.
                                                  -- The value will be made up of the procedure name without 
                                                  -- the "etl_get" prefix.
    xtable           NVARCHAR(64) NOT NULL,       -- The extraction table contributing to the result set.
    xid              NVARCHAR(16) NOT NULL,       -- The name of the extraction id column in xtable.
    prune            TINYINT NOT NULL,            -- Indicates whether or not etl_prepare should prune 
                                                  -- old rows or not.
    status           TINYINT NOT NULL DEFAULT(0), -- 1 = Extraction data is prepared and TDW has not successfully 
                                                  -- received it.
                                                  -- 0 = No extraction data is waiting for TDW to receive.
    last_xid         DECIMAL(18) NULL DEFAULT(0), -- The extraction id of the most recent extraction record 
                                                  -- successfully received by TDW.  This column is set by the 
                                                  -- tU_twh_ExtractCtrl table trigger when the status column is 
                                                  -- changed to 0 by TDW
    last_mtime       DATETIME NULL DEFAULT(NULL), -- The database time when last_xid was set.
    this_xid         DECIMAL(18) NULL DEFAULT(0), -- The extraction id of the most recent extraction record prepared for TDW.  
                                                  -- This column is set when etl_getnewpubobjs, etl_getnewprivobjs, 
                                                  -- etl_getupdateattr, or etl_getdeletedobjs are called.
    this_mtime       DATETIME NULL DEFAULT(NULL), -- The database time when this_xid was set.
    receipt_mtime    DATETIME NULL DEFAULT(NULL)  -- The database time when TDW last set the status flag to 0.
  )

  INSERT INTO twh_ExtractCtrl (name, xtable, xid, prune)
    SELECT 'newpubobjs', 'twh_ResourceCreate', 'xid', 1

  INSERT INTO twh_ExtractCtrl (name, xtable, xid, prune)
    SELECT 'newprivobjs', 'twh_ResourceCreate', 'xid', 1

  INSERT INTO twh_ExtractCtrl (name, xtable, xid, prune)
    SELECT 'updateattr', 'twh_AttrUpdate', 'xid', 1

  INSERT INTO twh_ExtractCtrl (name, xtable, xid, prune)
    SELECT 'deletedobjs', 'twh_ResourceDelete', 'xid', 1

  INSERT INTO twh_ExtractCtrl (name, xtable, xid, prune)
    SELECT 'deletedlinks', 'twh_LinkDelete', 'xid', 1

  INSERT INTO twh_ExtractCtrl (name, xtable, xid, prune)
    SELECT 'newpubobjs', 'twh_LinkCreate', 'xid', 1

  INSERT INTO twh_ExtractCtrl (name, xtable, xid, prune)
    SELECT 'newprivobjs', 'twh_LinkCreate', 'xid', 1

  INSERT INTO twh_ExtractCtrl (name, xtable, xid, prune)
    SELECT 'events', 'event', 'tstamp', 0

  INSERT INTO twh_ExtractCtrl (name, xtable, xid, prune)
    SELECT 'ash', 'AlertStateHistory', 'ash_id', 0

  INSERT INTO twh_ExtractCtrl (name, xtable, xid, prune)
    SELECT 'notehistory', 'NoteHistory', 'ident', 0
END

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'tU_twh_ExtractCtrl' AND type = 'TR')
  DROP TRIGGER tU_twh_ExtractCtrl

PRINT 'Creating trigger tU_twh_ExtractCtrl'
GO

-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/twh_extractctrl.sql
-- SID: 1.10
-- Date Modified: 04/10/21 09:07:12

-- Update timestamps when various columns are updated.

CREATE TRIGGER tU_twh_ExtractCtrl ON twh_ExtractCtrl FOR UPDATE WITH APPEND
AS

  -- update last_mtime when last_xid is changed
  IF UPDATE(last_xid)
    UPDATE data SET last_mtime = CURRENT_TIMESTAMP
      FROM twh_ExtractCtrl AS data
      JOIN inserted        i    ON i.name = data.name

  -- update this_mtime when this_xid is changed
  IF UPDATE(this_xid)
    UPDATE data SET this_mtime = CURRENT_TIMESTAMP
      FROM twh_ExtractCtrl AS data
      JOIN inserted        i    ON i.name = data.name

  -- update receipt_mtime when status is changed to 0
  IF UPDATE(status)
  BEGIN
    UPDATE data SET receipt_mtime = CURRENT_TIMESTAMP, data.last_xid = data.this_xid, data.last_mtime = CURRENT_TIMESTAMP
      FROM twh_ExtractCtrl AS data
      JOIN inserted        i    ON i.name = data.name
      JOIN deleted         d    ON d.name = i.name
      WHERE i.status = 0 AND d.status = 1

    -- the twh_date table has some extraction timestamps that should also be updated when certain result sets are processed
    UPDATE date SET prevlastenddate = lastenddate
      FROM twh_date AS date
      JOIN inserted        i    ON date.loadtype = 'link/' + i.name
      JOIN deleted         d    ON d.name = i.name
      WHERE i.status = 0 AND d.status = 1

    -- the twh_gmt_time table has gmt-offsets that should also be updated when result sets are processed
    UPDATE gmt_time SET last_gmt = this_gmt, last_date = this_date, mtime = CURRENT_TIMESTAMP
      FROM twh_gmt_time AS gmt_time
      JOIN inserted        i    ON gmt_time.loadtype = i.name
      JOIN deleted         d    ON d.name = i.name
      WHERE i.status = 0 AND d.status = 1
  END
GO