-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2003 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/twh_gmt_time.sql
-- SID: 1.4
-- Date Modified: 04/10/22 13:00:04

-- Module Description
--   Create table twh_gmt_time

USE Object
GO

PRINT 'Creating table twh_gmt_time'
GO
 
IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'twh_gmt_time' AND type = 'U')
BEGIN
  EXEC('CREATE TABLE twh_gmt_time(
        loadtype        NVARCHAR(50),
	last_gmt	INT,
        last_date       DATETIME,
	this_gmt	INT,
        this_date       DATETIME,
	mtime		DATETIME,
        status          TINYINT)')
END
GO

IF NOT EXISTS (SELECT * 
                 FROM syscolumns c
                 JOIN systypes   t ON t.xusertype = c.xusertype
                 WHERE c.id = OBJECT_ID('twh_gmt_time') AND c.name = 'loadtype')
  ALTER TABLE twh_gmt_time ADD loadtype NVARCHAR(50)

IF NOT EXISTS (SELECT * 
                 FROM syscolumns c
                 JOIN systypes   t ON t.xusertype = c.xusertype
                 WHERE c.id = OBJECT_ID('twh_gmt_time') AND c.name = 'last_date')
  ALTER TABLE twh_gmt_time ADD last_date DATETIME

IF NOT EXISTS (SELECT * 
                 FROM syscolumns c
                 JOIN systypes   t ON t.xusertype = c.xusertype
                 WHERE c.id = OBJECT_ID('twh_gmt_time') AND c.name = 'this_date')
  ALTER TABLE twh_gmt_time ADD this_date DATETIME
GO

DECLARE @gmoff INT
DECLARE @now DATETIME
DECLARE @gmt DATETIME

-- figure out current GMT offset
SELECT @now = CURRENT_TIMESTAMP, @gmt = GETUTCDATE()
SET @gmoff = DATEDIFF(mi, @now, @gmt)

RAISERROR('twh_gmt_time.sql calculated current GMT offset to be %d minutes (add this amount to a local time to arrive at GMT)', 0, 1, @gmoff)

IF NOT EXISTS (SELECT * FROM twh_gmt_time WHERE loadtype = 'ash')
  INSERT INTO twh_gmt_time (loadtype, last_gmt, last_date, this_gmt, this_date, mtime, status)
    SELECT 'ash', @gmoff, @now, NULL, NULL, NULL, 0

IF NOT EXISTS (SELECT * FROM twh_gmt_time WHERE loadtype = 'deletedobjs')
  INSERT INTO twh_gmt_time (loadtype, last_gmt, last_date, this_gmt, this_date, mtime, status)
    SELECT 'deletedobjs', @gmoff, @now, NULL, NULL, NULL, 0

IF NOT EXISTS (SELECT * FROM twh_gmt_time WHERE loadtype = 'deletelinks')
  INSERT INTO twh_gmt_time (loadtype, last_gmt, last_date, this_gmt, this_date, mtime, status)
    SELECT 'deletelinks', @gmoff, @now, NULL, NULL, NULL, 0

IF NOT EXISTS (SELECT * FROM twh_gmt_time WHERE loadtype = 'events')
  INSERT INTO twh_gmt_time (loadtype, last_gmt, last_date, this_gmt, this_date, mtime, status)
    SELECT 'events', @gmoff, @now, NULL, NULL, NULL, 0

IF NOT EXISTS (SELECT * FROM twh_gmt_time WHERE loadtype = 'newprivobjs')
  INSERT INTO twh_gmt_time (loadtype, last_gmt, last_date, this_gmt, this_date, mtime, status)
    SELECT 'newprivobjs', @gmoff, @now, NULL, NULL, NULL, 0

IF NOT EXISTS (SELECT * FROM twh_gmt_time WHERE loadtype = 'newpubobjs')
  INSERT INTO twh_gmt_time (loadtype, last_gmt, last_date, this_gmt, this_date, mtime, status)
    SELECT 'newpubobjs', @gmoff, @now, NULL, NULL, NULL, 0

IF NOT EXISTS (SELECT * FROM twh_gmt_time WHERE loadtype = 'notehistory')
  INSERT INTO twh_gmt_time (loadtype, last_gmt, last_date, this_gmt, this_date, mtime, status)
    SELECT 'notehistory', @gmoff, @now, NULL, NULL, NULL, 0

IF NOT EXISTS (SELECT * FROM twh_gmt_time WHERE loadtype = 'updateattr')
  INSERT INTO twh_gmt_time (loadtype, last_gmt, last_date, this_gmt, this_date, mtime, status)
    SELECT 'updateattr', @gmoff, @now, NULL, NULL, NULL, 0

PRINT 'Done.'
GO
