-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/twh_inventory.sql
-- SID: 1.5
-- Date Modified: 04/10/22 13:31:48

-- Create the table to be populated by TDW and used by
-- the etl_* procedures to perform synchronization and
-- obtain TDW component ids.
USE Object
IF NOT EXISTS (SELECT * FROM sysobjects WHERE type = 'U' AND name = 'twh_Inventory') BEGIN
  PRINT 'Creating twh_Inventory'

  CREATE TABLE twh_Inventory (
    cid              NVARCHAR(40) NOT NULL,               -- The class identifier of the resource type
    id               ObjID NOT NULL PRIMARY KEY(cid, id), -- The object id of the resource
    comp_id          DECIMAL(18,0) NOT NULL,              -- TDW component id for the resource
    name             NVARCHAR(255) NOT NULL,              -- The display name of the resource
    description      NVARCHAR(255) NULL,                  -- The description of the resource
    deleted          TINYINT DEFAULT (0),
    ctime            DATETIME DEFAULT(CURRENT_TIMESTAMP), -- The database time when the row was added to the table.
    mtime            DATETIME DEFAULT(CURRENT_TIMESTAMP)  -- The database time when the row was last updated.
  )

  CREATE INDEX twh_Inventory_comp_id ON twh_Inventory (comp_id)
END

GO

IF EXISTS (SELECT * FROM sysobjects WHERE type = 'TR' AND name = 'tU_twh_Inventory')
  DROP TRIGGER tU_twh_Inventory
PRINT 'Creating trigger tU_twh_Inventory'
GO

-- File: archive/db/sql/tables/twh_inventory.sql
-- SID: 1.5
-- Date Modified: 04/10/22 13:31:48

CREATE TRIGGER tU_twh_Inventory ON twh_Inventory FOR UPDATE WITH APPEND
AS
  IF NOT UPDATE(mtime)
    UPDATE inventory SET mtime = CURRENT_TIMESTAMP
      FROM twh_Inventory inventory
      JOIN inserted i ON i.cid = inventory.cid AND i.id = inventory.id
GO
