-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                       
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive/db/sql/tables/twh_tbsm_map.sql
-- SID: 1.1.2.6 
-- Date Modified: 04/09/09 18:11:04
-- Module Description
--   Create table ObjectStateHist
-- This table holds static data for mapping TBSM data model into TDW data model for reaources:
-- TBSM classes to TDW comp types
-- 2. TBSM object states to availibility state 
USE Object
GO

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'twh_tbsm_map' AND type = 'U')
BEGIN
  PRINT 'Creating table twh_tbsm_map'
  CREATE TABLE twh_tbsm_map
  (
	  id		      ObjID NOT NULL PRIMARY KEY IDENTITY(1,1),
  	map_type		ObjName, -- ObjectType, MSMTType, RelnType
  	map_type_id	ObjID,   -- ObjectType=1, MSMTType=3, RelnType=2
	  twh		      ObjName,
	  tbsm		    ObjName,
	  ctime		    DATETIME,
	  enabled		  TINYINT,
    priority    TINYINT
  )
END

IF NOT EXISTS (SELECT * FROM sysobjects WHERE name = 'u_twh_tbsm_map')
  ALTER TABLE twh_tbsm_map ADD CONSTRAINT u_twh_tbsm_map UNIQUE (map_type_id, twh, tbsm)
GO

IF NOT EXISTS (SELECT * FROM twh_tbsm_map)
BEGIN
  DECLARE @tcount INT
  SET @tcount = @@TRANCOUNT
  IF @tcount = 0 
    BEGIN TRANSACTION insert_data
  ELSE
    SAVE TRANSACTION insert_data

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'ObjectType', 1, 'BUSINESS_SYSTEM', 'LOB', CURRENT_TIMESTAMP, 1, 1 )

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'ObjectType', 1, 'SERVICE', 'EXES ', CURRENT_TIMESTAMP, 1, 1 )

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'ObjectType', 1, 'GTM_LOBC', 'LOBC', CURRENT_TIMESTAMP, 1, 1 )

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'ObjectType', 1, 'GTM_EXEI', 'EXEI', CURRENT_TIMESTAMP, 1, 1 )

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'ObjectType', 1, 'GTM_EXEU', 'EXEU', CURRENT_TIMESTAMP, 1, 1 )

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'RelnType', 2, 'PCHILD', 'LOBC_like', CURRENT_TIMESTAMP, 1, 1 )

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'RelnType', 2, 'SUPPRT', 'EVSL', CURRENT_TIMESTAMP, 1, 1 )

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'RelnType', 2, 'PCHILD', 'EVSS', CURRENT_TIMESTAMP, 1, 1 )

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'MSMTType', 2, 'Unavailable', 'Red', CURRENT_TIMESTAMP, 1, 1 )

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'MSMTType', 2, 'Degrading', 'Yellow', CURRENT_TIMESTAMP, 1, 1 )

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'MSMTType', 2, 'Available', 'Green', CURRENT_TIMESTAMP, 1, 1 )

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'MSMTType', 2, 'Unmanaged', 'Maint', CURRENT_TIMESTAMP, 1, 3 )

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'MSMTType', 2, 'Repairing', 'Owned', CURRENT_TIMESTAMP, 1, 2 )

  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'MSMTType', 2, 'Unknown', 'Unknown', CURRENT_TIMESTAMP, 1, 1 )

  IF @tcount = 0
    COMMIT TRANSACTION insert_data

END

IF NOT EXISTS (SELECT 1 FROM twh_tbsm_map WHERE map_type = 'RelnType' AND tbsm = 'EVSS')
  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'RelnType', 2, 'PCHILD', 'EVSS', CURRENT_TIMESTAMP, 1, 1 )

-- event type mapping
IF NOT EXISTS (SELECT 1 FROM twh_tbsm_map WHERE map_type = 'EventType' AND tbsm = 'MESG')
  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'EventType', 3, 'Message', 'MESG', CURRENT_TIMESTAMP, 1, 1 )
IF NOT EXISTS (SELECT 1 FROM twh_tbsm_map WHERE map_type = 'EventType' AND tbsm = 'EXCH')
  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'EventType', 3, 'Exception', 'EXCP', CURRENT_TIMESTAMP, 1, 1 )
IF NOT EXISTS (SELECT 1 FROM twh_tbsm_map WHERE map_type = 'EventType' AND tbsm = 'CHVH')
  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'EventType', 3, 'Child Event', 'CHEV', CURRENT_TIMESTAMP, 1, 1 )
-- cover the previous data
IF NOT EXISTS (SELECT 1 FROM twh_tbsm_map WHERE map_type = 'EventType' AND tbsm = 'EXCH')
  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'EventType', 3, 'Exception', 'EXCH', CURRENT_TIMESTAMP, 1, 1 )
IF NOT EXISTS (SELECT 1 FROM twh_tbsm_map WHERE map_type = 'EventType' AND tbsm = 'CHVH')
  INSERT INTO twh_tbsm_map(map_type, map_type_id, twh, tbsm, ctime, enabled, priority)
    VALUES ( 'EventType', 3, 'Child Event', 'CHVH', CURRENT_TIMESTAMP, 1, 1 )