-------------------------------------------------- {COPYRIGHT-TOP-RM}
-- Licensed Materials - Property of IBM                              
--                                                                   
-- Restricted Materials of IBM                                     
--                                                                   
-- 5724-C51                                                          
--                                                                   
-- (C) Copyright IBM Corp. 1999,2004 All Rights Reserved.            
--                                                                   
-- US Government Users Restricted Rights - Use, duplication or       
-- disclosure restricted by GSA ADP Schedule Contract with           
-- IBM Corp.                                                         
-------------------------------------------------- {COPYRIGHT-END-RM}
-- File: archive\db\sql\tracks\schedule_deleteoldevent.sql
-- SID: 1.1
-- Date Modified: 04/10/19 08:27:21
--
-- Module Description:
--  This file will create one scheduled task used to clean up a table used
--  by the TDW and traditional TBSM reporting systems:
--    - "Delete old entries in event table" job to Delete old entries in event table every day.
--  The job is designed to be run once a day in all configurations:
--    1) Only traditional TBSM used
--    2) Only TDW used
--    3) Both traditional TBSM and TDW used
--    4) Neither traditional TBSM nor TDW used

BEGIN TRANSACTION
   DECLARE @JobID BINARY(16)
   DECLARE @ReturnCode INT
   SELECT @ReturnCode = 0

IF (SELECT COUNT(*) FROM msdb.dbo.syscategories WHERE name = N'History Server Maintenance') < 1
   EXECUTE msdb.dbo.sp_add_category @name = N'History Server Maintenance'

IF (SELECT COUNT(*) FROM msdb.dbo.sysjobs WHERE name = N'Delete old entries in event table') > 0
BEGIN
   PRINT 'The job "Delete old entries in event table" is already defined.  It will not be changed.'
END

ELSE

BEGIN
PRINT N'Creating job "Delete old entries in event table"'

   -- Add the job
   EXECUTE @ReturnCode = msdb.dbo.sp_add_job @job_id = @JobID OUTPUT,
	@job_name = N'Delete old entries in event table',
	@owner_login_name = N'sa',
        @description = N'No description available',
        @category_name = N'History Server Maintenance',
        @enabled = 1,
	@notify_level_email = 0,
	@notify_level_page = 0,
	@notify_level_netsend = 0,
	@notify_level_eventlog = 2,
	@delete_level = 0
   IF (@@ERROR <> 0 OR @ReturnCode <>0) GOTO QuitWithRollback

   -- Add the job step
   EXECUTE @ReturnCode = msdb.dbo.sp_add_jobstep @job_id = @JobID,
	@step_id = 1,
	@step_name = N'Delete old entries in event table',
	@command = N'asisp_deleteoldevent',
	@database_name = N'Object',
	@server = N'',
	@database_user_name = N'',
	@subsystem = N'TSQL',
	@cmdexec_success_code = 0,
	@flags = 0,
	@retry_attempts = 0,
	@retry_interval = 1,
	@output_file_name = N'',
	@on_success_step_id = 0,
	@on_success_action = 1,
	@on_fail_step_id = 0,
	@on_fail_action = 2
   IF (@@ERROR <> 0 OR @ReturnCode <>0) GOTO QuitWithRollback

   -- Identify which step to start at 
   EXECUTE @ReturnCode = msdb.dbo.sp_update_job @job_id = @JobID, @start_step_id = 1
   IF (@@ERROR <> 0 OR @ReturnCode <>0) GOTO QuitWithRollback

   -- Add the job schedule
   EXECUTE @ReturnCode = msdb.dbo.sp_add_jobschedule @job_id = @JobID,
	@name = N'Copy Interval',
	@enabled = 1,
	@freq_type = 4,                          -- When job is to be run: 4=Daily
	@freq_interval = 1,                      -- Days the job is to be run: 1=Daily
	@freq_subday_type = 1,                   -- 1=Run at the specified time
	@freq_subday_interval = 0,               
	@freq_relative_interval = 0,
	@freq_recurrence_factor = 0,
 	@active_start_date = 20000410,
	@active_start_time = 010000,             -- Run at One a.m. 
	@active_end_date = 99991231,
	@active_end_time = 235959
   IF (@@ERROR <> 0 OR @ReturnCode <>0) GOTO QuitWithRollback

   -- Add the Target Servers	
   EXECUTE @ReturnCode = msdb.dbo.sp_add_jobserver @job_id = @JobID, @server_name = N'(local)'
   IF (@@ERROR <> 0 OR @ReturnCode <>0) GOTO QuitWithRollback

END
COMMIT TRANSACTION
GOTO EndSave
QuitWithRollback:
   IF (@@TRANCOUNT > 0) ROLLBACK TRANSACTION
EndSave:
GO
